@@ -31,6 +31,10 @@
 #include <algorithm>
 #include <string>
 
+#if __cplusplus >= 201703L
+#include <filesystem>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,16 +42,30 @@
 #include <sys/stat.h> /* for stat() */
 
 #ifdef _WIN32
+#define _CRT_INTERNAL_NONSTDC_NAMES 1
+#define WIN32_LEAN_AND_MEAN
+#ifndef NOMINMAX
+#define NOMINMAX
+#endif
 #include <windows.h>
 #endif
+#if !defined(S_ISREG) && defined(S_IFMT) && defined(S_IFREG)
+#define S_ISREG(m) (((m) & S_IFMT) == S_IFREG)
+#endif
+#if !defined(S_ISDIR) && defined(S_IFMT) && defined(S_IFDIR)
+#define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
+#endif
 
 #include "glk.h"
 #include "garglk.h"
 #include "glkstart.h"
 
+#include "format.h"
+#include "optional.hpp"
+
 #ifdef GARGLK
 std::string gli_workdir = ".";
-std::string gli_workfile;
+nonstd::optional<std::string> gli_workfile;
 
 const char *garglk_fileref_get_name(fileref_t *fref)
 {
@@ -169,9 +187,9 @@ frefid_t glk_fileref_create_temp(glui32 usage, glui32 rock)
 #ifdef _WIN32
     char tempdir[MAX_PATH];
     char filename[MAX_PATH];
-    GetTempPath(MAX_PATH, tempdir);
-    if(GetTempPath(MAX_PATH, tempdir) == 0 ||
-       GetTempFileName(tempdir, "glk", 0, filename) == 0)
+    GetTempPathA(MAX_PATH, tempdir);
+    if(GetTempPathA(MAX_PATH, tempdir) == 0 ||
+       GetTempFileNameA(tempdir, "glk", 0, filename) == 0)
     {
         gli_strict_warning("fileref_create_temp: unable to create temporary file");
         return NULL;
@@ -196,7 +214,7 @@ frefid_t glk_fileref_create_temp(glui32 usage, glui32 rock)
     fileref_t *fref;
     
     sprintf(filename, "/tmp/glktempfref-XXXXXX");
-    mktemp(filename);
+    close(mkstemp(filename));
     
 #endif
 
@@ -247,10 +265,16 @@ frefid_t glk_fileref_create_by_name(glui32 usage, char *name,
                            [&to_remove](const char &c) { return to_remove.find(c) != std::string::npos; }),
             buf.end());
 
-    if (buf.empty())
-        buf = "null";
+    auto dot = buf.find('.');
+    if (dot != std::string::npos) {
+        buf = buf.substr(0, dot);
+    }
 
-    buf = std::string(gli_workdir) + "/" + buf + gli_suffix_for_usage(usage);
+    if (buf.empty()) {
+        buf = "null";
+    }
+
+    buf = Format("{}/{}{}", gli_workdir, buf, gli_suffix_for_usage(usage));
 
     fref = gli_new_fileref(buf.c_str(), usage, rock);
 #else
@@ -331,21 +355,22 @@ frefid_t glk_fileref_create_by_prompt(glui32 usage, glui32 fmode,
             break;
     }
 
-    if (fmode == filemode_Read)
+    if (fmode == filemode_Read) {
         buf = garglk::winopenfile(prompt, filter);
-    else
+    } else {
         buf = garglk::winsavefile(prompt, filter);
+    }
 
     if (buf.empty()) {
         /* The player just hit return. It would be nice to provide a
             default value, but this implementation is too cheap. */
-        return NULL;
+        return nullptr;
     }
 
     if (fmode == filemode_Read) {
         /* According to recent spec discussion, we must silently return NULL if no such file exists. */
-        if (access(buf.c_str(), R_OK)) {
-            return NULL;
+        if (access(buf.c_str(), R_OK) == -1) {
+            return nullptr;
         }
     }
 
@@ -511,15 +536,26 @@ void glk_fileref_delete_file(fileref_t *fref)
 void glkunix_set_base_file(char *filename)
 {
 #ifdef GARGLK
+#if __cplusplus >= 201703L
+    auto parent = std::filesystem::path(filename).parent_path();
+    if (parent.empty()) {
+        parent = ".";
+    }
+
+    gli_workdir = parent.string();
+#else
     gli_workdir = filename;
     auto slash = gli_workdir.find_last_of('/');
-    if (slash == std::string::npos)
+    if (slash == std::string::npos) {
         slash = gli_workdir.find_last_of('\\');
+    }
 
-    if (slash != std::string::npos)
+    if (slash != std::string::npos) {
         gli_workdir.erase(slash);
-    else
+    } else {
         gli_workdir = ".";
+    }
+#endif
 
     gli_workfile = filename;
 #else
