@@ -112,11 +112,10 @@ void gli_delete_stream(stream_t *str)
     }
     
 #ifdef GARGLK
-    for (win = gli_window_iterate_treeorder(NULL);
-         win != NULL;
-         win = gli_window_iterate_treeorder(win)) {
-        if (win->echostr == str)
-            win->echostr = NULL;
+    for (win = gli_window_iterate_treeorder(nullptr); win != nullptr; win = gli_window_iterate_treeorder(win)) {
+        if (win->echostr == str) {
+            win->echostr = nullptr;
+        }
     }
 #else
     win = gli_window_get();
@@ -374,19 +373,41 @@ strid_t glk_stream_open_file_uni(fileref_t *fref, glui32 fmode,
 
 #ifdef GLK_MODULE_RESOURCE_STREAM
 
+#define gli_get_dataresource_info(num, ptr, len, isbinary) (false)
+
 strid_t glk_stream_open_resource(glui32 filenum, glui32 rock)
 {
     strid_t str;
     int isbinary;
     giblorb_err_t err;
     giblorb_result_t res;
-    giblorb_map_t *map = giblorb_get_resource_map();
-    if (!map)
-        return 0; /* Not running from a blorb file */
 
-    err = giblorb_load_resource(map, giblorb_method_Memory, &res, giblorb_ID_Data, filenum);
-    if (err)
-        return 0; /* Not found, or some other error */
+    if (gli_get_dataresource_info(filenum, &res.data.ptr, &res.length, &isbinary)) {
+        /* Success! */
+    }
+    else {
+        /* No command-line pathname; check blorb. */
+        
+        giblorb_map_t *map = giblorb_get_resource_map();
+        if (!map)
+            return 0; /* Not running from a blorb file */
+        
+        err = giblorb_load_resource(map, giblorb_method_Memory, &res, giblorb_ID_Data, filenum);
+        if (err)
+            return 0; /* Not found, or some other error */
+
+        /* Note that binary chunks are normally type BINA, but FORM
+           chunks also count as binary. (This allows us to embed AIFF
+           files as readable resources, for example.) */
+        
+        if (res.chunktype == giblorb_ID_TEXT)
+            isbinary = FALSE;
+        else if (res.chunktype == giblorb_ID_BINA
+            || res.chunktype == giblorb_make_id('F', 'O', 'R', 'M'))
+            isbinary = TRUE;
+        else
+            return 0; /* Unknown chunk type */
+    }
 
     /* We'll use the in-memory copy of the chunk data as the basis for
        our new stream. It's important to not call chunk_unload() until
@@ -397,18 +418,7 @@ strid_t glk_stream_open_resource(glui32 filenum, glui32 rock)
        would be to use the file on disk, but this requires some hacking
        into the file stream code (we'd need to open a new FILE*) and
        I don't feel like doing that.
-
-       Note that binary chunks are normally type BINA, but FORM
-       chunks also count as binary. (This allows us to embed AIFF
-       files as readable resources, for example.) */
-
-    if (res.chunktype == giblorb_ID_TEXT)
-        isbinary = FALSE;
-    else if (res.chunktype == giblorb_ID_BINA
-        || res.chunktype == giblorb_make_id('F', 'O', 'R', 'M'))
-        isbinary = TRUE;
-    else
-        return 0; /* Unknown chunk type */
+    */
 
     str = gli_new_stream(strtype_Resource,
         TRUE, FALSE, rock);
@@ -436,22 +446,30 @@ strid_t glk_stream_open_resource_uni(glui32 filenum, glui32 rock)
     int isbinary;
     giblorb_err_t err;
     giblorb_result_t res;
-    giblorb_map_t *map = giblorb_get_resource_map();
-    if (!map)
-        return 0; /* Not running from a blorb file */
-
-    err = giblorb_load_resource(map, giblorb_method_Memory, &res, giblorb_ID_Data, filenum);
-    if (err)
-        return 0; /* Not found, or some other error */
-
-    if (res.chunktype == giblorb_ID_TEXT)
-        isbinary = FALSE;
-    else if (res.chunktype == giblorb_ID_BINA
-        || res.chunktype == giblorb_make_id('F', 'O', 'R', 'M'))
-        isbinary = TRUE;
-    else
-        return 0; /* Unknown chunk type */
 
+    if (gli_get_dataresource_info(filenum, &res.data.ptr, &res.length, &isbinary)) {
+        /* Success! */
+    }
+    else {
+        /* No explicit path; check blorb. */
+        
+        giblorb_map_t *map = giblorb_get_resource_map();
+        if (!map)
+            return 0; /* Not running from a blorb file */
+        
+        err = giblorb_load_resource(map, giblorb_method_Memory, &res, giblorb_ID_Data, filenum);
+        if (err)
+            return 0; /* Not found, or some other error */
+        
+        if (res.chunktype == giblorb_ID_TEXT)
+            isbinary = FALSE;
+        else if (res.chunktype == giblorb_ID_BINA
+            || res.chunktype == giblorb_make_id('F', 'O', 'R', 'M'))
+            isbinary = TRUE;
+        else
+            return 0; /* Unknown chunk type */
+    }
+    
     str = gli_new_stream(strtype_Resource, 
         TRUE, FALSE, rock);
     if (!str) {
@@ -693,10 +711,9 @@ static void gli_put_char(stream_t *str, unsigned char ch)
 #ifdef GARGLK
             if (str->win->line_request || str->win->line_request_uni) {
                 if (gli_conf_safeclicks && gli_forceclick) {
-                    glk_cancel_line_event(str->win, NULL);
+                    glk_cancel_line_event(str->win, nullptr);
                     gli_forceclick = false;
-                }
-                else {
+                } else {
                     gli_strict_warning("put_char: window has pending line request");
                     break;
                 }
@@ -778,10 +795,9 @@ static void gli_put_char_uni(stream_t *str, glui32 ch)
 #ifdef GARGLK
             if (str->win->line_request || str->win->line_request_uni) {
                 if (gli_conf_safeclicks && gli_forceclick) {
-                    glk_cancel_line_event(str->win, NULL);
+                    glk_cancel_line_event(str->win, nullptr);
                     gli_forceclick = false;
-                }
-                else {
+                } else {
                     gli_strict_warning("put_char: window has pending line request");
                     break;
                 }
@@ -891,16 +907,16 @@ static void gli_put_buffer(stream_t *str, char *buf, glui32 len)
 #ifdef GARGLK
             if (str->win->line_request || str->win->line_request_uni) {
                 if (gli_conf_safeclicks && gli_forceclick) {
-                    glk_cancel_line_event(str->win, NULL);
+                    glk_cancel_line_event(str->win, nullptr);
                     gli_forceclick = false;
-                }
-                else {
+                } else {
                     gli_strict_warning("put_buffer: window has pending line request");
                     break;
                 }
             }
-            for (lx=0; lx<len; lx++)
-                gli_window_put_char_uni(str->win, (unsigned char)buf[lx]);
+            for (lx = 0; lx < len; lx++) {
+                gli_window_put_char_uni(str->win, static_cast<unsigned char>(buf[lx]));
+            }
 #else
             if (str->win->line_request) {
                 gli_strict_warning("put_buffer: window has pending line request");
@@ -1754,8 +1770,9 @@ glui32 glk_get_buffer_stream(stream_t *str, char *buf, glui32 len)
 glui32 gli_strlen_uni(const glui32 *s)
 {
     glui32 length = 0;
-    while (*s++)
+    while (*s++ != 0) {
         length++;
+    }
     return length;
 }
 
@@ -1764,8 +1781,9 @@ stream_t *gli_stream_open_window(window_t *win)
     stream_t *str;
 
     str = gli_new_stream(strtype_Window, false, true, 0);
-    if (!str)
-        return NULL;
+    if (str == nullptr) {
+        return nullptr;
+    }
 
     str->win = win;
     str->unicode = true;
@@ -1775,16 +1793,15 @@ stream_t *gli_stream_open_window(window_t *win)
 
 static void gli_set_hyperlink(stream_t *str, glui32 linkval)
 {
-    if (!str || !str->writable)
+    if (str == nullptr || !str->writable) {
         return;
-
-    switch (str->type)
-    {
-        case strtype_Window:
-            str->win->attr.hyper = linkval;
-            break;
     }
 
+    switch (str->type) {
+    case strtype_Window:
+        str->win->attr.hyper = linkval;
+        break;
+    }
 }
 
 void glk_set_hyperlink(glui32 linkval)
@@ -1794,8 +1811,7 @@ void glk_set_hyperlink(glui32 linkval)
 
 void glk_set_hyperlink_stream(stream_t *str, glui32 linkval)
 {
-    if (!str)
-    {
+    if (str == nullptr) {
         gli_strict_warning("set_hyperlink_stream: invalid ref");
         return;
     }
@@ -1804,31 +1820,33 @@ void glk_set_hyperlink_stream(stream_t *str, glui32 linkval)
 
 static void gli_set_style(stream_t *str, glui32 val)
 {
-    if (!str || !str->writable)
+    if (str == nullptr || !str->writable) {
         return;
+    }
 
-    if (val >= style_NUMSTYLES)
-        val = 0;
-    if (val < 0)
+    if (val < 0 || val >= style_NUMSTYLES) {
         val = 0;
+    }
 
-    switch (str->type)
-    {
-        case strtype_Window:
-            str->win->attr.style = val;
-            if (str->win->echostr)
-                gli_set_style(str->win->echostr, val);
-            break;
+    switch (str->type) {
+    case strtype_Window:
+        str->win->attr.style = val;
+        if (str->win->echostr != nullptr) {
+            gli_set_style(str->win->echostr, val);
+        }
+        break;
     }
 }
 
 static void gli_set_zcolors(stream_t *str, glui32 fg, glui32 bg)
 {
-    if (!str || !str->writable)
+    if (str == nullptr || !str->writable) {
         return;
+    }
 
-    if (!gli_conf_stylehint)
+    if (!gli_conf_stylehint) {
         return;
+    }
 
     Color fore((fg >> 16) & 0xff,
                (fg >> 8) & 0xff,
@@ -1838,63 +1856,44 @@ static void gli_set_zcolors(stream_t *str, glui32 fg, glui32 bg)
                (bg >> 8) & 0xff,
                (bg) & 0xff);
 
-    switch (str->type)
-    {
-        case strtype_Window:
-            if (fg != zcolor_Transparent && fg != zcolor_Cursor)
-            {
-                if (fg == zcolor_Default)
-                {
-                    str->win->attr.fgset = false;
-                    str->win->attr.fgcolor = Color(0, 0, 0);
-                    gli_override_fg_set = false;
-                    gli_override_fg_val = Color(0, 0, 0);
-                    gli_more_color = gli_more_save;
-                    gli_caret_color = gli_caret_save;
-                    gli_link_color = gli_link_save;
-                }
-                else if (fg != zcolor_Current)
-                {
-                    str->win->attr.fgset = true;
-                    str->win->attr.fgcolor = fore;
-                    gli_override_fg_set = true;
-                    gli_override_fg_val = fore;
-                    gli_more_color = fore;
-                    gli_caret_color = fore;
-                    gli_link_color = fore;
-                }
+    switch (str->type) {
+    case strtype_Window:
+        if (fg != zcolor_Transparent && fg != zcolor_Cursor) {
+            if (fg == zcolor_Default) {
+                str->win->attr.fgcolor.reset();
+                gli_override_fg.reset();
+                gli_more_color = gli_more_save;
+                gli_caret_color = gli_caret_save;
+                gli_link_color = gli_link_save;
+            } else if (fg != zcolor_Current) {
+                str->win->attr.fgcolor = fore;
+                gli_override_fg = fore;
+                gli_more_color = fore;
+                gli_caret_color = fore;
+                gli_link_color = fore;
             }
+        }
 
-            if (bg != zcolor_Transparent && bg != zcolor_Cursor)
-            {
-                if (bg == zcolor_Default)
-                {
-                    str->win->attr.bgset = false;
-                    str->win->attr.bgcolor = Color(0, 0, 0);
-                    gli_override_bg_set = false;
-                    gli_override_bg_val = Color(0, 0, 0);
-                    gli_window_color = gli_window_save;
-                    gli_border_color = gli_border_save;
-                }
-                else if (bg != zcolor_Current)
-                {
-                    str->win->attr.bgset = true;
-                    str->win->attr.bgcolor = back;
-                    gli_override_bg_set = true;
-                    gli_override_bg_val = back;
-                    gli_window_color = back;
-                    gli_border_color = back;
-                }
+        if (bg != zcolor_Transparent && bg != zcolor_Cursor) {
+            if (bg == zcolor_Default) {
+                str->win->attr.bgcolor.reset();
+                gli_override_bg.reset();
+                gli_window_color = gli_window_save;
+                gli_border_color = gli_border_save;
+            } else if (bg != zcolor_Current) {
+                str->win->attr.bgcolor = back;
+                gli_override_bg = back;
+                gli_window_color = back;
+                gli_border_color = back;
             }
+        }
 
-            if (fg == zcolor_Default && bg == zcolor_Default)
-                gli_override_reverse = false;
-            else
-                gli_override_reverse = true;
+        gli_override_reverse = fg != zcolor_Default || bg != zcolor_Default;
 
-            if (str->win->echostr)
-                gli_set_zcolors(str->win->echostr, fg, bg);
-            break;
+        if (str->win->echostr != nullptr) {
+            gli_set_zcolors(str->win->echostr, fg, bg);
+        }
+        break;
     }
 
     gli_force_redraw = true;
@@ -1907,8 +1906,7 @@ void garglk_set_zcolors(glui32 fg, glui32 bg)
 
 void garglk_set_zcolors_stream(stream_t *str, glui32 fg, glui32 bg)
 {
-    if (!str)
-    {
+    if (str == nullptr) {
         gli_strict_warning("set_style_stream: invalid ref");
         return;
     }
@@ -1917,19 +1915,21 @@ void garglk_set_zcolors_stream(stream_t *str, glui32 fg, glui32 bg)
 
 static void gli_set_reversevideo(stream_t *str, glui32 reverse)
 {
-    if (!str || !str->writable)
+    if (str == nullptr || !str->writable) {
         return;
+    }
 
-    if (!gli_conf_stylehint)
+    if (!gli_conf_stylehint) {
         return;
+    }
 
-    switch (str->type)
-    {
-        case strtype_Window:
-            str->win->attr.reverse = (reverse != 0);
-            if (str->win->echostr)
-                gli_set_reversevideo(str->win->echostr, reverse);
-            break;
+    switch (str->type) {
+    case strtype_Window:
+        str->win->attr.reverse = (reverse != 0);
+        if (str->win->echostr != nullptr) {
+            gli_set_reversevideo(str->win->echostr, reverse);
+        }
+        break;
     }
     gli_force_redraw = true;
 }
@@ -1941,8 +1941,7 @@ void garglk_set_reversevideo(glui32 reverse)
 
 void garglk_set_reversevideo_stream(stream_t *str, glui32 reverse)
 {
-    if (!str)
-    {
+    if (str == nullptr) {
         gli_strict_warning("set_style_stream: invalid ref");
         return;
     }
@@ -1954,32 +1953,36 @@ static glui32 gli_unput_buffer_uni(stream_t *str, const glui32 *buf, glui32 len)
     glui32 lx;
     const glui32 *cx;
 
-    if (!str || !str->writable)
+    if (str == nullptr || !str->writable) {
         return 0;
+    }
 
-    if (str->type == strtype_Window)
-    {
-        if (str->win->line_request || str->win->line_request_uni)
-        {
-            if (gli_conf_safeclicks && gli_forceclick)
-            {
-                glk_cancel_line_event(str->win, NULL);
+    if (str->win->line_request || str->win->line_request_uni) {
+        gli_strict_warning("unput_buffer: window has pending line request");
+        return 0;
+    }
+
+    if (str->type == strtype_Window) {
+        if (str->win->line_request || str->win->line_request_uni) {
+            if (gli_conf_safeclicks && gli_forceclick) {
+                glk_cancel_line_event(str->win, nullptr);
                 gli_forceclick = false;
-            }
-            else
-            {
+            } else {
                 gli_strict_warning("unput_buffer: window has pending line request");
                 return 0;
             }
         }
-        for (lx = 0, cx = buf + len - 1; lx < len; lx++, cx--)
-        {
-            if (!gli_window_unput_char_uni(str->win, *cx))
+
+        for (lx = 0, cx = buf + len - 1; lx < len; lx++, cx--) {
+            if (!gli_window_unput_char_uni(str->win, *cx)) {
                 break;
+            }
             str->writecount--;
         }
-        if (str->win->echostr)
+
+        if (str->win->echostr != nullptr) {
             gli_unput_buffer_uni(str->win->echostr, buf, len);
+        }
 
         return lx;
     }
@@ -1989,16 +1992,18 @@ static glui32 gli_unput_buffer_uni(stream_t *str, const glui32 *buf, glui32 len)
 
 static glui32 gli_unput_buffer(stream_t *str, const char *buf, glui32 len)
 {
-    glui32 *unicode = static_cast<glui32 *>(malloc(len));
-    if (unicode == NULL)
+    std::vector<glui32> unicode;
+    try {
+        unicode.reserve(len);
+    } catch (const std::bad_alloc &) {
         return 0;
+    }
 
-    for (glui32 i = 0; i < len; i++)
-        unicode[i] = (unsigned char)buf[i];
+    for (glui32 i = 0; i < len; i++) {
+        unicode.push_back(static_cast<unsigned char>(buf[i]));
+    }
 
-    glui32 n = gli_unput_buffer_uni(str, &unicode[0], len);
-
-    free(unicode);
+    glui32 n = gli_unput_buffer_uni(str, unicode.data(), len);
 
     return n;
 }
