@@ -23,12 +23,16 @@
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
+#include <deque>
 #include <fstream>
 #include <functional>
 #include <iomanip>
+#include <ios>
+#include <locale>
 #include <sstream>
 #include <stdexcept>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 #if __cplusplus >= 201703L
@@ -36,6 +40,10 @@
 #endif
 
 #ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#ifndef NOMINMAX
+#define NOMINMAX
+#endif
 #include <shlwapi.h>
 #else
 #include <fnmatch.h>
@@ -45,21 +53,50 @@
 #include <FindDirectory.h>
 #endif
 
+#include "format.h"
+#include "optional.hpp"
+
 #include "glk.h"
 #include "glkstart.h"
 #include "garglk.h"
 
 #include GARGLKINI_H
 
-float gli_backingscalefactor = 1.0f;
-float gli_zoom = 1.0f;
+struct ConfigError : public std::exception {
+    explicit ConfigError(std::string message_) : message(std::move(message_)) {
+    }
 
-FontFiles gli_conf_prop, gli_conf_mono, gli_conf_prop_override, gli_conf_mono_override;
+    std::string message;
+};
 
-std::string gli_conf_monofont = "Gargoyle Mono";
-std::string gli_conf_propfont = "Gargoyle Serif";
-float gli_conf_monosize = 12.6; // good size for Gargoyle Mono
-float gli_conf_propsize = 14.7; // good size for Gargoyle Serif
+std::string garglk::ConfigFile::format_type() const {
+    std::string status = "";
+    std::ifstream f(path);
+    if (!f.is_open()) {
+        status = ", non-existent";
+    }
+
+    switch (type) {
+    case Type::System:
+        return Format("[system{}]", status);
+    case Type::User:
+        return Format("[user{}]", status);
+    case Type::PerGame:
+        return Format("[game specific{}]", status);
+    default:
+        return "";
+    }
+}
+
+double gli_backingscalefactor = 1.0;
+double gli_zoom = 1.0;
+
+FontFiles gli_conf_prop, gli_conf_mono;
+
+std::string gli_conf_monofont = DEFAULT_MONO_FONT;
+std::string gli_conf_propfont = DEFAULT_PROP_FONT;
+double gli_conf_monosize = 12.6; // good size for Gargoyle Mono
+double gli_conf_propsize = 14.7; // good size for Gargoyle Serif
 
 Styles gli_tstyles{{
     {FontFace::propr(), Color(0xff, 0xff, 0xff), Color(0x00, 0x00, 0x00), false}, // Normal
@@ -89,34 +126,32 @@ Styles gli_gstyles{{
     {FontFace::monor(), Color(0xff, 0xff, 0xff), Color(0x60, 0x60, 0x60), false}, // User2
 }};
 
-const Styles gli_tstyles_def = gli_tstyles;
-const Styles gli_gstyles_def = gli_gstyles;
+Styles gli_tstyles_def = gli_tstyles;
+Styles gli_gstyles_def = gli_gstyles;
 
 std::vector<garglk::ConfigFile> garglk::all_configs;
 
 static FontFace font2idx(const std::string &font)
 {
-    if (font == "monor") {
-        return FontFace::monor();
-    } else if (font == "monob") {
-        return FontFace::monob();
-    } else if (font == "monoi") {
-        return FontFace::monoi();
-    } else if (font == "monoz") {
-        return FontFace::monoz();
-    } else if (font == "propr") {
-        return FontFace::propr();
-    } else if (font == "propb") {
-        return FontFace::propb();
-    } else if (font == "propi") {
-        return FontFace::propi();
-    } else if (font == "propz") {
-        return FontFace::propz();
+    const static std::unordered_map<std::string, FontFace> facemap = {
+        {"monor", FontFace::monor()},
+        {"monob", FontFace::monob()},
+        {"monoi", FontFace::monoi()},
+        {"monoz", FontFace::monoz()},
+        {"propr", FontFace::propr()},
+        {"propb", FontFace::propb()},
+        {"propi", FontFace::propi()},
+        {"propz", FontFace::propz()},
+    };
+
+    try {
+        return facemap.at(font);
+    } catch (const std::out_of_range &) {
+        throw ConfigError(Format("invalid font: {}", font));
     }
-    return FontFace::monor();
 }
 
-float gli_conf_gamma = 1.0;
+double gli_conf_gamma = 1.0;
 
 Color gli_window_color(0xff, 0xff, 0xff);
 Color gli_caret_color(0x00, 0x00, 0x00);
@@ -130,10 +165,8 @@ Color gli_border_save(0x00, 0x00, 0x00);
 Color gli_more_save(0x00, 0x60, 0x00);
 Color gli_link_save(0x00, 0x00, 0x60);
 
-bool gli_override_fg_set = false;
-Color gli_override_fg_val(0x00, 0x00, 0x00);
-bool gli_override_bg_set = false;
-Color gli_override_bg_val(0x00, 0x00, 0x00);
+nonstd::optional<Color> gli_override_fg;
+nonstd::optional<Color> gli_override_bg;
 bool gli_override_reverse = false;
 
 static std::string base_more_prompt = "— more —";
@@ -153,7 +186,9 @@ bool gli_conf_lcd = true;
 std::array<unsigned char, 5> gli_conf_lcd_weights = {28, 56, 85, 56, 28};
 
 int gli_wmarginx = 20;
+int gli_wmarginx_save = gli_wmarginx;
 int gli_wmarginy = 20;
+int gli_wmarginy_save = gli_wmarginy;
 int gli_wpaddingx = 0;
 int gli_wpaddingy = 0;
 int gli_wborderx = 0;
@@ -161,8 +196,6 @@ int gli_wbordery = 0;
 int gli_tmarginx = 0;
 int gli_tmarginy = 7;
 
-int gli_wmarginx_save = 15;
-int gli_wmarginy_save = 15;
 
 int gli_cols = 60;
 int gli_rows = 25;
@@ -173,8 +206,8 @@ bool gli_conf_lockrows = false;
 bool gli_conf_save_window_size = false;
 bool gli_conf_save_window_location = false;
 
-float gli_conf_propaspect = 1.0;
-float gli_conf_monoaspect = 1.0;
+double gli_conf_propaspect = 1.0;
+double gli_conf_monoaspect = 1.0;
 
 int gli_baseline = 15;
 int gli_leading = 20;
@@ -191,13 +224,28 @@ bool gli_conf_speak = false;
 bool gli_conf_speak_input = false;
 std::string gli_conf_speak_language;
 
+#ifdef GARGLK_DEFAULT_SOUNDFONT
+std::deque<std::string> gli_conf_soundfonts = {GARGLK_DEFAULT_SOUNDFONT};
+#else
+std::deque<std::string> gli_conf_soundfonts;
+#endif
+
+bool gli_conf_fluidsynth_chorus = true;
+bool gli_conf_fluidsynth_reverb = true;
+
 bool gli_conf_fullscreen = false;
 
+bool gli_wait_on_quit = true;
+
 bool gli_conf_stylehint = true;
 bool gli_conf_safeclicks = false;
 
 bool gli_conf_per_game_config = true;
 
+GamedataLocation gli_conf_gamedata_location = GamedataLocation::Default;
+
+Scaler gli_conf_scaler = Scaler::None;
+
 std::string garglk::downcase(const std::string &string)
 {
     std::string lowered;
@@ -214,6 +262,7 @@ static void parsecolor(const std::string &str, Color &rgb)
     try {
         rgb = gli_parse_color(str);
     } catch (const std::runtime_error &) {
+        throw ConfigError(Format("invalid color: {}", str));
     }
 }
 
@@ -238,14 +287,25 @@ static void parsecolor(const std::string &str, Color &rgb)
 // 6. <directory containing gargoyle/interpreter executable>/garglk.ini (Windows only)
 //
 // gamepath is the path to the game file being run
-std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = "")
+std::vector<garglk::ConfigFile> garglk::configs(const nonstd::optional<std::string> &gamepath = nonstd::nullopt)
 {
     std::vector<ConfigFile> configs;
-    if (!gamepath.empty()) {
+    if (gamepath.has_value()) {
+#if __cplusplus >= 201703L
+        // game file .ini
+        std::filesystem::path config(*gamepath);
+        config.replace_extension(".ini");
+        configs.emplace_back(config.string(), ConfigFile::Type::PerGame);
+
+        // game directory .ini
+        config = *gamepath;
+        config = config.parent_path() / "garglk.ini";
+        configs.emplace_back(config.string(), ConfigFile::Type::PerGame);
+#else
         std::string config;
 
         // game file .ini
-        config = gamepath;
+        config = *gamepath;
         auto dot = config.rfind('.');
         if (dot != std::string::npos) {
             config.replace(dot, std::string::npos, ".ini");
@@ -256,7 +316,7 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
         configs.emplace_back(config, ConfigFile::Type::PerGame);
 
         // game directory .ini
-        config = gamepath;
+        config = *gamepath;
         auto slash = config.find_last_of("/\\");
         if (slash != std::string::npos) {
             config.replace(slash + 1, std::string::npos, "garglk.ini");
@@ -265,25 +325,26 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
         }
 
         configs.emplace_back(config, ConfigFile::Type::PerGame);
+#endif
     }
 
 #if defined(__HAIKU__)
     char settings_dir[PATH_MAX + 1];
     if (find_directory(B_USER_SETTINGS_DIRECTORY, -1, false, settings_dir, sizeof settings_dir) == B_OK) {
-        configs.push_back(ConfigFile(std::string(settings_dir) + "/Gargoyle", ConfigFile::Type::User));
+        configs.emplace_back(Format("{}/Gargoyle", settings_dir), ConfigFile::Type::User);
     }
 #elif defined(_WIN32)
     // $APPDATA/Gargoyle/garglk.ini (Windows only). This has a higher
     // priority than $PWD/garglk.ini since it's a more "proper" location.
     const char *appdata = std::getenv("APPDATA");
     if (appdata != nullptr) {
-        configs.push_back(ConfigFile(std::string(appdata) + "/Gargoyle/garglk.ini", ConfigFile::Type::User));
+        configs.emplace_back(Format("{}/Gargoyle/garglk.ini", appdata), ConfigFile::Type::User);
     }
 
     // current directory .ini
     // Historically this has been the location of garglk.ini on Windows,
     // so treat it as a user config there.
-    configs.push_back(ConfigFile("garglk.ini", ConfigFile::Type::User));
+    configs.emplace_back("garglk.ini", ConfigFile::Type::User);
 #else
 
     const char *home = std::getenv("HOME");
@@ -293,7 +354,7 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
     // $HOME/garglk.ini. At some point this probably should move to somewhere in
     // $HOME/Library, but for now, make sure this config file is used.
     if (home != nullptr) {
-        configs.push_back(ConfigFile(std::string(home) + "/garglk.ini", ConfigFile::Type::User));
+        configs.emplace_back(Format("{}/garglk.ini", home), ConfigFile::Type::User);
     }
 #endif
 
@@ -303,16 +364,16 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
     if (xdg != nullptr && xdg[0] == '/') {
         xdg_path = xdg;
     } else if (home != nullptr) {
-        xdg_path = std::string(home) + "/.config";
+        xdg_path = Format("{}/.config", home);
     }
 
     if (!xdg_path.empty()) {
-        configs.emplace_back(xdg_path + "/garglk.ini", ConfigFile::Type::User);
+        configs.emplace_back(Format("{}/garglk.ini", xdg_path), ConfigFile::Type::User);
     }
 
     // $HOME/.garglkrc
     if (home != nullptr) {
-        configs.emplace_back(std::string(home) + "/.garglkrc", ConfigFile::Type::User);
+        configs.emplace_back(Format("{}/.garglkrc", home), ConfigFile::Type::User);
     }
 #endif
 
@@ -321,7 +382,7 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
     // default garglk.ini.
     const char *garglkini = std::getenv("GARGLK_RESOURCES");
     if (garglkini != nullptr) {
-        configs.emplace_back(std::string(garglkini) + "/garglk.ini", ConfigFile::Type::System);
+        configs.emplace_back(Format("{}/garglk.ini", garglkini), ConfigFile::Type::System);
     }
 #endif
 
@@ -333,8 +394,8 @@ std::vector<garglk::ConfigFile> garglk::configs(const std::string &gamepath = ""
 #ifdef _WIN32
     // install directory
     auto exedir = garglk::winappdir();
-    if (!exedir.empty()) {
-        configs.push_back(ConfigFile(exedir + "/garglk.ini", ConfigFile::Type::System));
+    if (exedir.has_value()) {
+        configs.emplace_back(Format("{}/garglk.ini", *exedir), ConfigFile::Type::System);
     }
 #endif
 
@@ -378,26 +439,26 @@ std::string garglk::user_config()
             std::filesystem::create_directories(fspath.parent_path());
         }
     } catch (const std::runtime_error &e) {
-        throw std::runtime_error("Unable to create parent directory for configuration file " + path + ": " + e.what());
+        throw std::runtime_error(Format("Unable to create parent directory for configuration file {}: {}", path, e.what()));
     }
 #endif
 
     std::ofstream f(path);
     if (!f.is_open()) {
-        throw std::runtime_error("Unable to open configuration file " + path + " for writing.");
+        throw std::runtime_error(Format("Unable to open configuration file {} for writing.", path));
     }
 
     f << garglkini;
 
     if (f.bad()) {
         std::remove(path.c_str());
-        throw std::runtime_error("Error writing to configuration file " + path + ".");
+        throw std::runtime_error(Format("Error writing to configuration file {}.", path));
     }
 
     return path;
 }
 
-void garglk::config_entries(const std::string &fname, bool accept_bare, const std::vector<std::string> &matches, const std::function<void(const std::string &cmd, const std::string &arg)> &callback)
+void garglk::config_entries(const std::string &fname, bool accept_bare, const std::vector<std::string> &matches, const std::function<void(const std::string &cmd, const std::string &arg, int lineno)> &callback)
 {
     std::string line;
     bool accept = accept_bare;
@@ -407,7 +468,15 @@ void garglk::config_entries(const std::string &fname, bool accept_bare, const st
         return;
     }
 
-    while (std::getline(f >> std::ws, line)) {
+    int lineno = 0;
+    while (std::getline(f, line)) {
+        lineno++;
+
+        // Strip leading whitespace
+        line.erase(line.begin(), std::find_if(line.begin(), line.end(), [](unsigned char c) {
+            return !std::isspace(c);
+        }));
+
         auto comment = line.find('#');
         if (comment != std::string::npos) {
             line.erase(comment);
@@ -424,7 +493,7 @@ void garglk::config_entries(const std::string &fname, bool accept_bare, const st
                 std::string pattern;
                 while (s >> std::quoted(pattern)) {
 #ifdef _WIN32
-                    if (PathMatchSpec(garglk::downcase(match).c_str(), garglk::downcase(pattern).c_str())) {
+                    if (PathMatchSpecA(garglk::downcase(match).c_str(), garglk::downcase(pattern).c_str())) {
 #else
                     if (fnmatch(garglk::downcase(pattern).c_str(), garglk::downcase(match).c_str(), 0) == 0) {
 #endif
@@ -448,26 +517,104 @@ void garglk::config_entries(const std::string &fname, bool accept_bare, const st
             std::getline(linestream >> std::ws, arg);
 
             if (linestream) {
-                callback(cmd, arg);
+                callback(cmd, arg, lineno);
             }
         }
     }
 }
 
-// Replace with std::clamp when moving to C++17.
-template<typename T>
-constexpr const T clamp(const T &v, const T &lo, const T &hi)
+template <typename T>
+T parse_number(const std::string &s)
 {
-    return (v < lo) ? lo : (hi < v) ? hi : v;
+    // C++17: std::from_chars; but check compatibility, as it seems some
+    // C++17 implementations didn't (initially) support doubles.
+    std::istringstream ss(s);
+    ss.imbue(std::locale::classic());
+
+    T val;
+
+    // Force decimal: this is for end-users, not programmers. If a user
+    // types 09, he probably expects 9, not an error; or 010 to mean 10
+    // rather than 8. And in the config file, there's no place where hex
+    // is the "natural" representation.
+    ss >> std::dec >> val;
+
+    // This check isn't completely perfect; a string like "+" will be
+    // considered out of range rather than invalid. istringstream
+    // doesn't have discrete enough error reporting to disginguish "all
+    // valid characters but not a complete number" from "all valid
+    // characters but out of range". This will be fixed with
+    // std::from_chars, if that's ever implemented, but at least an
+    // error will be given, even if it's slightly misleading.
+    if (s.empty() || !ss.eof()) {
+        throw ConfigError(Format("invalid number: {}", s));
+    } else if (ss.fail() && ss.eof()) {
+        throw ConfigError(Format("out of range: {}", s));
+    }
+
+    return val;
 }
 
-static void readoneconfig(const std::string &fname, const std::string &argv0, const std::string &gamefile)
+double parse_double(std::string s)
 {
-    std::vector<std::string> matches = {argv0, gamefile};
+    // Earlier versions of Gargoyle used std::stod which is locale
+    // aware; and while Gargoyle doesn't call setlocale(), it can't stop
+    // libraries from doing so, meaning the user's locale was consulted
+    // for the decimal separator character. The default config uses a
+    // period no matter what, meaning parse errors for users whose
+    // locale prefers a comma as the separator.
+    //
+    // Now parsing is always done in the classic (C) locale, but since
+    // config files out there might contain commas (and just because
+    // it's more user friendly in general), treat commas as periods.
+    std::replace(s.begin(), s.end(), ',', '.');
 
-    garglk::config_entries(fname, true, matches, [](const std::string &cmd, const std::string &arg) {
+    return parse_number<double>(s);
+}
+
+int parse_int(const std::string &s)
+{
+    return parse_number<int>(s);
+}
+
+template <typename T>
+const T &config_range(const T &value, const T &min, const T &max)
+{
+    if (value < min || value > max) {
+        throw ConfigError(Format("invalid value: {} (must be in the range {}-{})", value, min, max));
+    }
+
+    return value;
+}
+
+template <typename T>
+constexpr const T &config_atleast(const T &value, const T &min)
+{
+    if (value < min) {
+        throw ConfigError(Format("invalid value: {} (must be at least {})", value, min));
+    }
+
+    return value;
+}
+
+static void readoneconfig(const std::string &fname, const std::string &argv0, const nonstd::optional<std::string> &gamefile)
+{
+    std::vector<std::string> matches = {argv0};
+    if (gamefile.has_value()) {
+        matches.push_back(*gamefile);
+    }
+
+    garglk::config_entries(fname, true, matches, [&fname](const std::string &cmd, const std::string &arg, int lineno) {
         auto asbool = [](const std::string &arg) {
-            return std::stoi(arg) != 0;
+            if (arg == "0" || arg == "1") {
+                return arg == "1";
+            }
+
+            throw ConfigError(Format("invalid value: {} (must be 0 or 1)", arg));
+        };
+
+        auto warn = [&fname, &lineno](const std::string &msg) {
+            std::cerr << Format("{}:{}: {}", fname, lineno, msg) << std::endl;
         };
 
         try {
@@ -479,60 +626,60 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
             } else if (cmd == "morefont") {
                 gli_more_font = font2idx(arg);
             } else if (cmd == "morealign") {
-                gli_more_align = clamp(std::stoi(arg), 0, 2);
+                gli_more_align = config_range(parse_int(arg), 0, 2);
             } else if (cmd == "monoaspect") {
-                gli_conf_monoaspect = std::stof(arg);
+                gli_conf_monoaspect = parse_double(arg);
             } else if (cmd == "propaspect") {
-                gli_conf_propaspect = std::stof(arg);
+                gli_conf_propaspect = parse_double(arg);
             } else if (cmd == "monosize") {
-                gli_conf_monosize = std::stof(arg);
+                gli_conf_monosize = parse_double(arg);
             } else if (cmd == "monor") {
-                gli_conf_mono_override.r = arg;
+                gli_conf_mono.r.override = arg;
             } else if (cmd == "monob") {
-                gli_conf_mono_override.b = arg;
+                gli_conf_mono.b.override = arg;
             } else if (cmd == "monoi") {
-                gli_conf_mono_override.i = arg;
+                gli_conf_mono.i.override = arg;
             } else if (cmd == "monoz") {
-                gli_conf_mono_override.z = arg;
+                gli_conf_mono.z.override = arg;
             } else if (cmd == "monofont") {
                 gli_conf_monofont = arg;
             } else if (cmd == "propsize") {
-                gli_conf_propsize = std::stof(arg);
+                gli_conf_propsize = parse_double(arg);
             } else if (cmd == "propr") {
-                gli_conf_prop_override.r = arg;
+                gli_conf_prop.r.override = arg;
             } else if (cmd == "propb") {
-                gli_conf_prop_override.b = arg;
+                gli_conf_prop.b.override = arg;
             } else if (cmd == "propi") {
-                gli_conf_prop_override.i = arg;
+                gli_conf_prop.i.override = arg;
             } else if (cmd == "propz") {
-                gli_conf_prop_override.z = arg;
+                gli_conf_prop.z.override = arg;
             } else if (cmd == "propfont") {
                 gli_conf_propfont = arg;
             } else if (cmd == "leading") {
-                gli_leading = std::max(1, std::stoi(arg));
+                gli_leading = config_atleast(parse_int(arg), 1);
             } else if (cmd == "baseline") {
-                gli_baseline = std::max(0, std::stoi(arg));
+                gli_baseline = config_atleast(parse_int(arg), 0);
             } else if (cmd == "rows") {
-                gli_rows = std::max(1, std::stoi(arg));
+                gli_rows = config_atleast(parse_int(arg), 1);
             } else if (cmd == "cols") {
-                gli_cols = std::max(1, std::stoi(arg));
+                gli_cols = config_atleast(parse_int(arg), 1);
             } else if (cmd == "minrows") {
-                int r = std::max(0, std::stoi(arg));
+                int r = config_atleast(parse_int(arg), 0);
                 if (gli_rows < r) {
                     gli_rows = r;
                 }
             } else if (cmd == "maxrows") {
-                int r = std::max(0, std::stoi(arg));
+                int r = config_atleast(parse_int(arg), 0);
                 if (gli_rows > r) {
                     gli_rows = r;
                 }
             } else if (cmd == "mincols") {
-                int r = std::max(0, std::stoi(arg));
+                int r = config_atleast(parse_int(arg), 0);
                 if (gli_cols < r) {
                     gli_cols = r;
                 }
             } else if (cmd == "maxcols") {
-                int r = std::max(0, std::stoi(arg));
+                int r = config_atleast(parse_int(arg), 0);
                 if (gli_cols > r) {
                     gli_cols = r;
                 }
@@ -550,28 +697,30 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                         gli_conf_save_window_size = true;
                     } else if (entry == "location") {
                         gli_conf_save_window_location = true;
+                    } else {
+                        throw ConfigError(Format("invalid value: {}", entry));
                     }
                 }
             } else if (cmd == "wmarginx") {
-                gli_wmarginx = std::max(0, std::stoi(arg));
+                gli_wmarginx = config_atleast(parse_int(arg), 0);
                 gli_wmarginx_save = gli_wmarginx;
             } else if (cmd == "wmarginy") {
-                gli_wmarginy = std::max(0, std::stoi(arg));
+                gli_wmarginy = config_atleast(parse_int(arg), 0);
                 gli_wmarginy_save = gli_wmarginy;
             } else if (cmd == "wpaddingx") {
-                gli_wpaddingx = std::max(0, std::stoi(arg));
+                gli_wpaddingx = config_atleast(parse_int(arg), 0);
             } else if (cmd == "wpaddingy") {
-                gli_wpaddingy = std::max(0, std::stoi(arg));
+                gli_wpaddingy = config_atleast(parse_int(arg), 0);
             } else if (cmd == "wborderx") {
-                gli_wborderx = std::max(0, std::stoi(arg));
+                gli_wborderx = config_atleast(parse_int(arg), 0);
             } else if (cmd == "wbordery") {
-                gli_wbordery = std::max(0, std::stoi(arg));
+                gli_wbordery = config_atleast(parse_int(arg), 0);
             } else if (cmd == "tmarginx") {
-                gli_tmarginx = std::max(0, std::stoi(arg));
+                gli_tmarginx = config_atleast(parse_int(arg), 0);
             } else if (cmd == "tmarginy") {
-                gli_tmarginy = std::max(0, std::stoi(arg));
+                gli_tmarginy = config_atleast(parse_int(arg), 0);
             } else if (cmd == "gamma") {
-                gli_conf_gamma = std::max(0.0f, std::stof(arg));
+                gli_conf_gamma = config_atleast(parse_double(arg), 0.0);
             } else if (cmd == "caretcolor") {
                 parsecolor(arg, gli_caret_color);
                 parsecolor(arg, gli_caret_save);
@@ -587,25 +736,29 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
             } else if (cmd == "lcd") {
                 gli_conf_lcd = asbool(arg);
             } else if (cmd == "lcdfilter") {
-                garglk::set_lcdfilter(arg);
+                if (!garglk::set_lcdfilter(arg)) {
+                    throw ConfigError(Format("invalid value: {}", arg));
+                }
             } else if (cmd == "lcdweights") {
                 std::istringstream argstream(arg);
-                int weight;
+                std::string weight;
                 std::vector<unsigned char> weights;
 
                 while (argstream >> weight) {
-                    weights.push_back(weight);
+                    weights.push_back(config_range(parse_int(weight), 0, 255));
                 }
 
                 if (weights.size() == gli_conf_lcd_weights.size()) {
                     std::copy(weights.begin(), weights.end(), gli_conf_lcd_weights.begin());
+                } else {
+                    throw ConfigError(Format("invalid value: {} (must be 5 bytes)", arg));
                 }
             } else if (cmd == "caretshape") {
-                gli_caret_shape = clamp(std::stoi(arg), 0, 4);
+                gli_caret_shape = config_range(parse_int(arg), 0, 4);
             } else if (cmd == "linkstyle") {
                 gli_underline_hyperlinks = asbool(arg);
             } else if (cmd == "scrollwidth") {
-                gli_scroll_width = std::max(0, std::stoi(arg));
+                gli_scroll_width = config_atleast(parse_int(arg), 0);
             } else if (cmd == "scrollbg") {
                 parsecolor(arg, gli_scroll_bg);
             } else if (cmd == "scrollfg") {
@@ -613,11 +766,11 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
             } else if (cmd == "justify") {
                 gli_conf_justify = asbool(arg);
             } else if (cmd == "quotes") {
-                gli_conf_quotes = clamp(std::stoi(arg), 0, 2);
+                gli_conf_quotes = config_range(parse_int(arg), 0, 2);
             } else if (cmd == "dashes") {
-                gli_conf_dashes = clamp(std::stoi(arg), 0, 2);
+                gli_conf_dashes = config_range(parse_int(arg), 0, 2);
             } else if (cmd == "spaces") {
-                gli_conf_spaces = clamp(std::stoi(arg), 0, 2);
+                gli_conf_spaces = config_range(parse_int(arg), 0, 2);
             } else if (cmd == "caps") {
                 gli_conf_caps = asbool(arg);
             } else if (cmd == "graphics") {
@@ -626,24 +779,54 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                 gli_conf_sound = asbool(arg);
             } else if (cmd == "zbleep") {
                 std::istringstream argstream(arg);
-                int number, frequency;
-                double duration;
+                std::string number, frequency, duration;
 
                 if (argstream >> number >> duration >> frequency) {
-                    gli_bleeps.update(number, duration, frequency);
+                    gli_bleeps.update(
+                            config_range(parse_int(number), 1, 2),
+                            parse_double(duration),
+                            parse_int(frequency));
                 }
             } else if (cmd == "zbleep_file") {
                 std::istringstream argstream(arg);
-                int number;
-                std::string path;
+                std::string number, path;
 
                 if (argstream >> number >> path) {
-                    gli_bleeps.update(number, path);
+                    gli_bleeps.update(config_range(parse_int(number), 1, 2), path);
                 }
+            } else if (cmd == "soundfont") {
+                gli_conf_soundfonts.clear();
+
+                std::istringstream argstream(arg);
+                std::string soundfont;
+
+                while (argstream >> std::quoted(soundfont)) {
+                    gli_conf_soundfonts.push_front(soundfont);
+                }
+            } else if (cmd == "fluidsynth_reverb") {
+                gli_conf_fluidsynth_reverb = asbool(arg);
+            } else if (cmd == "fluidsynth_chorus") {
+                gli_conf_fluidsynth_chorus = asbool(arg);
             } else if (cmd == "fullscreen") {
                 gli_conf_fullscreen = asbool(arg);
             } else if (cmd == "zoom") {
-                gli_zoom = std::max(0.1f, std::stof(arg));
+                gli_zoom = config_atleast(parse_double(arg), 0.1);
+            } else if (cmd == "scaler") {
+#ifdef GARGLK_CONFIG_SCALERS
+                if (arg == "none") {
+                    gli_conf_scaler = Scaler::None;
+                } else if (arg == "hqx") {
+                    gli_conf_scaler = Scaler::HQX;
+                } else if (arg == "xbrz") {
+                    gli_conf_scaler = Scaler::XBRZ;
+                } else {
+                    throw ConfigError(Format("invalid value: {} (must be one of none, hqx, xbrz)", arg));
+                }
+#else
+                throw ConfigError("this build of Gargoyle does not have support for scalers");
+#endif
+            } else if (cmd == "wait_on_quit") {
+                gli_wait_on_quit = asbool(arg);
             } else if (cmd == "speak") {
                 gli_conf_speak = asbool(arg);
             } else if (cmd == "speak_input") {
@@ -655,7 +838,9 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
             } else if (cmd == "safeclicks") {
                 gli_conf_safeclicks = asbool(arg);
             } else if (cmd == "theme") {
-                garglk::theme::set(arg);
+                if (!garglk::theme::set(arg)) {
+                    throw ConfigError(Format("unknown theme: {}", arg));
+                }
             } else if (cmd == "tcolor" || cmd == "gcolor") {
                 std::istringstream argstream(arg);
                 std::string style, fg, bg;
@@ -669,7 +854,7 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                             parsecolor(bg, styles[i].bg);
                         }
                     } else {
-                        int i = std::stoi(style);
+                        int i = parse_int(style);
 
                         if (i >= 0 && i < style_NUMSTYLES) {
                             parsecolor(fg, styles[i].fg);
@@ -682,7 +867,7 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                 std::string style, font;
 
                 if (argstream >> style >> font) {
-                    int i = std::stoi(style);
+                    int i = config_range(parse_int(style), 0, style_NUMSTYLES - 1);
 
                     if (i >= 0 && i < style_NUMSTYLES) {
                         if (cmd[0] == 't') {
@@ -694,6 +879,16 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                 }
             } else if (cmd == "game_config") {
                 gli_conf_per_game_config = asbool(arg);
+            } else if (cmd == "gamedata_location") {
+                if (arg == "default") {
+                    gli_conf_gamedata_location = GamedataLocation::Default;
+                } else if (arg == "dedicated") {
+                    gli_conf_gamedata_location = GamedataLocation::Dedicated;
+                } else if (arg == "gamedir") {
+                    gli_conf_gamedata_location = GamedataLocation::Gamedir;
+                } else {
+                    throw ConfigError(Format("unknown gamedata location: {}", arg));
+                }
             } else if (cmd == "redraw_hack") {
                 gli_conf_redraw_hack = asbool(arg);
             } else if (cmd == "glyph_substitution_file") {
@@ -728,19 +923,25 @@ static void readoneconfig(const std::string &fname, const std::string &argv0, co
                                 gli_conf_glyph_substitution_files[fontface].push_back(file);
                             }
                         } catch (const std::out_of_range &) {
+                            throw ConfigError(Format("unknown font style: {}", style));
                         }
                     }
                 }
+            } else if (cmd == "terp") {
+                // Ignore: this is used by the launcher.
+            } else {
+                warn(Format("unknown configuration option: {}", cmd));
             }
-
-        // For now just ignore failure; in the future, probably log it.
-        } catch (const std::invalid_argument &) {
-        } catch (const std::out_of_range &) {
+        } catch (const ConfigError &e) {
+            warn(Format("{}: {}", cmd, e.message));
         }
     });
+
+    gli_tstyles_def = gli_tstyles;
+    gli_gstyles_def = gli_gstyles;
 }
 
-static void gli_read_config(int argc, char **argv)
+void gli_read_config(int argc, char **argv)
 {
 #if __cplusplus >= 201703L
     // load argv0 with name of executable without suffix
@@ -750,9 +951,12 @@ static void gli_read_config(int argc, char **argv)
         .string();
 
     // load gamefile with basename of last argument
-    std::string gamefile = std::filesystem::path(argv[argc - 1])
-        .filename()
-        .string();
+    nonstd::optional<std::string> gamefile;
+    if (argc > 1) {
+        gamefile = std::filesystem::path(argv[argc - 1])
+            .filename()
+            .string();
+    }
 #else
     auto basename = [](std::string path) {
         auto slash = path.find_last_of("/\\");
@@ -771,11 +975,14 @@ static void gli_read_config(int argc, char **argv)
     }
 
     // load gamefile with basename of last argument
-    std::string gamefile = basename(argv[argc - 1]);
+    nonstd::optional<std::string> gamefile;
+    if (argc > 1) {
+        gamefile = basename(argv[argc - 1]);
+    }
 #endif
 
     // load gamepath with the path to the story file itself
-    std::string gamepath;
+    nonstd::optional<std::string> gamepath;
     if (argc > 1) {
         gamepath = argv[argc - 1];
     }
@@ -807,9 +1014,23 @@ strid_t glkunix_stream_open_pathname(char *pathname, glui32 textmode, glui32 roc
     return gli_stream_open_pathname(pathname, false, (textmode != 0), rock);
 }
 
-void gli_startup(int argc, char *argv[])
+void garglk_startup(int argc, char *argv[])
 {
-    wininit(&argc, argv);
+    static bool initialized = false;
+
+    if (argc == 0) {
+        std::cerr << "argv[0] is null, aborting\n";
+        std::exit(EXIT_FAILURE);
+    }
+
+    if (initialized) {
+        gli_strict_warning("garglk_startup called multiple times");
+        return;
+    }
+
+    initialized = true;
+
+    wininit();
 
     if (argc > 1) {
         glkunix_set_base_file(argv[argc - 1]);
@@ -853,5 +1074,23 @@ void gli_startup(int argc, char *argv[])
     gli_initialize_sound();
 
     winopen();
-    gli_initialize_babel();
+
+    if (gli_workfile.has_value()) {
+        gli_initialize_babel(*gli_workfile);
+    }
+
+    // atexit() handlers should run before static destructors (see C++14
+    // 3.6.3p3):
+    //
+    // "If the completion of the initialization of an object with static
+    // storage duration is sequenced before a call to std::atexit [...],
+    // the call to the function passed to std::atexit is sequenced
+    // before the call to the destructor for the object."
+    //
+    // In general, this ought to obviate the need for setting
+    // gli_exiting in gli_exit(), but it's possible for atexit() to
+    // fail, so do it in both places.
+    if (std::atexit([]() { gli_exiting = true; }) != 0) {
+        gli_strict_warning("garglk_startup: unable to register atexit handler");
+    }
 }
