@@ -22,13 +22,14 @@
 #include <cmath>
 #include <cstddef>
 #include <functional>
-#include <sstream>
+#include <iostream>
 #include <stdexcept>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
+#include "format.h"
 #include "optional.hpp"
 
 #include "glk.h"
@@ -46,6 +47,8 @@
 
 using namespace std::literals;
 
+#define UNICODE_QUESTION_MARK 63
+
 #define GAMMA_BITS 11
 #define GAMMA_MAX ((1 << GAMMA_BITS) - 1)
 
@@ -53,6 +56,10 @@ using namespace std::literals;
 #define mulhigh(a, b) ((static_cast<int>(a) * (b) + (1 << (GAMMA_BITS - 1)) - 1) / GAMMA_MAX)
 #define grayscale(r, g, b) ((30 * (r) + 59 * (g) + 11 * (b)) / 100)
 
+static std::string convert_ft_error(FT_Error err, const std::string &basemsg);
+
+namespace {
+
 struct Bitmap {
     int w, h, lsb, top, pitch;
     std::vector<unsigned char> data;
@@ -63,29 +70,55 @@ struct FontEntry {
     std::array<Bitmap, GLI_SUBPIX> glyph;
 };
 
+struct UniqueFaceDeleter {
+    void operator()(FT_Face face) const {
+        FT_Done_Face(face);
+    }
+};
+
+using UniqueFace = std::unique_ptr<std::remove_pointer_t<FT_Face>, UniqueFaceDeleter>;
+
 class Font {
 public:
-    Font(FontFace fontface, FT_Face face, const std::string &fontpath);
+    class LoadError : public std::exception {
+    public:
+        LoadError(FT_Error err, std::string filename, const std::string &basemsg) :
+            m_filename(std::move(filename)),
+            m_what(convert_ft_error(err, basemsg))
+        {
+        }
+
+        const std::string &filename() const {
+            return m_filename;
+        }
+
+        const char *what() const noexcept override {
+            return m_what.c_str();
+        }
+
+    private:
+        std::string m_filename;
+        std::string m_what;
+    };
+
+    Font(FontFace fontface, UniqueFace face, const std::string &fontpath);
 
     FontEntry getglyph(glui32 cid);
     int charkern(glui32 c0, glui32 c1);
-    const FT_Face &face() {
+    const UniqueFace &face() {
         return m_face;
     }
 
-    FontFace fontface() {
-        return m_fontface;
-    }
-
 private:
-    FontFace m_fontface;
-    FT_Face m_face = nullptr;
+    UniqueFace m_face;
     bool m_make_bold = false;
     bool m_make_oblique = false;
     bool m_kerned = false;
     std::unordered_map<unsigned long long, int> m_kerncache;
 };
 
+}
+
 //
 // Globals
 //
@@ -109,7 +142,7 @@ static FT_Matrix ftmat;
 static bool use_freetype_preset_filter = false;
 static FT_LcdFilter freetype_preset_filter = FT_LCD_FILTER_DEFAULT;
 
-void garglk::set_lcdfilter(const std::string &filter)
+bool garglk::set_lcdfilter(const std::string &filter)
 {
     use_freetype_preset_filter = true;
 
@@ -121,9 +154,14 @@ void garglk::set_lcdfilter(const std::string &filter)
         freetype_preset_filter = FT_LCD_FILTER_LIGHT;
     } else if (filter == "legacy") {
         freetype_preset_filter = FT_LCD_FILTER_LEGACY;
+    } else if (filter == "custom") {
+        use_freetype_preset_filter = false;
     } else {
         use_freetype_preset_filter = false;
+        return false;
     }
+
+    return true;
 }
 
 //
@@ -142,51 +180,37 @@ static std::string convert_ft_error(FT_Error err, const std::string &basemsg)
 #endif
 
     if (errstr == nullptr) {
-        return basemsg + " (error code " + std::to_string(err) + ")";
+        return Format("{} (error code {})", basemsg, err);
     } else {
-        return basemsg + ": " + errstr;
+        return Format("{}: {}", basemsg, errstr);
     }
 }
 
-class FreetypeError : public std::exception {
-public:
-    FreetypeError(FT_Error err, const std::string &basemsg) :
-        m_what(convert_ft_error(err, basemsg))
-    {
-    }
-
-    const char *what() const noexcept override {
-        return m_what.c_str();
-    }
-
-private:
-    std::string m_what;
-};
+namespace {
 
 FontEntry Font::getglyph(glui32 cid)
 {
     FT_Vector v;
     int err;
     glui32 gid;
-    int x;
     FontEntry entry;
     std::size_t datasize;
 
-    gid = FT_Get_Char_Index(m_face, cid);
+    gid = FT_Get_Char_Index(m_face.get(), cid);
     if (gid == 0) {
-        throw std::out_of_range("no glyph for " + std::to_string(gid));
+        throw std::out_of_range(Format("no glyph for {}", cid));
     }
 
-    for (x = 0; x < GLI_SUBPIX; x++) {
+    for (int x = 0; x < GLI_SUBPIX; x++) {
         v.x = (x * 64) / GLI_SUBPIX;
         v.y = 0;
 
-        FT_Set_Transform(m_face, nullptr, &v);
+        FT_Set_Transform(m_face.get(), nullptr, &v);
 
-        err = FT_Load_Glyph(m_face, gid,
+        err = FT_Load_Glyph(m_face.get(), gid,
                 FT_LOAD_NO_BITMAP | FT_LOAD_NO_HINTING);
         if (err != 0) {
-            throw FreetypeError(err, "Error in FT_Load_Glyph");
+            throw std::runtime_error(convert_ft_error(err, "FT_Load_Glyph"));
         }
 
         if (m_make_bold) {
@@ -210,7 +234,7 @@ FontEntry Font::getglyph(glui32 cid)
         }
 
         if (err != 0) {
-            throw FreetypeError(err, "Error in FT_Render_Glyph");
+            throw std::runtime_error(convert_ft_error(err, "FT_Render_Glyph"));
         }
 
         datasize = m_face->glyph->bitmap.pitch * m_face->glyph->bitmap.rows;
@@ -227,50 +251,42 @@ FontEntry Font::getglyph(glui32 cid)
     return entry;
 }
 
-// Look for a user-specified font. This will be either based on a font
-// family (propfont or monofont), or specific font files (e.g. propr,
-// monor, etc).
-static std::string font_path_user(const std::string &path, const std::string &)
-{
-    return path;
 }
 
 // Look in a system-wide location for the fallback Gargoyle fonts; on
-// Unix this is generally somewhere like /usr/share/fonts/gargoyle
-// (although this can be changed at build time), and on Windows it's the
-// install directory (e.g. "C:\Program Files (x86)\Gargoyle").
-static std::string font_path_fallback_system(const std::string &, const std::string &fallback)
+// Unix this is generally somewhere like /usr/share/gargoyle (although
+// this can be changed at build time), and on Windows it's the install
+// directory (e.g. "C:\Program Files (x86)\Gargoyle").
+static nonstd::optional<std::string> font_path_fallback_system(const std::string &fallback)
 {
 #ifdef _WIN32
     char directory[256];
     DWORD dsize = sizeof directory;
     if (RegGetValueA(HKEY_LOCAL_MACHINE, "Software\\Tor Andersson\\Gargoyle", "Directory", RRF_RT_REG_SZ, nullptr, directory, &dsize) != ERROR_SUCCESS) {
-        return "";
+        return nonstd::nullopt;
     }
 
-    return std::string(directory) + "\\" + fallback;
-#elif defined(GARGLK_CONFIG_FONT_PATH)
-    return std::string(GARGLK_CONFIG_FONT_PATH) + "/" + fallback;
+    return Format("{}\\{}", directory, fallback);
 #else
-    return "";
+    return Format("{}/{}", garglk::windatadir(), fallback);
 #endif
 }
 
 // Look in a platform-specific location for the fonts. This is typically
 // the same directory that the executable is in, but can be anything the
 // platform code deems appropriate.
-static std::string font_path_fallback_platform(const std::string &, const std::string &fallback)
+static nonstd::optional<std::string> font_path_fallback_platform(const std::string &fallback)
 {
     return garglk::winfontpath(fallback);
 }
 
 // As a last-ditch effort, look in the current directory for the fonts.
-static std::string font_path_fallback_local(const std::string &, const std::string &fallback)
+static nonstd::optional<std::string> font_path_fallback_local(const std::string &fallback)
 {
     return fallback;
 }
 
-static std::string fontface_to_name(FontFace &fontface)
+static std::string fontface_to_name(FontFace fontface)
 {
     std::string type = fontface.monospace ? "Mono" : "Proportional";
     std::string style = (fontface.bold && fontface.italic) ? "Bold Italic" :
@@ -278,20 +294,18 @@ static std::string fontface_to_name(FontFace &fontface)
                         (fontface.italic                 ) ? "Italic" :
                                                              "Regular";
 
-    return type + " " + style;
+    return Format("{} {}", type, style);
 }
 
-static Font make_font(FontFace fontface, const std::string &fallback)
+static Font make_font(FontFace fontface, const std::string &fallback, std::vector<std::string> &problem_fonts)
 {
-    std::string fontpath;
-    std::vector<std::function<std::string(const std::string &path, const std::string &fallback)>> font_paths = {
-        font_path_user,
+    std::vector<std::function<nonstd::optional<std::string>(const std::string &fallback)>> font_paths = {
         font_path_fallback_system,
         font_path_fallback_platform,
         font_path_fallback_local,
     };
 
-    std::string path =
+    const auto &fontfiles =
         fontface == FontFace::monor() ? gli_conf_mono.r :
         fontface == FontFace::monob() ? gli_conf_mono.b :
         fontface == FontFace::monoi() ? gli_conf_mono.i :
@@ -302,15 +316,37 @@ static Font make_font(FontFace fontface, const std::string &fallback)
         fontface == FontFace::propz() ? gli_conf_prop.z :
                                         gli_conf_mono.r;
 
-    for (const auto &get_font_path : font_paths) {
-        auto fontpath = get_font_path(path, fallback);
-        FT_Face face;
-        if (!fontpath.empty() && FT_New_Face(ftlib, fontpath.c_str(), 0, &face) == 0) {
-            return {fontface, face, fontpath};
+    const auto &path = fontfiles.fontpath();
+    const auto &override = fontfiles.override;
+
+    // First look for a user-specified font. This will be either based
+    // on a font family (propfont or monofont), or specific font files
+    // (e.g. propr, monor, etc).
+    nonstd::optional<Font::LoadError> error;
+    FT_Face face;
+    if (path.has_value() && FT_New_Face(ftlib, path->c_str(), 0, &face) == 0) {
+        try {
+            return {fontface, UniqueFace(face), *path};
+        } catch (const Font::LoadError &e) {
+            error = e;
         }
     }
 
-    garglk::winabort("Unable to find font " + (fontface.monospace ? gli_conf_monofont : gli_conf_propfont) + " for " + fontface_to_name(fontface) + ", and fallback " + fallback + " not found");
+    // If no user font can be loaded, try to find a fallback.
+    for (const auto &get_font_path : font_paths) {
+        auto fontpath = get_font_path(fallback);
+        if (fontpath.has_value() && FT_New_Face(ftlib, fontpath->c_str(), 0, &face) == 0) {
+            if (error.has_value()) {
+                problem_fonts.push_back(Format("Unable to load font file \"{}\" ({}): using fallback {}.", error->filename(), error->what(), *fontpath));
+            }
+            return {fontface, UniqueFace(face), *fontpath};
+        }
+    }
+
+    garglk::winabort(Format("Unable to find font \"{}\" for {}, and fallback {} not found",
+                override.value_or(fontface.monospace ? gli_conf_monofont : gli_conf_propfont),
+                fontface_to_name(fontface),
+                fallback));
 }
 
 static std::vector<Font> make_substitution_fonts(FontFace fontface)
@@ -320,19 +356,17 @@ static std::vector<Font> make_substitution_fonts(FontFace fontface)
 
     auto files = gli_conf_glyph_substitution_files[fontface];
 
-    for (const auto &datadir : garglk::winappdata()) {
-        files.push_back(datadir + "/unifont.otf");
-        files.push_back(datadir + "/unifont_upper.otf");
-    }
 
-    files.emplace_back("./unifont.otf");
-    files.emplace_back("./unifont_upper.otf");
+    for (const auto &path : {garglk::windatadir(), "."s}) {
+        files.push_back(Format("{}/unifont.otf", path));
+        files.push_back(Format("{}/unifont_upper.otf", path));
+    }
 
     for (const auto &file : files) {
         if (FT_New_Face(ftlib, file.c_str(), 0, &face) == 0) {
             try {
-                fonts.emplace_back(fontface, face, file);
-            } catch (const FreetypeError &) {
+                fonts.emplace_back(fontface, UniqueFace(face), file);
+            } catch (const Font::LoadError &) {
             }
         }
     }
@@ -340,14 +374,13 @@ static std::vector<Font> make_substitution_fonts(FontFace fontface)
     return fonts;
 }
 
-Font::Font(FontFace fontface, FT_Face face, const std::string &fontpath) :
-    m_fontface(fontface),
-    m_face(face)
+Font::Font(FontFace fontface, UniqueFace face, const std::string &fontpath) :
+    m_face(std::move(face))
 {
     int err = 0;
-    float aspect, size;
+    double aspect, size;
 
-    if (m_fontface.monospace) {
+    if (fontface.monospace) {
         aspect = gli_conf_monoaspect;
         size = gli_conf_monosize;
     } else {
@@ -361,26 +394,26 @@ Font::Font(FontFace fontface, FT_Face face, const std::string &fontpath) :
         auto ext = afmbuf.substr(dot);
         if (ext == ".pfa" || ext == ".PFA" || ext == ".pfb" || ext == ".PFB") {
             afmbuf.replace(dot, std::string::npos, ".afm");
-            FT_Attach_File(m_face, afmbuf.c_str());
+            FT_Attach_File(m_face.get(), afmbuf.c_str());
             afmbuf.replace(dot, std::string::npos, ".AFM");
-            FT_Attach_File(m_face, afmbuf.c_str());
+            FT_Attach_File(m_face.get(), afmbuf.c_str());
         }
     }
 
-    err = FT_Set_Char_Size(m_face, size * aspect * 64, size * 64, 72, 72);
+    err = FT_Set_Char_Size(m_face.get(), size * aspect * 64, size * 64, 72, 72);
     if (err != 0) {
-        throw FreetypeError(err, "Error in FT_Set_Char_Size for " + fontpath);
+        throw LoadError(err, fontpath, "FT_Set_Char_Size");
     }
 
-    err = FT_Select_Charmap(m_face, ft_encoding_unicode);
+    err = FT_Select_Charmap(m_face.get(), FT_ENCODING_UNICODE);
     if (err != 0) {
-        throw FreetypeError(err, "Error in FT_Select_CharMap for " + fontpath);
+        throw LoadError(err, fontpath, "FT_Select_CharMap");
     }
 
     m_kerned = FT_HAS_KERNING(m_face);
 
-    m_make_bold = m_fontface.bold && ((m_face->style_flags & FT_STYLE_FLAG_BOLD) == 0);
-    m_make_oblique = m_fontface.italic && ((m_face->style_flags & FT_STYLE_FLAG_ITALIC) == 0);
+    m_make_bold = fontface.bold && ((m_face->style_flags & FT_STYLE_FLAG_BOLD) == 0);
+    m_make_oblique = fontface.italic && ((m_face->style_flags & FT_STYLE_FLAG_ITALIC) == 0);
 }
 
 void gli_initialize_fonts()
@@ -392,7 +425,7 @@ void gli_initialize_fonts()
     }
 
     for (int i = 0; i <= GAMMA_MAX; i++) {
-        gammainv[i] = std::round(std::pow(i / static_cast<float>(GAMMA_MAX), 1.0 / gli_conf_gamma) * 255.0);
+        gammainv[i] = std::round(std::pow(i / static_cast<double>(GAMMA_MAX), 1.0 / gli_conf_gamma) * 255.0);
     }
 
     err = FT_Init_FreeType(&ftlib);
@@ -400,36 +433,42 @@ void gli_initialize_fonts()
         garglk::winabort(convert_ft_error(err, "Unable to initialize FreeType"));
     }
 
-    fontload();
-    garglk::fontreplace(gli_conf_monofont, FontType::Monospace);
-    garglk::fontreplace(gli_conf_propfont, FontType::Proportional);
-    fontunload();
+    std::vector<std::string> problem_fonts;
 
-    // If the user provided specific fonts, swap them in
-    if (!gli_conf_mono_override.r.empty()) {
-        gli_conf_mono.r = gli_conf_mono_override.r;
+    // Gargoyle ships "Gargoyle Mono" and "Gargoyle Serif" fonts, which
+    // can be installed system-wide; it also sets the default mono and
+    // prop fonts to these values. However, it's certainly not required
+    // for these fonts to be installed system-wide, so it has a fallback
+    // mechanism, which, if Gargoyle is installed/configured properly,
+    // will fall back to "known-good" locations of these fonts.
+    //
+    // The problem is that Gargoyle has, in the past, shipped with a
+    // garglk.ini specifying these fonts. That's always been fine,
+    // because Gargoyle falls back silently if they're not installed,
+    // and nobody's the wiser. But Gargoyle always fell back silently if
+    // _any_ user-specified font couldn't be located, which meant that
+    // if a user mistyped a font, the fallback would be substituted
+    // without warning. Gargoyle now warns about missing fonts, but that
+    // leads to the problem of Gargoyle warning about its own specified
+    // fonts not being installed, if indeed they're not. This wouldn't
+    // be a problem if users didn't already have garglk.ini, but they
+    // do, so it is.
+    //
+    // At the moment the solution is to elide the warning when either
+    // of Gargoyle's own fonts is requested, i.e. the fonts listed in
+    // the default garglk.ini. This is a bit ugly since it means
+    // Gargoyle treats two specific font names as special. Ideally the
+    // default fonts would be empty and Gargoyle would use that to mean
+    // "built-in/fallback fonts", but with existing configs out there,
+    // it's not meant to be.
+    fontload();
+    if (!garglk::fontreplace(gli_conf_monofont, FontType::Monospace) && gli_conf_monofont != DEFAULT_MONO_FONT) {
+        problem_fonts.push_back(Format("Unable to find monospace font \"{}\", using fallback.", gli_conf_monofont));
     }
-    if (!gli_conf_mono_override.b.empty()) {
-        gli_conf_mono.b = gli_conf_mono_override.b;
-    }
-    if (!gli_conf_mono_override.i.empty()) {
-        gli_conf_mono.i = gli_conf_mono_override.i;
-    }
-    if (!gli_conf_mono_override.z.empty()) {
-        gli_conf_mono.z = gli_conf_mono_override.z;
-    }
-    if (!gli_conf_prop_override.r.empty()) {
-        gli_conf_prop.r = gli_conf_prop_override.r;
-    }
-    if (!gli_conf_prop_override.b.empty()) {
-        gli_conf_prop.b = gli_conf_prop_override.b;
-    }
-    if (!gli_conf_prop_override.i.empty()) {
-        gli_conf_prop.i = gli_conf_prop_override.i;
-    }
-    if (!gli_conf_prop_override.z.empty()) {
-        gli_conf_prop.z = gli_conf_prop_override.z;
+    if (!garglk::fontreplace(gli_conf_propfont, FontType::Proportional) && gli_conf_propfont != DEFAULT_PROP_FONT) {
+        problem_fonts.push_back(Format("Unable to find proportional font \"{}\", using fallback.", gli_conf_propfont));
     }
+    fontunload();
 
     // Set up the Unicode fallback font.
     auto faces = {FontFace::propr(), FontFace::propi(), FontFace::propb(), FontFace::propz(),
@@ -444,8 +483,8 @@ void gli_initialize_fonts()
     ftmat.xy = 0x03000L;
     ftmat.yy = 0x10000L;
 
-    auto make_entry = [](FontFace face, const std::string &filename) {
-        return std::make_pair(face, make_font(face, filename));
+    auto make_entry = [&problem_fonts](FontFace face, const std::string &fallback) {
+        return std::make_pair(face, make_font(face, fallback, problem_fonts));
     };
 
     try {
@@ -462,8 +501,13 @@ void gli_initialize_fonts()
 
         gli_cellh = gli_leading;
         gli_cellw = (entry.adv + GLI_SUBPIX - 1) / GLI_SUBPIX;
-    } catch (const FreetypeError &e) {
-        garglk::winabort(e.what());
+    } catch (const Font::LoadError &e) {
+        garglk::winabort(Format("Unable to load font file \"{}\" ({}).", e.filename(), e.what()));
+    }
+
+    if (!problem_fonts.empty()) {
+        auto msg = garglk::join(problem_fonts, "\n\n");
+        garglk::winwarning("Font error", msg);
     }
 }
 
@@ -538,23 +582,21 @@ static void draw_pixel_lcd_gamma(int x, int y, const unsigned char *alpha, const
                                    gammainv[fg[2] + mulhigh(static_cast<int>(bg[2]) - fg[2], invalf[2])]);
 }
 
-static void draw_bitmap_gamma(const Bitmap *b, int x, int y, const Color &rgb)
+static void draw_bitmap_gamma(const Bitmap &b, int x, int y, const Color &rgb)
 {
-    int i, k, c;
-    for (k = 0; k < b->h; k++) {
-        for (i = 0; i < b->w; i++) {
-            c = b->data[k * b->pitch + i];
-            draw_pixel_gamma(x + b->lsb + i, y - b->top + k, c, rgb);
+    for (int k = 0; k < b.h; k++) {
+        for (int i = 0; i < b.w; i++) {
+            auto c = b.data[k * b.pitch + i];
+            draw_pixel_gamma(x + b.lsb + i, y - b.top + k, c, rgb);
         }
     }
 }
 
-static void draw_bitmap_lcd_gamma(const Bitmap *b, int x, int y, const Color &rgb)
+static void draw_bitmap_lcd_gamma(const Bitmap &b, int x, int y, const Color &rgb)
 {
-    int i, j, k;
-    for (k = 0; k < b->h; k++) {
-        for (i = 0, j = 0; i < b->w; i += 3, j++) {
-            draw_pixel_lcd_gamma(x + b->lsb + j, y - b->top + k, b->data.data() + k * b->pitch + i, rgb);
+    for (int k = 0; k < b.h; k++) {
+        for (int i = 0, j = 0; i < b.w; i += 3, j++) {
+            draw_pixel_lcd_gamma(x + b.lsb + j, y - b.top + k, b.data.data() + k * b.pitch + i, rgb);
         }
     }
 }
@@ -596,16 +638,16 @@ int Font::charkern(glui32 c0, glui32 c1)
     } catch (const std::out_of_range &) {
     }
 
-    g0 = FT_Get_Char_Index(m_face, c0);
-    g1 = FT_Get_Char_Index(m_face, c1);
+    g0 = FT_Get_Char_Index(m_face.get(), c0);
+    g1 = FT_Get_Char_Index(m_face.get(), c1);
 
     if (g0 == 0 || g1 == 0) {
         return 0;
     }
 
-    err = FT_Get_Kerning(m_face, g0, g1, FT_KERNING_UNFITTED, &v);
+    err = FT_Get_Kerning(m_face.get(), g0, g1, FT_KERNING_UNFITTED, &v);
     if (err != 0) {
-        throw FreetypeError(err, "Error in FT_Get_Kerning");
+        throw std::runtime_error(convert_ft_error(err, "FT_Get_Kerning"));
     }
 
     int value = (v.x * GLI_SUBPIX) / 64.0;
@@ -648,7 +690,7 @@ static int gli_string_impl(int x, FontFace fontface, const glui32 *s, std::size_
             });
         }
 
-        if (it != ligatures.end() && FT_Get_Char_Index(f.face(), it->second) != 0) {
+        if (it != ligatures.end() && FT_Get_Char_Index(f.face().get(), it->second) != 0) {
             c = it->second;
             s += it->first.size();
             n -= it->first.size();
@@ -657,55 +699,55 @@ static int gli_string_impl(int x, FontFace fontface, const glui32 *s, std::size_
             n--;
         }
 
-        auto glyph = [&f, &fontface](glui32 c) -> std::shared_ptr<const FontEntry> {
-            // Cache all glyphs. If a glyph is not found, return null.
-            static std::unordered_map<std::pair<FontFace, glui32>, std::shared_ptr<FontEntry>> fallback_cache;
+        // Return a FontEntry corresponding to the specific glyph. If
+        // that glyph is unavailable, log a warning and select a
+        // question mark instead. If a question mark can't be loaded,
+        // abort with an error message. Lookups are cached.
+        auto glyph = [&f, &fontface](glui32 c) -> const FontEntry & {
+            static std::unordered_map<std::pair<FontFace, glui32>, FontEntry> fallback_cache;
 
             auto key = std::make_pair(fontface, c);
 
             auto it = fallback_cache.find(key);
             if (it == fallback_cache.end()) {
-                std::shared_ptr<FontEntry> entry;
-
                 try {
-                    entry = std::make_shared<FontEntry>(f.getglyph(c));
+                    it = fallback_cache.emplace(key, f.getglyph(c)).first;
                 } catch (const std::out_of_range &) {
                     for (auto &font : glyph_substitution_fonts[fontface]) {
                         try {
-                            entry = std::make_shared<FontEntry>(font.getglyph(c));
+                            it = fallback_cache.emplace(key, font.getglyph(c)).first;
                             break;
                         } catch (const std::out_of_range &) {
                         }
                     }
                 }
 
-                it = fallback_cache.emplace(key, entry).first;
+                if (it == fallback_cache.end()) {
+                    auto msg = Format("Unable to look up glyph {} for {}", c, fontface_to_name(fontface));
+                    std::cerr << msg << std::endl;
+                    try {
+                        it = fallback_cache.emplace(key, f.getglyph(UNICODE_QUESTION_MARK)).first;
+                    } catch (const std::out_of_range &) {
+                        garglk::winabort(Format("{}, and substituting '?' failed", msg));
+                    }
+                }
             }
 
             return it->second;
         };
 
-        std::shared_ptr<const FontEntry> entry;
-        try {
-            entry = glyph(c);
-        } catch (const std::out_of_range &) {
-            entry = glyph('?');
-        }
-
-        if (entry == nullptr) {
-            garglk::winabort("unable to look up glyph " + std::to_string(c) + " for " + fontface_to_name(fontface));
-        }
-
         if (prev != -1) {
             x += f.charkern(prev, c);
         }
 
-        callback(x, entry->glyph);
+        const auto &entry = glyph(c);
+
+        callback(x, entry.glyph);
 
         if (spw >= 0 && c == ' ') {
             x += spw;
         } else {
-            x += entry->adv;
+            x += entry.adv;
         }
 
         prev = c;
@@ -722,9 +764,9 @@ int gli_draw_string_uni(int x, int y, FontFace face, const Color &rgb,
         int sx = x % GLI_SUBPIX;
 
         if (gli_conf_lcd) {
-            draw_bitmap_lcd_gamma(&glyphs[sx], px, y, rgb);
+            draw_bitmap_lcd_gamma(glyphs[sx], px, y, rgb);
         } else {
-            draw_bitmap_gamma(&glyphs[sx], px, y, rgb);
+            draw_bitmap_gamma(glyphs[sx], px, y, rgb);
         }
     });
 }
@@ -755,7 +797,7 @@ void gli_draw_caret(int x, int y)
     }
 }
 
-void gli_draw_picture(picture_t *pic, int x0, int y0, int dx0, int dy0, int dx1, int dy1)
+void gli_draw_picture(const picture_t *pic, int x0, int y0, int dx0, int dy0, int dx1, int dy1)
 {
     int x1, y1, sx0, sy0, sx1, sy1;
     int w, h;
