@@ -17,58 +17,59 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include <memory>
-#include <sstream>
 #include <string>
 #include <vector>
 
 #include <fontconfig/fontconfig.h>
 
+#include "format.h"
+#include "optional.hpp"
+
+#include "font.h"
 #include "glk.h"
 #include "garglk.h"
 
-static bool initialized = false;
+static FcConfig *cfg;
 
-static std::string findfont(const std::string &fontname)
+static nonstd::optional<std::string> findfont(const std::string &fontname)
 {
     FcChar8 *strval = nullptr;
 
     auto p = garglk::unique(FcNameParse(reinterpret_cast<const FcChar8 *>(fontname.c_str())), FcPatternDestroy);
     if (p == nullptr) {
-        return "";
+        return nonstd::nullopt;
     }
 
     auto os = garglk::unique(FcObjectSetBuild(FC_FILE, static_cast<char *>(nullptr)), FcObjectSetDestroy);
     if (os == nullptr) {
-        return "";
+        return nonstd::nullopt;
     }
 
-    auto fs = garglk::unique(FcFontList(nullptr, p.get(), os.get()), FcFontSetDestroy);
+    auto fs = garglk::unique(FcFontList(cfg, p.get(), os.get()), FcFontSetDestroy);
     if (fs->nfont == 0) {
-        return "";
+        return nonstd::nullopt;
     }
 
     if (FcPatternGetString(fs->fonts[0], FC_FILE, 0, &strval) == FcResultTypeMismatch || strval == nullptr) {
-        return "";
+        return nonstd::nullopt;
     }
 
     return reinterpret_cast<char *>(strval);
 }
 
-static std::string find_font_by_styles(const std::string &basefont, const std::vector<std::string> &styles, const std::vector<std::string> &weights, const std::vector<std::string> &slants)
+static nonstd::optional<std::string> find_font_by_styles(const std::string &basefont, const std::vector<std::string> &styles, const std::vector<int> &weights, const std::vector<std::string> &slants)
 {
     // Prefer normal width fonts, but if there aren't any, allow whatever fontconfig finds.
-    std::vector<std::string> widths = {":normal", ""};
+    std::vector<std::string> widths = {":width=100", ""};
 
     for (const auto &width : widths) {
         for (const auto &style : styles) {
             for (const auto &weight : weights) {
                 for (const auto &slant : slants) {
-                    std::ostringstream fontname;
-                    std::string fontpath;
+                    auto fontname = Format("{}:style={}:weight={}:{}{}", basefont, style, weight, slant, width);
 
-                    fontname << basefont << ":style=" << style << ":weight=" << weight << ":" << slant << width;
-                    fontpath = findfont(fontname.str());
-                    if (!fontpath.empty()) {
+                    auto fontpath = findfont(fontname);
+                    if (fontpath.has_value()) {
                         return fontpath;
                     }
                 }
@@ -76,28 +77,13 @@ static std::string find_font_by_styles(const std::string &basefont, const std::v
         }
     }
 
-    return "";
+    return nonstd::nullopt;
 }
 
-void garglk::fontreplace(const std::string &font, FontType type)
+bool garglk::fontreplace(const std::string &font, FontType type)
 {
-    if (!initialized || font.empty()) {
-        return;
-    }
-
-    std::string sysfont;
-    std::string *r, *b, *i, *z;
-
-    if (type == FontType::Monospace) {
-        r = &gli_conf_mono.r;
-        b = &gli_conf_mono.b;
-        i = &gli_conf_mono.i;
-        z = &gli_conf_mono.z;
-    } else {
-        r = &gli_conf_prop.r;
-        b = &gli_conf_prop.b;
-        i = &gli_conf_prop.i;
-        z = &gli_conf_prop.z;
+    if (cfg == nullptr || font.empty()) {
+        return false;
     }
 
     // Although there are 4 "main" types of font (Regular, Italic, Bold, Bold
@@ -132,52 +118,49 @@ void garglk::fontreplace(const std::string &font, FontType type)
     // return weights that are not medium (100); explicitly listing the
     // weight value doesn't have this problem.
 
-    // Book, Regular, Medium
-    std::vector<std::string> regular_weights = {"75", "80", "100"};
-    // Demibold/Semibold, Bold, Extrabold, Black
-    std::vector<std::string> bold_weights = {"180", "200", "205", "210"};
+    std::vector<int> regular_weights = {FC_WEIGHT_REGULAR, FC_WEIGHT_BOOK, FC_WEIGHT_MEDIUM};
+    std::vector<int> bold_weights = {FC_WEIGHT_BOLD, FC_WEIGHT_EXTRABOLD, FC_WEIGHT_DEMIBOLD, FC_WEIGHT_BLACK};
 
     std::vector<std::string> roman_slants = {"roman"};
     std::vector<std::string> italic_slants = {"italic", "oblique"};
 
     // regular or roman
-    sysfont = find_font_by_styles(font, regular_styles, regular_weights, roman_slants);
-    if (!sysfont.empty()) {
-        *r = sysfont;
-        *b = sysfont;
-        *i = sysfont;
-        *z = sysfont;
+    auto sysfont = find_font_by_styles(font, regular_styles, regular_weights, roman_slants);
+
+    // A regular font is required to exist, so bail if it doesn't (even
+    // if bold, italic, and/or bold italic are available). Without this,
+    // it'd be possible to get a setup where, for example, the bold font
+    // is one family and all other fonts are the fallback. Bold and
+    // italic variations can be created out of a regular font, so it's
+    // OK if those don't exist.
+    if (!sysfont.has_value()) {
+        return false;
     }
 
-    // bold
+    FontFiller filler(type);
+
+    filler.add(FontFiller::Style::Regular, sysfont);
+
     sysfont = find_font_by_styles(font, bold_styles, bold_weights, roman_slants);
-    if (!sysfont.empty()) {
-        *b = sysfont;
-        *z = sysfont;
-    }
+    filler.add(FontFiller::Style::Bold, sysfont);
 
-    // italic or oblique
     sysfont = find_font_by_styles(font, italic_styles, regular_weights, italic_slants);
-    if (!sysfont.empty()) {
-        *i = sysfont;
-        *z = sysfont;
-    }
+    filler.add(FontFiller::Style::Italic, sysfont);
 
-    // bold italic or bold oblique
     sysfont = find_font_by_styles(font, bold_italic_styles, bold_weights, italic_slants);
-    if (!sysfont.empty()) {
-        *z = sysfont;
-    }
+    filler.add(FontFiller::Style::BoldItalic, sysfont);
+
+    return filler.fill();
 }
 
 void fontload()
 {
-    initialized = FcInit() != 0;
+    cfg = FcInitLoadConfigAndFonts();
 }
 
 void fontunload()
 {
-    if (initialized) {
-        FcFini();
+    if (cfg != nullptr) {
+        FcConfigDestroy(cfg);
     }
 }
