@@ -19,147 +19,19 @@
 #import <Cocoa/Cocoa.h>
 
 #include <cstdlib>
+#include <string>
 
+#include "font.h"
 #include "glk.h"
 #include "garglk.h"
 
-enum class FontStyle {
-    Roman,
-    Bold,
-    Italic,
-    BoldItalic
-};
-
-static bool gli_sys_monor = false;
-static bool gli_sys_monob = false;
-static bool gli_sys_monoi = false;
-static bool gli_sys_monoz = false;
-
-static void monofont(const std::string &file, FontStyle style)
-{
-    switch (style) {
-    case FontStyle::Roman:
-        if (!gli_sys_monor) {
-            gli_conf_mono.r = file;
-
-            if (!gli_sys_monob) {
-                gli_conf_mono.b = file;
-            }
-
-            if (!gli_sys_monoi) {
-                gli_conf_mono.i = file;
-            }
-
-            if (!gli_sys_monoz && !gli_sys_monoi && !gli_sys_monob) {
-                gli_conf_mono.z = file;
-            }
-
-            gli_sys_monor = true;
-        }
-        return;
-
-    case FontStyle::Bold:
-        if (!gli_sys_monob) {
-            gli_conf_mono.b = file;
-
-            if (!gli_sys_monoz && !gli_sys_monoi) {
-                gli_conf_mono.z = file;
-            }
-
-            gli_sys_monob = true;
-        }
-        return;
-
-    case FontStyle::Italic:
-        if (!gli_sys_monoi) {
-            gli_conf_mono.i = file;
-
-            if (!gli_sys_monoz) {
-                gli_conf_mono.z = file;
-            }
-
-            gli_sys_monoi = true;
-        }
-        return;
-
-    case FontStyle::BoldItalic:
-        if (!gli_sys_monoz) {
-            gli_conf_mono.z = file;
-            gli_sys_monoz = true;
-        }
-        return;
-    }
-}
-
-static bool gli_sys_propr = false;
-static bool gli_sys_propb = false;
-static bool gli_sys_propi = false;
-static bool gli_sys_propz = false;
-
-static void propfont(const std::string &file, FontStyle style)
-{
-    switch (style) {
-    case FontStyle::Roman: {
-        if (!gli_sys_propr) {
-            gli_conf_prop.r = file;
-
-            if (!gli_sys_propb) {
-                gli_conf_prop.b = file;
-            }
-
-            if (!gli_sys_propi) {
-                gli_conf_prop.i = file;
-            }
-
-            if (!gli_sys_propz && !gli_sys_propi && !gli_sys_propb) {
-                gli_conf_prop.z = file;
-            }
-
-            gli_sys_propr = true;
-        }
-        return;
-    }
-
-    case FontStyle::Bold:
-        if (!gli_sys_propb) {
-            gli_conf_prop.b = file;
-
-            if (!gli_sys_propz && !gli_sys_propi) {
-                gli_conf_prop.z = file;
-            }
-
-            gli_sys_propb = true;
-        }
-        return;
-
-    case FontStyle::Italic:
-        if (!gli_sys_propi) {
-            gli_conf_prop.i = file;
-
-            if (!gli_sys_propz) {
-                gli_conf_prop.z = file;
-            }
-
-            gli_sys_propi = true;
-        }
-        return;
-
-    case FontStyle::BoldItalic:
-        if (!gli_sys_propz) {
-            gli_conf_prop.z = file;
-            gli_sys_propz = true;
-        }
-        return;
-    }
-}
-
 static NSMutableArray *gli_registered_fonts = nil;
 static NSDistributedLock *gli_font_lock = nil;
 
-void garglk::fontreplace(const std::string &font, FontType type)
+bool garglk::fontreplace(const std::string &font, FontType type)
 {
     if (font.empty()) {
-        return;
+        return false;
     }
 
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
@@ -171,18 +43,9 @@ void garglk::fontreplace(const std::string &font, FontType type)
     NSArray *fontMatches =
         [fontFamilyDescriptor matchingFontDescriptorsWithMandatoryKeys: nil];
 
+    FontFiller filler(type);
+
     for (NSFontDescriptor *sysfont in fontMatches) {
-        // find style for font
-        FontStyle style = FontStyle::Roman;
-
-        if (([sysfont symbolicTraits] & NSFontBoldTrait) && ([sysfont symbolicTraits] & NSFontItalicTrait)) {
-            style = FontStyle::BoldItalic;
-        } else if ([sysfont symbolicTraits] & NSFontBoldTrait) {
-            style = FontStyle::Bold;
-        } else if ([sysfont symbolicTraits] & NSFontItalicTrait) {
-            style = FontStyle::Italic;
-        }
-
         // find path for font
         CFURLRef urlRef = static_cast<CFURLRef>(CTFontDescriptorCopyAttribute((CTFontDescriptorRef)sysfont, kCTFontURLAttribute));
         if (!urlRef) {
@@ -195,16 +58,17 @@ void garglk::fontreplace(const std::string &font, FontType type)
             NSString *fontPath = (__bridge NSString *)fontPathRef;
             NSLog(@"fontPath: %@", fontPath);
 
-            std::string filebuf = [fontPath UTF8String];
+            std::string file = [fontPath UTF8String];
+            auto traits = [sysfont symbolicTraits];
 
-            switch (type) {
-            case FontType::Monospace:
-                monofont(filebuf, style);
-                break;
-
-            case FontType::Proportional:
-                propfont(filebuf, style);
-                break;
+            if ((traits & NSFontBoldTrait) && (traits & NSFontItalicTrait)) {
+                filler.add(FontFiller::Style::BoldItalic, file);
+            } else if (traits & NSFontBoldTrait) {
+                filler.add(FontFiller::Style::Bold, file);
+            } else if (traits & NSFontItalicTrait) {
+                filler.add(FontFiller::Style::Italic, file);
+            } else {
+                filler.add(FontFiller::Style::Regular, file);
             }
 
             CFRelease(fontPathRef);
@@ -214,6 +78,8 @@ void garglk::fontreplace(const std::string &font, FontType type)
     }
 
     [pool drain];
+
+    return filler.fill();
 }
 
 void fontload()
