@@ -17,10 +17,17 @@
 // along with Gargoyle; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#define NOMINMAX
 #include <windows.h>
 
 #include <cstdio>
+#include <cstdlib>
+#include <string>
 
+#include "format.h"
+#include "optional.hpp"
+
+#include "font.h"
 #include "glk.h"
 #include "garglk.h"
 
@@ -31,90 +38,34 @@ static bool find_font_file(const std::string &facename, std::string &filepath);
 
 static HDC hdc;
 
-struct Fonts {
-    bool rset = false, bset = false, iset = false, zset = false;
-    std::string r, b, i, z;
-} monofonts, propfonts;
-
-static void fill_in_fonts(Fonts &fonts, std::string &r, std::string &b, std::string &i, std::string &z)
-{
-#define FILL(font) do { if (!fonts.font.empty()) font = fonts.font; } while(0);
-    FILL(r);
-    FILL(b);
-    FILL(i);
-    FILL(z);
-#undef FILL
-}
-
-static int CALLBACK cb_handler(ENUMLOGFONTEX *lpelfe, Fonts *fonts)
+static int CALLBACK font_cb(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *, int, LPARAM filler_)
 {
+    FontFiller *filler = reinterpret_cast<FontFiller *>(filler_);
     std::string filepath;
     std::string style = reinterpret_cast<char *>(lpelfe->elfStyle);
 
-    *fonts = Fonts();
-
     if (!find_font_file(reinterpret_cast<char *>(lpelfe->elfFullName), filepath)) {
         return 1;
     }
 
-    if (!fonts->rset && (style == "Regular" || style == "Roman")) {
-        fonts->r = filepath;
-        fonts->rset = true;
-
-        if (!fonts->bset) {
-            fonts->b = filepath;
-        }
-
-        if (!fonts->iset) {
-            fonts->i = filepath;
-        }
-
-        if (!fonts->zset && !fonts->iset && !fonts->bset) {
-            fonts->z = filepath;
-        }
-    }
-
-    else if (!fonts->bset && style == "Bold") {
-        fonts->b = filepath;
-        fonts->bset = true;
-
-        if (!fonts->zset && !fonts->iset) {
-            fonts->z = filepath;
-        }
-    }
-
-    else if (!fonts->iset && (style == "Italic" || style == "Oblique")) {
-        fonts->i = filepath;
-        fonts->iset = true;
-
-        if (!fonts->zset) {
-            fonts->z = filepath;
-        }
-    }
-
-    else if (!fonts->zset && (style == "Bold Italic" || style == "Bold Oblique" || style == "BoldOblique" || style == "BoldItalic")) {
-        fonts->z = filepath;
-        fonts->zset = true;
+    if (style == "Regular" || style == "Roman") {
+        filler->add(FontFiller::Style::Regular, filepath);
+    } else if (style == "Bold") {
+        filler->add(FontFiller::Style::Bold, filepath);
+    } else if (style == "Italic" || style == "Oblique") {
+        filler->add(FontFiller::Style::Italic, filepath);
+    } else if (style == "Bold Italic" || style == "Bold Oblique" || style == "BoldItalic" || style == "BoldOblique") {
+        filler->add(FontFiller::Style::BoldItalic, filepath);
     }
 
     return 0;
 }
 
-static int CALLBACK monofont_cb(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, int FontType, LPARAM lParam)
-{
-    return cb_handler(lpelfe, &monofonts);
-}
-
-static int CALLBACK propfont_cb(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, int FontType, LPARAM lParam)
-{
-    return cb_handler(lpelfe, &propfonts);
-}
-
 static std::string make_font_filepath(const std::string &filename)
 {
     // create the absolute path to the font file
     if (filename.find(':') == std::string::npos && std::getenv("SYSTEMROOT") != nullptr) {
-        return std::string(std::getenv("SYSTEMROOT")) + "\\Fonts\\" + filename;
+        return Format("{}\\Fonts\\{}", std::getenv("SYSTEMROOT"), filename);
     } else {
         return filename;
     }
@@ -128,27 +79,27 @@ static bool find_font_file_with_key(HKEY key, const char *subkey, const std::str
     char filename[256];
 
     // open the Fonts registry key
-    if (RegOpenKeyEx(key, subkey, 0, KEY_QUERY_VALUE, &hkey) != ERROR_SUCCESS) {
+    if (RegOpenKeyExA(key, subkey, 0, KEY_QUERY_VALUE, &hkey) != ERROR_SUCCESS) {
         return false;
     }
 
     // check for a TrueType font
-    face = facename + " (TrueType)";
+    face = Format("{} (TrueType)", facename);
 
     size = sizeof(filename);
 
-    if (RegQueryValueEx(hkey, face.c_str(), NULL, NULL, (PBYTE)filename, &size) == ERROR_SUCCESS) {
+    if (RegQueryValueExA(hkey, face.c_str(), nullptr, nullptr, (PBYTE)filename, &size) == ERROR_SUCCESS) {
         filepath = make_font_filepath(filename);
         RegCloseKey(hkey);
         return true;
     }
 
     // check for an OpenType font
-    face = facename + " (OpenType)";
+    face = Format("{} (OpenType)", facename);
 
     size = sizeof(filename);
 
-    if (RegQueryValueEx(hkey, face.c_str(), NULL, NULL, (PBYTE)filename, &size) == ERROR_SUCCESS) {
+    if (RegQueryValueExA(hkey, face.c_str(), nullptr, nullptr, (PBYTE)filename, &size) == ERROR_SUCCESS) {
         filepath = make_font_filepath(filename);
         RegCloseKey(hkey);
         return true;
@@ -172,13 +123,13 @@ static bool find_font_file(const std::string &facename, std::string &filepath)
     return find_font_file_with_key(HKEY_LOCAL_MACHINE, FONT_SUBKEY, facename, filepath);
 }
 
-void garglk::fontreplace(const std::string &font, FontType type)
+bool garglk::fontreplace(const std::string &font, FontType type)
 {
     if (font.empty()) {
-        return;
+        return false;
     }
 
-    LOGFONT logfont;
+    LOGFONTA logfont;
 
     ZeroMemory(&logfont, sizeof logfont);
     logfont.lfCharSet = DEFAULT_CHARSET;
@@ -186,20 +137,14 @@ void garglk::fontreplace(const std::string &font, FontType type)
 
     hdc = GetDC(0);
 
-    switch (type) {
-    case FontType::Monospace:
-        std::snprintf(logfont.lfFaceName, LF_FACESIZE, "%s", font.c_str());
-        EnumFontFamiliesEx(hdc, &logfont, (FONTENUMPROC)monofont_cb, 0, 0);
-        fill_in_fonts(monofonts, gli_conf_mono.r, gli_conf_mono.b, gli_conf_mono.i, gli_conf_mono.z);
-        break;
-    case FontType::Proportional:
-        std::snprintf(logfont.lfFaceName, LF_FACESIZE, "%s", font.c_str());
-        EnumFontFamiliesEx(hdc, &logfont, (FONTENUMPROC)propfont_cb, 0, 0);
-        fill_in_fonts(propfonts, gli_conf_prop.r, gli_conf_prop.b, gli_conf_prop.i, gli_conf_prop.z);
-        break;
-    }
+    FontFiller filler(type);
+
+    std::snprintf(logfont.lfFaceName, LF_FACESIZE, "%s", font.c_str());
+    EnumFontFamiliesExA(hdc, &logfont, reinterpret_cast<FONTENUMPROCA>(font_cb), reinterpret_cast<LPARAM>(&filler), 0);
 
     ReleaseDC(0, hdc);
+
+    return filler.fill();
 }
 
 void fontload()
