@@ -1,3 +1,4 @@
+// Copyright (C) 1998-2016 by Andrew Plotkin.
 // Copyright (C) 2006-2009 by Tor Andersson, Jesse McGrew.
 // Copyright (C) 2010 by Ben Cressey, Chris Spiegel.
 //
@@ -18,32 +19,34 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 // Private header file for the Gargoyle Implementation of the Glk API.
-// Glk API which this implements: version 0.7.3.
+// Glk API which this implements: version 0.7.6.
 // Glk designed by Andrew Plotkin <erkyrath@eblong.com>
 // http://www.eblong.com/zarf/glk/index.html
 
 #ifndef GARGLK_GARGLK_H
 #define GARGLK_GARGLK_H
 
-#ifdef __cplusplus
 #include <algorithm>
 #include <array>
 #include <cmath>
 #include <cstddef>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <deque>
 #include <exception>
-#include <fstream>
 #include <functional>
 #include <iostream>
+#include <map>
 #include <memory>
+#include <sstream>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
 #include "optional.hpp"
+#include "variant.hpp"
 
 #include "glk.h"
 #include "gi_dispa.h"
@@ -120,24 +123,7 @@ struct ConfigFile {
     ConfigFile(std::string path_, Type type_) : path(std::move(path_)), type(type_) {
     }
 
-    std::string format_type() const {
-        std::string status = "";
-        std::ifstream f(path);
-        if (!f.is_open()) {
-            status = ", non-existent";
-        }
-
-        switch (type) {
-        case Type::System:
-            return "[system" + status + "]";
-        case Type::User:
-            return "[user" + status + "]";
-        case Type::PerGame:
-            return "[game specific" + status + "]";
-        default:
-            return "";
-        }
-    }
+    std::string format_type() const;
 
     // The path to the file itself.
     std::string path;
@@ -164,7 +150,7 @@ extern std::vector<garglk::ConfigFile> all_configs;
 
 // C++17: std::clamp
 template <typename T>
-const T &clamp(const T &value, const T &min, const T &max)
+constexpr const T &clamp(const T &value, const T &min, const T &max)
 {
     return value < min ? min : value > max ? max : value;
 }
@@ -173,19 +159,23 @@ std::string winopenfile(const char *prompt, FileFilter filter);
 std::string winsavefile(const char *prompt, FileFilter filter);
 [[noreturn]]
 void winabort(const std::string &msg);
+void winwarning(const std::string &title, const std::string &msg);
 std::string downcase(const std::string &string);
-void fontreplace(const std::string &font, FontType type);
-std::vector<ConfigFile> configs(const std::string &gamepath);
-void config_entries(const std::string &fname, bool accept_bare, const std::vector<std::string> &matches, const std::function<void(const std::string &cmd, const std::string &arg)> &callback);
+bool fontreplace(const std::string &font, FontType type);
+std::vector<ConfigFile> configs(const nonstd::optional<std::string> &gamepath);
+void config_entries(const std::string &fname, bool accept_bare, const std::vector<std::string> &matches, const std::function<void(const std::string &cmd, const std::string &arg, int lineno)> &callback);
 std::string user_config();
-void set_lcdfilter(const std::string &filter);
-std::string winfontpath(const std::string &filename);
-std::vector<std::string> winappdata();
-std::string winappdir();
+bool set_lcdfilter(const std::string &filter);
+nonstd::optional<std::string> winfontpath(const std::string &filename);
+std::string windatadir();
+std::vector<std::string> winthemedirs();
+nonstd::optional<std::string> winlegacythemedir();
+nonstd::optional<std::string> winappdir();
+bool winisfullscreen();
 
 namespace theme {
 void init();
-void set(std::string name);
+bool set(std::string name);
 std::vector<std::string> paths();
 std::vector<std::string> names();
 }
@@ -196,6 +186,25 @@ std::unique_ptr<T, Deleter> unique(T *p, Deleter deleter)
     return std::unique_ptr<T, Deleter>(p, deleter);
 }
 
+bool read_file(const std::string &filename, std::vector<unsigned char> &buf);
+
+template <typename Iterable, typename DType>
+std::string join(const Iterable &values, const DType &delim)
+{
+    auto it = std::begin(values);
+    std::ostringstream result;
+
+    if (it != std::end(values)) {
+        result << *it++;
+    }
+
+    for (; it != std::end(values); ++it) {
+        result << delim << *it;
+    }
+
+    return result.str();
+}
+
 }
 
 template <std::size_t N>
@@ -208,9 +217,12 @@ class PixelView;
 // data; contrast this with PixelView which holds a pointer to a pixel.
 template <std::size_t N>
 class Pixel {
+    static_assert(N > 0, "Pixel size must be non-zero");
+
 public:
     template <typename... Args>
     explicit Pixel(Args... args) : m_pixel{static_cast<unsigned char>(args)...} {
+        static_assert(sizeof...(Args) == N, "Incorrect argument count for Pixel");
     }
 
     // This is intentionally _not_ explicit so that PixelViews can be
@@ -243,8 +255,8 @@ private:
 // is *not* owned by instances of this class. Instead, users of this
 // class must pass in a pointer to an appropriately-sized buffer (at
 // least N bytes) from/to which pixel data will be read/written.
-// Assigning a Pixel to a PixelView will overwrite the pixel pointer
-// with the new pixel data.
+// Assigning a Pixel to a PixelView will copy the Pixel's data; it does
+// _not_ point the PixelView to the Pixel.
 template <std::size_t N>
 class PixelView {
 public:
@@ -274,7 +286,7 @@ public:
     }
 
 private:
-    unsigned char *m_data;
+    unsigned char *const m_data;
 };
 
 template <std::size_t N>
@@ -295,12 +307,35 @@ public:
     }
 
 private:
-    unsigned char *m_row;
+    unsigned char *const m_row;
+};
+
+template <std::size_t N>
+class ConstRow {
+public:
+    explicit ConstRow(const unsigned char *row) : m_row(row) {
+    }
+
+    const unsigned char *operator[](std::size_t x) const {
+        return &m_row[x * N];
+    }
+
+private:
+    const unsigned char *const m_row;
 };
 
 template <std::size_t N>
 class Canvas {
 public:
+    Canvas() = default;
+
+    Canvas(int width, int height) :
+        m_width(width),
+        m_height(height)
+    {
+        resize(width, height, false);
+    }
+
     void resize(int width, int height, bool keep) {
         if (keep) {
             auto backup = m_pixels;
@@ -323,15 +358,15 @@ public:
         m_stride = width * N;
     }
 
-    int width() {
+    int width() const {
         return m_width;
     }
 
-    int height() {
+    int height() const {
         return m_height;
     }
 
-    int stride() {
+    int stride() const {
         return m_stride;
     }
 
@@ -349,10 +384,18 @@ public:
         return Row<N>(&m_pixels[y * stride()]);
     }
 
+    ConstRow<N> operator[](std::size_t y) const {
+        return ConstRow<N>(&m_pixels[y * stride()]);
+    }
+
     unsigned char *data() {
         return m_pixels.data();
     }
 
+    const unsigned char *data() const {
+        return m_pixels.data();
+    }
+
     std::size_t size() const {
         return m_pixels.size();
     }
@@ -360,6 +403,9 @@ public:
     void clear() {
         m_pixels.clear();
         m_pixels.shrink_to_fit();
+        m_width = 0;
+        m_height = 0;
+        m_stride = 0;
     }
 
 private:
@@ -383,10 +429,7 @@ public:
     std::vector<std::uint8_t> &at(int number);
 
 private:
-    std::unordered_map<int, nonstd::optional<std::vector<std::uint8_t>>> m_bleeps = {
-        {1, nonstd::nullopt},
-        {2, nonstd::nullopt},
-    };
+    std::array<nonstd::optional<std::vector<std::uint8_t>>, 2> m_bleeps;
 };
 
 extern Bleeps gli_bleeps;
@@ -462,13 +505,18 @@ struct rect_t {
 };
 
 struct picture_t {
-    picture_t(unsigned int id_, int w_, int h_, bool scaled_) : w(w_), h(h_), id(id_), scaled(scaled_) {
-        rgba.resize(w, h, false);
+    picture_t(unsigned long id_, Canvas<4> rgba_, bool scaled_) :
+        id(id_),
+        rgba(std::move(rgba_)),
+        w(rgba.width()),
+        h(rgba.height()),
+        scaled(scaled_)
+    {
     }
 
-    int w, h;
-    Canvas<4> rgba;
     unsigned long id;
+    Canvas<4> rgba;
+    int w, h;
     bool scaled;
 };
 
@@ -499,13 +547,13 @@ extern Canvas<3> gli_image_rgb;
 //
 
 extern std::string gli_workdir;
-extern std::string gli_workfile;
+extern nonstd::optional<std::string> gli_workfile;
 
 extern Styles gli_tstyles;
 extern Styles gli_gstyles;
 
-extern const Styles gli_tstyles_def;
-extern const Styles gli_gstyles_def;
+extern Styles gli_tstyles_def;
+extern Styles gli_gstyles_def;
 
 extern Color gli_window_color;
 extern Color gli_border_color;
@@ -519,10 +567,8 @@ extern Color gli_caret_save;
 extern Color gli_more_save;
 extern Color gli_link_save;
 
-extern bool gli_override_fg_set;
-extern Color gli_override_fg_val;
-extern bool gli_override_bg_set;
-extern Color gli_override_bg_val;
+extern nonstd::optional<Color> gli_override_fg;
+extern nonstd::optional<Color> gli_override_bg;
 extern bool gli_override_reverse;
 
 extern bool gli_underline_hyperlinks;
@@ -539,17 +585,23 @@ extern int gli_wpaddingy;
 extern int gli_tmarginx;
 extern int gli_tmarginy;
 
-extern float gli_backingscalefactor;
-extern float gli_zoom;
+extern double gli_backingscalefactor;
+extern double gli_zoom;
 
 extern bool gli_conf_lcd;
 extern std::array<unsigned char, 5> gli_conf_lcd_weights;
 
 extern bool gli_conf_graphics;
 extern bool gli_conf_sound;
+extern std::deque<std::string> gli_conf_soundfonts;
+
+extern bool gli_conf_fluidsynth_reverb;
+extern bool gli_conf_fluidsynth_chorus;
 
 extern bool gli_conf_fullscreen;
 
+extern bool gli_wait_on_quit;
+
 extern bool gli_conf_speak;
 extern bool gli_conf_speak_input;
 
@@ -581,18 +633,29 @@ extern int gli_baseline;
 extern int gli_leading;
 
 struct FontFiles {
-    std::string r, b, i, z;
-};
-extern std::string gli_conf_propfont;
-extern FontFiles gli_conf_prop, gli_conf_prop_override;
-extern std::string gli_conf_monofont;
-extern FontFiles gli_conf_mono, gli_conf_mono_override;
+    struct {
+        nonstd::optional<std::string> base;
+        nonstd::optional<std::string> override;
 
-extern float gli_conf_gamma;
-extern float gli_conf_propsize;
-extern float gli_conf_monosize;
-extern float gli_conf_propaspect;
-extern float gli_conf_monoaspect;
+        const nonstd::optional<std::string> &fontpath() const {
+            return override.has_value() ? override : base;
+        }
+    } r, b, i, z;
+};
+
+#define DEFAULT_MONO_FONT	"Gargoyle Mono"
+#define DEFAULT_PROP_FONT	"Gargoyle Serif"
+
+extern std::string gli_conf_propfont;
+extern FontFiles gli_conf_prop;
+extern std::string gli_conf_monofont;
+extern FontFiles gli_conf_mono;
+
+extern double gli_conf_gamma;
+extern double gli_conf_propsize;
+extern double gli_conf_monosize;
+extern double gli_conf_propaspect;
+extern double gli_conf_monoaspect;
 
 extern std::vector<glui32> gli_more_prompt;
 extern glui32 gli_more_prompt_len;
@@ -606,6 +669,21 @@ extern bool gli_claimselect;
 
 extern bool gli_conf_per_game_config;
 
+enum class GamedataLocation {
+    Default,
+    Dedicated,
+    Gamedir,
+};
+
+extern GamedataLocation gli_conf_gamedata_location;
+
+enum class Scaler {
+    None,
+    HQX,
+    XBRZ,
+};
+extern Scaler gli_conf_scaler;
+
 extern std::unordered_map<FontFace, std::vector<std::string>> gli_conf_glyph_substitution_files;
 
 // XXX See issue #730.
@@ -705,18 +783,14 @@ struct glk_fileref_struct {
 #undef hyper
 
 struct attr_t {
-    bool fgset = false;
-    bool bgset = false;
     bool reverse = false;
     glui32 style = 0;
-    Color fgcolor = Color(0, 0, 0);
-    Color bgcolor = Color(0, 0, 0);
+    nonstd::optional<Color> fgcolor;
+    nonstd::optional<Color> bgcolor;
     glui32 hyper = 0;
 
     bool operator!=(const attr_t &other) const {
-        return fgset != other.fgset ||
-               bgset != other.bgset ||
-               reverse != other.reverse ||
+        return reverse != other.reverse ||
                style != other.style ||
                fgcolor != other.fgcolor ||
                bgcolor != other.bgcolor ||
@@ -743,13 +817,19 @@ struct glk_window_struct {
     window_t *parent = nullptr; // pair window which contains this one
     rect_t bbox;
     int yadj = 0;
-    union {
-        window_textgrid_t *textgrid;
-        window_textbuffer_t *textbuffer;
-        window_graphics_t *graphics;
-        window_blank_t *blank;
-        window_pair_t *pair;
-    } window;
+    nonstd::variant<
+        std::unique_ptr<window_textgrid_t>,
+        std::unique_ptr<window_textbuffer_t>,
+        std::unique_ptr<window_graphics_t>,
+        std::unique_ptr<window_blank_t>,
+        std::unique_ptr<window_pair_t>
+    > window;
+
+    window_textgrid_t *wingrid() { return nonstd::get<std::unique_ptr<window_textgrid_t>>(window).get(); }
+    window_textbuffer_t *winbuffer() { return nonstd::get<std::unique_ptr<window_textbuffer_t>>(window).get(); }
+    window_graphics_t *wingraphics() { return nonstd::get<std::unique_ptr<window_graphics_t>>(window).get(); }
+    window_blank_t *winblank() { return nonstd::get<std::unique_ptr<window_blank_t>>(window).get(); }
+    window_pair_t *winpair() { return nonstd::get<std::unique_ptr<window_pair_t>>(window).get(); }
 
     stream_t *str;               // the window stream.
     stream_t *echostr = nullptr; // the window's echo stream, if any.
@@ -816,12 +896,18 @@ struct tgline_t {
 };
 
 struct window_textgrid_t {
-    window_textgrid_t(window_t *owner_, Styles styles_) :
-        owner(owner_),
-        styles(std::move(styles_))
+    explicit window_textgrid_t(window_t *owner_) :
+        owner(owner_)
     {
     }
 
+    ~window_textgrid_t() {
+        if (inbuf != nullptr && gli_unregister_arr != nullptr) {
+            const char *typedesc = (inunicode ? "&+#!Iu" : "&+#!Cn");
+            gli_unregister_arr(inbuf, inoriglen, const_cast<char *>(typedesc), inarrayrock);
+        }
+    }
+
     window_t *owner;
 
     int width = 0, height = 0;
@@ -837,10 +923,9 @@ struct window_textgrid_t {
     int incurs, inlen;
     attr_t origattr;
     gidispatch_rock_t inarrayrock;
-    std::vector<glui32> line_terminators;
 
     // style hints and settings
-    Styles styles;
+    Styles styles = gli_gstyles;
 };
 
 struct tbline_t {
@@ -848,6 +933,7 @@ struct tbline_t {
         chars.fill(' ');
     }
     int len = 0;
+    int flow_break_pos = -1;
     bool newline = false, dirty = false, repaint = false;
     std::shared_ptr<picture_t> lpic, rpic;
     glui32 lhyper = 0, rhyper = 0;
@@ -857,16 +943,21 @@ struct tbline_t {
 };
 
 struct window_textbuffer_t {
-    window_textbuffer_t(window_t *owner_, Styles styles_, int scrollback_) :
-        owner(owner_),
-        scrollback(scrollback_),
-        styles(std::move(styles_))
+    explicit window_textbuffer_t(window_t *owner_) :
+        owner(owner_)
     {
         lines.resize(scrollback);
         chars = lines[0].chars.data();
         attrs = lines[0].attrs.data();
     }
 
+    ~window_textbuffer_t() {
+        if (inbuf != nullptr && gli_unregister_arr != nullptr) {
+            const char *typedesc = (inunicode ? "&+#!Iu" : "&+#!Cn");
+            gli_unregister_arr(inbuf, inmax, const_cast<char *>(typedesc), inarrayrock);
+        }
+    }
+
     window_t *owner;
 
     int width = -1, height = -1;
@@ -904,15 +995,11 @@ struct window_textbuffer_t {
     attr_t origattr;
     gidispatch_rock_t inarrayrock;
 
-    bool echo_line_input = true;
-    std::vector<glui32> line_terminators;
-
     // style hints and settings
-    Styles styles;
+    Styles styles = gli_tstyles;
 
     // for copy selection
     std::vector<glui32> copybuf;
-    int copypos = 0;
 };
 
 struct window_graphics_t {
@@ -947,19 +1034,13 @@ extern gidispatch_rock_t gli_sound_get_channel_disprock(const channel_t *chan);
 [[noreturn]]
 extern void gli_exit(int status);
 
-extern window_blank_t *win_blank_create(window_t *win);
-extern void win_blank_destroy(window_blank_t *dwin);
-extern void win_blank_rearrange(window_t *win, rect_t *box);
+extern void win_blank_rearrange(window_t *win, const rect_t *box);
 extern void win_blank_redraw(window_t *win);
 
-extern window_pair_t *win_pair_create(window_t *win, glui32 method, window_t *key, glui32 size);
-extern void win_pair_destroy(window_pair_t *dwin);
-extern void win_pair_rearrange(window_t *win, rect_t *box);
+extern void win_pair_rearrange(window_t *win, const rect_t *box);
 extern void win_pair_redraw(window_t *win);
 extern void win_pair_click(window_pair_t *dwin, int x, int y);
 
-extern window_textgrid_t *win_textgrid_create(window_t *win);
-extern void win_textgrid_destroy(window_textgrid_t *dwin);
 extern void win_textgrid_rearrange(window_t *win, rect_t *box);
 extern void win_textgrid_redraw(window_t *win);
 extern void win_textgrid_putchar_uni(window_t *win, glui32 ch);
@@ -973,8 +1054,6 @@ extern void win_textgrid_click(window_textgrid_t *dwin, int x, int y);
 extern void gcmd_grid_accept_readchar(window_t *win, glui32 arg);
 extern void gcmd_grid_accept_readline(window_t *win, glui32 arg);
 
-extern window_textbuffer_t *win_textbuffer_create(window_t *win);
-extern void win_textbuffer_destroy(window_textbuffer_t *dwin);
 extern void win_textbuffer_rearrange(window_t *win, rect_t *box);
 extern void win_textbuffer_redraw(window_t *win);
 extern void win_textbuffer_putchar_uni(window_t *win, glui32 ch);
@@ -992,7 +1071,7 @@ extern bool gcmd_accept_scroll(window_t *win, glui32 arg);
 
 extern void gli_initialize_misc();
 extern void gli_initialize_windows();
-extern void gli_initialize_babel();
+extern void gli_initialize_babel(const std::string &filename);
 
 extern window_t *gli_window_iterate_treeorder(window_t *win);
 
@@ -1004,7 +1083,7 @@ extern bool gli_window_check_terminator(glui32 ch);
 extern void gli_window_refocus(window_t *win);
 
 extern void gli_windows_redraw();
-extern void gli_windows_size_change(int w, int h);
+extern void gli_windows_size_change(int w, int h, bool post_arrange_event);
 extern void gli_windows_unechostream(stream_t *str);
 
 extern void gli_window_click(window_t *win, int x, int y);
@@ -1012,6 +1091,7 @@ extern void gli_window_click(window_t *win, int x, int y);
 void gli_redraw_rect(int x0, int y0, int x1, int y1);
 
 void gli_input_handle_key(glui32 key);
+void gli_input_handle_key_paste(glui32 key);
 void gli_input_handle_click(int x, int y);
 void gli_event_store(glui32 type, window_t *win, glui32 val1, glui32 val2);
 
@@ -1035,54 +1115,48 @@ void gli_draw_rect(int x, int y, int w, int h, const Color &rgb);
 int gli_draw_string_uni(int x, int y, FontFace face, const Color &rgb, const glui32 *text, int len, int spacewidth);
 int gli_string_width_uni(FontFace face, const glui32 *text, int len, int spacewidth);
 void gli_draw_caret(int x, int y);
-void gli_draw_picture(picture_t *pic, int x, int y, int x0, int y0, int x1, int y1);
-
-void gli_startup(int argc, char *argv[]);
+void gli_draw_picture(const picture_t *pic, int x0, int y0, int dx0, int dy0, int dx1, int dy1);
 
 extern void gli_select(event_t *event, bool polled);
 #ifdef GARGLK_CONFIG_TICK
 extern void gli_tick();
 #endif
 
-void wininit(int *argc, char **argv);
+void wininit();
 void winopen();
 void wintitle();
 void winmore();
 void winrepaint(int x0, int y0, int x1, int y1);
 bool windark();
 void winexit();
-void winclipstore(const glui32 *text, int len);
+void winclipstore(const std::vector<glui32> &text);
 
 void fontload();
 void fontunload();
 
-void giblorb_get_resource(glui32 usage, glui32 resnum, std::FILE **file, long *pos, long *len, glui32 *type);
+bool giblorb_copy_resource(glui32 usage, glui32 resnum, glui32 &type, std::vector<unsigned char> &buf);
 
 std::shared_ptr<picture_t> gli_picture_load(unsigned long id);
 void gli_picture_store(const std::shared_ptr<picture_t> &pic);
 std::shared_ptr<picture_t> gli_picture_retrieve(unsigned long id, bool scaled);
-std::shared_ptr<picture_t> gli_picture_scale(picture_t *src, int newcols, int newrows);
+std::shared_ptr<picture_t> gli_picture_scale(const picture_t *src, int newcols, int newrows);
 void gli_piclist_increment();
 void gli_piclist_decrement();
 
-window_graphics_t *win_graphics_create(window_t *win);
-void win_graphics_destroy(window_graphics_t *dwin);
 void win_graphics_rearrange(window_t *win, rect_t *box);
-void win_graphics_get_size(window_t *win, glui32 *width, glui32 *height);
 void win_graphics_redraw(window_t *win);
 void win_graphics_click(window_graphics_t *dwin, int x, int y);
 
-bool win_graphics_draw_picture(window_graphics_t *dwin,
-  glui32 image, glsi32 xpos, glsi32 ypos,
-  bool scale, glui32 imagewidth, glui32 imageheight);
+bool win_graphics_draw_picture(std::shared_ptr<picture_t> pic, window_graphics_t *dwin,
+  glsi32 xpos, glsi32 ypos, glui32 imagewidth, glui32 imageheight);
 void win_graphics_erase_rect(window_graphics_t *dwin, bool whole, glsi32 x0, glsi32 y0, glui32 width, glui32 height);
 void win_graphics_fill_rect(window_graphics_t *dwin, glui32 color, glsi32 x0, glsi32 y0, glui32 width, glui32 height);
 void win_graphics_set_background_color(window_graphics_t *dwin, glui32 color);
 
-bool win_textbuffer_draw_picture(window_textbuffer_t *dwin, glui32 image, glui32 align, bool scaled, glui32 width, glui32 height);
+bool win_textbuffer_draw_picture(std::shared_ptr<picture_t> pic, window_textbuffer_t *dwin, glui32 window_width, glui32 align, glui32 width, glui32 height, glui32 maxwidth);
 void win_textbuffer_flow_break(window_textbuffer_t *win);
 
-void gli_calc_padding(window_t *win, int *x, int *y);
+void gli_read_config(int argc, char **argv);
 
 // unicode case mapping
 
@@ -1099,6 +1173,7 @@ using gli_decomp_block_t = glui32[2];
 // If the count is zero, there is no decomposition.
 
 void gli_putchar_utf8(glui32 val, std::FILE *fl);
+extern int gli_encode_utf8(glui32 val, char *buf, int len);
 glui32 gli_getchar_utf8(std::FILE *fl);
 glui32 gli_parse_utf8(const unsigned char *buf, glui32 buflen, glui32 *out, glui32 outlen);
 
@@ -1109,7 +1184,7 @@ glui32 gli_get_hyperlink(int x, int y);
 void gli_clear_selection();
 bool gli_check_selection(int x0, int y0, int x1, int y1);
 bool gli_get_selection(int x0, int y0, int x1, int y1, int *rx0, int *rx1);
-void gli_clipboard_copy(const glui32 *buf, int len);
+void gli_clipboard_copy(const std::vector<glui32> &buf);
 void gli_start_selection(int x, int y);
 void gli_resize_mask(unsigned int x, unsigned int y);
 void gli_move_selection(int x, int y);
@@ -1117,6 +1192,11 @@ void gli_notification_waiting();
 
 void gli_edit_config();
 
+nonstd::optional<std::vector<char>> gli_get_scrollback();
+std::vector<char> gli_get_text(window_textbuffer_t *dwin);
+
+const std::map<glui32, std::vector<unsigned char>> &gli_get_resource_map(glui32 usage);
+
 // A macro which reads and decodes one character of UTF-8. Needs no
 // explanation, I'm sure.
 //
@@ -1157,6 +1237,5 @@ void gli_edit_config();
             )) \
         )) \
     )
-#endif
 
 #endif
