@@ -22,11 +22,14 @@
 #define GARGLK_LAUNCHER_H
 
 #include <string>
+#include <vector>
+
+#include "optional.hpp"
 
 namespace garglk {
 
 void winmsg(const std::string &msg);
-bool winterp(const std::string &exe, const std::string &flags, const std::string &game);
+bool winterp(const std::string &exe, const std::vector<std::string> &flags, const std::string &game);
 bool rungame(const std::string &game);
 
 }
