@@ -16,6 +16,8 @@
 // along with Gargoyle; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "format.h"
+
 #include "garglk.h"
 #include "garversion.h"
 #include "launcher.h"
@@ -28,14 +30,20 @@
 #import <Cocoa/Cocoa.h>
 #import <OpenGL/gl.h>
 #import <OpenGL/glu.h>
-#include <mach-o/dyld.h>
+#include <libproc.h>
+#include <unistd.h>
 #import "sysmac.h"
 
-#define MaxBuffer 1024
+static const char *AppName = "Gargoyle " GARGOYLE_VERSION;
 
-static const char *AppName = GARGOYLE_NAME " " GARGOYLE_VERSION;
-
-static std::string winpath();
+// Qt's origin is the top-left, but Mac's is bottom left, so translate,
+// since the config format is determined by Qt.
+static int coord_flip(NSWindow *window)
+{
+    auto screen_height = [NSScreen mainScreen].frame.size.height;
+    auto content = [NSWindow contentRectForFrameRect: window.frame styleMask: window.styleMask];
+    return screen_height - content.size.height;
+}
 
 static const std::unordered_map<FileFilter, std::string> winfilters = {
     {FileFilter::Save, "glksave"},
@@ -162,11 +170,15 @@ static const std::unordered_map<FileFilter, std::string> winfilters = {
 - (void) moveWordBackward: (id) sender;
 - (void) moveWordForward: (id) sender;
 - (IBAction) performZoom: (id) sender;
+- (void) saveWindows;
 - (void) performRefresh: (NSNotification *) notice;
+- (void) performMove: (NSNotification *) notice;
 - (NSString *) openFileDialog: (NSString *) prompt
-                   fileFilter: (FileFilter) filter;
+                   fileFilter: (FileFilter) filter
+                      savedir: (NSString *) savedir;
 - (NSString *) saveFileDialog: (NSString *) prompt
-                   fileFilter: (FileFilter) filter;
+                   fileFilter: (FileFilter) filter
+                      savedir: (NSString *) savedir;
 - (pid_t) retrieveID;
 - (void) quit;
 @end
@@ -197,19 +209,22 @@ static const std::unordered_map<FileFilter, std::string> winfilters = {
     [self setAcceptsMouseMovedEvents: YES];
     lastMouseMove = 0;
 
-    [[NSNotificationCenter defaultCenter] addObserver: self
-                                             selector: @selector(performRefresh:)
-                                                 name: NSWindowDidDeminiaturizeNotification
-                                               object: self];
+    std::vector<NSString *> names = {
+        NSWindowDidDeminiaturizeNotification,
+        NSWindowDidChangeScreenNotification,
+        NSWindowDidEndLiveResizeNotification,
+    };
+
+    for (const auto &name : names) {
+        [[NSNotificationCenter defaultCenter] addObserver: self
+                                                 selector: @selector(performRefresh:)
+                                                     name: name
+                                                   object: self];
+    }
 
     [[NSNotificationCenter defaultCenter] addObserver: self
                                              selector: @selector(performRefresh:)
-                                                 name: NSWindowDidChangeScreenNotification
-                                               object: self];
-
-    [[NSNotificationCenter defaultCenter] addObserver: self
-                                             selector: @selector(performRefresh:)
-                                                 name: NSWindowDidEndLiveResizeNotification
+                                                 name: NSWindowDidMoveNotification
                                                object: self];
 
     return self;
@@ -467,21 +482,40 @@ static BOOL isTextbufferEvent(NSEvent *evt)
     [super performZoom: sender];
 }
 
-- (void) performRefresh: (NSNotification *) notice
+- (void) saveWindows
 {
-    if (gli_conf_save_window_size) {
+    if (gli_conf_save_window_location || gli_conf_save_window_size) {
         auto path = get_qt_plist_path();
         if (path != nil) {
             NSMutableDictionary *config = [NSMutableDictionary dictionaryWithContentsOfFile: path];
-            std::stringstream size;
-            int w = self.frame.size.width;
-            int h = self.frame.size.height;
+            if (config == nil) {
+                config = [NSMutableDictionary dictionary];
+            }
+
+            if (gli_conf_save_window_location) {
+                auto location = Format("@Point({} {})", self.frame.origin.x, coord_flip(self) - self.frame.origin.y);
+                [config setObject: [NSString stringWithUTF8String: location.c_str()] forKey: @"window.position"];
+            }
+
+            if (gli_conf_save_window_size) {
+                auto size = Format("@Size({} {})", self.frame.size.width, self.frame.size.height);
+                [config setObject: [NSString stringWithUTF8String: size.c_str()] forKey: @"window.size"];
+            }
+
+            if (gli_conf_save_window_location || gli_conf_save_window_size) {
+                auto is_fullscreen = (self.styleMask & NSWindowStyleMaskFullScreen) == NSWindowStyleMaskFullScreen;
+                auto as_object = [NSNumber numberWithBool: is_fullscreen];
+                [config setObject: as_object forKey: @"window.fullscreen"];
+            }
 
-            size << "@Size(" << w << " " << h << ")";
-            [config setObject: [NSString stringWithUTF8String: size.str().c_str()] forKey: @"window.size"];
             [config writeToFile: path atomically: YES];
         }
     }
+}
+
+- (void) performRefresh: (NSNotification *) notice
+{
+    [self saveWindows];
 
     [self sendEvent: [NSEvent otherEventWithType: NSApplicationDefined
                                         location: NSZeroPoint
@@ -494,8 +528,54 @@ static BOOL isTextbufferEvent(NSEvent *evt)
                                            data2: 0]];
 }
 
+- (void) performMove: (NSNotification *) notice
+{
+    [self saveWindows];
+}
+
+static NSURL *get_save_dir(NSString *filename)
+{
+
+    NSError *error = nil;
+    NSURL *dir = [[NSFileManager defaultManager] URLForDirectory: NSApplicationSupportDirectory
+                                                        inDomain: NSUserDomainMask
+                                               appropriateForURL: nil
+                                                          create: YES
+                                                           error: &error];
+
+    if (error != nil) {
+        return nil;
+    }
+
+    dir = [dir URLByAppendingPathComponent: @"Gargoyle/gamedata"];
+    dir = [dir URLByAppendingPathComponent: filename];
+    NSFileManager *fm = [NSFileManager defaultManager];
+    [fm createDirectoryAtURL: dir withIntermediateDirectories: YES attributes: nil error: &error];
+
+    if (error == nil) {
+        return dir;
+    } else {
+        return nil;
+    }
+}
+
+static void maybe_set_save_dir(NSSavePanel *panel, NSString *savedir)
+{
+    if (savedir == nil) {
+        return;
+    }
+
+    auto initial_directory = get_save_dir(savedir);
+
+    if (initial_directory != nil) {
+        NSURL *startingDirectoryURL = [NSURL fileURLWithPath: [initial_directory path]];
+        [panel setDirectoryURL: startingDirectoryURL];
+    }
+}
+
 - (NSString *) openFileDialog: (NSString *) prompt
                    fileFilter: (FileFilter) filter
+                      savedir: (NSString *) savedir
 {
     int result;
 
@@ -506,6 +586,8 @@ static BOOL isTextbufferEvent(NSEvent *evt)
     [openDlg setAllowsMultipleSelection: NO];
     [openDlg setTitle: prompt];
 
+    maybe_set_save_dir(openDlg, savedir);
+
     if (filter != FileFilter::Data) {
         NSArray *filterTypes = [NSArray arrayWithObject: [NSString stringWithCString: winfilters.at(filter).c_str()
                                                                             encoding: NSUTF8StringEncoding]];
@@ -523,6 +605,7 @@ static BOOL isTextbufferEvent(NSEvent *evt)
 
 - (NSString *) saveFileDialog: (NSString *) prompt
                    fileFilter: (FileFilter) filter
+                      savedir: (NSString *) savedir
 {
     int result;
 
@@ -531,6 +614,8 @@ static BOOL isTextbufferEvent(NSEvent *evt)
     [saveDlg setCanCreateDirectories: YES];
     [saveDlg setTitle: prompt];
 
+    maybe_set_save_dir(saveDlg, savedir);
+
     if (filter != FileFilter::Data) {
         NSArray *filterTypes = [NSArray arrayWithObject: [NSString stringWithCString: winfilters.at(filter).c_str()
                                                                             encoding: NSUTF8StringEncoding]];
@@ -614,6 +699,9 @@ static BOOL isTextbufferEvent(NSEvent *evt)
 }
 
 - (BOOL) initWindow: (pid_t) processID
+               move: (BOOL) move
+                  x: (unsigned int) x
+                  y: (unsigned int) y
               width: (unsigned int) width
              height: (unsigned int) height
          fullscreen: (BOOL) fullscreen
@@ -634,8 +722,15 @@ static BOOL isTextbufferEvent(NSEvent *evt)
                                                                  process: processID
                                                          backgroundColor: backgroundColor];
 
+    [window setMinSize:NSMakeSize(gli_wmarginx * 2, gli_wmarginy * 2)];
+
     [window makeKeyAndOrderFront: window];
-    [window center];
+    if (move) {
+        rect = NSMakeRect(x, coord_flip(window) - y, width, height);
+        [window setFrame: rect display: YES];
+    } else {
+        [window center];
+    }
     [window setReleasedWhenClosed: YES];
     [window setDelegate: self];
     if (fullscreen) {
@@ -781,11 +876,12 @@ static BOOL isTextbufferEvent(NSEvent *evt)
 - (NSString *) openWindowDialog: (pid_t) processID
                          prompt: (NSString *) prompt
                          filter: (FileFilter) filter
+                        savedir: (NSString *) savedir
 {
     GargoyleWindow *window = [windows objectForKey: [NSNumber numberWithInt: processID]];
 
     if (window) {
-        return [window openFileDialog: prompt fileFilter: filter];
+        return [window openFileDialog: prompt fileFilter: filter savedir: savedir];
     }
 
     return nil;
@@ -794,11 +890,12 @@ static BOOL isTextbufferEvent(NSEvent *evt)
 - (NSString *) saveWindowDialog: (pid_t) processID
                          prompt: (NSString *) prompt
                          filter: (FileFilter) filter
+                        savedir: (NSString *) savedir
 {
     GargoyleWindow *window = [windows objectForKey: [NSNumber numberWithInt: processID]];
 
     if (window) {
-        return [window saveFileDialog: prompt fileFilter: filter];
+        return [window saveFileDialog: prompt fileFilter: filter savedir: savedir];
     }
 
     return nil;
@@ -810,6 +907,18 @@ static BOOL isTextbufferEvent(NSEvent *evt)
     NSRunAlertPanel(@"Fatal error", @"%@", nil, nil, nil, prompt);
 }
 
+- (BOOL) isFullScreen: (pid_t) processID
+{
+    GargoyleWindow *window = [windows objectForKey: [NSNumber numberWithInt: processID]];
+    return (window.styleMask & NSWindowStyleMaskFullScreen) == NSWindowStyleMaskFullScreen;
+}
+
+- (void) toggleFullScreen: (pid_t) processID
+{
+    GargoyleWindow *window = [windows objectForKey: [NSNumber numberWithInt: processID]];
+    [window toggleFullScreen: self];
+}
+
 #define kArrowCursor        1
 #define kIBeamCursor        2
 #define kPointingHandCursor 3
@@ -968,16 +1077,15 @@ static BOOL isTextbufferEvent(NSEvent *evt)
 
 - (IBAction) toggle: (id) sender
 {
-    NSFileManager *fm = [NSFileManager defaultManager];
-    NSString *home = [NSString stringWithFormat: @"%@/%@", [[[NSProcessInfo processInfo] environment] objectForKey: @"HOME"], @"garglk.ini"];
-    NSString *main = [NSString stringWithFormat: @"%@/%@", [[NSBundle mainBundle] resourcePath], @"garglk.ini"];
-
-    if (![fm isWritableFileAtPath: home] && [fm isReadableFileAtPath: main]) {
-        [fm createFileAtPath: home contents: [NSData dataWithContentsOfFile: main] attributes: nullptr];
+    try {
+        auto config = [NSString stringWithUTF8String: garglk::user_config().c_str()];
+        NSTask *task = [[NSTask alloc] init];
+        task.launchPath = @"/usr/bin/open";
+        task.arguments = @[@"-t", config];
+        [task launch];
+    } catch (const std::runtime_error &e) {
+        garglk::winwarning("Warning", e.what());
     }
-
-    [[NSWorkspace sharedWorkspace] openFile: home
-                            withApplication: @"TextEdit"];
 }
 
 @end
@@ -991,12 +1099,9 @@ void garglk::winmsg(const std::string &msg)
 
 static std::string winpath()
 {
-    char tmp[MaxBuffer];
-    std::uint32_t exelen = sizeof tmp;
-    char exepath[MaxBuffer];
+    char exepath[PROC_PIDPATHINFO_MAXSIZE];
 
-    if (_NSGetExecutablePath(tmp, &exelen) != 0 ||
-        realpath(tmp, exepath) == nullptr) {
+    if (proc_pidpath(getpid(), exepath, sizeof exepath) == -1) {
         garglk::winmsg("Unable to locate executable path");
     }
 
@@ -1040,24 +1145,18 @@ static int winexec(const std::string &cmd, const std::vector<std::string> &args)
     return [proc isRunning];
 }
 
-bool garglk::winterp(const std::string &exe, const std::string &flags, const std::string &game)
+bool garglk::winterp(const std::string &exe, const std::vector<std::string> &flags, const std::string &game)
 {
     // get dir of executable
     auto interpreter_dir = winpath();
 
-    std::ostringstream cmd;
-    cmd << interpreter_dir << "/" << exe;
+    auto cmd = Format("{}/{}", interpreter_dir, exe);
 
-    std::vector<std::string> args;
+    auto args = flags;
 
-    if (flags.find('-') != std::string::npos) {
-        args.push_back(flags);
-        args.push_back(game);
-    } else {
-        args.push_back(game);
-    }
+    args.push_back(game);
 
-    if (!winexec(cmd.str(), args)) {
+    if (!winexec(cmd, args)) {
         garglk::winmsg("Could not start 'terp.\nSorry.");
         return false;
     }
@@ -1071,5 +1170,9 @@ int main(int argc, char **argv)
     [NSApplication sharedApplication];
     [NSApp setDelegate:[[GargoyleApp alloc] init]];
     [pool drain];
+
+    // Read the config file here so that the "save_window" settings are available.
+    gli_read_config(argc, argv);
+
     return NSApplicationMain(argc, const_cast<const char **>(argv));
 }
