@@ -45,7 +45,9 @@
 
 #include GARGLKINI_H
 
-static const char *AppName = GARGOYLE_NAME " " GARGOYLE_VERSION;
+static const char *AppName = "Gargoyle " GARGOYLE_VERSION;
+
+namespace {
 
 class Filter {
 public:
@@ -69,6 +71,8 @@ private:
     QStringList m_extensions;
 };
 
+}
+
 void garglk::winmsg(const std::string &msg)
 {
     QMessageBox::critical(nullptr, "Error", msg.c_str());
@@ -112,7 +116,7 @@ static QString winbrowsefile()
     return QFileDialog::getOpenFileName(nullptr, AppName, "", filter_string, nullptr, options);
 }
 
-bool garglk::winterp(const std::string &exe, const std::string &flags, const std::string &game)
+bool garglk::winterp(const std::string &exe, const std::vector<std::string> &flags, const std::string &game)
 {
     // Find the directory that contains the interpreters. By default
     // this is GARGLK_CONFIG_INTERPRETER_DIR but if that is not set, it
@@ -137,12 +141,10 @@ bool garglk::winterp(const std::string &exe, const std::string &flags, const std
     QString argv0 = QDir(interpreter_dir).absoluteFilePath(exe.c_str());
 
     QStringList args;
-
-    if (flags.find('-') != std::string::npos) {
-        args = QStringList({flags.c_str(), game.c_str()});
-    } else {
-        args = QStringList({game.c_str()});
+    for (const auto &flag : flags) {
+        args.push_back(QString::fromStdString(flag));
     }
+    args.push_back(QString::fromStdString(game));
 
     QProcess proc;
     proc.setProcessChannelMode(QProcess::ForwardedChannels);
@@ -290,9 +292,8 @@ int main(int argc, char **argv)
 {
     QApplication app(argc, argv);
 
-    app.setOrganizationName(GARGOYLE_ORGANIZATION);
-    app.setApplicationName(GARGOYLE_NAME);
-    app.setApplicationVersion(GARGOYLE_VERSION);
+    QApplication::setApplicationName("gargoyle");
+    QApplication::setApplicationVersion(GARGOYLE_VERSION);
 
     auto story = parse_args(app);
 
