@@ -26,11 +26,9 @@
 #include <algorithm>
 #include <array>
 #include <cmath>
-#include <cstdio>
-#include <cstdlib>
 #include <cstring>
 #include <functional>
-#include <new>
+#include <stdexcept>
 #include <string>
 #include <utility>
 #include <vector>
@@ -38,6 +36,8 @@
 #include <SDL.h>
 #include <SDL_mixer.h>
 
+#include "format.h"
+
 #include "glk.h"
 #include "garglk.h"
 
@@ -125,6 +125,17 @@ void gli_initialize_sound()
             return;
         }
 
+        std::string soundfonts;
+        for (const auto &soundfont : gli_conf_soundfonts) {
+            if (soundfont.find(';') == std::string::npos) {
+                soundfonts += soundfont + ';';
+            }
+        }
+
+        if (!soundfonts.empty()) {
+            Mix_SetSoundFonts(soundfonts.c_str());
+        }
+
         int channels = Mix_AllocateChannels(SDL_CHANNELS);
         Mix_GroupChannels(0, channels - 1, FREE);
         Mix_ChannelFinished(nullptr);
@@ -183,7 +194,7 @@ schanid_t glk_schannel_create_ext(glui32 rock, glui32 volume)
 static void cleanup_channel(schanid_t chan)
 {
     if (chan->sdl_rwops != nullptr) {
-        SDL_FreeRW(chan->sdl_rwops);
+        SDL_RWclose(chan->sdl_rwops);
         chan->sdl_rwops = nullptr;
     }
 
@@ -486,7 +497,13 @@ static int detect_format(const std::vector<unsigned char> &buf)
         {{0, {"OggS"}}, giblorb_ID_OGG},
 
         // mp3
-        {{0, {"\377\372"}}, giblorb_ID_MP3},
+        {{0, {"ID3"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xe2"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xe3"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xf2"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xf3"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xfa"}}, giblorb_ID_MP3},
+        {{0, {"\xff\xfb"}}, giblorb_ID_MP3},
     };
 
     for (const auto &entry : formats) {
@@ -495,7 +512,7 @@ static int detect_format(const std::vector<unsigned char> &buf)
         auto format = entry.second;
 
         if (std::any_of(magics.begin(), magics.end(), [&offset, &buf](const auto &magic) {
-            return offset + magic.size() < buf.size() &&
+            return offset + magic.size() <= buf.size() &&
                     std::memcmp(buf.data() + offset, magic.data(), magic.size()) == 0;
         }))
         {
@@ -520,54 +537,31 @@ static int load_bleep_resource(glui32 snd, std::vector<unsigned char> &buf)
 
 static glui32 load_sound_resource(glui32 snd, std::vector<unsigned char> &buf)
 {
-    long len;
-
-    if (giblorb_get_resource_map() == nullptr) {
-        std::string name;
-
-        name = gli_workdir + "/SND" + std::to_string(snd);
-
-        auto file = garglk::unique(std::fopen(name.c_str(), "rb"), fclose);
-        if (!file) {
-            return 0;
-        }
-
-        std::fseek(file.get(), 0, SEEK_END);
-
-        try {
-            buf.resize(std::ftell(file.get()));
-        } catch (const std::bad_alloc &) {
-            return 0;
-        }
-
-        std::rewind(file.get());
-        if (std::fread(buf.data(), 1, buf.size(), file.get()) != buf.size() && !std::feof(file.get())) {
-            return 0;
-        }
-
-        return detect_format(buf);
-    } else {
-        std::FILE *file;
+    if (giblorb_get_resource_map() != nullptr) {
         glui32 type;
-        long pos;
 
-        giblorb_get_resource(giblorb_ID_Snd, snd, &file, &pos, &len, &type);
-        if (file == nullptr) {
-            return 0;
-        }
-
-        try {
-            buf.resize(len);
-        } catch (const std::bad_alloc &) {
-            return 0;
-        }
-
-        std::fseek(file, pos, SEEK_SET);
-        if (std::fread(buf.data(), 1, buf.size(), file) != buf.size() && !std::feof(file)) {
+        if (!giblorb_copy_resource(giblorb_ID_Snd, snd, type, buf)) {
             return 0;
         }
 
         return type;
+    } else {
+        const auto &resource_map = gli_get_resource_map(giblorb_ID_Snd);
+        if (!resource_map.empty()) {
+            try {
+                buf = resource_map.at(snd);
+            } catch (const std::out_of_range &) {
+                return 0;
+            }
+        } else {
+            auto filename = Format("{}/SND{}", gli_workdir, snd);
+
+            if (!garglk::read_file(filename, buf)) {
+                return 0;
+            }
+        }
+
+        return detect_format(buf);
     }
 }
 
@@ -578,12 +572,12 @@ static glui32 play_sound(schanid_t chan)
     SDL_LockAudio();
     chan->status = CHANNEL_SOUND;
     chan->sdl_channel = Mix_GroupAvailable(FREE);
-    Mix_GroupChannel(chan->sdl_channel, BUSY);
-    SDL_UnlockAudio();
-    chan->sample = Mix_LoadWAV_RW(chan->sdl_rwops, false);
     if (chan->sdl_channel < 0) {
         gli_strict_warning("No available sound channels");
     }
+    Mix_GroupChannel(chan->sdl_channel, BUSY);
+    SDL_UnlockAudio();
+    chan->sample = Mix_LoadWAV_RW(chan->sdl_rwops, false);
     if (chan->sdl_channel >= 0 && chan->sample != nullptr) {
         SDL_LockAudio();
         sound_channels[chan->sdl_channel] = chan;
@@ -607,10 +601,8 @@ static glui32 play_sound(schanid_t chan)
 }
 
 // Start a mod music channel
-static glui32 play_mod(schanid_t chan, long len)
+static glui32 play_mod(schanid_t chan)
 {
-    std::FILE *file;
-    const char *tempdir;
     bool music_busy;
     int loop;
 
@@ -630,32 +622,7 @@ static glui32 play_mod(schanid_t chan, long len)
     }
 
     chan->status = CHANNEL_MUSIC;
-    // The fscking mikmod lib want to read the mod only from disk!
-    tempdir = std::getenv("TMPDIR");
-    if (tempdir == nullptr) {
-        tempdir = std::getenv("TEMP");
-        if (tempdir == nullptr) {
-            tempdir = std::getenv("TMP");
-            if (tempdir == nullptr) {
-                tempdir = ".";
-            }
-        }
-    }
-
-    // allocate size of string tempdir + "XXXXXX' + terminator
-    std::vector<char> tn(std::strlen(tempdir) + 7);
-    std::sprintf(tn.data(), "%sXXXXXX", tempdir);
-    int fd;
-    fd = mkstemp(tn.data());
-    if (fd == -1) {
-        gli_strict_warning("play mod failed: could not create temp file");
-        return 0;
-    }
-    file = fdopen(fd, "wb");
-    std::fwrite(chan->sdl_memory.data(), 1, len, file);
-    std::fclose(file);
-    chan->music = Mix_LoadMUS(tn.data());
-    std::remove(tn.data());
+    chan->music = Mix_LoadMUS_RW(chan->sdl_rwops, 0);
     if (chan->music != nullptr) {
         SDL_LockAudio();
         music_channel = chan;
@@ -678,7 +645,7 @@ static glui32 play_mod(schanid_t chan, long len)
     return 0;
 }
 
-glui32 glk_schannel_play_ext_impl(schanid_t chan, glui32 snd, glui32 repeats, glui32 notify, std::function<glui32(glui32, std::vector<unsigned char> &)> load_resource)
+static glui32 gli_schannel_play_ext(schanid_t chan, glui32 snd, glui32 repeats, glui32 notify, std::function<glui32(glui32, std::vector<unsigned char> &)> load_resource)
 {
     glui32 type;
     glui32 result = 0;
@@ -700,11 +667,7 @@ glui32 glk_schannel_play_ext_impl(schanid_t chan, glui32 snd, glui32 repeats, gl
     }
 
     // load sound resource into memory
-    try {
-        type = load_resource(snd, chan->sdl_memory);
-    } catch (const Bleeps::Empty &) {
-        return 1;
-    }
+    type = load_resource(snd, chan->sdl_memory);
 
     chan->sdl_rwops = SDL_RWFromConstMem(chan->sdl_memory.data(), chan->sdl_memory.size());
     chan->notify = notify;
@@ -722,7 +685,7 @@ glui32 glk_schannel_play_ext_impl(schanid_t chan, glui32 snd, glui32 repeats, gl
 
     case giblorb_ID_MOD:
     case giblorb_ID_MIDI:
-        result = play_mod(chan, chan->sdl_memory.size());
+        result = play_mod(chan);
         break;
 
     default:
@@ -739,7 +702,7 @@ glui32 glk_schannel_play_ext_impl(schanid_t chan, glui32 snd, glui32 repeats, gl
 
 glui32 glk_schannel_play_ext(schanid_t chan, glui32 snd, glui32 repeats, glui32 notify)
 {
-    return glk_schannel_play_ext_impl(chan, snd, repeats, notify, load_sound_resource);
+    return gli_schannel_play_ext(chan, snd, repeats, notify, load_sound_resource);
 }
 
 void glk_schannel_pause(schanid_t chan)
@@ -786,7 +749,6 @@ void glk_schannel_stop(schanid_t chan)
         return;
     }
     SDL_LockAudio();
-    chan->paused = false;
     glk_schannel_unpause(chan);
     SDL_UnlockAudio();
     switch (chan->status) {
@@ -816,6 +778,9 @@ void garglk_zbleep(glui32 number)
     }
 
     if (gli_bleep_channel != nullptr) {
-        glk_schannel_play_ext_impl(gli_bleep_channel, number, 1, 0, load_bleep_resource);
+        try {
+            gli_schannel_play_ext(gli_bleep_channel, number, 1, 0, load_bleep_resource);
+        } catch (const Bleeps::Empty &) {
+        }
     }
 }
