@@ -44,6 +44,7 @@
 #define NSKEY_H         0x04
 #define NSKEY_N         0x2d
 #define NSKEY_P         0x23
+#define NSKEY_S         0x01
 #define NSKEY_T         0x11
 #define NSKEY_U         0x20
 #define NSKEY_V         0x09
@@ -75,6 +76,9 @@
 @protocol GargoyleApp
 
 - (BOOL) initWindow: (pid_t) processID
+               move: (BOOL) move
+                  x: (unsigned int) x
+                  y: (unsigned int) y
               width: (unsigned int) width
              height: (unsigned int) height
          fullscreen: (BOOL) fullscreen
@@ -108,17 +112,22 @@
 
 - (NSString *) openWindowDialog: (pid_t) processID
                          prompt: (NSString *) prompt
-                         filter: (FileFilter) filter;
+                         filter: (FileFilter) filter
+                        savedir: (NSString *) savedir;
 
 - (NSString *) saveWindowDialog: (pid_t) processID
                          prompt: (NSString *) prompt
-                         filter: (FileFilter) filter;
+                         filter: (FileFilter) filter
+                        savedir: (NSString *) savedir;
 
 - (void) abortWindowDialog: (pid_t) processID
                     prompt: (NSString *) prompt;
 
 - (void) setCursor: (unsigned int) cursor;
 
+- (BOOL) isFullScreen: (pid_t) processID;
+- (void) toggleFullScreen: (pid_t) processID;
+
 @end
 
 NSString *get_qt_plist_path();
