@@ -18,6 +18,7 @@
 
 #include <algorithm>
 #include <cstdlib>
+#include <fstream>
 #include <functional>
 #include <map>
 #include <regex>
@@ -34,6 +35,9 @@
 #include "garglk.h"
 #include "garversion.h"
 
+#include "format.h"
+#include "optional.hpp"
+
 #import "Cocoa/Cocoa.h"
 #import "sysmac.h"
 
@@ -163,19 +167,38 @@ void garglk::winabort(const std::string &msg)
     std::exit(1);
 }
 
+void garglk::winwarning(const std::string &title, const std::string &msg)
+{
+    auto *nsTitle = [NSString stringWithCString: title.c_str() encoding: NSUTF8StringEncoding];
+    auto *nsMsg = [NSString stringWithCString: msg.c_str() encoding: NSUTF8StringEncoding];
+    NSRunAlertPanel(nsTitle, @"%@", nil, nil, nil, nsMsg);
+}
+
 void winexit()
 {
     [gargoyle closeWindow:processID];
     gli_exit(0);
 }
 
+static NSString *get_savedir(FileFilter filter)
+{
+    if (gli_conf_gamedata_location == GamedataLocation::Dedicated && gli_workfile.has_value()) {
+        return [NSString stringWithUTF8String: gli_workfile->c_str()];
+    } else if (gli_conf_gamedata_location == GamedataLocation::Gamedir) {
+        return [NSString stringWithUTF8String: gli_workdir.c_str()];
+    }
+
+    return nil;
+}
+
 std::string garglk::winopenfile(const char *prompt, FileFilter filter)
 {
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
     NSString *fileref = [gargoyle openWindowDialog: processID
                                             prompt: [NSString stringWithCString: prompt encoding: NSUTF8StringEncoding]
-                                            filter: filter];
+                                            filter: filter
+                                            savedir: get_savedir(filter)];
 
     std::string buf;
 
@@ -194,7 +217,8 @@ std::string garglk::winsavefile(const char *prompt, FileFilter filter)
 
     NSString *fileref = [gargoyle saveWindowDialog: processID
                                             prompt: [NSString stringWithCString: prompt encoding: NSUTF8StringEncoding]
-                                            filter: filter];
+                                            filter: filter
+                                           savedir: get_savedir(filter)];
 
     std::string buf;
 
@@ -207,9 +231,9 @@ std::string garglk::winsavefile(const char *prompt, FileFilter filter)
     return buf;
 }
 
-void winclipstore(const glui32 *text, int len)
+void winclipstore(const std::vector<glui32> &text)
 {
-    if (!len) {
+    if (text.empty()) {
         return;
     }
 
@@ -218,8 +242,8 @@ void winclipstore(const glui32 *text, int len)
         cliptext = nil;
     }
 
-    cliptext = [[NSString alloc] initWithBytes: text
-                                        length: (len * sizeof(glui32))
+    cliptext = [[NSString alloc] initWithBytes: text.data()
+                                        length: (text.size() * sizeof(glui32))
                                       encoding: UTF32StringEncoding];
 }
 
@@ -253,14 +277,17 @@ void winclipreceive()
                     case '\0':
                         return;
 
-                    case '\r':
-                    case '\n':
                     case '\b':
                     case '\t':
                         break;
 
+                    case '\r':
+                    case '\n':
+                        gli_input_handle_key_paste(keycode_Return);
+                        break;
+
                     default:
-                        gli_input_handle_key(ch);
+                        gli_input_handle_key_paste(ch);
                     }
                 }
             }
@@ -303,7 +330,7 @@ void winresize()
         return;
     }
 
-    gli_windows_size_change(vw, vh);
+    gli_windows_size_change(vw, vh, true);
 
     // redraw window content
     gli_refresh_needed = true;
@@ -352,7 +379,7 @@ static void winhandler(int signal)
     }
 }
 
-void wininit(int *argc, char **argv)
+void wininit()
 {
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     gli_backingscalefactor = BACKING_SCALE_FACTOR;
@@ -393,7 +420,7 @@ NSString *get_qt_plist_path()
     if (home != nullptr) {
         // Optimistically use the path that Qt uses with the hope/plan
         // of moving macOS to Qt one of these days.
-        auto path = std::string(home) + "/Library/Preferences/com.io-github-garglk.Gargoyle.plist";
+        auto path = Format("{}/Library/Preferences/com.io-github-garglk.Gargoyle.plist", home);
         return [NSString stringWithUTF8String: path.c_str()];
     }
 
@@ -404,42 +431,76 @@ void winopen()
 {
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
-    unsigned int defw = gli_wmarginx * 2 + gli_cellw * gli_cols / BACKING_SCALE_FACTOR;
-    unsigned int defh = gli_wmarginy * 2 + gli_cellh * gli_rows / BACKING_SCALE_FACTOR;
+    auto do_fullscreen = gli_conf_fullscreen;
+
+    bool move = false;
+    int x = 0;
+    int y = 0;
+    unsigned int width = gli_wmarginx * 2 + gli_cellw * gli_cols / BACKING_SCALE_FACTOR;
+    unsigned int height = gli_wmarginy * 2 + gli_cellh * gli_rows / BACKING_SCALE_FACTOR;
     NSColor *windowColor = [NSColor colorWithCalibratedRed: (float) gli_window_color[0] / 255.0f
                                                      green: (float) gli_window_color[1] / 255.0f
                                                       blue: (float) gli_window_color[2] / 255.0f
                                                      alpha: 1.0f];
 
-    if (gli_conf_save_window_size) {
-        auto path = get_qt_plist_path();
-        if (path != nil) {
-            NSDictionary *config = [NSDictionary dictionaryWithContentsOfFile: path];
-            NSString *sizeobj = config[@"window.size"];
+    auto path = get_qt_plist_path();
+    if (path != nil) {
+        NSDictionary *config = [NSDictionary dictionaryWithContentsOfFile: path];
 
-            if (sizeobj) {
-                std::string size = [sizeobj UTF8String];
-                std::regex size_re(R"(^@Size\((\d+) (\d+)\)$)");
+        if (gli_conf_save_window_location) {
+            NSString *positionobj = config[@"window.position"];
+            if (positionobj) {
+                std::string position = [positionobj UTF8String];
+                std::regex position_re(R"(^@Point\((\d+) (\d+)\)$)");
                 std::smatch cm;
-                if (std::regex_match(size, cm, size_re) && cm.size() == 3) {
-                    int w, h;
+                if (std::regex_match(position, cm, position_re) && cm.size() == 3) {
                     try {
-                        w = std::stoi(cm[1]);
-                        h = std::stoi(cm[2]);
+                        auto tmpx = std::stoi(cm[1]);
+                        auto tmpy = std::stoi(cm[2]);
 
-                        defw = w;
-                        defh = h;
+                        x = tmpx;
+                        y = tmpy;
+                        move = true;
                     } catch (...) {
                     }
                 }
             }
         }
+
+        if (gli_conf_save_window_size) {
+            NSString *sizeobj = config[@"window.size"];
+            if (sizeobj) {
+                std::string size = [sizeobj UTF8String];
+                std::regex size_re(R"(^@Size\((\d+) (\d+)\)$)");
+                std::smatch cm;
+                if (std::regex_match(size, cm, size_re) && cm.size() == 3) {
+                    try {
+                        auto tmpwidth = std::stoi(cm[1]);
+                        auto tmpheight = std::stoi(cm[2]);
+
+                        width = tmpwidth;
+                        height = tmpheight;
+                    } catch (...) {
+                    }
+                }
+            }
+        }
+
+        if (gli_conf_save_window_location || gli_conf_save_window_size) {
+            NSNumber *fsobj = config[@"window.fullscreen"];
+            if (fsobj) {
+                do_fullscreen = [fsobj boolValue];
+            }
+        }
     }
 
     [gargoyle initWindow: processID
-                   width: defw
-                  height: defh
-              fullscreen: gli_conf_fullscreen
+                    move: move
+                       x: x
+                       y: y
+                   width: width
+                  height: height
+              fullscreen: do_fullscreen
          backgroundColor: windowColor];
 
     wintitle();
@@ -476,10 +537,10 @@ void winrefresh()
     gli_refresh_needed = !refreshed;
 }
 
-static void show_info(NSString *title, const std::string &text)
+static void show_alert(NSAlertStyle style, NSString *title, const std::string &text)
 {
     NSAlert *alert = [[NSAlert alloc] init];
-    alert.alertStyle = NSAlertStyleInformational;
+    alert.alertStyle = style;
     alert.messageText = title;
     alert.informativeText = [NSString stringWithUTF8String: text.c_str()];
     alert.window.level = NSModalPanelWindowLevel;
@@ -487,18 +548,33 @@ static void show_info(NSString *title, const std::string &text)
     [alert release];
 }
 
+static void show_info(NSString *title, const std::string &text)
+{
+    show_alert(NSAlertStyleInformational, title, text);
+}
+
+static void show_warning(NSString *title, const std::string &text)
+{
+    show_alert(NSAlertStyleWarning, title, text);
+}
+
+static void show_error(NSString *title, const std::string &text)
+{
+    show_alert(NSAlertStyleCritical, title, text);
+}
+
 static void show_paths()
 {
     std::string text = "Configuration file paths:\n\n";
     for (const auto &config : garglk::all_configs) {
-        text += "• " + config.path + " " + config.format_type() + "\n";
+        text += Format("• {} {}\n", config.path, config.format_type());
     }
 
     text += "\nTheme paths:\n\n";
     auto theme_paths = garglk::theme::paths();
     std::reverse(theme_paths.begin(), theme_paths.end());
     for (const auto &path : theme_paths) {
-        text += "• " + path + "\n";
+        text += Format("• {}\n", path);
     }
 
     show_info(@"Paths", text);
@@ -511,7 +587,7 @@ static void show_themes()
 
     std::sort(theme_names.begin(), theme_names.end());
     for (const auto &theme_name : theme_names) {
-        text += "• " + theme_name + "\n";
+        text += Format("• {}\n", theme_name);
     }
 
     show_info(@"Themes", text);
@@ -585,6 +661,41 @@ void winkey(NSEvent *evt)
         {{0, NSKEY_F10},  []{ gli_input_handle_key(keycode_Func10); }},
         {{0, NSKEY_F11},  []{ gli_input_handle_key(keycode_Func11); }},
         {{0, NSKEY_F12},  []{ gli_input_handle_key(keycode_Func12); }},
+
+        // save transcript
+        {{NSEventModifierFlagShift | NSEventModifierFlagCommand, NSKEY_S}, []{
+            auto text = gli_get_scrollback();
+            if (text.has_value()) {
+                NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+
+                NSString *fileref = [gargoyle saveWindowDialog: processID
+                                                        prompt: @"Save transcript"
+                                                        filter: FileFilter::Text
+                                                       savedir: nil];
+
+                if (fileref != nullptr) {
+                    std::string filename = [fileref UTF8String];
+
+                    std::ofstream file(filename, std::ios::binary);
+                    if (file.is_open()) {
+                        if (!file.write(text->data(), text->size())) {
+                            show_error(@"Error", "Error writing entire transcript.");
+                        }
+                    } else {
+                        show_error(@"Error", "Unable to open file for writing.");
+                    }
+                }
+
+                [pool drain];
+            } else {
+                show_warning(@"Warning", "Could not find appropriate window for scrollback.");
+            }
+        }},
+
+        // toggle fullscreen
+        {{NSEventModifierFlagControl | NSEventModifierFlagCommand, NSKEY_F}, []{
+            [gargoyle toggleFullScreen: processID];
+        }},
     };
 
     try {
@@ -750,40 +861,60 @@ void winpoll()
     } while (evt);
 }
 
-std::string garglk::winfontpath(const std::string &filename)
+nonstd::optional<std::string> garglk::winfontpath(const std::string &filename)
 {
     char *resources = std::getenv("GARGLK_RESOURCES");
 
     if (resources != nullptr) {
-        return std::string(resources) + "/Fonts/" + filename;
+        return Format("{}/Fonts/{}", resources, filename);
     }
 
-    return "";
+    return nonstd::nullopt;
 }
 
-std::vector<std::string> garglk::winappdata()
+std::string garglk::windatadir()
+{
+    char *resources = std::getenv("GARGLK_RESOURCES");
+
+    if (resources != nullptr) {
+        return resources;
+    }
+
+    return ".";
+}
+
+std::vector<std::string> garglk::winthemedirs()
 {
     NSArray *appdir_paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
     char *resources = std::getenv("GARGLK_RESOURCES");
     std::vector<std::string> paths;
 
     if (resources != nullptr) {
-        paths.emplace_back(resources);
+        paths.push_back(Format("{}/themes", resources));
     }
 
     // This is what Qt returns for AppDataLocation (though Qt adds a
     // few more directories that aren't particularly relevant).
     for (NSString *appdir_path in appdir_paths) {
-        paths.push_back(std::string([appdir_path UTF8String]) + "/" + GARGOYLE_ORGANIZATION + "/" + GARGOYLE_NAME);
+        paths.push_back(Format("{}/Gargoyle/themes", [appdir_path UTF8String]));
     }
 
     return paths;
 }
 
-std::string garglk::winappdir()
+nonstd::optional<std::string> garglk::winlegacythemedir() {
+    return [[NSHomeDirectory() stringByAppendingPathComponent: @"Library/Application Support/io.github.garglk/Gargoyle/themes"] UTF8String];
+}
+
+nonstd::optional<std::string> garglk::winappdir()
 {
     // This is only used on Windows.
-    return "";
+    return nonstd::nullopt;
+}
+
+bool garglk::winisfullscreen()
+{
+    return [gargoyle isFullScreen: processID];
 }
 
 void gli_select(event_t *event, bool polled)
