@@ -24,8 +24,8 @@
 #include <QCursor>
 #include <QDesktopServices>
 #include <QDir>
-#include <QElapsedTimer>
 #include <QFileDialog>
+#include <QFileInfo>
 #include <QGraphicsView>
 #include <QLabel>
 #include <QList>
@@ -58,24 +58,37 @@
 #define GARGLK_KDE_HAS_APPLICATION_LAUNCHER
 #include <KApplicationTrader>
 #include <KIO/ApplicationLauncherJob>
+#if KIO_VERSION >= ((5 << 16) | (98 << 8))
+#include <KIO/JobUiDelegateFactory>
+#define create_delegate(window) (KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window))
+#else
 #include <KIO/JobUiDelegate>
+#define create_delegate(window) (new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window))
+#endif
 #else
 #include <KRun>
 #endif
 #endif
 
 #include <algorithm>
+#include <atomic>
+#include <chrono>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
 #include <iostream>
+#include <limits>
 #include <map>
 #include <stdexcept>
 #include <string>
+#include <thread>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
+#include "format.h"
+#include "optional.hpp"
+
 #include "sysqt.h"
 #include "moc_sysqt.cpp"
 
@@ -98,19 +111,23 @@ static const std::unordered_map<FileFilter, std::pair<QString, QString>> filters
 };
 
 static QApplication *app;
-static Window *window;
-static QElapsedTimer last_tick;
-static constexpr long long TICK_PERIOD_MILLIS = 10;
+static garglk::Window *window;
 
 static bool refresh_needed = true;
 
-static void handle_input(const QString &input)
+static constexpr int TICK_PERIOD_MILLIS = 10;
+static std::atomic<bool> process_events(false);
+
+static void handle_input(const QString &input, bool from_paste)
 {
+    auto fn = from_paste ? gli_input_handle_key_paste :
+                           gli_input_handle_key;
+
     for (const uint &c : input.toUcs4()) {
         if (c == '\r' || c == '\n') {
-            gli_input_handle_key(keycode_Return);
+            fn(keycode_Return);
         } else if (QChar::isPrint(c)) {
-            gli_input_handle_key(c);
+            fn(c);
         }
     }
 }
@@ -132,6 +149,12 @@ void garglk::winabort(const std::string &msg)
     gli_exit(EXIT_FAILURE);
 }
 
+void garglk::winwarning(const std::string &title, const std::string &msg)
+{
+    std::cerr << "warning: " << msg << std::endl;
+    QMessageBox::warning(nullptr, title.c_str(), msg.c_str());
+}
+
 void winexit()
 {
     gli_exit(0);
@@ -147,11 +170,29 @@ static std::string winchoosefile(const QString &prompt, FileFilter filter, Actio
     options |= QFileDialog::DontUseNativeDialog;
 #endif
 
+    QString dir = "";
+
+    if (gli_conf_gamedata_location == GamedataLocation::Dedicated && gli_workfile.has_value()) {
+        auto path = QFileInfo(QString::fromStdString(*gli_workfile));
+        if (!path.fileName().isEmpty()) {
+            QDir basedir = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+            QDir savepath = QDir(basedir.filePath("gamedata")).filePath(path.fileName());
+            if (savepath.mkpath(savepath.absolutePath())) {
+                dir = savepath.absolutePath();
+            }
+        }
+    } else if (gli_conf_gamedata_location == GamedataLocation::Gamedir) {
+        dir = QString::fromStdString(gli_workdir);
+    }
+
     if (action == Action::Open) {
         QString filterstring = QString("%1;;All files (*)").arg(filters.at(filter).first);
-        filename = QFileDialog::getOpenFileName(window, prompt, "", filterstring, nullptr, options);
+        filename = QFileDialog::getOpenFileName(window, prompt, dir, filterstring, nullptr, options);
     } else {
-        QString dir = QString("./Untitled.%1").arg(filters.at(filter).second);
+        if (dir == "") {
+            dir += ".";
+        }
+        dir += QString("/Untitled.%1").arg(filters.at(filter).second);
         filename = QFileDialog::getSaveFileName(window, prompt, dir, filters.at(filter).first, nullptr, options);
     }
 
@@ -180,9 +221,9 @@ std::string garglk::winsavefile(const char *prompt, FileFilter filter)
     return winchoosefile(realprompt, filter, Action::Save);
 }
 
-void winclipstore(const glui32 *text, int len)
+void winclipstore(const std::vector<glui32> &text)
 {
-    cliptext = QString::fromUcs4(reinterpret_cast<const char32_t *>(text), len);
+    cliptext = QString::fromUcs4(reinterpret_cast<const char32_t *>(text.data()), text.size());
 }
 
 static void winclipsend(QClipboard::Mode mode)
@@ -201,26 +242,43 @@ static void winclipreceive(QClipboard::Mode mode)
     QClipboard *clipboard = QGuiApplication::clipboard();
     QString text = clipboard->text(mode);
 
-    handle_input(text);
+    handle_input(text, true);
 }
 
-Window::Window() :
+garglk::Window::Window() :
     m_view(new View(this)),
     m_timer(new QTimer(this)),
-    m_settings(new QSettings(GARGOYLE_ORGANIZATION, GARGOYLE_NAME, this))
+    // Qt programs have an organization and name that can be set, and
+    // Gargoyle used to set these to "io.github.garglk" and "Gargoyle".
+    // The QSettings here followed that. However, Gargoyle now uses an
+    // empty organization and the name "gargoyle" (on Unix) so that
+    // directories are more conventionally-named, e.g. /usr/share/gargoyle
+    // instead of /usr/share/io.github.garglk/Gargoyle. But QSettings
+    // _requires_ an organization name. Given that this is a setting
+    // users aren't ever supposed to see anyhow, and that these exact
+    // names were used in the past, keep them the same so that older
+    // configurations can be loaded. Ideally this would probably just be
+    // "gargoyle" and "gargoyle" but aesthetics are nowhere near as
+    // important as not losing settings; and since nobody is going to
+    // see these names in the normal course of using Gargoyle, it
+    // doesn't really matter anyway.
+    m_settings(new QSettings("io.github.garglk", "Gargoyle", this))
 {
+    m_timer->setTimerType(Qt::TimerType::PreciseTimer);
     connect(m_timer, &QTimer::timeout, this, [&]() {
         m_timed_out = true;
     });
 }
 
-void Window::closeEvent(QCloseEvent *)
+void garglk::Window::closeEvent(QCloseEvent *)
 {
     gli_exit(0);
 }
 
-void Window::resizeEvent(QResizeEvent *event)
+void garglk::Window::resizeEvent(QResizeEvent *event)
 {
+    static bool first_resize = true;
+
     QMainWindow::resizeEvent(event);
 
     m_view->resize(event->size());
@@ -234,16 +292,25 @@ void Window::resizeEvent(QResizeEvent *event)
 
     refresh_needed = true;
 
-    gli_windows_size_change(newwid, newhgt);
+    // On startup, Qt posts a resize event as the window is created.
+    // This resize occurs before the Glk program even starts, so
+    // shouldn't create an arrange event.
+    gli_windows_size_change(newwid, newhgt, !first_resize);
 
     if (gli_conf_save_window_size) {
         m_settings->setValue("window/size", event->size());
     }
 
+    if (gli_conf_save_window_location || gli_conf_save_window_size) {
+        m_settings->setValue("window/fullscreen", ::window->isFullScreen());
+    }
+
     event->accept();
+
+    first_resize = false;
 }
 
-void Window::moveEvent(QMoveEvent *event)
+void garglk::Window::moveEvent(QMoveEvent *event)
 {
     if (gli_conf_save_window_location) {
         m_settings->setValue("window/position", event->pos());
@@ -252,7 +319,7 @@ void Window::moveEvent(QMoveEvent *event)
     event->accept();
 }
 
-QVariant View::inputMethodQuery(Qt::InputMethodQuery query) const
+QVariant garglk::View::inputMethodQuery(Qt::InputMethodQuery query) const
 {
     switch (query) {
     case Qt::ImEnabled:
@@ -264,7 +331,7 @@ QVariant View::inputMethodQuery(Qt::InputMethodQuery query) const
 
 // Handle compose key events (probably other input method events too).
 // See https://stackoverflow.com/questions/28793356/qt-and-dead-keys-in-a-custom-widget
-void View::inputMethodEvent(QInputMethodEvent *event)
+void garglk::View::inputMethodEvent(QInputMethodEvent *event)
 {
     if (!event->commitString().isEmpty()) {
         QKeyEvent key_event(QEvent::KeyPress, 0, Qt::NoModifier, event->commitString());
@@ -273,7 +340,7 @@ void View::inputMethodEvent(QInputMethodEvent *event)
     event->accept();
 }
 
-void View::refresh()
+void garglk::View::refresh()
 {
     if (!gli_drawselect) {
         gli_windows_redraw();
@@ -285,7 +352,7 @@ void View::refresh()
     refresh_needed = false;
 }
 
-void View::paintEvent(QPaintEvent *event)
+void garglk::View::paintEvent(QPaintEvent *event)
 {
     QImage image(gli_image_rgb.data(), gli_image_rgb.width(), gli_image_rgb.height(), gli_image_rgb.stride(), QImage::Format_RGB888);
     QPainter painter(this);
@@ -293,12 +360,17 @@ void View::paintEvent(QPaintEvent *event)
     event->accept();
 }
 
-void Window::start_timer(long ms)
+void garglk::Window::start_timer(unsigned long ms)
 {
     if (m_timer->isActive()) {
         m_timer->stop();
     }
 
+    // QTimer::setInterval() takes int, so limit to avoid wrapping.
+    if (ms > std::numeric_limits<int>::max()) {
+        ms = std::numeric_limits<int>::max();
+    }
+
     if (ms != 0) {
         m_timer->setInterval(ms);
         m_timer->start();
@@ -331,8 +403,8 @@ void gli_edit_config()
         QUrl url(QUrl::fromLocalFile(config.c_str()));
 #ifdef GARGLK_KDE_HAS_APPLICATION_LAUNCHER
         auto service = KApplicationTrader::preferredService("text/plain");
-        auto job = new KIO::ApplicationLauncherJob(service);
-        job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+        auto *job = new KIO::ApplicationLauncherJob(service);
+        job->setUiDelegate(create_delegate(window));
         job->setUrls({url});
         job->start();
 #else
@@ -363,7 +435,7 @@ static void show_paths()
         auto path = canonicalize(config.path);
         auto type = QString::fromStdString(config.format_type());
 
-        text += path + " " + type + "\n";
+        text += QString("%1 %2\n").arg(path).arg(type);
     }
 
     text += "</pre><p>Theme paths:</p><pre>";
@@ -392,27 +464,50 @@ static void show_themes()
     box.exec();
 }
 
-void View::keyPressEvent(QKeyEvent *event)
+// On Mac, Qt::ControlModifier means the command key. But for Emacs
+// keys, we need the actual control key.
+#ifdef Q_OS_MAC
+static constexpr Qt::KeyboardModifier RealCtrl = Qt::MetaModifier;
+#else
+static constexpr Qt::KeyboardModifier RealCtrl = Qt::ControlModifier;
+#endif
+
+void garglk::View::keyPressEvent(QKeyEvent *event)
 {
     Qt::KeyboardModifiers modmasked = event->modifiers() & (Qt::ShiftModifier | Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier);
 
     refresh_needed = true;
 
+    static const std::map<QKeySequence::StandardKey, std::function<void()>> sequences = {
+        {QKeySequence::Cut,                []{ winclipsend(QClipboard::Clipboard); }},
+        {QKeySequence::Copy,               []{ winclipsend(QClipboard::Clipboard); }},
+        {QKeySequence::Paste,              []{ winclipreceive(QClipboard::Clipboard); }},
+        {QKeySequence::MoveToPreviousWord, []{ gli_input_handle_key(keycode_SkipWordLeft); }},
+        {QKeySequence::MoveToNextWord,     []{ gli_input_handle_key(keycode_SkipWordRight); }},
+        {QKeySequence::Quit,               []{ gli_exit(0); }},
+        {QKeySequence::Delete,             []{ gli_input_handle_key(keycode_Erase); }},
+        {QKeySequence::MoveToStartOfLine,  []{ gli_input_handle_key(keycode_Home); }},
+        {QKeySequence::MoveToEndOfLine,    []{ gli_input_handle_key(keycode_End); }},
+        {QKeySequence::MoveToPreviousPage, []{ gli_input_handle_key(keycode_PageUp); }},
+        {QKeySequence::MoveToNextPage,     []{ gli_input_handle_key(keycode_PageDown); }},
+    };
+
     static const std::map<std::pair<decltype(modmasked), decltype(event->key())>, std::function<void()>> keys = {
-        {{Qt::ControlModifier, Qt::Key_A},     []{ gli_input_handle_key(keycode_Home); }},
-        {{Qt::ControlModifier, Qt::Key_B},     []{ gli_input_handle_key(keycode_Left); }},
-        {{Qt::ControlModifier, Qt::Key_C},     []{ winclipsend(QClipboard::Clipboard); }},
-        {{Qt::ControlModifier, Qt::Key_D},     []{ gli_input_handle_key(keycode_Erase); }},
-        {{Qt::ControlModifier, Qt::Key_E},     []{ gli_input_handle_key(keycode_End); }},
-        {{Qt::ControlModifier, Qt::Key_F},     []{ gli_input_handle_key(keycode_Right); }},
-        {{Qt::ControlModifier, Qt::Key_H},     []{ gli_input_handle_key(keycode_Delete); }},
-        {{Qt::ControlModifier, Qt::Key_N},     []{ gli_input_handle_key(keycode_Down); }},
-        {{Qt::ControlModifier, Qt::Key_P},     []{ gli_input_handle_key(keycode_Up); }},
-        {{Qt::ControlModifier, Qt::Key_U},     []{ gli_input_handle_key(keycode_Escape); }},
-        {{Qt::ControlModifier, Qt::Key_V},     []{ winclipreceive(QClipboard::Clipboard); }},
-        {{Qt::ControlModifier, Qt::Key_X},     []{ winclipsend(QClipboard::Clipboard); }},
-        {{Qt::ControlModifier, Qt::Key_Left},  []{ gli_input_handle_key(keycode_SkipWordLeft); }},
-        {{Qt::ControlModifier, Qt::Key_Right}, []{ gli_input_handle_key(keycode_SkipWordRight); }},
+        // Emacs keys.
+        {{RealCtrl, Qt::Key_A}, []{ gli_input_handle_key(keycode_Home); }},
+        {{RealCtrl, Qt::Key_B}, []{ gli_input_handle_key(keycode_Left); }},
+        {{RealCtrl, Qt::Key_D}, []{ gli_input_handle_key(keycode_Erase); }},
+        {{RealCtrl, Qt::Key_E}, []{ gli_input_handle_key(keycode_End); }},
+        {{RealCtrl, Qt::Key_F}, []{ gli_input_handle_key(keycode_Right); }},
+        {{RealCtrl, Qt::Key_H}, []{ gli_input_handle_key(keycode_Delete); }},
+        {{RealCtrl, Qt::Key_N}, []{ gli_input_handle_key(keycode_Down); }},
+        {{RealCtrl, Qt::Key_P}, []{ gli_input_handle_key(keycode_Up); }},
+        {{RealCtrl, Qt::Key_U}, []{ gli_input_handle_key(keycode_Escape); }},
+
+#ifdef Q_OS_WIN
+        // Qt doesn't assign Ctrl-Q to QKeySequence::Quit on Windows.
+        {{Qt::ControlModifier, Qt::Key_Q}, []{ gli_exit(0); }},
+#endif
 
 #ifdef __HAIKU__
         // For some reason, on Haiku, the "shifted" versions of comma/period are
@@ -436,17 +531,12 @@ void View::keyPressEvent(QKeyEvent *event)
         {{Qt::NoModifier, Qt::Key_Escape},    []{ gli_input_handle_key(keycode_Escape); }},
         {{Qt::NoModifier, Qt::Key_Tab},       []{ gli_input_handle_key(keycode_Tab); }},
         {{Qt::NoModifier, Qt::Key_Backspace}, []{ gli_input_handle_key(keycode_Delete); }},
-        {{Qt::NoModifier, Qt::Key_Delete},    []{ gli_input_handle_key(keycode_Erase); }},
         {{Qt::NoModifier, Qt::Key_Return},    []{ gli_input_handle_key(keycode_Return); }},
         {{Qt::NoModifier, Qt::Key_Enter},     []{ gli_input_handle_key(keycode_Return); }},
-        {{Qt::NoModifier, Qt::Key_Home},      []{ gli_input_handle_key(keycode_Home); }},
-        {{Qt::NoModifier, Qt::Key_End},       []{ gli_input_handle_key(keycode_End); }},
         {{Qt::NoModifier, Qt::Key_Left},      []{ gli_input_handle_key(keycode_Left); }},
         {{Qt::NoModifier, Qt::Key_Up},        []{ gli_input_handle_key(keycode_Up); }},
         {{Qt::NoModifier, Qt::Key_Right},     []{ gli_input_handle_key(keycode_Right); }},
         {{Qt::NoModifier, Qt::Key_Down},      []{ gli_input_handle_key(keycode_Down); }},
-        {{Qt::NoModifier, Qt::Key_PageUp},    []{ gli_input_handle_key(keycode_PageUp); }},
-        {{Qt::NoModifier, Qt::Key_PageDown},  []{ gli_input_handle_key(keycode_PageDown); }},
         {{Qt::NoModifier, Qt::Key_F1},        []{ gli_input_handle_key(keycode_Func1); }},
         {{Qt::NoModifier, Qt::Key_F2},        []{ gli_input_handle_key(keycode_Func2); }},
         {{Qt::NoModifier, Qt::Key_F3},        []{ gli_input_handle_key(keycode_Func3); }},
@@ -459,18 +549,67 @@ void View::keyPressEvent(QKeyEvent *event)
         {{Qt::NoModifier, Qt::Key_F10},       []{ gli_input_handle_key(keycode_Func10); }},
         {{Qt::NoModifier, Qt::Key_F11},       []{ gli_input_handle_key(keycode_Func11); }},
         {{Qt::NoModifier, Qt::Key_F12},       []{ gli_input_handle_key(keycode_Func12); }},
+
+        {{Qt::ShiftModifier | Qt::ControlModifier, Qt::Key_S}, []{
+            auto text = gli_get_scrollback();
+            if (text.has_value()) {
+                auto filename = QFileDialog::getSaveFileName(::window, "Save transcript", "transcript.txt", "Text files (*.txt)");
+                if (!filename.isNull()) {
+                    QFile file(filename);
+                    if (file.open(QIODevice::WriteOnly)) {
+                        std::size_t n = file.write(text->data(), text->size());
+                        if (n != text->size()) {
+                            QMessageBox::critical(nullptr, "Error", "Error writing entire transcript.");
+                        }
+                    } else {
+                        QMessageBox::critical(nullptr, "Error", "Unable to open file for writing.");
+                    }
+                }
+            } else {
+                QMessageBox::warning(nullptr, "Warning", "Could not find appropriate window for scrollback.");
+            }
+        }},
+
+        // Don't use QKeySequence::FullScreen here, as that takes over
+        // F11 on X, and F11 can be bound by Glk; and it doesn't bind
+        // Alt-Enter on X, either, meaning the existing behavior would
+        // change. For Mac, use Meta+Ctrl+F, and Alt-Enter everywhere
+        // else.
+#ifdef Q_OS_MAC
+        {{Qt::MetaModifier | Qt::ControlModifier, Qt::Key_F}, [this]{
+#else
+        {{Qt::AltModifier, Qt::Key_Return}, [this]{
+#endif
+            if (::window->isFullScreen()) {
+                if (m_fullscreen_from_maximized) {
+                    ::window->showMaximized();
+                } else {
+                    ::window->showNormal();
+                }
+            } else {
+                m_fullscreen_from_maximized = ::window->isMaximized();
+                ::window->showFullScreen();
+            }
+        }},
     };
 
+    for (const auto &pair : sequences) {
+        if (event->matches(pair.first)) {
+            pair.second();
+            return;
+        }
+    }
+
     try {
         keys.at(std::make_pair(modmasked, event->key()))();
         return;
     } catch (const std::out_of_range &) {
     }
 
-    handle_input(event->text());
+    handle_input(event->text(), false);
 }
 
-void View::mouseMoveEvent(QMouseEvent *event)
+void garglk::View::mouseMoveEvent(QMouseEvent *event)
 {
     // hyperlinks and selection
     if (gli_copyselect) {
@@ -487,7 +626,7 @@ void View::mouseMoveEvent(QMouseEvent *event)
     event->accept();
 }
 
-void View::mousePressEvent(QMouseEvent *event)
+void garglk::View::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton) {
         gli_input_handle_click(event->pos().x(), event->pos().y());
@@ -498,7 +637,7 @@ void View::mousePressEvent(QMouseEvent *event)
     event->accept();
 }
 
-void View::mouseReleaseEvent(QMouseEvent *event)
+void garglk::View::mouseReleaseEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton) {
         gli_copyselect = false;
@@ -509,7 +648,7 @@ void View::mouseReleaseEvent(QMouseEvent *event)
     event->accept();
 }
 
-void View::wheelEvent(QWheelEvent *event)
+void garglk::View::wheelEvent(QWheelEvent *event)
 {
     QPoint pixels = event->pixelDelta();
     QPoint degrees = event->angleDelta() / 8;
@@ -544,36 +683,51 @@ void View::wheelEvent(QWheelEvent *event)
     event->accept();
 }
 
-void wininit(int *, char **)
+void wininit()
 {
     // QApplication takes argc by reference (because it might modify
     // it), and thus requires it to live at least as long as the
-    // QApplication instance. However, the caller to this function is
-    // passing argc as the address of a function parameter which will no
-    // longer exist as soon as that function exits. This isn't really a
-    // Qt program in the normal sense, so doesn't need to provide Qt
-    // command-line arguments (such as -style). Create static dummy
-    // argument data here so it will live the entire life of the
-    // program and thus fulfull QApplication's requirements.
+    // QApplication instance. This isn't really a Qt program in the
+    // normal sense, so doesn't need to provide Qt command-line
+    // arguments (such as -style). Create static dummy argument data
+    // here so it will live the entire life of the program and thus
+    // fulfill QApplication's requirements.
     static int argc = 1;
     static char *argv[] = {const_cast<char *>("gargoyle"), nullptr};
     app = new QApplication(argc, argv);
-    app->setOrganizationName(GARGOYLE_ORGANIZATION);
-    app->setApplicationName(GARGOYLE_NAME);
-    app->setApplicationVersion(GARGOYLE_VERSION);
-    last_tick.start();
+    QApplication::setApplicationVersion(GARGOYLE_VERSION);
+
+    // The convention on Windows & Mac is capitalized directories,
+    // whereas on Unix it's all lowercase. Qt uses the application name
+    // when finding paths with QStandardDirectories, so set the expected
+    // name based on operating system.
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+    QApplication::setApplicationName("Gargoyle");
+#else
+    QApplication::setApplicationName("gargoyle");
+#endif
+
+    std::thread([]() {
+        while (true) {
+            std::this_thread::sleep_for(std::chrono::milliseconds(TICK_PERIOD_MILLIS));
+            process_events.store(true, std::memory_order_relaxed);
+        }
+    })
+    .detach();
 }
 
 void winopen()
 {
-    window = new Window();
+    window = new garglk::Window();
+
+    window->setMinimumSize(gli_wmarginx * 2, gli_wmarginy * 2);
 
     int defw = gli_wmarginx * 2 + gli_cellw * gli_cols;
     int defh = gli_wmarginy * 2 + gli_cellh * gli_rows;
     QSize size(defw, defh);
     if (gli_conf_save_window_size) {
         auto stored_size = window->settings()->value("window/size");
-        if (!stored_size.isNull()) {
+        if (stored_size.canConvert<QSize>()) {
             size = stored_size.toSize();
         }
     }
@@ -581,14 +735,23 @@ void winopen()
 
     if (gli_conf_save_window_location) {
         auto position = window->settings()->value("window/position");
-        if (!position.isNull()) {
+        if (position.canConvert<QPoint>()) {
             window->move(position.toPoint());
         }
     }
 
     wintitle();
 
-    if (gli_conf_fullscreen) {
+    bool do_fullscreen = gli_conf_fullscreen;
+
+    if (gli_conf_save_window_location || gli_conf_save_window_size) {
+        auto fullscreen = window->settings()->value("window/fullscreen");
+        if (fullscreen.canConvert<bool>()) {
+            do_fullscreen = fullscreen.toBool();
+        }
+    }
+
+    if (do_fullscreen) {
         window->showFullScreen();
     } else {
         window->show();
@@ -643,29 +806,42 @@ bool windark()
     return text_hsv_value > bg_hsv_value;
 }
 
-std::string garglk::winfontpath(const std::string &filename)
+nonstd::optional<std::string> garglk::winfontpath(const std::string &filename)
 {
-    return QCoreApplication::applicationDirPath().toStdString() + "/" + filename;
+    return Format("{}/{}", QCoreApplication::applicationDirPath().toStdString(), filename);
 }
 
-std::vector<std::string> garglk::winappdata()
+std::string garglk::windatadir()
+{
+#if defined(_WIN32)
+    return garglk::winappdir().value_or(".");
+#elif GARGLK_CONFIG_APPIMAGE
+    // For AppImages, hard-code the "known" path to app data (in this
+    // case that's <binary>/../share/gargoyle).
+    auto dir = QCoreApplication::applicationDirPath().toStdString();
+    return Format("{}/../share/gargoyle", dir);
+#elif defined(GARGLK_CONFIG_DATADIR)
+    return GARGLK_CONFIG_DATADIR;
+#else
+    return ".";
+#endif
+}
+
+std::vector<std::string> garglk::winthemedirs()
 {
     std::vector<std::string> paths;
 
     for (const auto &path : QStandardPaths::standardLocations(QStandardPaths::AppDataLocation)) {
-        paths.push_back(path.toStdString());
+        paths.push_back(path.toStdString() + "/themes");
     }
 
 #ifdef _WIN32
     // On Windows, also search the executable's directory.
-    paths.push_back(QCoreApplication::applicationDirPath().toStdString());
+    paths.push_back(QCoreApplication::applicationDirPath().toStdString() + "/themes");
 #endif
 
-    // For AppImages, hard-code the "known" path to app data (in this
-    // case that's <binary>/../share/io.github.garglk/Gargoyle).
 #if GARGLK_CONFIG_APPIMAGE
-    auto dir = QCoreApplication::applicationDirPath();
-    paths.push_back((dir + "/../share/" + GARGOYLE_ORGANIZATION + "/Gargoyle").toStdString());
+    paths.push_back(Format("{}/themes", garglk::windatadir()));
 #endif
 
     // QStandardPaths returns higher priority directories first: reverse
@@ -676,16 +852,44 @@ std::vector<std::string> garglk::winappdata()
     return paths;
 }
 
-std::string garglk::winappdir()
+nonstd::optional<std::string> garglk::winlegacythemedir() {
+#ifdef _WIN32
+    const char *appdata = std::getenv("APPDATA");
+    if (appdata == nullptr) {
+        return nonstd::nullopt;
+    }
+
+    return std::string(appdata) + "\\io.github.garglk\\Gargoyle\\themes";
+#else
+    QString user_share = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+    QDir legacy_themes = QDir(user_share).filePath("io.github.garglk/Gargoyle/themes");
+    return legacy_themes.path().toStdString();
+#endif
+}
+
+nonstd::optional<std::string> garglk::winappdir()
 {
     return QCoreApplication::applicationDirPath().toStdString();
 }
 
+bool garglk::winisfullscreen()
+{
+    return window->isFullScreen();
+}
+
 void gli_tick()
 {
-    if (last_tick.elapsed() > TICK_PERIOD_MILLIS) {
+    // Qt needs to keep processing events even in the absence of calls
+    // to glk_select(). Processing Qt events is expensive, so should not
+    // be done each tick (which generally happens each VM instruction).
+    // Originally this waited at least 10ms between calls, but the mere
+    // act of checking a timer each iteration was too expensive. Now a
+    // separate thread sits and atomically updates "process_events"
+    // every 10ms, since checking this atomic variable is much faster
+    // than checking a timer.
+    if (process_events.load(std::memory_order_relaxed)) {
         app->processEvents(QEventLoop::ExcludeUserInputEvents);
-        last_tick.start();
+        process_events.store(false, std::memory_order_relaxed);
     }
 }
 
@@ -717,4 +921,6 @@ void gli_select(event_t *event, bool polled)
         gli_dispatch_event(event, polled);
         window->reset_timeout();
     }
+
+    process_events.store(false, std::memory_order_relaxed);
 }
