@@ -5,12 +5,16 @@
 #include <QKeyEvent>
 #include <QMainWindow>
 #include <QMouseEvent>
+#include <QMoveEvent>
 #include <QPaintEvent>
 #include <QResizeEvent>
+#include <QSettings>
 #include <QTimer>
 #include <QWheelEvent>
 #include <QWidget>
 
+namespace garglk {
+
 class View : public QWidget
 {
     Q_OBJECT
@@ -23,6 +27,7 @@ public:
     }
 
     QVariant inputMethodQuery(Qt::InputMethodQuery query) const override;
+    void refresh();
 
 protected:
     void inputMethodEvent(QInputMethodEvent *event) override;
@@ -32,26 +37,36 @@ protected:
     void mousePressEvent(QMouseEvent *) override;
     void mouseReleaseEvent(QMouseEvent *) override;
     void wheelEvent(QWheelEvent *) override;
+
+private:
+    bool m_fullscreen_from_maximized = false;
 };
 
 class Window : public QMainWindow {
     Q_OBJECT
 public:
     Window();
-    ~Window();
 
-    void start_timer(long);
-    bool timed_out() { return m_timed_out; }
+    void refresh() { m_view->refresh(); }
+
+    void start_timer(unsigned long);
+    bool timed_out() const { return m_timed_out; }
     void reset_timeout() { m_timed_out = false; }
 
+    const QSettings *settings() { return m_settings; }
+
 protected:
     void closeEvent(QCloseEvent *) override;
     void resizeEvent(QResizeEvent *) override;
+    void moveEvent(QMoveEvent *) override;
 
 private:
-    View *m_view;
-    QTimer *m_timer;
+    View *const m_view;
+    QTimer *const m_timer;
+    QSettings *const m_settings;
     bool m_timed_out = false;
 };
 
+
+}
 #endif
