@@ -34,29 +34,11 @@ static void touch(window_textgrid_t *dwin, int line)
     winrepaint(win->bbox.x0, y, win->bbox.x1, y + gli_leading);
 }
 
-window_textgrid_t *win_textgrid_create(window_t *win)
-{
-    return new window_textgrid_t(win, gli_gstyles);
-}
-
-void win_textgrid_destroy(window_textgrid_t *dwin)
-{
-    if (dwin->inbuf != nullptr) {
-        if (gli_unregister_arr != nullptr) {
-            const char *typedesc = (dwin->inunicode ? "&+#!Iu" : "&+#!Cn");
-            (*gli_unregister_arr)(dwin->inbuf, dwin->inoriglen, const_cast<char *>(typedesc), dwin->inarrayrock);
-        }
-        dwin->inbuf = nullptr;
-    }
-
-    delete dwin;
-}
-
 void win_textgrid_rearrange(window_t *win, rect_t *box)
 {
     int newwid, newhgt;
     int k;
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     dwin->owner->bbox = *box;
 
     newwid = (box->x1 - box->x0) / gli_cellw;
@@ -77,83 +59,64 @@ void win_textgrid_rearrange(window_t *win, rect_t *box)
 
     for (k = 0; k < dwin->height; k++) {
         touch(dwin, k);
-        std::fill(&dwin->lines[k].chars[dwin->width], dwin->lines[k].chars.end(), ' ');
-        for (auto *attr = dwin->lines[k].attrs.begin() + dwin->width; attr != dwin->lines[k].attrs.end(); ++attr) {
+        std::fill(dwin->lines[k].chars.begin() + dwin->width, dwin->lines[k].chars.end(), ' ');
+        auto *attr_end = (&dwin->lines[k].attrs[0]) + dwin->lines[k].attrs.size();
+        for (auto *attr = &dwin->lines[k].attrs[dwin->width]; attr < attr_end; ++attr) {
             attr->clear();
         }
     }
 }
 
-void win_textgrid_redraw(window_t *win)
-{
-    window_textgrid_t *dwin = win->window.textgrid;
-    tgline_t *ln;
-    int x0, y0;
-    int x, y, w;
-    int i, a, b, k, o;
-    glui32 link;
+void win_textgrid_redraw(window_t *win) {
+    window_textgrid_t *dwin = win->wingrid();
+    int x0 = win->bbox.x0;
+    int y0 = win->bbox.y0;
 
-    x0 = win->bbox.x0;
-    y0 = win->bbox.y0;
+    // Draw a run of identically-styled cells over the interval [start,end) at (x,y).
+    // Return the width of the drawn area.
+    auto draw_run = [&dwin](const tgline_t *ln, int start, int end, int x, int y) -> int {
+        glui32 link    = ln->attrs[start].hyper;
+        auto   font    = ln->attrs[start].font(dwin->styles);
+        Color  fgcolor = link != 0 ? gli_link_color
+                                   : ln->attrs[start].fg(dwin->styles);
+        Color  bgcolor = ln->attrs[start].bg(dwin->styles);
+        int    w       = (end - start) * gli_cellw;
 
-    for (i = 0; i < dwin->height; i++) {
-        ln = &dwin->lines[i];
-        if (ln->dirty || gli_force_redraw) {
-            ln->dirty = false;
+        gli_draw_rect(x, y, w, gli_leading, bgcolor);
 
-            x = x0;
-            y = y0 + i * gli_leading;
-
-            // clear any stored hyperlink coordinates
-            gli_put_hyperlink(0, x0, y, x0 + gli_cellw * dwin->width, y + gli_leading);
-
-            a = 0;
-            for (b = 0; b < dwin->width; b++) {
-                if (ln->attrs[a] != ln->attrs[b]) {
-                    link = ln->attrs[a].hyper;
-                    auto font = ln->attrs[a].font(dwin->styles);
-                    Color fgcolor = link != 0 ? gli_link_color : ln->attrs[a].fg(dwin->styles);
-                    Color bgcolor = ln->attrs[a].bg(dwin->styles);
-                    w = (b - a) * gli_cellw;
-                    gli_draw_rect(x, y, w, gli_leading, bgcolor);
-                    o = x;
-                    for (k = a; k < b; k++) {
-                        gli_draw_string_uni(o * GLI_SUBPIX,
-                                y + gli_baseline, font, fgcolor,
-                                &ln->chars[k], 1, -1);
-                        o += gli_cellw;
-                    }
-                    if (link != 0) {
-                        if (gli_underline_hyperlinks) {
-                            gli_draw_rect(x, y + gli_baseline + 1, w,
-                                        1, gli_link_color);
-                        }
-                        gli_put_hyperlink(link, x, y, x + w, y + gli_leading);
-                    }
-                    x += w;
-                    a = b;
-                }
+        if (link != 0) {
+            if (gli_underline_hyperlinks) {
+                gli_draw_rect(x, y + gli_baseline + 1, w, 1, gli_link_color);
             }
-            link = ln->attrs[a].hyper;
-            auto font = ln->attrs[a].font(dwin->styles);
-            Color fgcolor = link != 0 ? gli_link_color : ln->attrs[a].fg(dwin->styles);
-            Color bgcolor = ln->attrs[a].bg(dwin->styles);
-            w = (b - a) * gli_cellw;
-            w += win->bbox.x1 - (x + w);
-            gli_draw_rect(x, y, w, gli_leading, bgcolor);
-            o = x;
-            for (k = a; k < b; k++) {
-                gli_draw_string_uni(o * GLI_SUBPIX,
-                                    y + gli_baseline, font, fgcolor,
-                                    &ln->chars[k], 1, -1);
-                o += gli_cellw;
-            }
-            if (link != 0) {
-                if (gli_underline_hyperlinks) {
-                    gli_draw_rect(x, y + gli_baseline + 1, w,
-                                1, gli_link_color);
-                }
-                gli_put_hyperlink(link, x, y, x + w, y + gli_leading);
+            gli_put_hyperlink(link, x, y, x + w, y + gli_leading);
+        }
+
+        for (int i = start; i < end; i++) {
+            gli_draw_string_uni(x * GLI_SUBPIX, y + gli_baseline, font, fgcolor, &ln->chars[i], 1, -1);
+            x += gli_cellw;
+        }
+
+        return w;
+    };
+
+    for (int i = 0; i < dwin->height; i++) {
+        tgline_t *ln = &dwin->lines[i];
+        if (!ln->dirty && !gli_force_redraw) {
+            continue;
+        }
+        ln->dirty = false;
+
+        int x = x0;
+        int y = y0 + i * gli_leading;
+
+        // Clear any stored hyperlink coordinates.
+        gli_put_hyperlink(0, x0, y, x0 + gli_cellw * dwin->width, y + gli_leading);
+
+        // Draw runs of contiguous cells that share the same styling.
+        for (int start = 0, end = 1; end <= dwin->width; end++) {
+            if (end == dwin->width || ln->attrs[end] != ln->attrs[start]) {
+                x += draw_run(ln, start, end, x, y);
+                start = end;
             }
         }
     }
@@ -161,7 +124,7 @@ void win_textgrid_redraw(window_t *win)
 
 void win_textgrid_putchar_uni(window_t *win, glui32 ch)
 {
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     tgline_t *ln;
 
     // Canonicalize the cursor position. That is, the cursor may have been
@@ -198,7 +161,7 @@ void win_textgrid_putchar_uni(window_t *win, glui32 ch)
 
 bool win_textgrid_unputchar_uni(window_t *win, glui32 ch)
 {
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     tgline_t *ln;
     int oldx = dwin->curx, oldy = dwin->cury;
 
@@ -246,13 +209,11 @@ bool win_textgrid_unputchar_uni(window_t *win, glui32 ch)
 
 void win_textgrid_clear(window_t *win)
 {
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     int k;
 
-    win->attr.fgset = gli_override_fg_set;
-    win->attr.bgset = gli_override_bg_set;
-    win->attr.fgcolor = gli_override_fg_set ? gli_override_fg_val : Color(0, 0, 0);
-    win->attr.bgcolor = gli_override_bg_set ? gli_override_bg_val : Color(0, 0, 0);
+    win->attr.fgcolor = gli_override_fg;
+    win->attr.bgcolor = gli_override_bg;
     win->attr.reverse = false;
 
     for (k = 0; k < dwin->height; k++) {
@@ -267,7 +228,7 @@ void win_textgrid_clear(window_t *win)
 
 void win_textgrid_move_cursor(window_t *win, int xpos, int ypos)
 {
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
 
     // If the values are negative, they're really huge positive numbers --
     // remember that they were cast from glui32. So set them huge and
@@ -318,7 +279,7 @@ void win_textgrid_click(window_textgrid_t *dwin, int sx, int sy)
 // Prepare the window for line input.
 static void win_textgrid_init_impl(window_t *win, void *buf, int maxlen, int initlen, bool unicode)
 {
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
 
     dwin->inunicode = unicode;
     dwin->inoriglen = maxlen;
@@ -348,7 +309,7 @@ static void win_textgrid_init_impl(window_t *win, void *buf, int maxlen, int ini
             if (unicode) {
                 ln->chars[dwin->inorgx + ix] = (static_cast<glui32 *>(buf))[ix];
             } else {
-                ln->chars[dwin->inorgx + ix] = (static_cast<char *>(buf))[ix];
+                ln->chars[dwin->inorgx + ix] = (static_cast<unsigned char *>(buf))[ix];
             }
         }
 
@@ -360,8 +321,6 @@ static void win_textgrid_init_impl(window_t *win, void *buf, int maxlen, int ini
         touch(dwin, dwin->inorgy);
     }
 
-    dwin->line_terminators = win->line_terminators;
-
     if (gli_register_arr != nullptr) {
         dwin->inarrayrock = (*gli_register_arr)(dwin->inbuf, dwin->inoriglen, const_cast<char *>(unicode ? "&+#!Iu" : "&+#!Cn"));
     }
@@ -377,15 +336,14 @@ void win_textgrid_init_line_uni(window_t *win, glui32 *buf, int maxlen, int init
     win_textgrid_init_impl(win, buf, maxlen, initlen, true);
 }
 
-// Abort line input, storing whatever's been typed so far.
-void win_textgrid_cancel_line(window_t *win, event_t *ev)
+static void accept_or_cancel(window_t *win, event_t *ev, bool echo)
 {
     int ix;
     void *inbuf;
     int inoriglen;
     bool inunicode;
     gidispatch_rock_t inarrayrock;
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     tgline_t *ln = &(dwin->lines[dwin->inorgy]);
 
     if (dwin->inbuf == nullptr) {
@@ -405,14 +363,14 @@ void win_textgrid_cancel_line(window_t *win, event_t *ev)
             }
             (static_cast<char *>(inbuf))[ix] = static_cast<char>(ch);
         }
-        if (win->echostr != nullptr) {
+        if (echo && win->echostr != nullptr) {
             gli_stream_echo_line(win->echostr, static_cast<char *>(inbuf), dwin->inlen);
         }
     } else {
         for (ix = 0; ix < dwin->inlen; ix++) {
             (static_cast<glui32 *>(inbuf))[ix] = ln->chars[dwin->inorgx + ix];
         }
-        if (win->echostr != nullptr) {
+        if (echo && win->echostr != nullptr) {
             gli_stream_echo_line_uni(win->echostr, static_cast<glui32 *>(inbuf), dwin->inlen);
         }
     }
@@ -428,7 +386,6 @@ void win_textgrid_cancel_line(window_t *win, event_t *ev)
 
     win->line_request = false;
     win->line_request_uni = false;
-    dwin->line_terminators.clear();
     dwin->inbuf = nullptr;
     dwin->inoriglen = 0;
     dwin->inmax = 0;
@@ -441,6 +398,12 @@ void win_textgrid_cancel_line(window_t *win, event_t *ev)
     }
 }
 
+// Abort line input, storing whatever's been typed so far.
+void win_textgrid_cancel_line(window_t *win, event_t *ev)
+{
+    accept_or_cancel(win, ev, false);
+}
+
 // Keybinding functions.
 
 // Any key, during character input. Ends character input.
@@ -473,80 +436,30 @@ void gcmd_grid_accept_readchar(window_t *win, glui32 arg)
 // Return or enter, during line input. Ends line input.
 static void acceptline(window_t *win, glui32 keycode)
 {
-    int ix;
-    void *inbuf;
-    int inoriglen;
-    bool inunicode;
-    gidispatch_rock_t inarrayrock;
-    window_textgrid_t *dwin = win->window.textgrid;
-    tgline_t *ln = &(dwin->lines[dwin->inorgy]);
+    event_t ev;
 
-    if (dwin->inbuf == nullptr) {
-        return;
+    accept_or_cancel(win, &ev, true);
+
+    if (!win->line_terminators.empty() && keycode != keycode_Return) {
+        ev.val2 = keycode;
     }
 
-    inbuf = dwin->inbuf;
-    inoriglen = dwin->inoriglen;
-    inarrayrock = dwin->inarrayrock;
-    inunicode = dwin->inunicode;
-
-    if (!inunicode) {
-        for (ix = 0; ix < dwin->inlen; ix++) {
-            (static_cast<char *>(inbuf))[ix] = static_cast<char>(ln->chars[dwin->inorgx + ix]);
-        }
-        if (win->echostr != nullptr) {
-            gli_stream_echo_line(win->echostr, static_cast<char *>(inbuf), dwin->inlen);
-        }
-    } else {
-        for (ix = 0; ix < dwin->inlen; ix++) {
-            (static_cast<glui32 *>(inbuf))[ix] = ln->chars[dwin->inorgx + ix];
-        }
-        if (win->echostr != nullptr) {
-            gli_stream_echo_line_uni(win->echostr, static_cast<glui32 *>(inbuf), dwin->inlen);
-        }
-    }
-
-    dwin->cury = dwin->inorgy + 1;
-    dwin->curx = 0;
-    win->attr = dwin->origattr;
-
-    if (!dwin->line_terminators.empty()) {
-        glui32 val2 = keycode;
-        if (val2 == keycode_Return) {
-            val2 = 0;
-        }
-        gli_event_store(evtype_LineInput, win, dwin->inlen, val2);
-        dwin->line_terminators.clear();
-    } else {
-        gli_event_store(evtype_LineInput, win, dwin->inlen, 0);
-    }
-    win->line_request = false;
-    win->line_request_uni = false;
-    dwin->inbuf = nullptr;
-    dwin->inoriglen = 0;
-    dwin->inmax = 0;
-    dwin->inorgx = 0;
-    dwin->inorgy = 0;
-
-    if (gli_unregister_arr != nullptr) {
-        const char *typedesc = (inunicode ? "&+#!Iu" : "&+#!Cn");
-        (*gli_unregister_arr)(inbuf, inoriglen, const_cast<char *>(typedesc), inarrayrock);
-    }
+    gli_event_store(ev.type, ev.win, ev.val1, ev.val2);
 }
 
 // Any regular key, during line input.
 void gcmd_grid_accept_readline(window_t *win, glui32 arg)
 {
     int ix;
-    window_textgrid_t *dwin = win->window.textgrid;
+    window_textgrid_t *dwin = win->wingrid();
     tgline_t *ln = &(dwin->lines[dwin->inorgy]);
 
     if (dwin->inbuf == nullptr) {
         return;
     }
 
-    if (!dwin->line_terminators.empty() && gli_window_check_terminator(arg)) {
-        if (std::find(dwin->line_terminators.begin(), dwin->line_terminators.end(), arg) != dwin->line_terminators.end()) {
+    if (!win->line_terminators.empty() && gli_window_check_terminator(arg)) {
+        if (std::find(win->line_terminators.begin(), win->line_terminators.end(), arg) != win->line_terminators.end()) {
             acceptline(win, arg);
             return;
         }
@@ -628,14 +541,14 @@ void gcmd_grid_accept_readline(window_t *win, glui32 arg)
 
     case keycode_Return:
         acceptline(win, arg);
-        break;
+        return;
 
     default:
         if (dwin->inlen >= dwin->inmax) {
             return;
         }
 
-        if (arg < 32 || arg > 0xff) {
+        if (arg < 32 || arg > 0x10ffff) {
             return;
         }
 
