@@ -37,6 +37,8 @@ static void
 put_text_uni(window_textbuffer_t *dwin, glui32 *buf, int len, int pos, int oldlen);
 static bool
 put_picture(window_textbuffer_t *dwin, const std::shared_ptr<picture_t> &pic, glui32 align, glui32 linkval);
+static void
+scrolloneline(window_textbuffer_t *dwin, bool forced, bool flow_break);
 
 static void touch(window_textbuffer_t *dwin, int line)
 {
@@ -58,34 +60,37 @@ static void touchscroll(window_textbuffer_t *dwin)
     }
 }
 
-window_textbuffer_t *win_textbuffer_create(window_t *win)
+std::vector<char> gli_get_text(window_textbuffer_t *dwin)
 {
-    return new window_textbuffer_t(win, gli_tstyles, SCROLLBACK);
-}
+    int s = dwin->scrollmax < SCROLLBACK ? dwin->scrollmax : SCROLLBACK - 1;
 
-void win_textbuffer_destroy(window_textbuffer_t *dwin)
-{
-    if (dwin->inbuf != nullptr) {
-        if (gli_unregister_arr != nullptr) {
-            const char *typedesc = (dwin->inunicode ? "&+#!Iu" : "&+#!Cn");
-            (*gli_unregister_arr)(dwin->inbuf, dwin->inmax, const_cast<char *>(typedesc), dwin->inarrayrock);
+    std::vector<char> text;
+    for (int lineidx = s; lineidx >= 0; lineidx--) {
+        auto line = dwin->lines[lineidx];
+        for (int charidx = 0; charidx < line.len; charidx++) {
+            std::array<char, 4> buf;
+            auto n = gli_encode_utf8(line.chars[charidx], buf.data(), 4);
+            for (int i = 0; i < n; i++) {
+                text.push_back(buf[i]);
+            }
+        }
+        // Only add newline if this was an actual paragraph break, not a wrapped line
+        if (line.newline) {
+            text.push_back(0x0a); // Unicode linefeed
         }
-        dwin->inbuf = nullptr;
     }
 
-    dwin->owner = nullptr;
-
-    delete dwin;
+    return text;
 }
 
 static void reflow(window_t *win)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     int inputbyte = -1;
     attr_t curattr;
     attr_t oldattr;
     int i, k, p, s;
-    int x;
+    int x, f;
 
     if (dwin->height < 4 || dwin->width < 20) {
         return;
@@ -99,6 +104,7 @@ static void reflow(window_t *win)
     std::vector<std::shared_ptr<picture_t>> pictbuf;
     std::vector<glui32> hyperbuf;
     std::vector<int> offsetbuf;
+    std::vector<int> flowbreakbuf;
 
     // allocate temp buffers
     try {
@@ -108,6 +114,7 @@ static void reflow(window_t *win)
         pictbuf.resize(SCROLLBACK);
         hyperbuf.resize(SCROLLBACK);
         offsetbuf.resize(SCROLLBACK);
+        flowbreakbuf.resize(SCROLLBACK);
     } catch (const std::bad_alloc &) {
         return;
     }
@@ -118,6 +125,7 @@ static void reflow(window_t *win)
     curattr.clear();
 
     x = 0;
+    f = 0;
     p = 0;
     s = dwin->scrollmax < SCROLLBACK ? dwin->scrollmax : SCROLLBACK - 1;
 
@@ -142,6 +150,12 @@ static void reflow(window_t *win)
             x++;
         }
 
+        auto flow_break_pos = dwin->lines[k].flow_break_pos;
+        if (flow_break_pos >= 0) {
+            flowbreakbuf[f] = p + flow_break_pos;
+            f++;
+        }
+
         for (i = 0; i < dwin->lines[k].len; i++) {
             attrbuf[p] = curattr = dwin->lines[k].attrs[i];
             charbuf[p] = dwin->lines[k].chars[i];
@@ -164,6 +178,7 @@ static void reflow(window_t *win)
     // and dump text back
 
     x = 0;
+    f = 0;
     for (i = 0; i < p; i++) {
         if (i == inputbyte) {
             break;
@@ -175,6 +190,11 @@ static void reflow(window_t *win)
             x++;
         }
 
+        if (flowbreakbuf[f] == i) {
+            scrolloneline(dwin, false, true);
+            f++;
+        }
+
         win_textbuffer_putchar_uni(win, charbuf[i]);
     }
 
@@ -195,7 +215,7 @@ static void reflow(window_t *win)
 
 void win_textbuffer_rearrange(window_t *win, rect_t *box)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     int newwid, newhgt;
     int rnd;
 
@@ -231,11 +251,7 @@ void win_textbuffer_rearrange(window_t *win, rect_t *box)
         }
         touchscroll(dwin);
 
-        // allocate copy buffer
-        dwin->copybuf.resize(dwin->height * TBLINELEN);
-        std::fill(dwin->copybuf.begin(), dwin->copybuf.end(), 0);
-
-        dwin->copypos = 0;
+        dwin->copybuf.clear();
     }
 }
 
@@ -270,7 +286,7 @@ static int calcwidth(window_textbuffer_t *dwin,
 
 void win_textbuffer_redraw(window_t *win)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     tbline_t ln;
     int linelen;
     int nsp, spw, pw;
@@ -350,7 +366,7 @@ void win_textbuffer_redraw(window_t *win)
         linelen = ln.len;
 
         // kill spaces at the end unless they're a different color
-        Color color = gli_override_bg_set ? gli_window_color : win->bgcolor;
+        Color color = gli_override_bg.has_value() ? gli_window_color : win->bgcolor;
         while (i > 0 && linelen > 1 && ln.chars[linelen - 1] == ' '
                 && ln.attrs[linelen - 1].bgcolor == color
                 && !ln.attrs[linelen - 1].reverse) {
@@ -429,14 +445,12 @@ void win_textbuffer_redraw(window_t *win)
             if (selchar) {
                 for (tsc = lsc; tsc <= rsc; tsc++) {
                     ln.attrs[tsc].reverse = !ln.attrs[tsc].reverse;
-                    dwin->copybuf[dwin->copypos] = ln.chars[tsc];
-                    dwin->copypos++;
+                    dwin->copybuf.push_back(ln.chars[tsc]);
                 }
             }
-            // add newline if we reach the end of the line
-            if (ln.len == 0 || ln.len == (rsc + 1)) {
-                dwin->copybuf[dwin->copypos] = '\n';
-                dwin->copypos++;
+            // add newline only if this is a real paragraph break, not just a wrapped line
+            if ((ln.len == 0 || ln.len == (rsc + 1)) && ln.newline) {
+                dwin->copybuf.push_back('\n');
             }
         }
 
@@ -445,7 +459,7 @@ void win_textbuffer_redraw(window_t *win)
                 x1 / GLI_SUBPIX, y + gli_leading);
 
         // fill in background colors
-        color = gli_override_bg_set ? gli_window_color : win->bgcolor;
+        color = gli_override_bg.has_value() ? gli_window_color : win->bgcolor;
         gli_draw_rect(x0 / GLI_SUBPIX, y,
                 (x1 - x0) / GLI_SUBPIX, gli_leading,
                 color);
@@ -493,7 +507,7 @@ void win_textbuffer_redraw(window_t *win)
         }
         x += w;
 
-        color = gli_override_bg_set ? gli_window_color : win->bgcolor;
+        color = gli_override_bg.has_value() ? gli_window_color : win->bgcolor;
         gli_draw_rect(x / GLI_SUBPIX, y,
                 x1 / GLI_SUBPIX - x / GLI_SUBPIX, gli_leading,
                 color);
@@ -543,7 +557,7 @@ void win_textbuffer_redraw(window_t *win)
         gli_put_hyperlink(0, x0 / GLI_SUBPIX, y,
                 x1/GLI_SUBPIX, y + gli_leading);
 
-        Color color = gli_override_bg_set ? gli_window_color : win->bgcolor;
+        Color color = gli_override_bg.has_value() ? gli_window_color : win->bgcolor;
         gli_draw_rect(x / GLI_SUBPIX, y,
                 x1 / GLI_SUBPIX - x / GLI_SUBPIX, gli_leading,
                 color);
@@ -553,12 +567,11 @@ void win_textbuffer_redraw(window_t *win)
 
         if (gli_more_align == 1) { // center
             x = x0 + SLOP + (x1 - x0 - w - SLOP * 2) / 2;
-        }
-        if (gli_more_align == 2) { // right
+        } else if (gli_more_align == 2) { // right
             x = x1 - SLOP - w;
         }
 
-        color = gli_override_fg_set ? gli_more_color : win->fgcolor;
+        color = gli_override_fg.has_value() ? gli_more_color : win->fgcolor;
         gli_draw_string_uni(x, y + gli_baseline,
                 gli_more_font, color,
                 gli_more_prompt.data(), gli_more_prompt_len, -1);
@@ -657,13 +670,10 @@ void win_textbuffer_redraw(window_t *win)
     }
 
     // send selected text to clipboard
-    if (selbuf && dwin->copypos != 0) {
+    if (selbuf && !dwin->copybuf.empty()) {
         gli_claimselect = true;
-        gli_clipboard_copy(dwin->copybuf.data(), dwin->copypos);
-        for (i = 0; i < dwin->copypos; i++) {
-            dwin->copybuf[i] = 0;
-        }
-        dwin->copypos = 0;
+        gli_clipboard_copy(dwin->copybuf);
+        dwin->copybuf.clear();
     }
 
     // no more prompt means all text has been seen
@@ -691,6 +701,7 @@ static void scrollresize(window_textbuffer_t *dwin)
         dwin->lines[i].lhyper = 0;
         dwin->lines[i].rhyper = 0;
         dwin->lines[i].len = 0;
+        dwin->lines[i].flow_break_pos = -1;
         dwin->lines[i].newline = false;
         dwin->lines[i].chars.fill(' ');
         dwin->lines[i].attrs.fill(attr_t{});
@@ -699,44 +710,64 @@ static void scrollresize(window_textbuffer_t *dwin)
     dwin->scrollback += SCROLLBACK;
 }
 
-static void scrolloneline(window_textbuffer_t *dwin, bool forced)
+static void scrolloneline(window_textbuffer_t *dwin, bool forced, bool flow_break = false)
 {
     int i;
+    int lines_to_scroll = 1;
 
-    dwin->lastseen++;
-    dwin->scrollmax++;
+    if (flow_break) {
+        if (dwin->ladjn == 0 && dwin->radjn == 0) {
+            auto &line = dwin->lines[0];
+            line.flow_break_pos = line.len;
+            return;
+        }
 
-    if (dwin->scrollmax > dwin->scrollback - 1
-            || dwin->lastseen > dwin->scrollback - 1) {
+        lines_to_scroll = std::max(dwin->ladjn, dwin->radjn);
+    }
+
+    dwin->lastseen += lines_to_scroll;
+    dwin->scrollmax += lines_to_scroll;
+
+    if (dwin->scrollmax > dwin->scrollback - lines_to_scroll
+            || dwin->lastseen > dwin->scrollback - lines_to_scroll) {
         scrollresize(dwin);
     }
 
     if (dwin->lastseen >= dwin->height) {
-        dwin->scrollpos++;
+        dwin->scrollpos += lines_to_scroll;
     }
 
-    if (dwin->scrollpos > dwin->scrollmax - dwin->height + 1) {
-        dwin->scrollpos = dwin->scrollmax - dwin->height + 1;
+    if (dwin->scrollpos > dwin->scrollmax - dwin->height + lines_to_scroll) {
+        dwin->scrollpos = dwin->scrollmax - dwin->height + lines_to_scroll;
     }
     if (dwin->scrollpos < 0) {
         dwin->scrollpos = 0;
     }
 
-    if (forced) {
+    if (forced || flow_break) {
         dwin->dashed = 0;
     }
     dwin->spaced = 0;
 
-    dwin->lines[0].len = dwin->numchars;
-    dwin->lines[0].newline = forced;
+    auto &line_0 = dwin->lines[0];
+    line_0.len = dwin->numchars;
+    line_0.newline = forced;
+    if (flow_break) {
+        line_0.flow_break_pos = line_0.len;
+    }
 
-    for (i = dwin->scrollback - 1; i > 0; i--) {
-        dwin->lines[i] = dwin->lines[i - 1];
+    for (i = dwin->scrollback - 1; i > lines_to_scroll - 1; i--) {
+        dwin->lines[i] = dwin->lines[i - lines_to_scroll];
         if (i < dwin->height) {
             touch(dwin, i);
         }
     }
 
+    if (flow_break) {
+        dwin->radjn = 0;
+        dwin->ladjn = 0;
+    }
+
     if (dwin->radjn != 0) {
         dwin->radjn--;
     }
@@ -750,17 +781,20 @@ static void scrolloneline(window_textbuffer_t *dwin, bool forced)
         dwin->ladjw = 0;
     }
 
-    touch(dwin, 0);
-    dwin->lines[0].len = 0;
-    dwin->lines[0].newline = false;
-    dwin->lines[0].lm = dwin->ladjw;
-    dwin->lines[0].rm = dwin->radjw;
-    dwin->lines[0].lpic.reset();
-    dwin->lines[0].rpic.reset();
-    dwin->lines[0].lhyper = 0;
-    dwin->lines[0].rhyper = 0;
-    dwin->lines[0].chars.fill(' ');
-    dwin->lines[0].attrs.fill(attr_t{});
+    for (i = 0; i < lines_to_scroll; i++) {
+        touch(dwin, i);
+        dwin->lines[i].len = 0;
+        dwin->lines[i].flow_break_pos = -1;
+        dwin->lines[i].newline = false;
+        dwin->lines[i].lm = dwin->ladjw;
+        dwin->lines[i].rm = dwin->radjw;
+        dwin->lines[i].lpic.reset();
+        dwin->lines[i].rpic.reset();
+        dwin->lines[i].lhyper = 0;
+        dwin->lines[i].rhyper = 0;
+        dwin->lines[i].chars.fill(' ');
+        dwin->lines[i].attrs.fill(attr_t{});
+    }
 
     dwin->numchars = 0;
 
@@ -863,7 +897,7 @@ static bool leftquote(std::uint32_t c)
 
 void win_textbuffer_putchar_uni(window_t *win, glui32 ch)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     std::array<glui32, TBLINELEN> bchars;
     std::array<attr_t, TBLINELEN> battrs;
     int pw;
@@ -898,7 +932,7 @@ void win_textbuffer_putchar_uni(window_t *win, glui32 ch)
     pw = (win->bbox.x1 - win->bbox.x0 - gli_tmarginx * 2 - gli_scroll_width) * GLI_SUBPIX;
     pw = pw - 2 * SLOP - dwin->radjw - dwin->ladjw;
 
-    Color color = gli_override_bg_set ? gli_window_color : win->bgcolor;
+    Color color = gli_override_bg.has_value() ? gli_window_color : win->bgcolor;
 
     // oops ... overflow
     if (dwin->numchars + 1 >= TBLINELEN) {
@@ -980,7 +1014,7 @@ void win_textbuffer_putchar_uni(window_t *win, glui32 ch)
         }
 
         // turn (per sp x) into (per sp sp x)
-        if (gli_conf_spaces == 2) {
+        else if (gli_conf_spaces == 2) {
             if (ch == '.') {
                 dwin->spaced = 1;
             } else if (ch == ' ' && dwin->spaced == 1) {
@@ -1034,7 +1068,7 @@ void win_textbuffer_putchar_uni(window_t *win, glui32 ch)
 
 bool win_textbuffer_unputchar_uni(window_t *win, glui32 ch)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     if (dwin->numchars > 0 && glk_char_to_upper(dwin->chars[dwin->numchars - 1]) == glk_char_to_upper(ch)) {
         dwin->numchars--;
         touch(dwin, 0);
@@ -1045,13 +1079,11 @@ bool win_textbuffer_unputchar_uni(window_t *win, glui32 ch)
 
 void win_textbuffer_clear(window_t *win)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     int i;
 
-    win->attr.fgset = gli_override_fg_set;
-    win->attr.bgset = gli_override_bg_set;
-    win->attr.fgcolor = gli_override_fg_set ? gli_override_fg_val : Color(0x00, 0x00, 0x00);
-    win->attr.bgcolor = gli_override_bg_set ? gli_override_bg_val : Color(0x00, 0x00, 0x00);
+    win->attr.fgcolor = gli_override_fg;
+    win->attr.bgcolor = gli_override_bg;
     win->attr.reverse = false;
 
     dwin->ladjw = dwin->radjw = 0;
@@ -1064,6 +1096,7 @@ void win_textbuffer_clear(window_t *win)
 
     for (i = 0; i < dwin->scrollback; i++) {
         dwin->lines[i].len = 0;
+        dwin->lines[i].flow_break_pos = -1;
 
         dwin->lines[i].lpic.reset();
         dwin->lines[i].rpic.reset();
@@ -1089,7 +1122,7 @@ void win_textbuffer_clear(window_t *win)
 // Prepare the window for line input.
 static void win_textbuffer_init_impl(window_t *win, void *buf, int maxlen, int initlen, bool unicode)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     int pw;
 
     gli_tts_flush();
@@ -1126,9 +1159,6 @@ static void win_textbuffer_init_impl(window_t *win, void *buf, int maxlen, int i
         }
     }
 
-    dwin->echo_line_input = win->echo_line_input;
-    dwin->line_terminators = win->line_terminators;
-
     if (gli_register_arr != nullptr) {
         dwin->inarrayrock = (*gli_register_arr)(dwin->inbuf, maxlen, const_cast<char *>(unicode ? "&+#!Iu" : "&+#!Cn"));
     }
@@ -1147,7 +1177,7 @@ void win_textbuffer_init_line_uni(window_t *win, glui32 *buf, int maxlen, int in
 // Abort line input, storing whatever's been typed so far.
 void win_textbuffer_cancel_line(window_t *win, event_t *ev)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     gidispatch_rock_t inarrayrock;
     int ix;
     int len;
@@ -1196,11 +1226,10 @@ void win_textbuffer_cancel_line(window_t *win, event_t *ev)
 
     win->line_request = false;
     win->line_request_uni = false;
-    win->line_terminators.clear();
     dwin->inbuf = nullptr;
     dwin->inmax = 0;
 
-    if (dwin->echo_line_input) {
+    if (win->echo_line_input) {
         win_textbuffer_putchar_uni(win, '\n');
     } else {
         dwin->numchars = dwin->infence;
@@ -1218,7 +1247,7 @@ void win_textbuffer_cancel_line(window_t *win, event_t *ev)
 // Any key, when text buffer is scrolled.
 bool gcmd_accept_scroll(window_t *win, glui32 arg)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     int pageht = dwin->height - 2; // 1 for prompt, 1 for overlap
     bool startpos = dwin->scrollpos != 0;
 
@@ -1268,7 +1297,7 @@ bool gcmd_accept_scroll(window_t *win, glui32 arg)
 // Any key, during character input. Ends character input.
 void gcmd_buffer_accept_readchar(window_t *win, glui32 arg)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
     glui32 key;
 
     if (dwin->height < 2) {
@@ -1315,7 +1344,7 @@ static void acceptline(window_t *win, glui32 keycode)
     int inmax;
     bool inunicode;
     gidispatch_rock_t inarrayrock;
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
 
     if (dwin->inbuf == nullptr) {
         return;
@@ -1382,13 +1411,12 @@ static void acceptline(window_t *win, glui32 keycode)
 
     win->attr = dwin->origattr;
 
-    if (!dwin->line_terminators.empty()) {
+    if (!win->line_terminators.empty()) {
         glui32 val2 = keycode;
         if (val2 == keycode_Return) {
             val2 = 0;
         }
         gli_event_store(evtype_LineInput, win, len, val2);
-        dwin->line_terminators.clear();
     } else {
         gli_event_store(evtype_LineInput, win, len, 0);
     }
@@ -1397,7 +1425,7 @@ static void acceptline(window_t *win, glui32 keycode)
     dwin->inbuf = nullptr;
     dwin->inmax = 0;
 
-    if (dwin->echo_line_input) {
+    if (win->echo_line_input) {
         win_textbuffer_putchar_uni(win, '\n');
     } else {
         dwin->numchars = dwin->infence;
@@ -1413,7 +1441,7 @@ static void acceptline(window_t *win, glui32 keycode)
 // Any key, during line input.
 void gcmd_buffer_accept_readline(window_t *win, glui32 arg)
 {
-    window_textbuffer_t *dwin = win->window.textbuffer;
+    window_textbuffer_t *dwin = win->winbuffer();
 
     if (dwin->height < 2) {
         dwin->scrollpos = 0;
@@ -1430,8 +1458,8 @@ void gcmd_buffer_accept_readline(window_t *win, glui32 arg)
         return;
     }
 
-    if (!dwin->line_terminators.empty() && gli_window_check_terminator(arg)) {
-        if (std::find(dwin->line_terminators.begin(), dwin->line_terminators.end(), arg) != dwin->line_terminators.end()) {
+    if (!win->line_terminators.empty() && gli_window_check_terminator(arg)) {
+        if (std::find(win->line_terminators.begin(), win->line_terminators.end(), arg) != win->line_terminators.end()) {
             acceptline(win, arg);
             return;
         }
@@ -1584,9 +1612,11 @@ static bool put_picture(window_textbuffer_t *dwin, const std::shared_ptr<picture
 
         dwin->radjw = (pic->w + gli_tmarginx) * GLI_SUBPIX;
         dwin->radjn = (pic->h + gli_cellh - 1) / gli_cellh;
-        dwin->lines[0].rpic = pic;
-        dwin->lines[0].rm = dwin->radjw;
-        dwin->lines[0].rhyper = linkval;
+        auto &line = dwin->lines[0];
+        line.rpic = pic;
+        line.rm = dwin->radjw;
+        line.rhyper = linkval;
+        line.flow_break_pos = -1;
     }
 
     else {
@@ -1600,9 +1630,11 @@ static bool put_picture(window_textbuffer_t *dwin, const std::shared_ptr<picture
 
         dwin->ladjw = (pic->w + gli_tmarginx) * GLI_SUBPIX;
         dwin->ladjn = (pic->h + gli_cellh - 1) / gli_cellh;
-        dwin->lines[0].lpic = pic;
-        dwin->lines[0].lm = dwin->ladjw;
-        dwin->lines[0].lhyper = linkval;
+        auto &line = dwin->lines[0];
+        line.lpic = pic;
+        line.lm = dwin->ladjw;
+        line.lhyper = linkval;
+        line.flow_break_pos = -1;
 
         if (align != imagealign_MarginLeft) {
             win_textbuffer_flow_break(dwin);
@@ -1612,27 +1644,21 @@ static bool put_picture(window_textbuffer_t *dwin, const std::shared_ptr<picture
     return true;
 }
 
-bool win_textbuffer_draw_picture(window_textbuffer_t *dwin,
-    glui32 image, glui32 align, bool scaled, glui32 width, glui32 height)
+bool win_textbuffer_draw_picture(std::shared_ptr<picture_t> pic, window_textbuffer_t *dwin, glui32 window_width,
+    glui32 align, glui32 width, glui32 height, glui32 maxwidth)
 {
     glui32 hyperlink;
 
-    auto pic = gli_picture_load(image);
-
-    if (!pic) {
-        return false;
+    if (maxwidth != 0) {
+        auto limit = window_width * (maxwidth / 65536.0);
+        if (width > limit) {
+            double scaleby = static_cast<double>(limit) / width;
+            width *= scaleby;
+            height *= scaleby;
+        }
     }
 
-    if (!dwin->owner->image_loaded) {
-        gli_piclist_increment();
-        dwin->owner->image_loaded = true;
-    }
-
-    if (scaled) {
-        pic = gli_picture_scale(pic.get(), width, height);
-    } else {
-        pic = gli_picture_scale(pic.get(), gli_zoom_int(pic->w), gli_zoom_int(pic->h));
-    }
+    pic = gli_picture_scale(pic.get(), gli_zoom_int(width), gli_zoom_int(height));
 
     hyperlink = dwin->owner->attr.hyper;
 
@@ -1641,9 +1667,7 @@ bool win_textbuffer_draw_picture(window_textbuffer_t *dwin,
 
 void win_textbuffer_flow_break(window_textbuffer_t *dwin)
 {
-    while (dwin->ladjn != 0 || dwin->radjn != 0) {
-        win_textbuffer_putchar_uni(dwin->owner, '\n');
-    }
+    scrolloneline(dwin, false, true);
 }
 
 void win_textbuffer_click(window_textbuffer_t *dwin, int sx, int sy)
