@@ -10,9 +10,14 @@
 
 set -e
 
-while getopts "t" o
+frankendrift="OFF"
+
+while getopts "ft" o
 do
     case "${o}" in
+        f)
+            frankendrift="ON"
+            ;;
         t)
             NO_TADS_ICON=1
             ;;
@@ -35,7 +40,7 @@ PKG_DEB_ROOT=${DEB_ROOT_DIR}/${PKG_NAME}/debian
 #Build Gargoyle
 mkdir build-debian
 pushd build-debian
-cmake .. -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr
+cmake .. -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr -DQT_VERSION=5 -DWITH_FRANKENDRIFT="${frankendrift}"
 make -j$(nproc)
 make install DESTDIR=${PKG_DIR}
 popd
@@ -139,7 +144,7 @@ pushd ${DEB_ROOT_DIR}/${PKG_NAME}
 export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${PKG_DIR}/usr/lib/${PROJECT}
 
 #Determine dependencies and write out final control file.
-dpkg-shlibdeps ./debian/tmp/usr/libexec/${PROJECT}/* ./debian/tmp/usr/lib/*/libgarglk.so
+dpkg-shlibdeps ./debian/tmp/usr/libexec/${PROJECT}/* ./debian/tmp/usr/lib/*/libgarglk.so ./debian/tmp/usr/lib/*/libgarglk-gpl2.so
 dpkg-gencontrol -v${VERSION}
 
 #Remove the control template, copy the package contents to final packaging location, remove temp directories as needed.
