@@ -7,6 +7,21 @@
 
 set -e
 
+frankendrift="OFF"
+
+while getopts "f" o
+do
+    case "${o}" in
+        f)
+            frankendrift_spec="%{_libexecdir}/gargoyle/FrankenDrift.GlkRunner.Gargoyle"
+            frankendrift="ON"
+            ;;
+        *)
+            fatal "Usage: $0 [-f]"
+            ;;
+    esac
+done
+
 VERSION=$(<VERSION)
 
 # Holds the name of the root directory containing the necessary structure to
@@ -19,7 +34,7 @@ PKG_DIR=/tmp/${PKG_NAME}
 #Build Gargoyle
 mkdir build-rpm
 pushd build-rpm
-cmake .. -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr
+cmake .. -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr -DWITH_FRANKENDRIFT="${frankendrift}"
 make -j$(nproc)
 make install DESTDIR=${PKG_DIR}
 popd
@@ -67,7 +82,8 @@ cp -v garglk.ini "%{buildroot}%{_sysconfdir}/"
 %files
 %{_sysconfdir}/garglk.ini
 %{_libdir}/libgarglk.so
-%{_libdir}/libgarglkmain.a
+%{_libdir}/libgarglk-gpl2.so
+${frankendrift_spec}
 %{_libexecdir}/gargoyle/advsys
 %{_libexecdir}/gargoyle/magnetic
 %{_libexecdir}/gargoyle/agility
@@ -85,46 +101,44 @@ cp -v garglk.ini "%{buildroot}%{_sysconfdir}/"
 %{_libexecdir}/gargoyle/tadsr
 %{_libexecdir}/gargoyle/level9
 %{_bindir}/gargoyle
-%{_includedir}/garglk/gi_blorb.h
-%{_includedir}/garglk/glk.h
-%{_includedir}/garglk/glkstart.h
-%{_libdir}/pkgconfig/garglk.pc
 %{_datarootdir}/applications/io.github.garglk.Gargoyle.desktop
 %{_datarootdir}/applications/io.github.garglk.GargoyleEditConfig.desktop
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Mono-Bold-Italic.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Mono-Bold.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Mono-Italic.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Mono.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Serif-Bold-Italic.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Serif-Bold.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Serif-Italic.ttf
-%{_datarootdir}/fonts/gargoyle/Gargoyle-Serif.ttf
-%{_datarootdir}/io.github.garglk/Gargoyle/themes/Blue.json
-"%{_datarootdir}/io.github.garglk/Gargoyle/themes/Breeze Darker.json"
-"%{_datarootdir}/io.github.garglk/Gargoyle/themes/Lectrote Dark.json"
-"%{_datarootdir}/io.github.garglk/Gargoyle/themes/Lectrote Sepia.json"
-"%{_datarootdir}/io.github.garglk/Gargoyle/themes/Lectrote Slate.json"
-%{_datarootdir}/io.github.garglk/Gargoyle/themes/Pencil.json
-%{_datarootdir}/io.github.garglk/Gargoyle/themes/Zoom.json
-%{_datarootdir}/io.github.garglk/Gargoyle/themes/dark.json
-%{_datarootdir}/io.github.garglk/Gargoyle/themes/light.json
-%{_datarootdir}/io.github.garglk/Gargoyle/unifont.otf
-%{_datarootdir}/io.github.garglk/Gargoyle/unifont_upper.otf
+%{_datarootdir}/gargoyle/themes/Blue.json
+"%{_datarootdir}/gargoyle/themes/Breeze Darker.json"
+"%{_datarootdir}/gargoyle/themes/Lectrote Dark.json"
+"%{_datarootdir}/gargoyle/themes/Lectrote Sepia.json"
+"%{_datarootdir}/gargoyle/themes/Lectrote Slate.json"
+%{_datarootdir}/gargoyle/Gargoyle-Mono-Bold-Italic.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Mono-Bold.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Mono-Italic.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Mono.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Serif-Bold-Italic.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Serif-Bold.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Serif-Italic.ttf
+%{_datarootdir}/gargoyle/Gargoyle-Serif.ttf
+%{_datarootdir}/gargoyle/themes/Pencil.json
+%{_datarootdir}/gargoyle/themes/Zoom.json
+%{_datarootdir}/gargoyle/themes/dark.json
+%{_datarootdir}/gargoyle/themes/light.json
+%{_datarootdir}/gargoyle/unifont.otf
+%{_datarootdir}/gargoyle/unifont_upper.otf
 %{_datarootdir}/icons/io.github.garglk.Gargoyle.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-adrift.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-advsys.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-agt.png
-%{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-alan.png
+%{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-alan-adventure-game.png
+%{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-alan3-adventure-game.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-blorb.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-glulx.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-hugo-image.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-level9.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-magscroll.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-tads.png
+%{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-t3vm-image.png
 %{_datarootdir}/icons/hicolor/32x32/mimetypes/application-x-zmachine.png
 %{_datarootdir}/man/man6/gargoyle.6.gz
 %{_datarootdir}/metainfo/io.github.garglk.Gargoyle.appdata.xml
-%{_datarootdir}/mime/packages/interactive-fiction.xml
+%{_datarootdir}/mime/packages/gargoyle.xml
 
 %clean
 %if "%{clean}" != ""
