@@ -1,34 +1,109 @@
 #!/bin/bash
 
-set -e
+set -eu
 
-# XXX Temporary hack for a broken sdl2_mixer or readline package in Homebrew.
-# sdl2_mixer's pkg-config file requires readline, but the readline pkg-config
-# file isn't installed to /usr/local/lib/pkgconfig, meaning it can't be found.
-# I don't know the logic behind what gets symlinked into /usr/local/lib/pkgconfig
-# so I don't know who's at fault here, but the hacky way around it is to set
-# $PKG_CONFIG_PATH to point to readline's pkg-config directory. This only
-# happens if $PKG_CONFIG_PATH isn't already set.
-if ! pkg-config readline --exists; then
-    if [[ -z "${PKG_CONFIG_PATH}" ]]; then
-        export PKG_CONFIG_PATH="$(dirname $(find /usr/local/Cellar/readline -name readline.pc|tail -1))"
-    fi
-fi
+fatal() {
+    echo "${@}" >&2
+    exit 1
+}
+
+GARGOYLE_CLEAN=
+GARGOYLE_FRANKENDRIFT="OFF"
+GARGOYLE_NO_DMG=
+GARGOYLE_CMAKE_EXTRAS=""
+
+while getopts "cfn" o
+do
+    case "${o}" in
+        c)
+            GARGOYLE_CLEAN=1
+            ;;
+        f)
+            GARGOYLE_FRANKENDRIFT="ON"
+            ;;
+        n)
+            GARGOYLE_NO_DMG=1
+            ;;
+        *)
+            fatal "Usage: $0 [-f]"
+            ;;
+    esac
+done
 
 # Use Homebrew if available. Alternately, you could just set the variable to
 # either yes or no.
+MAC_USEHOMEBREW=${MAC_USEHOMEBREW:-}
 if [ "${MAC_USEHOMEBREW}" == "" ]; then
   MAC_USEHOMEBREW=no
   brew --prefix > /dev/null 2>&1 && MAC_USEHOMEBREW=yes
 fi
 
 if [ "${MAC_USEHOMEBREW}" == "yes" ]; then
+  command -v brew &> /dev/null || fatal "Homebrew requested but not found"
   HOMEBREW_OR_MACPORTS_LOCATION="$(brew --prefix)"
 else
-  HOMEBREW_OR_MACPORTS_LOCATION="$(pushd "$(dirname "$(which port)")/.." > /dev/null ; pwd -P ; popd > /dev/null)"
+  command -v port &> /dev/null || fatal "Neither Homebrew nor MacPorts is available"
+  HOMEBREW_OR_MACPORTS_LOCATION="$(cd "$(dirname "$(which port)")/.." && pwd)"
+fi
+
+HOST_ARCH="$(uname -m)"
+
+if [[ "${MAC_USEHOMEBREW}" == "yes" ]]
+then
+    echo "Probing Homebrew architecture..."
+    HOMEBREW_ARCH=$(brew config | grep "^macOS:" | cut -d "-" -f2)
+else
+    HOMEBREW_ARCH=""
+fi
+
+# If the Homebrew architecture in $PATH is not the same as the current
+# architecture, assume a cross compile. Cross compiling is currently only
+# supported on Homebrew, and only from arm64 to x86_64.
+case "${HOMEBREW_ARCH}" in
+    # Building for x86_64
+    "x86_64")
+        case "${HOST_ARCH}" in
+            x86_64)
+                ;;
+            arm64)
+                echo "Cross compiling to ${HOMEBREW_ARCH} from ${HOST_ARCH}"
+                GARGOYLE_CMAKE_EXTRAS="-DCMAKE_OSX_ARCHITECTURES=${HOMEBREW_ARCH} -DCMAKE_PREFIX_PATH=${HOMEBREW_OR_MACPORTS_LOCATION}"
+                ;;
+            *)
+                fatal "Don't know how to cross compile for ${TARGET_ARCH} from ${HOST_ARCH}"
+                ;;
+        esac
+
+        TARGET_ARCH="${HOMEBREW_ARCH}"
+        ;;
+
+    # Building for arm64
+    "arm64")
+        [[ "${HOST_ARCH}" != "arm64" ]] && fatal "Don't know how to cross compile to ${HOMEBREW_ARCH} from ${HOST_ARCH}"
+
+        TARGET_ARCH="${HOMEBREW_ARCH}"
+        ;;
+
+    # No support for cross compiling on MacPorts at the moment. Assume the
+    # current archutecture.
+    "")
+        TARGET_ARCH="${HOST_ARCH}"
+        ;;
+
+    *)
+        fatal "Unknown target architecture: ${HOMEBREW_ARCH}"
+        ;;
+esac
+
+# Ensure a sane environment (mainly be certain GNU programs aren't visible).
+export PATH="${HOMEBREW_OR_MACPORTS_LOCATION}/bin:/usr/bin:/bin:/usr/sbin"
+
+if [[ "$(sw_vers -productVersion)" =~ ^10\.([0-9]+) && ${BASH_REMATCH[1]} -lt 15 ]]; then
+  MACOS_MIN_VER="10.13"
+else
+  MACOS_MIN_VER="10.15"
 fi
 
-MACOS_MIN_VER="10.13"
 echo "MACOS_MIN_VER $MACOS_MIN_VER"
 
 # Use as many CPU cores as possible
@@ -37,7 +112,7 @@ NUMJOBS=$(sysctl -n hw.ncpu)
 GARGDIST=build/dist
 BUNDLE=Gargoyle.app/Contents
 
-GARVERSION=$(cat VERSION)
+GARVERSION=$(<VERSION)
 
 rm -rf Gargoyle.app
 mkdir -p "$BUNDLE/MacOS"
@@ -46,10 +121,12 @@ mkdir -p "$BUNDLE/Resources/Fonts"
 mkdir -p "$BUNDLE/Resources/themes"
 mkdir -p "$BUNDLE/PlugIns"
 
+[[ -n "${GARGOYLE_CLEAN}" ]] && rm -rf build-osx build/dist
+
 rm -rf $GARGDIST
 mkdir -p build-osx
 cd build-osx
-cmake .. -DBUILD_SHARED_LIBS=OFF -DCMAKE_OSX_DEPLOYMENT_TARGET=${MACOS_MIN_VER} -DDIST_INSTALL=ON -DCMAKE_BUILD_TYPE=Release -DCMAKE_FIND_FRAMEWORK=LAST -DCMAKE_EXPORT_COMPILE_COMMANDS=1
+cmake .. -DBUILD_SHARED_LIBS=ON -DCMAKE_OSX_DEPLOYMENT_TARGET=${MACOS_MIN_VER} -DDIST_INSTALL=ON -DCMAKE_BUILD_TYPE=Release -DCMAKE_FIND_FRAMEWORK=LAST -DCMAKE_EXPORT_COMPILE_COMMANDS=1 -DWITH_FRANKENDRIFT="${GARGOYLE_FRANKENDRIFT}" ${GARGOYLE_CMAKE_EXTRAS}
 make "-j${NUMJOBS}"
 make install
 cd -
@@ -116,7 +193,9 @@ do
   find "${GARGDIST}" -type f -name '*.dylib' -exec install_name_tool -change "@executable_path/$(basename "{}")" "@executable_path/../Frameworks/$(basename "{}")" "${file_path}" \;
 done
 
-# exit
+# Ensure interpreters can find libgarglk
+find Gargoyle.app/Contents/PlugIns/ -type f -exec install_name_tool -add_rpath '@executable_path/../Frameworks' {} \;
+install_name_tool -add_rpath '@executable_path/../Frameworks' "$BUNDLE/MacOS/Gargoyle"
 
 echo "Copying additional support files..."
 /usr/bin/sed -E -e "s/INSERT_VERSION_HERE/$GARVERSION/" garglk/launcher.plist > $BUNDLE/Info.plist
@@ -132,7 +211,10 @@ cp themes/*.json $BUNDLE/Resources/themes
 
 codesign -s - -f --deep Gargoyle.app
 
-echo "Creating DMG..."
-hdiutil create -fs "HFS+J" -ov -srcfolder Gargoyle.app/ "gargoyle-$GARVERSION-mac.dmg"
+if [[ -z "${GARGOYLE_NO_DMG}" ]]
+then
+    echo "Creating DMG..."
+    hdiutil create -fs "HFS+J" -ov -srcfolder Gargoyle.app/ "gargoyle-$GARVERSION-$TARGET_ARCH.dmg"
+fi
 
 echo "Done."
