@@ -125,7 +125,10 @@ void sys(fpos, len)
 
   getstr(fpos, len);            /* Returns address to string on stack */
   command = (char *)pop();
+  // Gargoyle will not allow games to run arbitrary programs.
+#ifndef GARGLK
   int tmp = system(command);
+#endif
   free(command);
 }
 
