@@ -5,7 +5,7 @@
 
 #include "types.h"
 
-void tokenize(uint16_t text, uint16_t parse, uint16_t dictaddr, bool flag);
+void tokenize(uint16_t text, uint16_t parse, uint16_t dictaddr, bool ignore_unknown);
 
 void ztokenise();
 void zencode_text();
