@@ -1,18 +1,6 @@
-// Copyright 2010-2021 Chris Spiegel.
+// Copyright 2010-2025 Chris Spiegel.
 //
-// This file is part of Bocfel.
-//
-// Bocfel is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License, version
-// 2 or 3, as published by the Free Software Foundation.
-//
-// Bocfel is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Bocfel. If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: MIT
 
 #include <climits>
 #include <cstdio>
@@ -35,10 +23,32 @@ extern "C" {
 #include "unicode.h"
 #include "util.h"
 
+#ifdef ZTERP_NO_STDIO
+#pragma message("warning: macro ZTERP_NO_STDIO is deprecated in favor of ZTERP_GLK_NO_STDIO")
+#define ZTERP_GLK_NO_STDIO
+#endif
+
+#ifdef ZTERP_GLK_NO_STDIO
+#ifndef ZTERP_GLK_UNIX
+#error ZTERP_GLK_NO_STDIO requires a Unix Glk
+#endif
+
+extern "C" {
+#include <glkstart.h>
+}
+
+#ifndef GLKUNIX_FILEREF_CREATE_UNCLEANED
+#error ZTERP_GLK_NO_STDIO requires the extension glkunix_fileref_create_by_name_uncleaned
+// Prototype so that usage of this function doesn’t cause a compile error.
+frefid_t glkunix_fileref_create_by_name_uncleaned(glui32 usage, const char *name, glui32 rock);
+#endif
+#endif
+
 // Due to C++’s less-than-ideal type system, there’s no way to guarantee
 // that an enum class actually contains a valid value. When checking the
 // value of the I/O object’s type, this method is used as a sort of
 // run-time type checker.
+[[noreturn]]
 void IO::bad_type() const {
     die("internal error: unknown IO type %d", static_cast<int>(m_type));
 }
@@ -66,10 +76,12 @@ bool IO::textmode() const {
 // prompt. This is a headache.
 //
 // Prompting is assumed to be necessary if “filename” is null.
-IO::IO(const std::string *filename, Mode mode, Purpose purpose) :
+IO::IO(const std::string *filename, Mode mode, Purpose purpose, StreamRock namedglkrock) :
     m_mode(mode),
     m_purpose(purpose)
 {
+
+#if !defined(ZTERP_GLK) || !defined(ZTERP_GLK_NO_STDIO)
     char smode[] = "wb";
 
     if (m_mode == Mode::ReadOnly) {
@@ -81,62 +93,38 @@ IO::IO(const std::string *filename, Mode mode, Purpose purpose) :
     if (textmode()) {
         smode[1] = 0;
     }
+#endif
 
     // No need to prompt.
     if (filename != nullptr) {
-        m_type = Type::StandardIO;
-        m_file = File(std::fopen(filename->c_str(), smode), true);
-        if (m_file.stdio == nullptr) {
-            throw OpenError();
+        // In no-stdio mode, always use Glk. Otherwise, if Glk is
+        // enabled, use Glk I/O if a rock is provided. If no rock is
+        // provided, use stdio.
+#ifdef ZTERP_GLK_NO_STDIO
+        open_as_glk([&filename](glui32 usage, glui32) {
+            return glkunix_fileref_create_by_name_uncleaned(usage, filename->c_str(), 0);
+        }, namedglkrock);
+#else
+#if defined(ZTERP_GLK)
+        if (namedglkrock != StreamRock::None) {
+            open_as_glk([&filename](glui32 usage, glui32 filemode) {
+                return glk_fileref_create_by_name(usage, const_cast<char *>(filename->c_str()), 0);
+            }, namedglkrock);
+        } else
+#endif
+        {
+            m_type = Type::StandardIO;
+            m_file = File(std::fopen(filename->c_str(), smode), true);
+            if (m_file.stdio == nullptr) {
+                throw OpenError();
+            }
         }
+#endif
     } else { // Prompt.
 #ifdef ZTERP_GLK
-        frefid_t ref;
-        glui32 usage, filemode;
-        usage = fileusage_BinaryMode;
-
-        switch (m_purpose) {
-        case Purpose::Data:
-            usage |= fileusage_Data;
-            break;
-        case Purpose::Save:
-            usage |= fileusage_SavedGame;
-            break;
-        case Purpose::Transcript:
-            usage |= fileusage_Transcript;
-            break;
-        case Purpose::Input:
-            usage |= fileusage_InputRecord;
-            break;
-        default:
-            throw OpenError();
-        }
-
-        switch (m_mode) {
-        case Mode::ReadOnly:
-            filemode = filemode_Read;
-            break;
-        case Mode::WriteOnly:
-            filemode = filemode_Write;
-            break;
-        case Mode::Append:
-            filemode = filemode_WriteAppend;
-            break;
-        default:
-            throw OpenError();
-        }
-
-        ref = glk_fileref_create_by_prompt(usage, filemode, 0);
-        if (ref == nullptr) {
-            throw OpenError();
-        }
-
-        m_type = Type::Glk;
-        m_file = File(glk_stream_open_file(ref, filemode, 0));
-        glk_fileref_destroy(ref);
-        if (m_file.glk == nullptr) {
-            throw OpenError();
-        }
+        open_as_glk([](glui32 usage, glui32 filemode) {
+            return glk_fileref_create_by_prompt(usage, filemode, 0);
+        }, namedglkrock);
 #else
         std::string fn, prompt;
 
@@ -171,6 +159,56 @@ IO::IO(const std::string *filename, Mode mode, Purpose purpose) :
     }
 }
 
+#ifdef ZTERP_GLK
+void IO::open_as_glk(const std::function<frefid_t(glui32 usage, glui32 filemode)> &create_fref, StreamRock rock)
+{
+    glui32 usage = fileusage_BinaryMode, filemode;
+
+    switch (m_purpose) {
+    case Purpose::Data:
+        usage |= fileusage_Data;
+        break;
+    case Purpose::Save:
+        usage |= fileusage_SavedGame;
+        break;
+    case Purpose::Transcript:
+        usage |= fileusage_Transcript;
+        break;
+    case Purpose::Input:
+        usage |= fileusage_InputRecord;
+        break;
+    default:
+        throw OpenError();
+    }
+
+    switch (m_mode) {
+    case Mode::ReadOnly:
+        filemode = filemode_Read;
+        break;
+    case Mode::WriteOnly:
+        filemode = filemode_Write;
+        break;
+    case Mode::Append:
+        filemode = filemode_WriteAppend;
+        break;
+    default:
+        throw OpenError();
+    }
+
+    frefid_t ref = create_fref(usage, filemode);
+    if (ref == nullptr) {
+        throw OpenError();
+    }
+
+    m_type = Type::Glk;
+    m_file = File(glk_stream_open_file(ref, filemode, static_cast<glui32>(rock)));
+    glk_fileref_destroy(ref);
+    if (m_file.glk == nullptr) {
+        throw OpenError();
+    }
+}
+#endif
+
 // Instead of being file-backed, indicate that this I/O object is
 // memory-backed. This allows internal save states (for @save_undo as
 // well as meta-saves created by /ps) to use Quetzal as their save
@@ -179,16 +217,15 @@ IO::IO(const std::string *filename, Mode mode, Purpose purpose) :
 // eliminates the need for code duplication.
 //
 // The I/O object starts out with the contents of the passed-in buffer,
-// which may be empty. The offset always starts at 0.
+// which may be empty.
 IO::IO(std::vector<uint8_t> buf, Mode mode) :
     m_file(File(std::move(buf))),
     m_type(Type::Memory),
     m_mode(mode),
     m_purpose(Purpose::Data)
 {
-    // Append isn’t used with memory-backed I/O, so it’s not supported.
-    if (m_mode != Mode::ReadOnly && m_mode != Mode::WriteOnly) {
-        throw OpenError();
+    if (mode == Mode::Append) {
+        m_file.backing.offset = m_file.backing.memory.size();
     }
 }
 
@@ -196,7 +233,7 @@ IO::IO(std::vector<uint8_t> buf, Mode mode) :
 // represents the state of the “file” at the time the function is
 // called. The reference is only valid until the next call to an I/O
 // method on this same I/O instance.
-const std::vector<uint8_t> &IO::get_memory()
+const std::vector<uint8_t> &IO::get_memory() const
 {
     if (m_type != Type::Memory) {
         throw std::runtime_error("not a memory object");
@@ -261,7 +298,7 @@ void IO::seek(long offset, SeekFrom whence)
     }
 }
 
-long IO::tell()
+long IO::tell() const
 {
     switch (m_type) {
     case Type::StandardIO: {
@@ -374,7 +411,7 @@ size_t IO::write(const void *buf, size_t n)
         Backing *b = &m_file.backing;
         auto remaining = b->memory.size() - b->offset;
 
-        if (m_mode != Mode::WriteOnly) {
+        if (m_mode != Mode::WriteOnly && m_mode != Mode::Append) {
             return 0;
         }
 
@@ -424,7 +461,7 @@ uint16_t IO::read16()
 
     read_exact(buf, sizeof buf);
 
-    return ((static_cast<uint16_t>(buf[0]) << 8)) | (static_cast<uint16_t>(buf[1]));
+    return (static_cast<uint16_t>(buf[0]) << 8) | (static_cast<uint16_t>(buf[1]));
 }
 
 uint32_t IO::read32()
@@ -441,7 +478,7 @@ uint32_t IO::read32()
 
 void IO::write8(uint8_t v)
 {
-    return write_exact(&v, sizeof v);
+    write_exact(&v, sizeof v);
 }
 
 void IO::write16(uint16_t v)
@@ -451,7 +488,7 @@ void IO::write16(uint16_t v)
     buf[0] = v >> 8;
     buf[1] = v & 0xff;
 
-    return write_exact(buf, sizeof buf);
+    write_exact(buf, sizeof buf);
 }
 
 void IO::write32(uint32_t v)
@@ -463,7 +500,7 @@ void IO::write32(uint32_t v)
     buf[2] = (v >>  8) & 0xff;
     buf[3] = (v >>  0) & 0xff;
 
-    return write_exact(buf, sizeof buf);
+    write_exact(buf, sizeof buf);
 }
 
 // getc() and putc() are meant to operate in terms of characters, not
@@ -481,7 +518,7 @@ long IO::getc(bool limit16)
 {
     long ret;
     uint8_t c;
-    struct NotUnicode : std::exception {};
+    class NotUnicode : public std::exception {};
 
     try {
         c = read8();
@@ -493,7 +530,7 @@ long IO::getc(bool limit16)
     auto read_byte = [this] {
         uint8_t b = read8();
 
-        if ((b & 0x80) != 0x80) {
+        if ((b & 0xc0) != 0x80) {
             throw NotUnicode();
         } else {
             return b;
@@ -506,15 +543,24 @@ long IO::getc(bool limit16)
         } else if ((c & 0xe0) == 0xc0) { // Two bytes.
             ret = (c & 0x1f) << 6;
             ret |= (read_byte() & 0x3f);
+            if (ret < 0x80) {
+                throw NotUnicode();
+            }
         } else if ((c & 0xf0) == 0xe0) { // Three bytes.
             ret = (c & 0x0f) << 12;
             ret |= (read_byte() & 0x3f) << 6;
             ret |= (read_byte() & 0x3f);
+            if (ret < 0x800) {
+                throw NotUnicode();
+            }
         } else if ((c & 0xf8) == 0xf0) { // Four bytes.
             ret = (static_cast<long>(c) & 0x07) << 18;
             ret |= (read_byte() & 0x3f) << 12;
             ret |= (read_byte() & 0x3f) << 6;
             ret |= (read_byte() & 0x3f);
+            if (ret < 0x10000) {
+                throw NotUnicode();
+            }
         } else { // Invalid value.
             ret = UNICODE_REPLACEMENT;
         }
@@ -524,7 +570,7 @@ long IO::getc(bool limit16)
         return UNICODE_REPLACEMENT;
     }
 
-    if (limit16 && ret > UINT16_MAX) {
+    if (ret > (limit16 ? UINT16_MAX : 0x10ffff)) {
         ret = UNICODE_REPLACEMENT;
     }
 
@@ -568,8 +614,9 @@ void IO::putc(uint32_t c)
 // characters terminated by a Unicode linefeed (0xa). If EOF is
 // encountered at any point (including after characters have been read,
 // but before a linefeed), EndOfFile is thrown, which means that all
-// lines, including the last one, must end in a linefeed. Any characters
-// read before the EOF can be considered lost and unrecoverable.
+// lines, including the last one, must end in a linefeed. If the last
+// line does not end in a linefeed, its characters will be read, but
+// ultimately discarded, so can be considered lost and unrecoverable.
 std::vector<uint16_t> IO::readline()
 {
     std::vector<uint16_t> result;
@@ -591,7 +638,7 @@ std::vector<uint16_t> IO::readline()
     return result;
 }
 
-long IO::filesize()
+long IO::filesize() const
 {
     switch (m_type) {
     case Type::StandardIO:
@@ -602,9 +649,16 @@ long IO::filesize()
     case Type::Memory:
         return m_file.backing.memory.size();
 #ifdef ZTERP_GLK
-    case Type::Glk:
-        break;
+    case Type::Glk: {
+        auto orig = glk_stream_get_position(m_file.glk.get());
+        glk_stream_set_position(m_file.glk.get(), 0, seekmode_End);
+        long size = glk_stream_get_position(m_file.glk.get());
+        glk_stream_set_position(m_file.glk.get(), orig, seekmode_Start);
+        return size;
+    }
 #endif
+    default:
+        bad_type();
     }
 
     return -1;
