@@ -1,22 +1,10 @@
-// Copyright 2013-2021 Chris Spiegel.
+// Copyright 2013-2024 Chris Spiegel.
 //
-// This file is part of Bocfel.
-//
-// Bocfel is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License, version
-// 2 or 3, as published by the Free Software Foundation.
-//
-// Bocfel is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Bocfel. If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: MIT
 
-#include <algorithm>
-#include <array>
 #include <cstring>
+#include <map>
+#include <memory>
 #include <new>
 #include <set>
 #include <sstream>
@@ -28,12 +16,14 @@
 #include "io.h"
 #include "memory.h"
 #include "objects.h"
+#include "options.h"
 #include "osdep.h"
 #include "process.h"
 #include "screen.h"
 #include "stack.h"
 #include "stash.h"
 #include "types.h"
+#include "unicode.h"
 #include "util.h"
 #include "zterp.h"
 
@@ -41,12 +31,12 @@ using namespace std::literals;
 
 static bool uni_isdigit(char c)
 {
-    return c > '0' && c < '9';
+    return c >= '0' && c <= '9';
 }
 
 static bool uni_isxdigit(char c)
 {
-    return "0123456789abcdefg"s.find(c) != std::string::npos;
+    return "0123456789abcdef"s.find(c) != std::string::npos;
 }
 
 static void try_user_save(const char *desc)
@@ -84,17 +74,12 @@ static void try_user_drop(size_t i)
     }
 }
 
-static void meta_status_putc(uint8_t c)
-{
-    screen_print(std::string{static_cast<char>(c)});
-}
-
 static std::vector<uint8_t> debug_change_memory;
 static std::set<uint16_t> debug_change_invalid;
 
 static void meta_debug_change_start()
 {
-    debug_change_memory.assign(memory, memory + header.static_start);
+    debug_change_memory.assign(memory.begin(), memory.begin() + header.static_start);
     debug_change_invalid.clear();
     screen_puts("[Debug change reset]");
 }
@@ -133,7 +118,7 @@ static void meta_debug_change_inc_dec(const std::string &string)
             screen_puts("[No changes]");
         }
 
-        std::memcpy(debug_change_memory.data(), memory, header.static_start);
+        std::copy(memory.begin(), memory.begin() + header.static_start, debug_change_memory.begin());
     }
 }
 
@@ -158,7 +143,7 @@ static bool meta_debug_scan(const std::string &string)
         invalid_addr.clear();
         screen_puts("[Debug scan reset]");
     } else if (string == "show") {
-        for (size_t addr = 0; addr < header.static_start - 2; addr++) {
+        for (uint16_t addr = 0; addr < header.static_start - 2; addr++) {
             if (invalid_addr.find(addr) == invalid_addr.end()) {
                 if (is_global(addr) || !in_globals(addr)) {
                     screen_puts(addrstring(addr));
@@ -166,10 +151,8 @@ static bool meta_debug_scan(const std::string &string)
             }
         }
     } else if (uni_isdigit(string[0]) || (string[0] == '-' && uni_isdigit(string[1]))) {
-        long value;
         bool valid;
-
-        value = parseint(string, 0, valid);
+        long value = parseint(string, 0, valid);
         if (!valid) {
             return false;
         }
@@ -183,7 +166,7 @@ static bool meta_debug_scan(const std::string &string)
                 value += 0x10000;
             }
 
-            for (size_t addr = 0; addr < header.static_start - 2; addr++) {
+            for (uint16_t addr = 0; addr < header.static_start - 2; addr++) {
                 if (word(addr) != value) {
                     invalid_addr.insert(addr);
                 }
@@ -232,10 +215,8 @@ static bool validate_address(long addr, bool print)
 
 static bool meta_debug_print(const std::string &string)
 {
-    long addr;
     bool valid;
-
-    addr = parse_address(string, valid);
+    long addr = parse_address(string, valid);
 
     if (!valid) {
         return false;
@@ -244,29 +225,28 @@ static bool meta_debug_print(const std::string &string)
         return true;
     }
 
-    screen_printf("%ld (0x%lx)\n", static_cast<long>(as_signed(word(addr))), static_cast<unsigned long>(word(addr)));
+    screen_printf("%ld (0x%04lx)\n", static_cast<long>(as_signed(word(addr))), static_cast<unsigned long>(word(addr)));
 
     return true;
 }
 
 #ifndef ZTERP_NO_CHEAT
-// This is an array of pairs (frozen, value), where the specified
-// address is frozen to “value” if “frozen” is true.
-static std::array<std::pair<bool, uint16_t>, UINT16_MAX + 1> freeze;
+// Map addresses to frozen values.
+static std::map<uint16_t, uint16_t> frozen_addresses;
 
-bool cheat_add(std::string how, bool print)
+bool cheat_add(const std::string &how, bool print)
 {
-    char *type, *addrstr, *valstr;
+    std::istringstream ss(how);
+    std::string type, addrstr, valstr;
 
-    type = std::strtok(&how[0], ":");
-    addrstr = std::strtok(nullptr, ":");
-    valstr = std::strtok(nullptr, "");
-
-    if (type == nullptr || addrstr == nullptr || valstr == nullptr) {
+    if (!std::getline(ss, type, ':') ||
+        !std::getline(ss, addrstr, ':') ||
+        !std::getline(ss, valstr))
+    {
         return false;
     }
 
-    if (std::strcmp(type, "freeze") == 0 || std::strcmp(type, "freezew") == 0) {
+    if (type == "freeze" || type == "freezew") {
         long addr;
         long value;
         bool valid;
@@ -291,7 +271,7 @@ bool cheat_add(std::string how, bool print)
             return true;
         }
 
-        freeze[addr] = {true, value};
+        frozen_addresses[addr] = value;
     } else {
         return false;
     }
@@ -305,49 +285,41 @@ bool cheat_add(std::string how, bool print)
 
 static bool cheat_remove(uint16_t addr)
 {
-    if (!freeze[addr].first) {
-        return false;
-    }
-
-    freeze[addr] = {false, 0};
-    return true;
+    return frozen_addresses.erase(addr) > 0;
 }
 
 bool cheat_find_freeze(uint32_t addr, uint16_t &val)
 {
-    if (addr > UINT16_MAX || !freeze[addr].first) {
+    auto found = frozen_addresses.find(addr);
+    if (found == frozen_addresses.end()) {
         return false;
     }
 
-    val = freeze[addr].second;
+    val = found->second;
 
     return true;
 }
 
-static bool meta_debug_freeze(std::string string)
+static bool meta_debug_freeze(const std::string &string)
 {
-    char *addrstr, *valstr;
+    std::istringstream ss(string);
+    std::string addrstr, valstr;
 
-    addrstr = std::strtok(&string[0], " ");
-    if (addrstr == nullptr) {
-        return false;
-    }
-    valstr = std::strtok(nullptr, "");
-    if (valstr == nullptr) {
+    if (!std::getline(ss, addrstr, ' ') ||
+        !std::getline(ss, valstr))
+    {
         return false;
     }
 
-    std::string cheat = "freeze:"s + addrstr + ":" + valstr;
+    std::string cheat = "freeze:" + addrstr + ":" + valstr;
 
     return cheat_add(cheat, true);
 }
 
 static bool meta_debug_unfreeze(const std::string &string)
 {
-    long addr;
     bool valid;
-
-    addr = parse_address(string, valid);
+    long addr = parse_address(string, valid);
     if (!valid) {
         return false;
     }
@@ -366,17 +338,12 @@ static bool meta_debug_unfreeze(const std::string &string)
 
 static bool meta_debug_show_freeze()
 {
-    bool any_frozen = false;
-
-    for (size_t addr = 0; addr < freeze.size(); addr++) {
-        if (freeze[addr].first) {
-            any_frozen = true;
-            screen_printf("%s: %lu\n", addrstring(addr).c_str(), static_cast<unsigned long>(freeze[addr].second));
-        }
-    }
-
-    if (!any_frozen) {
+    if (frozen_addresses.empty()) {
         screen_puts("[No frozen values]");
+    } else {
+        for (const auto &pair : frozen_addresses) {
+            screen_printf("%s: %lu\n", addrstring(pair.first).c_str(), static_cast<unsigned long>(pair.second));
+        }
     }
 
     return true;
@@ -384,36 +351,33 @@ static bool meta_debug_show_freeze()
 #endif
 
 #ifndef ZTERP_NO_WATCHPOINTS
-static std::array<bool, UINT16_MAX + 1> watch_addresses;
+static std::set<uint16_t> watch_addresses;
 
 static void watch_add(uint16_t addr)
 {
-    watch_addresses[addr] = true;
+    watch_addresses.insert(addr);
 }
 
 static void watch_all()
 {
-    watch_addresses.fill(true);
+    for (unsigned long addr = 0; addr < UINT16_MAX + 1UL; addr++) {
+        watch_addresses.insert(addr);
+    }
 }
 
 static bool watch_remove(uint16_t addr)
 {
-    if (watch_addresses[addr]) {
-        watch_addresses[addr] = false;
-        return true;
-    } else {
-        return false;
-    }
+    return watch_addresses.erase(addr) == 1;
 }
 
 static void watch_none()
 {
-    watch_addresses.fill(false);
+    watch_addresses.clear();
 }
 
 void watch_check(uint16_t addr, unsigned long oldval, unsigned long newval)
 {
-    if (watch_addresses[addr] && oldval != newval) {
+    if (watch_addresses.find(addr) != watch_addresses.end() && oldval != newval) {
         screen_printf("[%s changed: %lu -> %lu (pc = 0x%lx)]\n", addrstring(addr).c_str(), oldval, newval, current_instruction);
     }
 }
@@ -429,10 +393,8 @@ static bool meta_debug_watch_helper(const std::string &string, bool do_watch)
             screen_puts("[Not watching any addresses for changes]");
         }
     } else {
-        long addr;
         bool valid;
-
-        addr = parse_address(string, valid);
+        long addr = parse_address(string, valid);
 
         if (!valid) {
             return false;
@@ -468,19 +430,14 @@ static bool meta_debug_unwatch(const std::string &string)
 
 static bool meta_debug_show_watch()
 {
-    bool any_watched = false;
-
-    for (size_t addr = 0; addr < watch_addresses.size(); addr++) {
-        if (watch_addresses[addr]) {
-            any_watched = true;
+    if (watch_addresses.empty()) {
+        screen_puts("[No watched values]");
+    } else {
+        for (const auto &addr : watch_addresses) {
             screen_puts(addrstring(addr));
         }
     }
 
-    if (!any_watched) {
-        screen_puts("[No watched values]");
-    }
-
     return true;
 }
 #endif
@@ -505,7 +462,7 @@ static void meta_debug_help()
             "watch [address]: report any changes to the word at [address]\n"
             "watch all: report any changes to words at all addresses\n"
             "unwatch [address]: stop watching [address]\n"
-            "unwatch all: stop watching all adddresses\n"
+            "unwatch all: stop watching all addresses\n"
             "show_watch: show all watched-for addresses\n"
 #endif
             "\nAddresses are either absolute, specified in hexadecimal with an optional leading 0x, or global variables. Global variables have the syntax Gxx, where xx is a hexadecimal value in the range [00, ef], corresponding to global variables 0 to 239.\n"
@@ -552,6 +509,13 @@ static void meta_debug(const std::string &string)
     }
 }
 
+static void meta_info()
+{
+    screen_printf("File: %s\n", game_file.c_str());
+    screen_printf("Z-machine version: %d\n", memory[0]);
+    screen_printf("ID: %s\n", get_story_id().c_str());
+}
+
 static std::vector<char> meta_notes;
 
 IFF::TypeID meta_write_bfnt(IO &savefile)
@@ -563,7 +527,7 @@ IFF::TypeID meta_write_bfnt(IO &savefile)
     savefile.write32(0); // Version
     savefile.write_exact(meta_notes.data(), meta_notes.size());
 
-    return IFF::TypeID(&"Bfnt");
+    return IFF::TypeID("Bfnt");
 }
 
 void meta_read_bfnt(IO &io, uint32_t size)
@@ -609,26 +573,25 @@ void meta_read_bfnt(IO &io, uint32_t size)
     }
 }
 
-static std::vector<char> meta_notes_backup;
+class NotesStasher : public Stasher {
+public:
+    void backup() override {
+        m_notes = meta_notes;
+    }
 
-static void notes_stash_backup()
-{
-    meta_notes_backup = meta_notes;
+    bool restore() override {
+        meta_notes = m_notes;
 
-    meta_notes.clear();
-}
+        return true;
+    }
 
-static bool notes_stash_restore()
-{
-    meta_notes = meta_notes_backup;
+    void free() override {
+        m_notes.clear();
+    }
 
-    return true;
-}
-
-static void notes_stash_free()
-{
-    meta_notes_backup.clear();
-}
+private:
+    std::vector<char> m_notes;
+};
 
 // Try to parse a meta command. If input should be re-requested, return
 // <MetaResult::Rerequest, "">. If a portion of the passed-in string
@@ -654,7 +617,7 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
             }
         }
 
-        auto buf = io.get_memory();
+        const auto &buf = io.get_memory();
         converted = rtrim(std::string(buf.begin(), buf.end()));
     } catch (const IO::OpenError &) {
         return {MetaResult::Rerequest, ""};
@@ -686,13 +649,13 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
             screen_puts("[No save found, unable to undo]");
         }
     } else if ZEROARG("scripton") {
-        if (output_stream(OSTREAM_SCRIPT, 0)) {
+        if (output_stream(OSTREAM_TRANSCRIPT, 0)) {
             screen_puts("[Transcripting on]");
         } else {
             screen_puts("[Transcripting failed]");
         }
     } else if ZEROARG("scriptoff") {
-        output_stream(-OSTREAM_SCRIPT, 0);
+        output_stream(-OSTREAM_TRANSCRIPT, 0);
         screen_puts("[Transcripting off]");
     } else if ZEROARG("recon") {
         if (output_stream(OSTREAM_RECORD, 0)) {
@@ -745,10 +708,8 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
         } else if (rest[0] == 0) {
             restore_or_drop(0);
         } else {
-            long saveno;
             bool valid;
-
-            saveno = parseint(rest, 10, valid);
+            long saveno = parseint(rest, 10, valid);
             if (!valid || saveno < 1) {
                 screen_puts("[Invalid index]");
                 return {MetaResult::Rerequest, ""};
@@ -757,9 +718,11 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
             restore_or_drop(saveno - 1);
         }
     } else if ZEROARG("ls") {
-        list_saves(SaveStackType::User, screen_puts);
+        list_saves(SaveStackType::User);
     } else if ZEROARG("savetranscript") {
         screen_save_persistent_transcript();
+    } else if ZEROARG("showtranscript") {
+        screen_show_persistent_transcript();
     } else if ZEROARG("notes") {
         screen_puts("[Editing notes]");
         screen_flush();
@@ -773,7 +736,9 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
         if (meta_notes.empty()) {
             screen_puts("[No notes taken]");
         } else {
-            screen_printf("[Start of notes]\n%.*s\n[End of notes]\n", static_cast<int>(meta_notes.size()), meta_notes.data());
+            screen_puts("[Start of notes]");
+            screen_puts(std::string(meta_notes.data(), meta_notes.size()));
+            screen_puts("[End of notes]");
         }
     } else if ZEROARG("savenotes") {
         if (meta_notes.empty()) {
@@ -797,7 +762,9 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
         } else {
             long first = as_signed(variable(0x11)), second = as_signed(variable(0x12));
 
-            print_object(variable(0x10), meta_status_putc);
+            print_object(variable(0x10), [](uint8_t c) {
+                screen_putc(zscii_to_unicode[c]);
+            });
 
             screen_print("\n");
 
@@ -817,7 +784,7 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
         screen_puts("[Meta commands disabled]");
     } else if (command == "say") {
         return {MetaResult::Say, rest};
-    } else if (command == "config") {
+    } else if ZEROARG("config") {
         auto config_file = zterp_os_rcfile(true);
         if (config_file != nullptr) {
             screen_puts("[Editing configuration file]");
@@ -831,10 +798,18 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
         } else {
             screen_puts("[Cannot determine configuration file location]");
         }
+    } else if ZEROARG("fixed") {
+        if (screen_toggle_force_fixed()) {
+            screen_puts("[Force fixed font on]");
+        } else {
+            screen_puts("[Force fixed font off]");
+        }
     } else if (command == "debug") {
         meta_debug(rest);
     } else if ZEROARG("quit") {
         zquit();
+    } else if ZEROARG("info") {
+        meta_info();
     } else {
         if (command != "help") {
             screen_printf("Unknown command: /%s\n\n", command.c_str());
@@ -866,8 +841,10 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
                 "/disable: disable these commands for the rest of this session\n"
                 "/say [command]: pretend like [command] was typed\n"
                 "/config: open the config file in a text editor\n"
+                "/fixed: toggle forced fixed-width font in the main window\n"
                 "/debug [...]: perform a debugging operation\n"
                 "/quit: quit immediately (as if the @quit opcode were executed)\n"
+                "/info: display some info about the current game\n"
                 );
     }
 
@@ -877,6 +854,6 @@ std::pair<MetaResult, std::string> handle_meta_command(const uint16_t *string, u
 void init_meta(bool first_run)
 {
     if (first_run) {
-        stash_register(notes_stash_backup, notes_stash_restore, notes_stash_free);
+        stash_register(std::make_unique<NotesStasher>());
     }
 }
