@@ -1,20 +1,7 @@
-// Copyright 2010-2021 Chris Spiegel.
+// Copyright 2010-2025 Chris Spiegel.
 //
-// This file is part of Bocfel.
-//
-// Bocfel is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License, version
-// 2 or 3, as published by the Free Software Foundation.
-//
-// Bocfel is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Bocfel. If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: MIT
 
-#include <cerrno>
 #include <climits>
 #include <cstdio>
 #include <cstdlib>
@@ -23,6 +10,7 @@
 #include <iomanip>
 #include <iostream>
 #include <memory>
+#include <random>
 #include <sstream>
 #include <stdexcept>
 #include <thread>
@@ -31,17 +19,22 @@
 
 #ifdef ZTERP_GLK
 extern "C" {
-#include "glk.h"
+#include <glk.h>
 }
 #endif
 
 #include "io.h"
+#include "options.h"
 #include "osdep.h"
 #include "screen.h"
 #include "types.h"
 #include "util.h"
 #include "zterp.h"
 
+#if defined(ZTERP_UNIX) || defined(ZTERP_WIN32) || defined(ZTERP_DOS)
+#error The operating system macros have been renamed to have a ZTERP_OS prefix.
+#endif
+
 using namespace std::literals;
 
 // OS-specific functions should all be collected in this file for
@@ -82,14 +75,16 @@ using namespace std::literals;
 //
 // The Z-machine allow games to save and load arbitrary files. Bocfel
 // confines these files to a single (per-game) directory to avoid games
-// overwriting unrelated files. This function, given such an arbitrary
-// filename, returns a pointer to a string containing a full path to a
-// file which represents the passed-in filename. The file need not
-// exist, but its containing directory must. A null pointer is returned
-// on failure. The file must not escape its containing directory, which
-// may require platform-specific methods, but at the very least means
-// that a file containing directory separators (e.g. '/' on Unix) must
-// be rejected.
+// overwriting unrelated files. This function, given a filename, returns
+// a pointer to a string containing a full path to a file which
+// represents the passed-in filename. The file need not exist, but its
+// containing directory must. A null pointer is returned on failure. The
+// file must not escape its containing directory, which may require
+// platform-specific methods, but at the very least means that a file
+// containing directory separators (e.g. '/' on Unix) must be rejected
+// or sanitized. The incoming filename is guaranteed to consist of ASCII
+// printable characters, i.e. in the range 32-126, but may need to be
+// further sanitized depending on the needs of the target platform.
 //
 // void zterp_os_edit_file(const std::string &filename)
 //
@@ -103,6 +98,12 @@ using namespace std::literals;
 // return the contents of the saved file. On failure, std::runtime_error
 // is thrown.
 //
+// void zterp_os_show_transcript(const std::vector<char> &transcript)
+//
+// Identical to zterp_os_edit_notes, except nothing is returned. This is
+// meant to show the persistent transcript in the user’s editor, so it
+// can be easily read, searched, etc.
+//
 // The following functions are useful for non-Glk builds only. They
 // provide for some handling of screen functions that is normally taken
 // care of by Glk.
@@ -140,40 +141,41 @@ using namespace std::literals;
 // ╔══════════════════════════════════════════════════════════════════════════════╗
 // ║ Shared functions                                                             ║
 // ╚══════════════════════════════════════════════════════════════════════════════╝
-#if (defined(ZTERP_WIN32) || defined(ZTERP_DOS)) && !defined(ZTERP_GLK)
+#if (defined(ZTERP_OS_WIN32) || defined(ZTERP_OS_DOS)) && !defined(ZTERP_GLK)
 static void ansi_set_style(const Style &style, const Color &fg, const Color &bg)
 {
-    std::printf("\33[0m");
+    std::cout << "\33[0";
 
     if (style.test(STYLE_ITALIC)) {
-        std::printf("\33[4m");
+        std::cout << ";4";
     }
     if (style.test(STYLE_REVERSE)) {
-        std::printf("\33[7m");
+        std::cout << ";7";
     }
     if (style.test(STYLE_BOLD)) {
-        std::printf("\33[1m");
+        std::cout << ";1";
     }
 
     if (fg.mode == Color::Mode::ANSI) {
-        std::printf("\33[%dm", 28 + fg.value);
+        std::cout << ";" << (28 + fg.value);
     }
     if (bg.mode == Color::Mode::ANSI) {
-        std::printf("\33[%dm", 38 + bg.value);
+        std::cout << ";" << (38 + bg.value);
     }
+
+    std::cout << "m";
 }
 #endif
 
-#if defined(ZTERP_UNIX) || defined(ZTERP_WIN32)
+#if defined(ZTERP_OS_UNIX) || defined(ZTERP_OS_WIN32)
 static std::vector<char> read_file(const std::string &filename)
 {
     std::vector<char> new_file;
 
     try {
         IO io(&filename, IO::Mode::ReadOnly, IO::Purpose::Data);
-        long size;
 
-        size = io.filesize();
+        long size = io.filesize();
         if (size == -1) {
             throw std::exception();
         }
@@ -195,10 +197,7 @@ static std::vector<char> read_file(const std::string &filename)
 // ╔══════════════════════════════════════════════════════════════════════════════╗
 // ║ Unix functions                                                               ║
 // ╚══════════════════════════════════════════════════════════════════════════════╝
-#ifdef ZTERP_UNIX
-#include <cstring>
-#include <vector>
-
+#ifdef ZTERP_OS_UNIX
 #include <spawn.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -211,7 +210,7 @@ static std::vector<char> read_file(const std::string &filename)
 
 static std::string unique_name()
 {
-    auto slash = game_file.find_last_of('/');
+    auto slash = game_file.rfind('/');
     std::string basename;
 
     if (slash != std::string::npos) {
@@ -266,23 +265,20 @@ std::unique_ptr<std::string> zterp_os_rcfile(bool create_parent)
 
     config_file = std::make_unique<std::string>(std::string(settings_dir) + "/bocfel/bocfelrc");
 #else
-    const char *home;
-    const char *config_home;
-
-    home = std::getenv("HOME");
+    auto home = zterp_getenv("HOME");
     if (home != nullptr) {
         // This is the legacy location of the config file.
-        auto s = std::string(home) + "/.bocfelrc";
+        auto s = *home + "/.bocfelrc";
         if (access(s.c_str(), R_OK) == 0) {
             return std::make_unique<std::string>(s);
         }
     }
 
-    config_home = std::getenv("XDG_CONFIG_HOME");
-    if (config_home != nullptr && config_home[0] == '/') {
-        config_file = std::make_unique<std::string>(std::string(config_home) + "/bocfel/bocfelrc");
+    auto config_home = zterp_getenv("XDG_CONFIG_HOME");
+    if (config_home != nullptr && config_home->find('/') == 0) {
+        config_file = std::make_unique<std::string>(*config_home + "/bocfel/bocfelrc");
     } else if (home != nullptr) {
-        config_file = std::make_unique<std::string>(std::string(home) + "/.config/bocfel/bocfelrc");
+        config_file = std::make_unique<std::string>(*home + "/.config/bocfel/bocfelrc");
     } else {
         return nullptr;
     }
@@ -307,18 +303,17 @@ static std::unique_ptr<std::string> data_file(const std::string &filename)
 
     auto name = std::make_unique<std::string>(std::string(settings_dir) + "/bocfel/" + filename);
 #else
-    const char *data_home;
     std::unique_ptr<std::string> name;
 
-    data_home = std::getenv("XDG_DATA_HOME");
-    if (data_home != nullptr && data_home[0] == '/') {
-        name = std::make_unique<std::string>(std::string(data_home) + "/bocfel/" + filename);
+    auto data_home = zterp_getenv("XDG_DATA_HOME");
+    if (data_home != nullptr && (*data_home)[0] == '/') {
+        name = std::make_unique<std::string>(*data_home + "/bocfel/" + filename);
     } else {
-        char *home = std::getenv("HOME");
+        auto home = zterp_getenv("HOME");
         if (home == nullptr) {
             return nullptr;
         }
-        name = std::make_unique<std::string>(std::string(home) + "/.local/share/bocfel/" + filename);
+        name = std::make_unique<std::string>(*home + "/.local/share/bocfel/" + filename);
     }
 #endif
 
@@ -331,16 +326,26 @@ static std::unique_ptr<std::string> data_file(const std::string &filename)
 
 std::unique_ptr<std::string> zterp_os_autosave_name()
 {
-    std::string filename = "autosave/"s + unique_name();
-
-    return data_file(filename);
+    if (options.autosave_directory != nullptr) {
+        std::string filename = *options.autosave_directory + "/"s + unique_name();
+        if (!mkdir_p(filename)) {
+            return nullptr;
+        }
+        return std::make_unique<std::string>(filename);
+    } else {
+        std::string filename = "autosave/"s + unique_name();
+        return data_file(filename);
+    }
 }
 #define have_zterp_os_autosave_name
 
-std::unique_ptr<std::string> zterp_os_aux_file(const std::string &filename)
+std::unique_ptr<std::string> zterp_os_aux_file(const std::string &filename_)
 {
-    if (filename.find('/') != std::string::npos) {
-        return nullptr;
+    std::string filename = filename_;
+    for (auto &c : filename) {
+        if (c == '/') {
+            c = '_';
+        }
     }
 
     return data_file("auxiliary/"s + unique_name() + "/" + filename);
@@ -427,7 +432,7 @@ void zterp_os_edit_file(const std::string &filename)
 
 #ifdef ZTERP_GLK_TICK
     while (waitpid(pid, &status, WNOHANG) != pid) {
-        std::this_thread::sleep_for(100ms);
+        std::this_thread::sleep_for(10ms);
         glk_tick();
     }
 #else
@@ -445,26 +450,21 @@ void zterp_os_edit_file(const std::string &filename)
 #endif
 
     if (WEXITSTATUS(status) != 0) {
-        std::ostringstream errstr;
-
-        errstr << "editor had exit status " << WEXITSTATUS(status);
-        throw std::runtime_error(errstr.str());
+        throw std::runtime_error("editor had exit status " + std::to_string(WEXITSTATUS(status)));
     }
 }
 #define have_zterp_os_edit_file
 
-class TempFD {
+class TempFile {
 public:
-    explicit TempFD(const std::string &tmpl)
+    explicit TempFile(const std::string &tmpl)
     {
-        const char *tmpdir;
-
-        tmpdir = std::getenv("TMPDIR");
+        auto tmpdir = zterp_getenv("TMPDIR");
         if (tmpdir == nullptr) {
-            tmpdir = "/tmp";
+            tmpdir = std::make_unique<std::string>("/tmp");
         }
 
-        m_path = std::string(tmpdir) + "/" + tmpl + ".XXXXXX";
+        m_path = *tmpdir + "/" + tmpl + ".XXXXXX";
 
         m_fd = mkstemp(&m_path[0]);
         if (m_fd == -1) {
@@ -472,16 +472,23 @@ public:
         }
     }
 
-    TempFD(const TempFD &) = delete;
-    TempFD &operator=(const TempFD &) = delete;
+    TempFile(const TempFile &) = delete;
+    TempFile &operator=(const TempFile &) = delete;
 
-    ~TempFD() {
-        close(m_fd);
-        std::remove(m_path.c_str());
+    void write(const std::vector<char> &data) const {
+        size_t bytes = 0;
+        while (bytes < data.size()) {
+            ssize_t n = ::write(m_fd, &data[bytes], data.size() - bytes);
+            if (n == -1) {
+                throw std::runtime_error("unable to write to temporary file");
+            }
+            bytes += n;
+        }
     }
 
-    int fd() const {
-        return m_fd;
+    ~TempFile() {
+        close(m_fd);
+        std::remove(m_path.c_str());
     }
 
     const std::string &path() const {
@@ -495,23 +502,24 @@ private:
 
 std::vector<char> zterp_os_edit_notes(const std::vector<char> &notes)
 {
-    size_t bytes = 0;
-    TempFD tempfd("bocfel.notes");
+    TempFile tempfile("bocfel.notes");
 
-    while (bytes < notes.size()) {
-        ssize_t n = write(tempfd.fd(), &notes[bytes], notes.size() - bytes);
-        if (n == -1) {
-            throw std::runtime_error("unable to write to temporary file");
-        }
-        bytes += n;
-    }
+    tempfile.write(notes);
+    zterp_os_edit_file(tempfile.path());
 
-    zterp_os_edit_file(tempfd.path());
-
-    return read_file(tempfd.path());
+    return read_file(tempfile.path());
 }
 #define have_zterp_os_edit_notes
 
+void zterp_os_show_transcript(const std::vector<char> &transcript)
+{
+    TempFile tempfile("bocfel.transcript");
+
+    tempfile.write(transcript);
+    zterp_os_edit_file(tempfile.path());
+}
+#define have_zterp_os_show_transcript
+
 #ifndef ZTERP_GLK
 
 #ifndef ZTERP_NO_CURSES
@@ -532,8 +540,18 @@ std::pair<unsigned int, unsigned int> zterp_os_get_screen_size()
 #define have_zterp_os_get_screen_size
 #endif
 
+static const char *capstr(const char *cap)
+{
+    char *ret = tigetstr(const_cast<char *>(cap));
+    if (ret == nullptr || ret == reinterpret_cast<char *>(-1)) {
+        return nullptr;
+    }
+
+    return ret;
+}
+
 static const char *ital = nullptr, *rev = nullptr, *bold = nullptr, *none = nullptr;
-static char *fg_string = nullptr, *bg_string = nullptr;
+static const char *fg_string = nullptr, *bg_string = nullptr;
 static bool have_colors = false;
 static bool have_24bit_rgb = false;
 void zterp_os_init_term()
@@ -543,16 +561,16 @@ void zterp_os_init_term()
     }
 
     // prefer italics over underline for emphasized text
-    ital = tgetstr(const_cast<char *>("ZH"), nullptr);
+    ital = capstr("sitm");
     if (ital == nullptr) {
-        ital = tgetstr(const_cast<char *>("us"), nullptr);
+        ital = capstr("smul");
     }
-    rev  = tgetstr(const_cast<char *>("mr"), nullptr);
-    bold = tgetstr(const_cast<char *>("md"), nullptr);
-    none = tgetstr(const_cast<char *>("me"), nullptr);
+    rev  = capstr("rev");
+    bold = capstr("bold");
+    none = capstr("sgr0");
 
-    fg_string = tgetstr(const_cast<char *>("AF"), nullptr);
-    bg_string = tgetstr(const_cast<char *>("AB"), nullptr);
+    fg_string = capstr("setaf");
+    bg_string = capstr("setab");
 
     have_colors = none != nullptr && fg_string != nullptr && bg_string != nullptr;
 
@@ -597,7 +615,7 @@ static void set_color(const char *string, const Color &color)
     switch (color.mode) {
     case Color::Mode::ANSI:
         if (color.value >= 2 && color.value <= 9) {
-            putp(tparm(const_cast<char *>(string), color.value - 2));
+            putp(tparm(const_cast<char *>(string), color.value - 2, 0, 0, 0, 0, 0, 0, 0, 0));
         }
         break;
     case Color::Mode::True:
@@ -611,7 +629,7 @@ static void set_color(const char *string, const Color &color)
             uint16_t transformed = color.value < 4 ? 0 :
                                    color.value < 8 ? 8 :
                                    color.value;
-            putp(tparm(const_cast<char *>(string), screen_convert_color(transformed)));
+            putp(tparm(const_cast<char *>(string), screen_convert_color(transformed), 0, 0, 0, 0, 0, 0, 0, 0));
         }
         break;
     }
@@ -649,7 +667,7 @@ void zterp_os_set_style(const Style &style, const Color &fg, const Color &bg)
 // ╔══════════════════════════════════════════════════════════════════════════════╗
 // ║ Windows functions                                                            ║
 // ╚══════════════════════════════════════════════════════════════════════════════╝
-#elif defined(ZTERP_WIN32)
+#elif defined(ZTERP_OS_WIN32)
 #include <direct.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -659,6 +677,10 @@ void zterp_os_set_style(const Style &style, const Color &fg, const Color &bg)
 #define ENABLE_VIRTUAL_TERMINAL_PROCESSING 0x0004
 #endif
 
+#ifdef _MSC_VER
+#pragma comment(lib, "shell32.lib")
+#endif
+
 long zterp_os_filesize(std::FILE *fp)
 {
     struct _stat st;
@@ -687,7 +709,7 @@ static bool mkdir_p(const std::string &filename)
             *p = 0;
             std::string component = std::string(drive) + path;
             _mkdir(component.c_str());
-            if (_stat(component.c_str(), &st) == -1 || (st.st_mode & _S_IFDIR) != S_IFDIR) {
+            if (_stat(component.c_str(), &st) == -1 || (st.st_mode & S_IFDIR) != S_IFDIR) {
                 return false;
             }
             *p = slash;
@@ -699,14 +721,12 @@ static bool mkdir_p(const std::string &filename)
 
 std::unique_ptr<std::string> zterp_os_rcfile(bool create_parent)
 {
-    const char *appdata;
-
-    appdata = std::getenv("APPDATA");
+    auto appdata = zterp_getenv("APPDATA");
     if (appdata == nullptr) {
         return nullptr;
     }
 
-    auto config = std::make_unique<std::string>(std::string(appdata) + "\\Bocfel\\bocfel.ini");
+    auto config = std::make_unique<std::string>(*appdata + "\\Bocfel\\bocfel.ini");
 
     if (create_parent && !mkdir_p(*config)) {
         return nullptr;
@@ -729,14 +749,12 @@ std::unique_ptr<std::string> unique_name()
 
 static std::unique_ptr<std::string> data_file(const std::string &filename)
 {
-    const char *appdata;
-
-    appdata = std::getenv("APPDATA");
+    auto appdata = zterp_getenv("APPDATA");
     if (appdata == nullptr) {
         return nullptr;
     }
 
-    auto name = std::make_unique<std::string>(std::string(appdata) + "\\Bocfel\\" + filename);
+    auto name = std::make_unique<std::string>(*appdata + "\\Bocfel\\" + filename);
 
     if (!mkdir_p(*name)) {
         return nullptr;
@@ -757,15 +775,43 @@ std::unique_ptr<std::string> zterp_os_autosave_name()
 }
 #define have_zterp_os_autosave_name
 
-std::unique_ptr<std::string> zterp_os_aux_file(const std::string &filename)
+static std::string ascii_toupper(std::string s)
 {
-    auto basename = unique_name();
-
-    if (basename == nullptr) {
-        return nullptr;
+    for (auto &c : s) {
+        if (c >= 97 && c <= 122) {
+            c -= 32;
+        }
     }
 
-    if (basename->find_first_of("/\\") != std::string::npos) {
+    return s;
+}
+
+std::unique_ptr<std::string> zterp_os_aux_file(const std::string &filename_)
+{
+    std::string filename = filename_;
+    std::string upper = ascii_toupper(filename);
+
+    static const std::vector<std::string> reserved_names = {
+        "CON", "PRN", "AUX", "NUL", "COM1", "COM2", "COM3", "COM4",
+        "COM5", "COM6", "COM7", "COM8", "COM9", "LPT1", "LPT2",
+        "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9",
+    };
+
+    for (const auto &reserved_name : reserved_names) {
+        if (upper == reserved_name || upper.find(reserved_name + ".") == 0) {
+            return nullptr;
+        }
+    }
+
+    static const std::string invalid_characters = "<>:\"/\\|?*";
+    for (auto &c : filename) {
+        if (invalid_characters.find(c) != std::string::npos) {
+            c = '_';
+        }
+    }
+
+    auto basename = unique_name();
+    if (basename == nullptr) {
         return nullptr;
     }
 
@@ -790,8 +836,8 @@ void zterp_os_edit_file(const std::string &filename)
         throw std::runtime_error("unable to launch text editor");
     }
 
-#ifdef ZTERP_GLK
-    while (WaitForSingleObject(si.hProcess, 100) != 0) {
+#ifdef ZTERP_GLK_TICK
+    while (WaitForSingleObject(si.hProcess, 10) != 0) {
         glk_tick();
     }
 #else
@@ -810,8 +856,14 @@ public:
     Remover(const Remover &) = delete;
     Remover &operator=(const Remover &) = delete;
 
+    Remover(Remover &&other) noexcept : m_filename(std::move(other.m_filename)) {
+        other.m_filename.clear();
+    }
+
     ~Remover() {
-        std::remove(m_filename.c_str());
+        if (!m_filename.empty()) {
+            std::remove(m_filename.c_str());
+        }
     }
 
     const std::string &filename() {
@@ -822,7 +874,19 @@ private:
     std::string m_filename;
 };
 
-std::vector<char> zterp_os_edit_notes(const std::vector<char> &notes)
+static std::string generate_random_prefix()
+{
+    const char charset[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    std::default_random_engine rng(std::random_device{}());
+    std::uniform_int_distribution<> dist(0, sizeof(charset) - 2);
+    std::string prefix;
+    for (size_t i = 0; i < 12; ++i) {
+        prefix += charset[dist(rng)];
+    }
+    return prefix;
+}
+
+static Remover create_temp_file(const std::string &filename, const std::vector<char> &data)
 {
     char tempdir[300];
 
@@ -831,22 +895,36 @@ std::vector<char> zterp_os_edit_notes(const std::vector<char> &notes)
         throw std::runtime_error("unable to find temporary directory");
     }
 
-    Remover notes_name(std::string(tempdir) + "\\bocfel_notes.txt");
+    Remover remover(std::string(tempdir) + "\\" + generate_random_prefix() + "-" + filename);
     try {
-        IO io(&notes_name.filename(), IO::Mode::WriteOnly, IO::Purpose::Data);
-        io.write_exact(notes.data(), notes.size());
+        IO io(&remover.filename(), IO::Mode::WriteOnly, IO::Purpose::Data);
+        io.write_exact(data.data(), data.size());
     } catch (const IO::OpenError &) {
         throw std::runtime_error("unable to create temporary file");
     } catch (const IO::IOError &) {
         throw std::runtime_error("unable to write to temporary file");
     }
 
+    return remover;
+}
+
+std::vector<char> zterp_os_edit_notes(const std::vector<char> &notes)
+{
+    auto notes_name = create_temp_file("bocfel_notes.txt", notes);
+
     zterp_os_edit_file(notes_name.filename());
 
     return read_file(notes_name.filename());
 }
 #define have_zterp_os_edit_notes
 
+void zterp_os_show_transcript(const std::vector<char> &transcript)
+{
+    auto transcript_name = create_temp_file("bocfel_transcript.txt", transcript);
+    zterp_os_edit_file(transcript_name.filename());
+}
+#define have_zterp_os_show_transcript
+
 #ifndef ZTERP_GLK
 std::pair<unsigned int, unsigned int> zterp_os_get_screen_size()
 {
@@ -903,7 +981,7 @@ void zterp_os_set_style(const Style &style, const Color &fg, const Color &bg)
 // ╔══════════════════════════════════════════════════════════════════════════════╗
 // ║ DOS functions                                                                ║
 // ╚══════════════════════════════════════════════════════════════════════════════╝
-#elif defined(ZTERP_DOS)
+#elif defined(ZTERP_OS_DOS)
 #ifndef ZTERP_GLK
 // This assumes ANSI.SYS (or equivalent) is loaded.
 bool zterp_os_have_style(StyleBit)
@@ -964,8 +1042,23 @@ std::unique_ptr<std::string> zterp_os_autosave_name()
 #endif
 
 #ifndef have_zterp_os_aux_file
-std::unique_ptr<std::string> zterp_os_aux_file(const std::string &)
+
+#ifdef ZTERP_GLK_UNIX
+extern "C" {
+#include <glkstart.h>
+}
+#endif
+
+std::unique_ptr<std::string> zterp_os_aux_file(const std::string &filename)
 {
+#if defined(ZTERP_GLK_UNIX) && defined(GLKUNIX_FILEREF_GET_FILENAME)
+    frefid_t fref = glk_fileref_create_by_name(fileusage_Data | fileusage_BinaryMode, const_cast<char *>(filename.c_str()), 0);
+    if (fref != nullptr) {
+        auto result = std::make_unique<std::string>(glkunix_fileref_get_filename(fref));
+        glk_fileref_destroy(fref);
+        return result;
+    }
+#endif
     return nullptr;
 }
 #endif
@@ -977,6 +1070,13 @@ std::vector<char> zterp_os_edit_notes(const std::vector<char> &)
 }
 #endif
 
+#ifndef have_zterp_os_show_transcript
+void zterp_os_show_transcript(const std::vector<char> &transcript)
+{
+    throw std::runtime_error("transcript display unimplemented on this platform");
+}
+#endif
+
 #ifndef ZTERP_GLK
 #ifndef have_zterp_os_get_screen_size
 std::pair<unsigned int, unsigned int> zterp_os_get_screen_size()
