@@ -1,30 +1,21 @@
-// Copyright 2010-2021 Chris Spiegel.
+// Copyright 2010-2024 Chris Spiegel.
 //
-// This file is part of Bocfel.
-//
-// Bocfel is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License, version
-// 2 or 3, as published by the Free Software Foundation.
-//
-// Bocfel is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Bocfel. If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: MIT
 
 #include <algorithm>
 #include <array>
+#include <chrono>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
 #include <deque>
-#include <map>
+#include <iomanip>
+#include <locale>
 #include <memory>
 #include <new>
 #include <sstream>
 #include <string>
+#include <unordered_map>
 #include <utility>
 #include <vector>
 
@@ -34,6 +25,7 @@
 #include "io.h"
 #include "memory.h"
 #include "meta.h"
+#include "options.h"
 #include "osdep.h"
 #include "process.h"
 #include "random.h"
@@ -43,6 +35,10 @@
 #include "util.h"
 #include "zterp.h"
 
+#ifdef ZTERP_GLK
+#include "glkautosave.h"
+#endif
+
 using namespace std::literals;
 
 enum class StoreWhere {
@@ -101,16 +97,24 @@ public:
 
 private:
     static std::string format_time() {
-        char formatted_time[128];
-        time_t when = time(nullptr);
-        struct tm *tm;
+        auto now = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
+        auto *lt = std::localtime(&now);
 
-        tm = std::localtime(&when);
-        if (tm == nullptr || std::strftime(formatted_time, sizeof formatted_time, "%c", tm) == 0) {
-            std::snprintf(formatted_time, sizeof formatted_time, "<no time information>");
+        if (lt == nullptr) {
+            return "<no time information>";
+        } else {
+            std::ostringstream formatted_time;
+
+            try {
+                formatted_time.imbue(std::locale(""));
+            } catch (...) {
+                // If the locale is invalid, don’t worry about it.
+            }
+
+            formatted_time << std::put_time(lt, "%c");
+
+            return formatted_time.str();
         }
-
-        return formatted_time;
     }
 };
 
@@ -148,7 +152,7 @@ struct SaveStack {
         states.shrink_to_fit();
     }
 };
-static std::map<SaveStackType, SaveStack> save_stacks;
+static std::unordered_map<SaveStackType, SaveStack, EnumClassHash> save_stacks;
 
 bool seen_save_undo = false;
 
@@ -200,11 +204,11 @@ void store_variable(uint16_t var, uint16_t n)
     }
 }
 
-uint16_t *stack_top_element()
+static uint16_t &stack_top_element()
 {
     ZASSERT(sp > CURRENT_FRAME->sp, "stack underflow");
 
-    return sp - 1;
+    return sp[-1];
 }
 
 void zpush()
@@ -221,7 +225,7 @@ void zpull()
 
         // The z-spec 1.1 requires indirect variable references to the stack not to push/pop
         if (zargs[0] == 0) {
-            *stack_top_element() = v;
+            stack_top_element() = v;
         } else {
             store_variable(zargs[0], v);
         }
@@ -231,6 +235,9 @@ void zpull()
         } else {
             uint16_t slots = user_word(zargs[0]) + 1;
 
+            ZASSERT(slots != 0, "user stack counter overflow");
+            ZASSERT(zargs[0] + (2UL * slots) <= header.static_end - 2, "user stack access out of bounds");
+
             v = user_word(zargs[0] + (2 * slots));
 
             user_store_word(zargs[0], slots);
@@ -244,7 +251,7 @@ void zload()
 {
     // The z-spec 1.1 requires indirect variable references to the stack not to push/pop
     if (zargs[0] == 0) {
-        store(*stack_top_element());
+        store(stack_top_element());
     } else {
         store(variable(zargs[0]));
     }
@@ -254,7 +261,7 @@ void zstore()
 {
     // The z-spec 1.1 requires indirect variable references to the stack not to push/pop
     if (zargs[0] == 0) {
-        *stack_top_element() = zargs[1];
+        stack_top_element() = zargs[1];
     } else {
         store_variable(zargs[0], zargs[1]);
     }
@@ -262,8 +269,6 @@ void zstore()
 
 static void call(StoreWhere store_where)
 {
-    uint32_t jmp_to;
-    uint8_t nlocals;
     uint16_t where;
 
     if (zargs[0] == 0) {
@@ -274,10 +279,10 @@ static void call(StoreWhere store_where)
         return;
     }
 
-    jmp_to = unpack_routine(zargs[0]);
+    uint32_t jmp_to = unpack_routine(zargs[0]);
     ZASSERT(jmp_to < memory_size - 1, "call to invalid address 0x%lx", static_cast<unsigned long>(jmp_to));
 
-    nlocals = byte(jmp_to++);
+    uint8_t nlocals = byte(jmp_to++);
     ZASSERT(nlocals <= 15, "too many (%d) locals at 0x%lx", nlocals, static_cast<unsigned long>(jmp_to) - 1);
 
     if (zversion <= 4) {
@@ -318,23 +323,25 @@ void start_v6()
     call(StoreWhere::None);
 }
 
-#ifdef ZTERP_GLK
-uint16_t internal_call(uint16_t routine)
+uint16_t internal_call(uint16_t routine, std::vector<uint16_t> args)
 {
-    std::vector<uint16_t> saved_args(zargs.begin(), zargs.begin() + znargs);
+    auto saved_zargs = zargs;
+    auto saved_znargs = znargs;
 
-    znargs = 1;
+    ZASSERT(args.size() < 8, "internal error: too many arguments");
+
+    znargs = 1 + args.size();
     zargs[0] = routine;
+    std::copy(args.begin(), args.end(), &zargs[1]);
     call(StoreWhere::Push);
 
     process_instructions();
 
-    std::copy(saved_args.begin(), saved_args.end(), zargs.begin());
-    znargs = saved_args.size();
+    zargs = saved_zargs;
+    znargs = saved_znargs;
 
     return pop_stack();
 }
-#endif
 
 void zcall_store()
 {
@@ -347,13 +354,11 @@ void zcall_nostore()
 
 void do_return(uint16_t retval)
 {
-    uint16_t where;
-
     ZASSERT(NFRAMES > 1, "return attempted outside of a function");
 
     pc = CURRENT_FRAME->pc;
     sp = CURRENT_FRAME->sp;
-    where = CURRENT_FRAME->where;
+    uint16_t where = CURRENT_FRAME->where;
     fp--;
 
     if (where <= 0xff) {
@@ -418,6 +423,7 @@ void zcheck_arg_count()
     branch_if(zargs[0] <= CURRENT_FRAME->nargs);
 }
 
+#ifndef ZTERP_NO_V6
 void zpop_stack()
 {
     if (znargs == 1) {
@@ -443,6 +449,7 @@ void zpush_stack()
 
     branch_if(true);
 }
+#endif
 
 // Compress dynamic memory according to Quetzal. On failure,
 // std::bad_alloc is thrown.
@@ -492,7 +499,7 @@ static bool uncompress_memory(const uint8_t *compressed, uint32_t size)
 {
     uint32_t memory_index = 0;
 
-    std::memcpy(memory, dynamic_memory, header.static_start);
+    std::copy(dynamic_memory.begin(), dynamic_memory.begin() + header.static_start, memory.begin());
 
     for (uint32_t i = 0; i < size; i++) {
         if (compressed[i] != 0) {
@@ -525,7 +532,7 @@ static IFF::TypeID write_ifhd(IO &savefile)
     savefile.write8((pc >>  8) & 0xff);
     savefile.write8((pc >>  0) & 0xff);
 
-    return IFF::TypeID(&"IFhd");
+    return IFF::TypeID("IFhd");
 }
 
 // Store the filename in an IntD chunk.
@@ -538,15 +545,15 @@ static IFF::TypeID write_intd(IO &savefile)
     savefile.write_exact("    ", 4);
     savefile.write_exact(game_file.c_str(), game_file.size());
 
-    return IFF::TypeID(&"IntD");
+    return IFF::TypeID("IntD");
 }
 
 static IFF::TypeID write_mem(IO &savefile)
 {
     std::vector<uint8_t> compressed;
     uint32_t memsize = header.static_start;
-    const uint8_t *mem = memory;
-    IFF::TypeID type = IFF::TypeID(&"UMem");
+    const uint8_t *mem = memory.data();
+    IFF::TypeID type = IFF::TypeID("UMem");
 
     try {
         compressed = compress_memory();
@@ -555,7 +562,7 @@ static IFF::TypeID write_mem(IO &savefile)
         if (compressed.size() < header.static_start) {
             mem = compressed.data();
             memsize = compressed.size();
-            type = IFF::TypeID(&"CMem");
+            type = IFF::TypeID("CMem");
         }
     } catch (const std::bad_alloc &) {
     }
@@ -572,13 +579,11 @@ static IFF::TypeID write_stks(IO &savefile)
     // calculate the evaluation stack used by the current routine.
     fp->sp = sp;
     for (CallFrame *p = BASE_OF_FRAMES; p != fp; p++) {
-        uint8_t flags;
-
         savefile.write8((p->pc >> 16) & 0xff);
         savefile.write8((p->pc >>  8) & 0xff);
         savefile.write8((p->pc >>  0) & 0xff);
 
-        flags = p->nlocals;
+        uint8_t flags = p->nlocals;
         if (p->where > 0xff) {
             flags |= 0x10;
         }
@@ -606,7 +611,7 @@ static IFF::TypeID write_stks(IO &savefile)
         }
     }
 
-    return IFF::TypeID(&"Stks");
+    return IFF::TypeID("Stks");
 }
 
 static IFF::TypeID write_anno(IO &savefile)
@@ -614,7 +619,7 @@ static IFF::TypeID write_anno(IO &savefile)
     std::string anno = "Interpreter: Bocfel "s + ZTERP_VERSION;
     savefile.write_exact(anno.c_str(), anno.size());
 
-    return IFF::TypeID(&"ANNO");
+    return IFF::TypeID("ANNO");
 }
 
 static IFF::TypeID write_args(IO &savefile, SaveOpcode saveopcode)
@@ -625,7 +630,7 @@ static IFF::TypeID write_args(IO &savefile, SaveOpcode saveopcode)
         savefile.write16(zargs[i]);
     }
 
-    return IFF::TypeID(&"Args");
+    return IFF::TypeID("Args");
 }
 
 static void write_undo_msav(IO &savefile, SaveStackType type)
@@ -656,31 +661,30 @@ static IFF::TypeID write_undo(IO &savefile)
 {
     write_undo_msav(savefile, SaveStackType::Game);
 
-    return IFF::TypeID(&"Undo");
+    return IFF::TypeID("Undo");
 }
 
 static IFF::TypeID write_msav(IO &savefile)
 {
     write_undo_msav(savefile, SaveStackType::User);
 
-    return IFF::TypeID(&"MSav");
+    return IFF::TypeID("MSav");
 }
 
 template<typename... Types>
 static void write_chunk(IO &io, IFF::TypeID (*writefunc)(IO &savefile, Types... args), Types... args)
 {
-    long chunk_pos, end_pos, size;
-    IFF::TypeID type;
-
-    chunk_pos = io.tell();
-    io.seek(8, IO::SeekFrom::Current); // skip past type and size
-    type = writefunc(io, args...);
+    long chunk_pos = io.tell();
+    // Type and size, to be filled in below.
+    io.write32(0);
+    io.write32(0);
+    auto type = writefunc(io, args...);
     if (type.empty()) {
         io.seek(chunk_pos, IO::SeekFrom::Start);
         return;
     }
-    end_pos = io.tell();
-    size = end_pos - chunk_pos - 8;
+    long end_pos = io.tell();
+    long size = end_pos - chunk_pos - 8;
     io.seek(chunk_pos, IO::SeekFrom::Start);
     io.write32(type.val());
     io.write32(size);
@@ -694,13 +698,12 @@ static void write_chunk(IO &io, IFF::TypeID (*writefunc)(IO &savefile, Types...
 // format of the save state is the same (that is, the IFhd, IntD, and
 // CMem/UMem chunks are identical). The type of the save file itself is
 // BFZS instead of IFZS to prevent the files from being used by a normal
-// @restore (as they are not compatible). See `Quetzal.md` for a
+// @restore (as they are not compatible). See Quetzal.md for a
 // description of how BFZS differs from IFZS.
 static bool save_quetzal(IO &savefile, SaveType savetype, SaveOpcode saveopcode, bool on_save_stack)
 {
     try {
-        long file_size;
-        bool is_bfzs = savetype == SaveType::Meta || savetype == SaveType::Autosave;
+        bool is_bfzs = savetype == SaveType::Meta || savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib;
 
         savefile.write_exact("FORM", 4);
         savefile.write32(0); // to be filled in
@@ -711,17 +714,19 @@ static bool save_quetzal(IO &savefile, SaveType savetype, SaveOpcode saveopcode,
         write_chunk(savefile, write_mem);
         write_chunk(savefile, write_stks);
         write_chunk(savefile, write_anno);
-        write_chunk(savefile, meta_write_bfnt);
 
         // When saving to a stack (either for undo or for in-memory saves),
-        // don’t store history or persistent transcripts. History is
-        // pointless, since the user can see this history already, and
-        // persistent transcripts want to track what actually happened. If
-        // the user types UNDO, for example, that should be reflected in the
-        // transcript.
-        if (on_save_stack) {
+        // don’t store history, persistent transcripts, or notes. History is
+        // pointless, since the user can see this history already. Persistent
+        // transcripts want to track what actually happened. If the user types
+        // UNDO, for example, that should be reflected in the transcript. And
+        // notes shouldn’t be cleared just because the user undid a turn; if a
+        // user doesn’t want the notes, he can delete them, but he can’t bring
+        // back notes that he never wanted deleted.
+        if (!on_save_stack) {
             write_chunk(savefile, screen_write_bfhs);
             write_chunk(savefile, screen_write_bfts);
+            write_chunk(savefile, meta_write_bfnt);
         }
 
         // When restoring a meta save, @read will be called to bring the user
@@ -736,13 +741,13 @@ static bool save_quetzal(IO &savefile, SaveType savetype, SaveOpcode saveopcode,
             write_chunk(savefile, screen_write_scrn);
         }
 
-        if (savetype == SaveType::Autosave) {
+        if (savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib) {
             write_chunk(savefile, write_undo);
             write_chunk(savefile, write_msav);
             write_chunk(savefile, random_write_rand);
         }
 
-        file_size = savefile.tell();
+        long file_size = savefile.tell();
         savefile.seek(4, IO::SeekFrom::Start);
         savefile.write32(file_size - 8); // entire file size minus 8 (FORM + size)
 
@@ -756,7 +761,7 @@ static void read_mem(IFF &iff)
 {
     uint32_t size;
 
-    if (iff.find(IFF::TypeID(&"CMem"), size)) {
+    if (iff.find(IFF::TypeID("CMem"), size)) {
         std::vector<uint8_t> buf;
 
         // Dynamic memory is 64KB, and a worst-case save should take up
@@ -780,12 +785,12 @@ static void read_mem(IFF &iff)
         if (!uncompress_memory(buf.data(), size)) {
             throw RestoreError("memory cannot be uncompressed");
         }
-    } else if (iff.find(IFF::TypeID(&"UMem"), size)) {
+    } else if (iff.find(IFF::TypeID("UMem"), size)) {
         if (size != header.static_start) {
             throw RestoreError("memory size mismatch");
         }
         try {
-            iff.io()->read_exact(memory, header.static_start);
+            iff.io()->read_exact(memory.data(), header.static_start);
         } catch (const IO::OpenError &) {
             throw RestoreError("unexpected eof reading memory");
         }
@@ -798,7 +803,7 @@ static void read_stks(IFF &iff)
 {
     uint32_t size, n = 0, frameno = 0;
 
-    if (!iff.find(IFF::TypeID(&"Stks"), size)) {
+    if (!iff.find(IFF::TypeID("Stks"), size)) {
         throw RestoreError("no stacks chunk found");
     }
     if (size == 0) {
@@ -810,10 +815,7 @@ static void read_stks(IFF &iff)
 
     while (n < size) {
         uint8_t frame[8];
-        uint8_t nlocals;
-        uint16_t nstack;
         uint8_t nargs = 0;
-        uint32_t frame_pc;
 
         try {
             iff.io()->read_exact(frame, sizeof frame);
@@ -822,14 +824,14 @@ static void read_stks(IFF &iff)
         }
         n += sizeof frame;
 
-        nlocals = frame[3] & 0xf;
-        nstack = (frame[6] << 8) | frame[7];
+        uint8_t nlocals = frame[3] & 0xf;
+        uint16_t nstack = (frame[6] << 8) | frame[7];
         frame[5]++;
         while ((frame[5] >>= 1) != 0) {
             nargs++;
         }
 
-        frame_pc = (static_cast<uint32_t>(frame[0]) << 16) | (static_cast<uint32_t>(frame[1]) << 8) | static_cast<uint32_t>(frame[2]);
+        uint32_t frame_pc = (static_cast<uint32_t>(frame[0]) << 16) | (static_cast<uint32_t>(frame[1]) << 8) | static_cast<uint32_t>(frame[2]);
         if (frame_pc >= memory_size) {
             throw RestoreError(fstring("frame #%lu pc out of range (0x%lx)", static_cast<unsigned long>(frameno), static_cast<unsigned long>(frame_pc)));
         }
@@ -875,7 +877,7 @@ static void read_args(IFF &iff, SaveOpcode &saveopcode)
     uint32_t size;
     uint8_t saveopcode_temp;
 
-    if (!iff.find(IFF::TypeID(&"Args"), size)) {
+    if (!iff.find(IFF::TypeID("Args"), size)) {
         throw RestoreError("no meta save Args chunk found");
     }
 
@@ -891,15 +893,23 @@ static void read_args(IFF &iff, SaveOpcode &saveopcode)
 
     // @read takes between 1 and 4 operands, @read_char takes
     // between 1 and 3.
+    // @save and @restore take none. (Args are possible in z5,
+    // but I haven’t implemented that yet.)
     switch (saveopcode) {
     case SaveOpcode::Read:
         if (size != 2 && size != 4 && size != 6 && size != 8) {
-            throw RestoreError(fstring("invalid Args size: %lu", static_cast<unsigned long>(size)));
+            throw RestoreError(fstring("invalid Args size for %d: %lu", static_cast<int>(saveopcode), static_cast<unsigned long>(size)));
         }
         break;
     case SaveOpcode::ReadChar:
         if (size != 2 && size != 4 && size != 6) {
-            throw RestoreError(fstring("invalid Args size: %lu", static_cast<unsigned long>(size)));
+            throw RestoreError(fstring("invalid Args size for %d: %lu", static_cast<int>(saveopcode), static_cast<unsigned long>(size)));
+        }
+        break;
+    case SaveOpcode::Save:
+    case SaveOpcode::Restore:
+        if (size != 0) {
+            throw RestoreError(fstring("invalid Args size for %d: %lu", static_cast<int>(saveopcode), static_cast<unsigned long>(size)));
         }
         break;
     default:
@@ -923,11 +933,13 @@ static void read_bfzs_specific(IFF &iff, SaveType savetype, SaveOpcode &saveopco
 
     read_args(iff, saveopcode);
 
-    if (savetype == SaveType::Autosave && iff.find(IFF::TypeID(&"Rand"), size)) {
+    if ((savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib) &&
+        iff.find(IFF::TypeID("Rand"), size))
+    {
         random_read_rand(*iff.io());
     }
 
-    if (iff.find(IFF::TypeID(&"Scrn"), size)) {
+    if (iff.find(IFF::TypeID("Scrn"), size)) {
         // Restoring cannot fail after this, because this function
         // actively touches the screen (changing window
         // configuration, etc). It would be possible to stash the
@@ -953,13 +965,12 @@ static void read_undo_msav(IO &savefile, uint32_t size, SaveStackType type)
 
     try {
         uint32_t version = savefile.read32();
-        uint32_t count;
         SaveStack &save_stack = save_stacks[type];
         size_t actual_size = 0;
         if (version != 0) {
             return;
         }
-        count = savefile.read32();
+        uint32_t count = savefile.read32();
 
         actual_size += 4 + 4;
 
@@ -970,13 +981,11 @@ static void read_undo_msav(IO &savefile, uint32_t size, SaveStackType type)
         for (uint32_t i = 0; i < count; i++) {
             uint8_t savetype = static_cast<uint8_t>(SaveType::Meta);
             std::string desc;
-            uint32_t quetzal_size;
             std::vector<uint8_t> quetzal;
 
             if (type == SaveStackType::Game) {
                 savetype = savefile.read8();
-                if ((static_cast<SaveType>(savetype) != SaveType::Normal && static_cast<SaveType>(savetype) != SaveType::Meta)) {
-
+                if (static_cast<SaveType>(savetype) != SaveType::Normal && static_cast<SaveType>(savetype) != SaveType::Meta) {
                     return;
                 }
 
@@ -992,7 +1001,7 @@ static void read_undo_msav(IO &savefile, uint32_t size, SaveStackType type)
                 actual_size += 4 + desc.size();
             }
 
-            quetzal_size = savefile.read32();
+            uint32_t quetzal_size = savefile.read32();
             quetzal.resize(quetzal_size);
             savefile.read_exact(quetzal.data(), quetzal_size);
 
@@ -1061,38 +1070,39 @@ static bool instruction_has_stack_argument(uint32_t addr)
     return false;
 }
 
-static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, SaveOpcode &saveopcode)
+static bool restore_quetzal(const std::shared_ptr<IO> &savefile, SaveType savetype, SaveOpcode &saveopcode, bool close_window)
 {
     std::unique_ptr<IFF> iff;
     uint32_t size;
     uint8_t ifhd[13];
     uint32_t newpc;
-    bool is_bfzs = savetype == SaveType::Meta || savetype == SaveType::Autosave;
+    bool is_bfzs = savetype == SaveType::Meta || savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib;
     bool is_bfms = false;
     Stash stash;
+    uint16_t flags2 = word(0x10);
 
     saveopcode = SaveOpcode::None;
 
     if (is_bfzs) {
         try {
-            iff = std::make_unique<IFF>(savefile, IFF::TypeID(&"BFZS"));
+            iff = std::make_unique<IFF>(savefile, IFF::TypeID("BFZS"));
         } catch (const IFF::InvalidFile &) {
             try {
-                iff = std::make_unique<IFF>(savefile, IFF::TypeID(&"BFMS"));
+                iff = std::make_unique<IFF>(savefile, IFF::TypeID("BFMS"));
                 is_bfms = true;
             } catch (const IFF::InvalidFile &) {
             }
         }
     } else {
         try {
-            iff = std::make_unique<IFF>(savefile, IFF::TypeID(&"IFZS"));
+            iff = std::make_unique<IFF>(savefile, IFF::TypeID("IFZS"));
         } catch (const IFF::InvalidFile &) {
         }
     }
 
     try {
         if (iff == nullptr ||
-            !iff->find(IFF::TypeID(&"IFhd"), size) ||
+            !iff->find(IFF::TypeID("IFhd"), size) ||
             size != 13) {
 
             throw RestoreError("corrupted save file or not a save file at all");
@@ -1120,12 +1130,12 @@ static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, Sav
             throw RestoreError("detected incompatible meta save: please file a bug report at https://bocfel.org/issues");
         }
 
-        if (is_bfzs && savetype == SaveType::Autosave) {
-            if (iff->find(IFF::TypeID(&"Undo"), size)) {
+        if (is_bfzs && (savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib)) {
+            if (iff->find(IFF::TypeID("Undo"), size)) {
                 read_undo(*iff->io(), size);
             }
 
-            if (iff->find(IFF::TypeID(&"MSav"), size)) {
+            if (iff->find(IFF::TypeID("MSav"), size)) {
                 read_msav(*iff->io(), size);
             }
         }
@@ -1135,11 +1145,15 @@ static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, Sav
         read_mem(*iff);
         read_stks(*iff);
 
-        if (iff->find(IFF::TypeID(&"Bfnt"), size)) {
+        if (iff->find(IFF::TypeID("Bfnt"), size)) {
             meta_read_bfnt(*iff->io(), size);
         }
 
-        if (iff->find(IFF::TypeID(&"Bfhs"), size)) {
+        if (iff->find(IFF::TypeID("Bfhs"), size)) {
+            // In the regular restore case, we display the history (from
+            // the save file) unless this is disabled by the -H option.
+            // We always redisplay after an autosave, unless we’re doing
+            // a library-state autosave, in which case we don’t need to.
             if (savetype == SaveType::Autosave || !options.disable_history_playback) {
                 try {
                     long start = iff->io()->tell();
@@ -1155,10 +1169,10 @@ static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, Sav
             }
         } else if (savetype == SaveType::Autosave) {
             warning("unable to find history record");
-            screen_printf(">");
+            screen_print(">");
         }
 
-        if (iff->find(IFF::TypeID(&"Bfts"), size)) {
+        if (iff->find(IFF::TypeID("Bfts"), size)) {
             screen_read_bfts(*iff->io(), size);
         }
 
@@ -1183,7 +1197,7 @@ static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, Sav
         // before anything happens, it is guaranteed that the stacks were
         // empty before the restore process started. This is faster and
         // simpler than stashing.
-        if (is_bfzs && savetype == SaveType::Autosave) {
+        if (is_bfzs && (savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib)) {
             save_stacks[SaveStackType::Game].clear();
             save_stacks[SaveStackType::User].clear();
         }
@@ -1193,6 +1207,45 @@ static bool restore_quetzal(std::shared_ptr<IO> savefile, SaveType savetype, Sav
 
     pc = newpc;
 
+    // §8.6.1.3
+    // Except that for AutosaveLib, we’ll restore the upper window.
+    if (close_window && zversion == 3 && savetype != SaveType::AutosaveLib) {
+        close_upper_window();
+    }
+
+    // For user saves, the game won’t know a restore occurred, so it
+    // won’t redraw the status line. Set the “redraw status” bit of
+    // Flags 2 to indicate a redraw is requested. The Z-Machine
+    // Standards Document says this bit is for V6 only, but Infocom’s
+    // documentation says V4+, and A Mind Forever Voyaging (which is V4)
+    // checks it.
+    if (zversion >= 4 && (savetype == SaveType::Autosave || savetype == SaveType::Meta)) {
+        flags2 |= FLAGS2_STATUS;
+    }
+
+    if (savetype == SaveType::AutosaveLib) {
+        // Use the save file’s FLAGS2_TRANSCRIPT.
+        if (word(0x10) & FLAGS2_TRANSCRIPT)
+            flags2 |= FLAGS2_TRANSCRIPT;
+        else
+            flags2 &= ~FLAGS2_TRANSCRIPT;
+    }
+
+    // §6.1.2.2: The save might be from a different interpreter with
+    // different capabilities, so update the header to indicate what the
+    // current capabilities are.
+    //
+    // Even for in-game saves (either via the game or user stack),
+    // rewrite the header: the interpreter will probably be the same,
+    // but the screen size might have changed. And, in fact, since these
+    // stacks can be stored in autosaves, it’s technically possible they
+    // COULD be running in another interpreter, or at least a future
+    // version of this one.
+    write_header();
+
+    // §6.1.2: Flags 2 should be preserved.
+    store_word(0x10, flags2);
+
     return true;
 }
 
@@ -1200,7 +1253,7 @@ static std::shared_ptr<IO> open_savefile(SaveType savetype, IO::Mode mode)
 {
     std::unique_ptr<std::string> filename;
 
-    if (savetype == SaveType::Autosave) {
+    if (savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib) {
         filename = zterp_os_autosave_name();
         if (filename == nullptr) {
             return nullptr;
@@ -1210,7 +1263,7 @@ static std::shared_ptr<IO> open_savefile(SaveType savetype, IO::Mode mode)
     try {
         return std::make_shared<IO>(filename.get(), mode, IO::Purpose::Save);
     } catch (const IO::OpenError &) {
-        if (savetype != SaveType::Autosave) {
+        if (!(savetype == SaveType::Autosave || savetype == SaveType::AutosaveLib)) {
             warning("unable to open save file");
         }
 
@@ -1227,11 +1280,19 @@ bool do_save(SaveType savetype, SaveOpcode saveopcode)
         return false;
     }
 
-    if (!save_quetzal(*savefile, savetype, saveopcode, true)) {
+    if (!save_quetzal(*savefile, savetype, saveopcode, false)) {
         warning("error while writing save file");
         return false;
     }
 
+#ifdef ZTERP_GLK
+    if (savetype == SaveType::AutosaveLib) {
+        if (!glkautosave_library_autosave()) {
+            return false;
+        }
+    }
+#endif
+
     return true;
 }
 
@@ -1245,6 +1306,20 @@ void zsave()
         return;
     }
 
+    // Autosave before blocking on the fileref prompt. (Which will
+    // certainly happen down in the guts of do_save(), because there
+    // is no suggested filename.)
+    //
+    // Yes, it’s goofy to call do_save() before do_save(), but that’s
+    // what happens if you want to autosave every time the Z-machine
+    // waits for input.
+    //
+    // (Note that we might have arrived here from zsave5().)
+    //
+    if (options.autosave && options.autosave_librarystate) {
+        do_save(SaveType::AutosaveLib, SaveOpcode::Save);
+    }
+
     bool success = do_save(SaveType::Normal, SaveOpcode::None);
 
     if (zversion <= 3) {
@@ -1260,43 +1335,34 @@ void zsave()
 // this is.
 bool do_restore(SaveType savetype, SaveOpcode &saveopcode)
 {
-    uint16_t flags2;
-    bool success;
+#ifdef ZTERP_GLK
+    if (savetype == SaveType::AutosaveLib) {
+        if (!glkautosave_library_autorestore()) {
+            return false;
+        }
+    }
+#endif
 
     auto savefile = open_savefile(savetype, IO::Mode::ReadOnly);
     if (savefile == nullptr) {
         return false;
     }
 
-    flags2 = word(0x10);
-
-    success = restore_quetzal(savefile, savetype, saveopcode);
-
-    if (success) {
-        // §8.6.1.3
-        if (zversion == 3) {
-            close_upper_window();
-        }
-
-        // §6.1.2.2: The save might be from a different interpreter with
-        // different capabilities, so update the header to indicate what the
-        // current capabilities are...
-        write_header();
-
-        // ...except that flags2 should be preserved (§6.1.2).
-        store_word(0x10, flags2);
-
-        // Redraw the status line in games that use one.
-        if (zversion <= 3) {
-            zshow_status();
-        }
-    }
-
-    return success;
+    return restore_quetzal(savefile, savetype, saveopcode, true);
 }
 
 void zrestore()
 {
+    // Autosave before blocking on the fileref prompt. (Which will
+    // certainly happen down in the guts of do_restore(), because there
+    // is no suggested filename.)
+    //
+    // (Note that we might have arrived here from restore5().)
+    //
+    if (options.autosave && options.autosave_librarystate) {
+        do_save(SaveType::AutosaveLib, SaveOpcode::Restore);
+    }
+
     SaveOpcode saveopcode;
     bool success = do_restore(SaveType::Normal, saveopcode);
 
@@ -1326,7 +1392,7 @@ SaveResult push_save(SaveStackType type, SaveType savetype, SaveOpcode saveopcod
     try {
         IO savefile(std::vector<uint8_t>(), IO::Mode::WriteOnly);
 
-        if (!save_quetzal(savefile, savetype, saveopcode, false)) {
+        if (!save_quetzal(savefile, savetype, saveopcode, true)) {
             return SaveResult::Failure;
         }
 
@@ -1346,15 +1412,12 @@ SaveResult push_save(SaveStackType type, SaveType savetype, SaveOpcode saveopcod
 bool pop_save(SaveStackType type, size_t saveno, SaveOpcode &saveopcode)
 {
     SaveStack &s = save_stacks[type];
-    uint16_t flags2;
     std::shared_ptr<IO> savefile;
 
     if (saveno >= s.states.size()) {
         return false;
     }
 
-    flags2 = word(0x10);
-
     for (size_t i = 0; i < saveno; i++) {
         s.states.pop_front();
     }
@@ -1368,21 +1431,7 @@ bool pop_save(SaveStackType type, size_t saveno, SaveOpcode &saveopcode)
         return false;
     }
 
-    if (!restore_quetzal(savefile, p.savetype, saveopcode)) {
-        return false;
-    }
-
-    // §6.1.2.2: As with @restore, header values marked with Rst (in
-    // §11) should be reset. Unlike with @restore, it can be assumed
-    // that the game was saved by the same interpreter, but it cannot be
-    // assumed that the screen size is the same as it was at the time
-    // @save_undo was called.
-    write_header();
-
-    // §6.1.2: Flags 2 should be preserved.
-    store_word(0x10, flags2);
-
-    return true;
+    return restore_quetzal(savefile, p.savetype, saveopcode, false);
 }
 
 // Wrapper around trim_saves which reports failure if the specified save
@@ -1400,25 +1449,24 @@ bool drop_save(SaveStackType type, size_t i)
     return true;
 }
 
-void list_saves(SaveStackType type, void (*printer)(const std::string &))
+void list_saves(SaveStackType type)
 {
-    SaveStack &s = save_stacks[type];
+    const SaveStack &s = save_stacks[type];
     auto nsaves = s.states.size();
 
     if (nsaves == 0) {
-        printer("[No saves available]");
+        screen_puts("[No saves available]");
         return;
     }
 
     for (auto p = s.states.crbegin(); p != s.states.crend(); ++p) {
-        std::ostringstream ss;
+        std::string desc = std::to_string(nsaves) + ". " + (p->desc.empty() ? "<no description>" : p->desc);
 
-        ss << nsaves << ". " << (p->desc.empty() ? "<no description>" : p->desc);
         if (nsaves == 1) {
-            ss << " *";
+            desc += " *";
         }
 
-        printer(ss.str());
+        screen_puts(desc);
 
         nsaves--;
     }
@@ -1482,108 +1530,112 @@ void zrestore_undo()
     }
 }
 
-static std::vector<uint16_t> zargs_stash;
-
-static void args_stash_backup()
-{
-    zargs_stash.assign(zargs.begin(), zargs.begin() + znargs);
-}
-
-static bool args_stash_restore()
-{
-    std::copy(zargs_stash.begin(), zargs_stash.end(), zargs.begin());
-    znargs = zargs_stash.size();
-
-    return true;
-}
-
-static void args_stash_free()
-{
-    zargs_stash.clear();
-}
-
-static std::unique_ptr<std::vector<uint8_t>> memory_backup;
-
-static void memory_stash_backup()
-{
-    try {
-        memory_backup = std::make_unique<std::vector<uint8_t>>(memory, memory + header.static_start);
-    } catch (std::bad_alloc &) {
-        memory_backup.reset();
-    }
-}
-
-static bool memory_stash_restore()
-{
-    if (memory_backup == nullptr) {
-        return false;
+class ArgsStasher : public Stasher {
+public:
+    void backup() override {
+        m_zargs.assign(zargs.begin(), zargs.begin() + znargs);
     }
 
-    std::copy(memory_backup->begin(), memory_backup->end(), memory);
+    bool restore() override {
+        std::copy(m_zargs.begin(), m_zargs.end(), zargs.begin());
+        znargs = m_zargs.size();
 
-    return true;
-}
-
-static void memory_stash_free()
-{
-    memory_backup.reset();
-}
-
-static std::unique_ptr<std::vector<uint16_t>> stack_backup;
-
-static void stack_stash_backup()
-{
-    try {
-        stack_backup = std::make_unique<std::vector<uint16_t>>(stack, sp);
-    } catch (const std::bad_alloc &) {
-        stack_backup.reset();
-    }
-}
-
-static bool stack_stash_restore()
-{
-    if (stack_backup == nullptr) {
-        return false;
+        return true;
     }
 
-    sp = BASE_OF_STACK + stack_backup->size();
-    std::copy(stack_backup->begin(), stack_backup->end(), stack);
-
-    return true;
-}
-
-static void stack_stash_free()
-{
-    stack_backup.reset();
-}
-
-static std::unique_ptr<std::vector<CallFrame>> frames_backup;
-
-static void frames_stash_backup()
-{
-    try {
-        frames_backup = std::make_unique<std::vector<CallFrame>>(frames, fp);
-    } catch (const std::bad_alloc &) {
-        frames_backup.reset();
-    }
-}
-
-static bool frames_stash_restore()
-{
-    if (frames_backup == nullptr) {
-        return false;
+    void free() override {
+        m_zargs.clear();
     }
 
-    fp = BASE_OF_FRAMES + frames_backup->size();
-    std::copy(frames_backup->begin(), frames_backup->end(), frames);
+private:
+    std::vector<uint16_t> m_zargs;
+};
 
-    return true;
-}
+class MemoryStasher : public Stasher {
+public:
+    void backup() override {
+        try {
+            m_memory = std::make_unique<std::vector<uint8_t>>(memory.begin(), memory.begin() + header.static_start);
+        } catch (std::bad_alloc &) {
+            m_memory.reset();
+        }
+    }
 
-static void frames_stash_free()
-{
-    frames_backup.reset();
-}
+    bool restore() override {
+        if (m_memory == nullptr) {
+            return false;
+        }
+
+        std::copy(m_memory->begin(), m_memory->end(), memory.begin());
+
+        return true;
+    }
+
+    void free() override {
+        m_memory.reset();
+    }
+
+private:
+    std::unique_ptr<std::vector<uint8_t>> m_memory;
+};
+
+class StackStasher : public Stasher {
+public:
+    void backup() override {
+        try {
+            m_stack = std::make_unique<std::vector<uint16_t>>(stack, sp);
+        } catch (const std::bad_alloc &) {
+            m_stack.reset();
+        }
+    }
+
+    bool restore() override {
+        if (m_stack == nullptr) {
+            return false;
+        }
+
+        sp = BASE_OF_STACK + m_stack->size();
+        std::copy(m_stack->begin(), m_stack->end(), stack);
+
+        return true;
+    }
+
+    void free() override {
+        m_stack.reset();
+    }
+
+private:
+    std::unique_ptr<std::vector<uint16_t>> m_stack;
+};
+
+class FrameStasher : public Stasher {
+public:
+    void backup() override {
+        try {
+            m_frames = std::make_unique<std::vector<CallFrame>>(frames, fp);
+        } catch (const std::bad_alloc &) {
+            m_frames.reset();
+        }
+    }
+
+    bool restore() override {
+        if (m_frames == nullptr) {
+            return false;
+        }
+
+        fp = BASE_OF_FRAMES + m_frames->size();
+        std::copy(m_frames->begin(), m_frames->end(), frames);
+
+        return true;
+    }
+
+    void free() override {
+        m_frames.reset();
+    }
+
+private:
+    std::unique_ptr<std::vector<CallFrame>> m_frames;
+};
 
 // Replace with std::clamp when switching to C++17.
 template <typename T>
@@ -1608,20 +1660,20 @@ void init_stack(bool first_run)
         }
         TOP_OF_STACK = &stack[options.eval_stack_size];
 
-        options.call_stack_size = clamp<size_t>(options.call_stack_size, 1, std::min<size_t>(0xffff, (SIZE_MAX / sizeof *frames) - sizeof *frames));
+        options.call_stack_size = clamp<size_t>(options.call_stack_size, 1, std::min<size_t>(0xffff, (SIZE_MAX / sizeof *frames) - 1));
         try {
-            // One extra to help with saving (thus the subtraction of
-            // sizeof *frames above).
+            // One extra to help with saving (thus the subtraction of 1
+            // above).
             frames = new CallFrame[options.call_stack_size + 1];
         } catch (std::bad_alloc &) {
             die("unable to allocate %lu bytes for the call stack", (options.call_stack_size + 1) * static_cast<unsigned long>(sizeof *frames));
         }
         TOP_OF_FRAMES = &frames[options.call_stack_size];
 
-        stash_register(args_stash_backup, args_stash_restore, args_stash_free);
-        stash_register(memory_stash_backup, memory_stash_restore, memory_stash_free);
-        stash_register(stack_stash_backup, stack_stash_restore, stack_stash_free);
-        stash_register(frames_stash_backup, frames_stash_restore, frames_stash_free);
+        stash_register(std::make_unique<ArgsStasher>());
+        stash_register(std::make_unique<MemoryStasher>());
+        stash_register(std::make_unique<StackStasher>());
+        stash_register(std::make_unique<FrameStasher>());
     }
 
     sp = BASE_OF_STACK;
