@@ -4,7 +4,7 @@
 #define ZTERP_STACK_H
 
 #include <stdexcept>
-#include <string>
+#include <vector>
 
 #include "types.h"
 
@@ -13,8 +13,7 @@ static constexpr unsigned long DEFAULT_CALL_DEPTH = 0x400;
 
 class RestoreError : public std::runtime_error {
 public:
-    explicit RestoreError(const std::string &msg) : std::runtime_error(msg) {
-    }
+    using std::runtime_error::runtime_error;
 };
 
 extern bool seen_save_undo;
@@ -23,24 +22,26 @@ void init_stack(bool first_run);
 
 uint16_t variable(uint16_t var);
 void store_variable(uint16_t var, uint16_t n);
-uint16_t *stack_top_element();
 
 void start_v6();
-#ifdef ZTERP_GLK
-uint16_t internal_call(uint16_t routine);
-#endif
+uint16_t internal_call(uint16_t routine, std::vector<uint16_t> args = {});
 void do_return(uint16_t retval);
 
 enum class SaveType {
     Normal,
     Meta,
     Autosave,
+    AutosaveLib,
 };
 
 enum class SaveOpcode {
     None = -1,
     Read = 0,
     ReadChar = 1,
+    // Yes, we can autosave in the middle of a @save or @restore! It’s the
+    // fileref prompt that does it, not the save operation.
+    Save = 2,
+    Restore = 3,
 };
 
 bool do_save(SaveType savetype, SaveOpcode saveopcode);
@@ -60,7 +61,7 @@ enum class SaveResult {
 SaveResult push_save(SaveStackType type, SaveType savetype, SaveOpcode saveopcode, const char *desc);
 bool pop_save(SaveStackType type, size_t saveno, SaveOpcode &saveopcode);
 bool drop_save(SaveStackType type, size_t i);
-void list_saves(SaveStackType type, void (*printer)(const std::string &));
+void list_saves(SaveStackType type);
 
 void zpush();
 void zpull();
