@@ -4,7 +4,10 @@
 #define ZTERP_UTIL_H
 
 #include <cstdarg>
+#include <functional>
+#include <memory>
 #include <string>
+#include <type_traits>
 
 #include "types.h"
 
@@ -21,6 +24,17 @@
 #define zprintflike(f, a)
 #endif
 
+// Allow scoped enums to be used as hash keys.
+// NOTE: Remove when switching to C++17, as C++17 provides this.
+struct EnumClassHash {
+    template <typename T>
+    std::enable_if_t<std::is_enum<T>::value, std::size_t>
+    operator()(T enumValue) const {
+        using UnderlyingType = std::underlying_type_t<T>;
+        return std::hash<UnderlyingType>{}(static_cast<UnderlyingType>(enumValue));
+    }
+};
+
 int16_t as_signed(uint16_t n);
 
 #ifndef ZTERP_NO_SAFETY_CHECKS
@@ -40,19 +54,13 @@ zprintflike(1, 2)
 void die(const char *fmt, ...);
 void help();
 
-enum class ArgStatus {
-    Ok,
-    Help,
-    Fail,
-};
-
-extern ArgStatus arg_status;
-void process_arguments(int argc, char **argv);
 long parseint(const std::string &s, int base, bool &valid);
 std::string vstring(const char *fmt, std::va_list ap);
 zprintflike(1, 2)
 std::string fstring(const char *fmt, ...);
 std::string ltrim(const std::string &s);
 std::string rtrim(const std::string &s);
+void parse_grouped_file(std::ifstream &f, const std::function<void(const std::string &line, int lineno)> &callback);
+std::unique_ptr<std::string> zterp_getenv(const std::string &name);
 
 #endif
