@@ -1,37 +1,23 @@
-// Copyright 2009-2021 Chris Spiegel.
+// Copyright 2009-2024 Chris Spiegel.
 //
-// This file is part of Bocfel.
-//
-// Bocfel is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License, version
-// 2 or 3, as published by the Free Software Foundation.
-//
-// Bocfel is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Bocfel. If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: MIT
 
 #include <algorithm>
 #include <array>
 #include <climits>
-#include <clocale>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include <fstream>
+#include <exception>
 #include <iomanip>
 #include <ios>
 #include <iostream>
-#include <istream>
-#include <map>
+#include <memory>
 #include <new>
 #include <set>
 #include <sstream>
-#include <stdexcept>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "zterp.h"
@@ -41,6 +27,7 @@
 #include "io.h"
 #include "memory.h"
 #include "meta.h"
+#include "options.h"
 #include "osdep.h"
 #include "patches.h"
 #include "process.h"
@@ -56,8 +43,9 @@
 #include <glk.h>
 #endif
 
+using namespace std::literals;
+
 std::string game_file;
-Options options;
 
 static std::string story_id;
 
@@ -70,11 +58,7 @@ const std::string &get_story_id()
 //
 // Z-machine versions 7 and 8 are identical to version 5 but for a
 // couple of tiny details. They are thus classified as version 5.
-//
-// zwhich stores the actual version (1–8) for the few rare times where
-// this knowledge is necessary.
 int zversion;
-static int zwhich;
 
 Header header;
 
@@ -104,12 +88,12 @@ static unsigned long unpack_multiplier;
 
 uint32_t unpack_routine(uint16_t addr)
 {
-    return (addr * unpack_multiplier) + header.R_O;
+    return (addr * unpack_multiplier) + header.routines_offset;
 }
 
 uint32_t unpack_string(uint16_t addr)
 {
-    return (addr * unpack_multiplier) + header.S_O;
+    return (addr * unpack_multiplier) + header.strings_offset;
 }
 
 void store(uint16_t v)
@@ -123,16 +107,35 @@ static void initialize_games()
 {
     // All V1234 games from Andrew Plotkin’s Obsessively Complete
     // Infocom Catalog: https://eblong.com/infocom/
-    std::set<std::string> infocom1234 = {
+    static const std::set<std::string> infocom1234 = {
 #include "v1234.h"
     };
 
-    std::map<Game, std::set<std::string>> gamemap = {
+    static const std::set<std::string> mysterious = {
+        "1-041209-e04a",
+        "1-041209-831b",
+        "1-041209-beb7",
+        "1-041209-c3e0",
+        "1-041209-2003",
+        "1-041209-f23c",
+        "1-041209-e38a",
+        "1-041209-90d8",
+        "1-041209-e644",
+        "1-041209-480b",
+        "1-041209-b1de",
+    };
+
+    static const std::vector<std::pair<Game, std::set<std::string>>> gamemap = {
         { Game::Infocom1234, infocom1234 },
-        { Game::Journey, { "83-890706" } },
+        { Game::Arthur, { "54-890606", "63-890622", "74-890714" } },
+        { Game::Journey, { "26-890316", "30-890322", "77-890616", "83-890706" } },
         { Game::LurkingHorror, { "203-870506", "219-870912", "221-870918" } },
         { Game::Planetfall, { "1-830517", "20-830708", "26-831014", "29-840118", "37-851003", "39-880501" } },
+        { Game::Shogun, { "292-890314", "295-890321", "311-890510", "322-890706" } },
         { Game::Stationfall, { "1-861017", "63-870218", "87-870326", "107-870430" } },
+        { Game::ZorkZero, { "296-881019", "366-890323", "383-890602", "393-890714" } },
+        { Game::ZorkZeroDOS, { "393-890714" } },
+        { Game::MysteriousAdventures, mysterious },
     };
 
     for (const auto &pair : gamemap) {
@@ -166,7 +169,7 @@ static void find_id()
 
     std::ostringstream ss;
     ss << header.release << "-" << serial;
-    if (std::strchr("012345679", serial[0]) != nullptr &&
+    if ("012345679"s.find(serial[0]) != std::string::npos &&
         serial != "000000" &&
         serial != "999999" &&
         serial != "------") {
@@ -178,233 +181,12 @@ static void find_id()
     initialize_games();
 }
 
-static void read_config()
-{
-    std::string line;
-    bool story_matches = true;
-
-    auto file = zterp_os_rcfile(false);
-    if (file == nullptr) {
-        return;
-    }
-
-    std::ifstream f(*file);
-    if (!f.is_open()) {
-        return;
-    }
-
-    int lineno = 0;
-    while (std::getline(f, line)) {
-        lineno++;
-
-        line = ltrim(line);
-
-        auto comment = line.find('#');
-        if (comment != std::string::npos) {
-            line.erase(comment);
-        }
-        line = rtrim(line);
-
-        if (line.empty()) {
-            continue;
-        }
-
-        if (line[0] == '[' && line.back() == ']') {
-            std::string id = line.substr(1, line.size() - 2);
-            story_matches = id == story_id;
-
-            continue;
-        }
-
-        if (!story_matches) {
-            continue;
-        }
-
-        auto equal = line.find('=');
-        if (equal == std::string::npos) {
-            std::cerr << *file << ":" << lineno << ": expected '='" << std::endl;
-            continue;
-        }
-
-        auto key = rtrim(line.substr(0, equal));
-        if (key.empty()) {
-            std::cerr << *file << ":" << lineno << ": no key" << std::endl;
-            continue;
-        }
-
-        auto val = ltrim(line.substr(equal + 1));
-        if (val.empty()) {
-            std::cerr << *file << ":" << lineno << ": no value" << std::endl;
-            continue;
-        }
-
-        try {
-            std::map<std::string, std::string> legacy_names = {
-                {"notes_editor", "editor"},
-                {"max_saves", "undo_slots"},
-            };
-
-            auto newkey = legacy_names.at(key);
-            std::cerr << *file << ":" << lineno << ": warning: configuration option " << key;
-            if (newkey.empty()) {
-                std::cerr << " no longer exists" << std::endl;
-                continue;
-            } else {
-                std::cerr << " is deprecated; it has been renamed " << newkey << std::endl;
-            }
-            key = newkey;
-        } catch (const std::out_of_range &) {
-        }
-
-#define START()		if (false) do { } while (false)
-
-#define BOOL(name)	else if (key == #name) do { \
-    if (val == "0") { \
-        options.name = false; \
-    } else if (val == "1") { \
-        options.name = true; \
-    } else { \
-        std::cerr << *file << ":" << lineno << ": invalid value for " << #name << ": must be 0 or 1" << std::endl; \
-    } \
-} while (false)
-
-#define NUMHELP(name, wrapper)	else if (key == #name) do { \
-    char *endptr; \
-    unsigned long n; \
-    n = std::strtoul(val.c_str(), &endptr, 10); \
-    if (val[0] == '-' || *endptr != 0) { \
-        std::cerr << *file << ":" << lineno << ": invalid value for " << #name << ": must be a non-negative number" << std::endl; \
-    } else { \
-        options.name = wrapper(n); \
-    } \
-} while (false)
-
-#define NUMBER(name)	NUMHELP(name, [](unsigned long num) { return num; })
-
-#define OPTNUM(name)	NUMHELP(name, std::make_unique<unsigned long>)
-
-#define STRING(name)	else if (key == #name) do { \
-    options.name = std::make_unique<std::string>(val); \
-} while (false)
-
-#define CHAR(name)	else if (key == #name) do { \
-    if (val.size() != 1) { \
-        std::cerr << *file << ":" << lineno << ": invalid value for " << #name << ": must be a single character" << std::endl; \
-    } else { \
-        options.name = val[0]; \
-    } \
-} while (false)
-
-#ifdef GLK_MODULE_GARGLKTEXT
-#define COLOR(name, num)else if (key == "color_" #name) do { \
-    char *endptr; \
-    unsigned long color; \
-    color = std::strtoul(val.c_str(), &endptr, 16); \
-    if (val[0] == '-' || *endptr != 0 || color > 0xffffff) { \
-        std::cerr << *file << ":" << lineno << ": invalid value for color_" << #name << ": must be a hexadecimal number ranging from 0x000000 to 0xffffff" << std::endl; \
-    } else { \
-        update_color(num, color); \
-    } \
-} while (false)
-#else
-#define COLOR(name, num)else if (key == "color_" #name) do { } while (false)
-#endif
-
-        // Some configuration options are available only in certain
-        // compile-come configurations. For those options, silently
-        // ignore them in configurations where they are not valid. This
-        // allows configuration files to be shared between such builds
-        // without dispaying diagnostics for options which are valid in
-        // some builds but not others.
-
-        START();
-
-        NUMBER(eval_stack_size);
-        NUMBER(call_stack_size);
-        BOOL  (disable_color);
-        BOOL  (disable_timed);
-        BOOL  (disable_sound);
-        BOOL  (enable_escape);
-        STRING(escape_string);
-        BOOL  (disable_fixed);
-        BOOL  (assume_fixed);
-        BOOL  (disable_graphics_font);
-        BOOL  (enable_alt_graphics);
-        BOOL  (disable_history_playback);
-        BOOL  (disable_term_keys);
-        STRING(username);
-        BOOL  (disable_meta_commands);
-        NUMBER(undo_slots);
-        NUMBER(int_number);
-        CHAR  (int_version);
-        BOOL  (disable_patches);
-        BOOL  (replay_on);
-        STRING(replay_name);
-        BOOL  (record_on);
-        STRING(record_name);
-        BOOL  (transcript_on);
-        STRING(transcript_name);
-        BOOL  (disable_abbreviations);
-        BOOL  (enable_censorship);
-        BOOL  (overwrite_transcript);
-        BOOL  (override_undo);
-        OPTNUM(random_seed);
-        STRING(random_device);
-
-        BOOL(autosave);
-        BOOL(persistent_transcript);
-        STRING(editor);
-
-        COLOR(black,   2);
-        COLOR(red,     3);
-        COLOR(green,   4);
-        COLOR(yellow,  5);
-        COLOR(blue,    6);
-        COLOR(magenta, 7);
-        COLOR(cyan,    8);
-        COLOR(white,   9);
-
-        else if (key == "cheat") {
-#ifndef ZTERP_NO_CHEAT
-            if (!cheat_add(val, false)) {
-                std::cerr << *file << ":" << lineno << ": syntax error in cheat" << std::endl;
-            }
-#endif
-        }
-
-        else if (key == "patch") {
-            try {
-                apply_user_patch(val);
-            } catch (const PatchStatus::SyntaxError &) {
-                std::cerr << *file << ":" << lineno << ": syntax error in patch" << std::endl;
-            } catch (const PatchStatus::NotFound &) {
-                std::cerr << *file << ":" << lineno << ": patch does not apply to this story" << std::endl;
-            }
-        }
-
-        else {
-            std::cerr << *file << ":" << lineno << ": unknown configuration option: " << key << std::endl;
-        }
-
-#undef START
-#undef BOOL
-#undef NUMHELP
-#undef NUMBER
-#undef OPTNUM
-#undef STRING
-#undef CHAR
-#undef COLOR
-    }
-}
-
 static bool have_statuswin = false;
 static bool have_upperwin  = false;
 
 static void write_flags1()
 {
-    uint8_t flags1;
-
-    flags1 = byte(0x01);
+    uint8_t flags1 = byte(0x01);
 
     if (zversion == 3) {
         flags1 |= FLAGS1_NOSTATUS;
@@ -434,6 +216,11 @@ static void write_flags1()
 
         if (zversion == 6) {
             flags1 &= ~(FLAGS1_PICTURES | FLAGS1_SOUND);
+#if defined(GLK_MODULE_IMAGE) && defined(ZTERP_GLK_BLORB)
+            if (glk_gestalt(gestalt_Graphics, 0)) {
+                flags1 |= FLAGS1_PICTURES;
+            }
+#endif
             if (sound_loaded()) {
                 flags1 |= FLAGS1_SOUND;
             }
@@ -540,10 +327,6 @@ void write_header()
     if (zversion >= 4) {
         unsigned int width, height;
 
-        // Interpreter number & version.
-        if (options.int_number < 1 || options.int_number > 11) {
-            options.int_number = DEFAULT_INT_NUMBER;
-        }
         store_byte(0x1e, options.int_number);
         store_byte(0x1f, options.int_version);
 
@@ -582,7 +365,7 @@ void write_header()
 static void process_alphabet_table()
 {
     if (zversion == 1) {
-        std::memcpy(&atable[26 * 2], " 0123456789.,!?_#'\"/\\<-:()", 26);
+        std::memcpy(&atable[26 * 2], R"#( 0123456789.,!?_#'"/\<-:()")#", 26);
     } else if (zversion >= 5 && word(0x34) != 0) {
         if (word(0x34) + 26 * 3 > memory_size) {
             die("corrupted story: alphabet table out of range");
@@ -665,22 +448,32 @@ static void process_story(IO &io, long offset)
     }
 
     try {
-        io.read_exact(memory, memory_size);
+        io.read_exact(memory.data(), memory_size);
     } catch (const IO::IOError &) {
         die("unable to read from story file");
     }
 
     zversion = byte(0x00);
-    if (zversion < 1 || zversion > 8) {
-        die("only z-code versions 1-8 are supported");
-    }
 
-    zwhich = zversion;
+#ifdef ZTERP_NO_V6
+    if (zversion == 6) {
+        die("support for version 6 has been disabled in this build");
+    }
+#endif
+
+    // The actual Z-machine version (differentiating 7 and 8 from 5) is
+    // only needed in this function, and only for 3 things:
+    //
+    // • The unpack multiplier
+    // • File length
+    // • Routine & string offsets for V6/7
+    auto actual_zversion = zversion;
+
     if (zversion == 7 || zversion == 8) {
         zversion = 5;
     }
 
-    switch (zwhich) {
+    switch (actual_zversion) {
     case 1: case 2: case 3:
         unpack_multiplier = 2;
         break;
@@ -691,7 +484,7 @@ static void process_story(IO &io, long offset)
         unpack_multiplier = 8;
         break;
     default:
-        die("unhandled z-machine version: %d", zwhich);
+        die("only z-code versions 1-8 are supported");
     }
 
     header.pc = word(0x06);
@@ -754,7 +547,7 @@ static void process_story(IO &io, long offset)
         die("corrupted story: abbreviation table out of range");
     }
 
-    header.file_length = word(0x1a) * (zwhich <= 3 ? 2UL : zwhich <= 5 ? 4UL : 8UL);
+    header.file_length = word(0x1a) * (actual_zversion <= 3 ? 2UL : actual_zversion <= 5 ? 4UL : 8UL);
     if (header.file_length > memory_size) {
         die("story's reported size (%lu) greater than file size (%lu)", static_cast<unsigned long>(header.file_length), static_cast<unsigned long>(memory_size));
     }
@@ -763,27 +556,21 @@ static void process_story(IO &io, long offset)
 
     calculate_checksum(io, offset);
 
-    if (zwhich == 6 || zwhich == 7) {
-        header.R_O = word(0x28) * 8UL;
-        header.S_O = word(0x2a) * 8UL;
+    if (actual_zversion == 6 || actual_zversion == 7) {
+        header.routines_offset = word(0x28) * 8UL;
+        header.strings_offset = word(0x2a) * 8UL;
     }
 
     if (zversion >= 5 && !options.disable_term_keys) {
         header.terminating_characters_table = word(0x2e);
     }
 
-    if (dynamic_memory == nullptr) {
-        try {
-            dynamic_memory = new uint8_t[header.static_start];
-        } catch (const std::bad_alloc &) {
-            die("unable to allocate memory for dynamic memory");
-        }
-        std::memcpy(dynamic_memory, memory, header.static_start);
+    try {
+        dynamic_memory.assign(memory.begin(), memory.begin() + header.static_start);
+    } catch (const std::bad_alloc &) {
+        die("unable to allocate memory for dynamic memory");
     }
 
-#ifdef ZTERP_GLK
-#endif
-
     process_alphabet_table();
     read_header_extension_table();
 
@@ -791,9 +578,12 @@ static void process_story(IO &io, long offset)
     // story is known, and the ID of the current story is not known until
     // the file has been processed; so do both of those here.
     find_id();
+#ifndef ZTERP_NO_OPTIONS
     if (!options.disable_config) {
-        read_config();
+        options.read_config();
     }
+    options.read_envvars();
+#endif
 
     // Most options directly set their respective variables, but a few
     // require intervention. Delay that intervention until here so that
@@ -811,6 +601,25 @@ static void process_story(IO &io, long offset)
         apply_patches();
     }
 
+    // This is a slightly “hidden” feature to allow consumers of Bocfel
+    // (e.g. Gargoyle and Spatterlight) to provide their own patches for
+    // games. If the environment variable $BOCFEL_PATCH_FILE is set,
+    // patches will be loaded from that file. Otherwise, if a patch file
+    // path was selected at compile time, it will be used.
+    const char *patch_file = std::getenv("BOCFEL_PATCH_FILE");
+    if (patch_file != nullptr) {
+        patch_load_file(patch_file);
+    } else {
+#ifdef ZTERP_PATCH_FILE
+        patch_load_file(ZTERP_PATCH_FILE);
+#endif
+    }
+
+    if (!options.disable_v6_hacks) {
+        create_graphicswin();
+        apply_v6_patches();
+    }
+
     if (zversion <= 3) {
         have_statuswin = create_statuswin();
     }
@@ -834,7 +643,6 @@ static void process_story(IO &io, long offset)
 
 void start_story()
 {
-    uint16_t flags2;
     static bool first_run = true;
 
     pc = header.pc;
@@ -844,8 +652,8 @@ void start_story()
     // used at the initial program start, but Flags2 should be preserved
     // there, as well: the pictures bit and the transcript bit might
     // have been set during story processing, and those should persist.
-    flags2 = word(0x10);
-    std::memcpy(memory, dynamic_memory, header.static_start);
+    uint16_t flags2 = word(0x10);
+    std::copy(dynamic_memory.begin(), dynamic_memory.begin() + header.static_start, memory.begin());
     store_word(0x10, flags2);
 
     write_header();
@@ -899,7 +707,7 @@ void zverify()
     branch_if(checksum_verified);
 }
 
-static std::shared_ptr<IO> open_savefile(IO::Mode mode)
+static IO open_savefile(IO::Mode mode)
 {
     // When this is null, the user is prompted for a filename.
     std::unique_ptr<std::string> suggested;
@@ -912,37 +720,32 @@ static std::shared_ptr<IO> open_savefile(IO::Mode mode)
         for (size_t i = 0; i < user_byte(addr); i++) {
             uint8_t c = user_byte(addr + i + 1);
 
-            // Convert to uppercase per §7.6.1.1; and according to the
-            // description for @save in §15, the characters are ASCII.
-            if (c >= 97 && c <= 122) {
-                c -= 32;
+            // Convert non-printable (ASCII) characters to underscores.
+            if (c >= 32 && c <= 126) {
+                filename.push_back(c);
+            } else {
+                filename.push_back('_');
             }
-
-            filename.push_back(c);
         }
 
-        if (!filename.empty()) {
-            if (filename.find('.') == std::string::npos) {
-                filename += ".AUX";
-            }
+        if (filename.empty()) {
+            filename = "NULL";
+        }
 
-            auto aux = zterp_os_aux_file(filename);
-            if (aux != nullptr) {
-                suggested = std::make_unique<std::string>(*aux);
-            }
+        auto aux = zterp_os_aux_file(filename);
+        if (aux != nullptr) {
+            suggested = std::make_unique<std::string>(*aux);
         }
     }
 
     // If there is a suggested filename and “prompt” is 1, this should
     // prompt the user with the suggested filename, but Glk doesn’t
     // support that.
-    return std::make_shared<IO>(suggested.get(), mode, IO::Purpose::Data);
+    return IO(suggested.get(), mode, IO::Purpose::Data);
 }
 
 void zsave5()
 {
-    std::shared_ptr<IO> savefile;
-
     if (znargs == 0) {
         zsave();
         return;
@@ -951,26 +754,16 @@ void zsave5()
     ZASSERT(zargs[0] + zargs[1] < memory_size, "attempt to save beyond the end of memory");
 
     try {
-        savefile = open_savefile(IO::Mode::WriteOnly);
-    } catch (const IO::OpenError &) {
-        store(0);
-        return;
-    }
-
-    try {
-        savefile->write_exact(&memory[zargs[0]], zargs[1]);
+        auto savefile = open_savefile(IO::Mode::WriteOnly);
+        savefile.write_exact(&memory[zargs[0]], zargs[1]);
         store(1);
-    } catch (const IO::IOError &) {
+    } catch (const IO::Error &) {
         store(0);
     }
 }
 
 void zrestore5()
 {
-    std::shared_ptr<IO> savefile;
-    std::vector<uint8_t> buf;
-    size_t n;
-
     if (znargs == 0) {
         zrestore();
         return;
@@ -982,25 +775,20 @@ void zrestore5()
     }
 
     try {
-        savefile = open_savefile(IO::Mode::ReadOnly);
+        auto savefile = open_savefile(IO::Mode::ReadOnly);
+        std::vector<uint8_t> buf(zargs[1]);
+
+        size_t n = savefile.read(buf.data(), zargs[1]);
+        for (size_t i = 0; i < n; i++) {
+            user_store_byte(zargs[0] + i, buf[i]);
+        }
+
+        store(n);
     } catch (const IO::OpenError &) {
         store(0);
-        return;
-    }
-
-    try {
-        buf.resize(zargs[1]);
     } catch (const std::bad_alloc &) {
         store(0);
-        return;
     }
-
-    n = savefile->read(buf.data(), zargs[1]);
-    for (size_t i = 0; i < n; i++) {
-        user_store_byte(zargs[0] + i, buf[i]);
-    }
-
-    store(n);
 }
 
 void zpiracy()
@@ -1019,17 +807,6 @@ static void real_main(int argc, char **argv)
         long offset = 0;
     } story;
 
-    // strftime() is given the %c conversion specification, which is
-    // locale-aware, so honor the user’s time locale.
-    std::setlocale(LC_TIME, "");
-
-    // It’s too early to properly set up all tables (neither the alphabet
-    // nor Unicode table has been read from the story file), but it’s
-    // possible for messages to be displayed to the user before a story is
-    // even loaded, so at least the basic tables need to be created so
-    // that non-Unicode platforms have proper translations available.
-    setup_tables();
-
 #ifdef ZTERP_GLK
     if (!create_mainwin()) {
         return;
@@ -1037,18 +814,38 @@ static void real_main(int argc, char **argv)
 #endif
 
 #ifndef ZTERP_GLK
-    process_arguments(argc, argv);
+    options.process_arguments(argc, argv);
 #endif
 
-    if (arg_status != ArgStatus::Ok) {
-        help();
-        if (arg_status == ArgStatus::Help) {
-            throw Exit(0);
-        } else {
-            throw Exit(EXIT_FAILURE);
+    if (arg_status.any()) {
+#ifdef ZTERP_GLK
+        glk_set_style(style_Preformatted);
+#endif
+
+#ifndef ZTERP_NO_OPTIONS
+        for (const auto &error : options.errors()) {
+            screen_puts(error);
         }
+
+        if (arg_status.test(ArgStatus::BadOption)) {
+            if (!options.errors().empty()) {
+                screen_puts("");
+            }
+
+            options.help();
+        }
+#endif
+
+        throw Exit(EXIT_FAILURE);
     }
 
+#ifndef ZTERP_NO_OPTIONS
+    if (options.show_help) {
+        options.help();
+        throw Exit(0);
+    }
+#endif
+
 #ifndef ZTERP_GLK
     zterp_os_init_term();
 #endif
@@ -1070,14 +867,13 @@ static void real_main(int argc, char **argv)
         if (config != nullptr) {
             screen_printf("Configuration file: %s\n", config->c_str());
         } else {
-            screen_printf("Cannot determine configuration file location\n");
+            screen_puts("Cannot determine configuration file location");
         }
 
         throw Exit(0);
     }
 
     if (game_file.empty()) {
-        help();
         die("no story provided");
     }
 
@@ -1092,10 +888,10 @@ static void real_main(int argc, char **argv)
 
         const auto *chunk = blorb.find(Blorb::Usage::Exec, 0);
         if (chunk == nullptr) {
-            die("no EXEC resource found");
+            die("no Exec resource found");
         }
-        if (chunk->type != IFF::TypeID(&"ZCOD")) {
-            if (chunk->type == IFF::TypeID(&"GLUL")) {
+        if (chunk->type != IFF::TypeID("ZCOD")) {
+            if (chunk->type == IFF::TypeID("GLUL")) {
                 die("Glulx stories are not supported (try git or glulxe)");
             }
 
@@ -1152,11 +948,11 @@ static void real_main(int argc, char **argv)
     // 2OP, requiring two more bytes to be read. At this point the opcode
     // will be looked up, resulting in an illegal instruction error.
     try {
-        memory = new uint8_t[memory_size + 22];
+        memory.resize(memory_size + 22);
     } catch (const std::bad_alloc &) {
         die("unable to allocate memory for story file");
     }
-    memset(memory + memory_size, 0, 22);
+    std::fill(memory.begin() + memory_size, memory.begin() + memory_size + 22, 0);
 
     process_story(*story.io, story.offset);
 
@@ -1172,6 +968,10 @@ static void real_main(int argc, char **argv)
         // story or by the user, this will activate them.
         user_store_word(0x10, word(0x10));
 
+        if (zversion == 6 && options.warn_on_v6) {
+            show_message("Version 6 of the Z-machine is only partially supported. Be aware that the game might not function properly.");
+        }
+
         setup_opcodes();
         process_loop();
     }
@@ -1183,24 +983,29 @@ void glk_main()
 int main(int argc, char **argv)
 #endif
 {
+    std::set_terminate([]() {
+        try {
+            std::rethrow_exception(std::current_exception());
+        } catch (const std::exception &e) {
+            std::cerr << "Unhandled exception: " << e.what() << std::endl;
+        } catch (...) {
+            std::cerr << "Unhandled exception" << std::endl;
+        }
+
+        std::abort();
+    });
+
 #ifdef ZTERP_GLK
     try {
         real_main();
     } catch (const Exit &) {
+    }
 #else
     try {
         real_main(argc, argv);
         return 0;
     } catch (const Exit &exit) {
         return exit.code();
-#endif
-    } catch (const std::exception &e) {
-        std::cerr << "Unhandled exception: " << e.what() << std::endl;
-    } catch (...) {
-        std::cerr << "Unhandled exception" << std::endl;
     }
-
-#ifndef ZTERP_GLK
-    return EXIT_FAILURE;
 #endif
 }
