@@ -3,7 +3,7 @@
 * Magnetic - Magnetic Scrolls Interpreter.
 *
 * Written by Niclas Karlsson <nkarlsso@abo.fi>,
-*            David Kinder <davidk.kinder@virgin.net>,
+*            David Kinder <davidk@davidkinder.co.uk>,
 *            Stefan Meier <Stefan.Meier@if-legends.org> and
 *            Paul David Doherty <pdd@if-legends.org>
 *
@@ -21,7 +21,7 @@
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
-*     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 \****************************************************************************/
 
@@ -38,15 +38,7 @@
 #include <stdint.h>
 
 typedef uint8_t type8;
-// The name implies a signed type, but this appears to generally be used
-// where "char" normally would (i.e. for C strings); that is, the fact
-// that it is signed does not appear to matter, and making it signed
-// causes all sorts of diagnostics due to the fact that "signed char"
-// and "char" are different types, even if "char" is signed. In any
-// case, making it "char" causes a lot fewer warnings, and regardless of
-// whether it's signed or plain char, the "solution" will be casting,
-// and this requires fewer casts.
-typedef char type8s;
+typedef int8_t type8s;
 
 typedef uint16_t type16;
 typedef int16_t type16s;
@@ -341,7 +333,7 @@ struct ms_hint
 {
   type16  elcount;
   type16  nodetype;
-  type8s * content;
+  type8 * content;
   type16  links[MAX_HITEMS];
   type16  parent;
 };
