@@ -5,17 +5,17 @@
 //  Created by Petter Sjölund on 2022-06-04.
 //
 
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include "glk.h"
 
 #include "common.h"
 #include "definitions.h"
-#include "restorestate.h"
 #include "extracommands.h"
 #include "parseinput.h"
+#include "restorestate.h"
 
 char **InputWordStrings = NULL;
 int WordsInInput = 0;
@@ -28,9 +28,10 @@ DictWord *Nouns;
 DictWord *Adverbs;
 DictWord *Prepositions;
 
-int LastVerb = 0, LastNoun = 0, LastPrep = 0, LastPartp = 0, LastNoun2 = 0, LastAdverb = 0;
+int CurrentVerb = 0, CurrentNoun = 0, CurrentPrep = 0, CurrentPartp = 0, CurrentNoun2 = 0, CurrentAdverb = 0, LastVerb = 0, LastNoun = 0, LastPrep = 0, LastPartp = 0, LastNoun2 = 0, LastAdverb = 0;
 
-void FreeInputWords(void) {
+void FreeInputWords(void)
+{
     WordIndex = 0;
     if (InputWordStrings != NULL) {
         for (int i = 0; i < WordsInInput && InputWordStrings[i] != NULL; i++) {
@@ -43,7 +44,8 @@ void FreeInputWords(void) {
     WordsInInput = 0;
 }
 
-int CompareUpToHashSign(char *word1, char *word2) {
+int CompareUpToHashSign(char *word1, char *word2)
+{
     size_t len1 = strlen(word1);
     size_t len2 = strlen(word2);
     size_t longest = MAX(len1, len2);
@@ -58,7 +60,8 @@ int CompareUpToHashSign(char *word1, char *word2) {
     return 1;
 }
 
-int YesOrNo(void) {
+int YesOrNo(void)
+{
     glk_request_char_event(Bottom);
 
     event_t ev;
@@ -136,10 +139,11 @@ static int MatchingChars(const char *synonym, const char *original)
             return 0;
     }
 
-    return (i == strlen(synonym) && (original[i] == 0 || original[i] == ' ' || (i > 0 && original[i-1] == 0)));
+    return (i == strlen(synonym) && (original[i] == 0 || original[i] == ' ' || (i > 0 && original[i - 1] == 0)));
 }
 
-static int IsSynonymMatch(const char *synonym, const char *original) {
+static int IsSynonymMatch(const char *synonym, const char *original)
+{
     char c = synonym[0];
     if (c == 0)
         return -1;
@@ -154,13 +158,13 @@ static int IsSynonymMatch(const char *synonym, const char *original) {
     return -1;
 }
 
-static char *StripDoubleSpaces(char *result, int *len) {
+static char *StripDoubleSpaces(char *result, int *len)
+{
     int found = 0;
     do {
         found = 0;
         for (int i = 0; i < *len - 1; i++) {
-            if (isspace(result[i]) && isspace(result[i + 1]))
-            {
+            if (isspace(result[i]) && isspace(result[i + 1])) {
                 found = 1;
                 char new[512];
                 if (i > 0)
@@ -182,11 +186,11 @@ static char *StripDoubleSpaces(char *result, int *len) {
     return result;
 }
 
-static char *StripPunctuation(char *buf, int *len) {
+static char *StripPunctuation(char *buf, int *len)
+{
     for (int i = 0; i < *len; i++) {
         char c = buf[i];
-        if (c == '.' || c == ',' || c == ':' || c == ';' || c == '\'' || c == '\"' || c == '-' || c == '!')
-        {
+        if (c == '.' || c == ',' || c == ':' || c == ';' || c == '\'' || c == '\"' || c == '-' || c == '!') {
             buf[i] = ' ';
         }
     }
@@ -196,7 +200,6 @@ static char *StripPunctuation(char *buf, int *len) {
     return final;
 }
 
-
 static char *ConcatStrings(const char *head, int headlen, const char *mid, int midlen, const char *tail, int taillen)
 {
     int totallen = headlen + midlen + taillen;
@@ -210,7 +213,6 @@ static char *ConcatStrings(const char *head, int headlen, const char *mid, int m
     return result;
 }
 
-
 static char *ReplaceString(char *source, const char *pattern, const char *replacement, int *startpos)
 {
     char *result = NULL;
@@ -241,7 +243,7 @@ static char *ReplaceString(char *source, const char *pattern, const char *replac
 static char *ReplaceSynonyms(char *buf, int *len)
 {
     char *result = MemAlloc(*len + 1);
-    for(int i = 0; i < *len; i++)
+    for (int i = 0; i < *len; i++)
         result[i] = toupper(buf[i]);
     result = StripDoubleSpaces(result, len);
     int finallen = (int)strlen(result);
@@ -314,7 +316,8 @@ static char **SplitIntoWords(const char *string, int length)
 
 WordToken *TokenWords = NULL;
 
-static int IntendedAsVerb(int i) {
+static int IntendedAsVerb(int i)
+{
     if (i == InitialIndex || i == 0)
         return 1;
     /* if the word before was an adverb and the one before that was not a verb */
@@ -323,7 +326,8 @@ static int IntendedAsVerb(int i) {
     return 0;
 }
 
-static void WordNotFoundError(int i) {
+static void WordNotFoundError(int i)
+{
     if (TokenWords[i].Type == ADVERB_TYPE)
         Display(Bottom, "%s do what?\n", InputWordStrings[i]);
     else if (IntendedAsVerb(i))
@@ -332,7 +336,8 @@ static void WordNotFoundError(int i) {
         Display(Bottom, "I don't know what %s means.\n", InputWordStrings[i]);
 }
 
-static int TokenizeInputWords(void) {
+static int TokenizeInputWords(void)
+{
     if (TokenWords != NULL)
         free(TokenWords);
     int word_not_found = -1;
@@ -445,23 +450,22 @@ static int TokenizeInputWords(void) {
     return 1;
 }
 
-int IsNextParticiple(int partp, int noun2) {
+int IsNextParticiple(int partp, int noun2)
+{
     if (WordIndex >= WordsInInput) {
         return 0;
     }
     if (partp == 1) { // None
         return 0;
     }
-    if (TokenWords[WordIndex].Type == NOUN_TYPE && partp == 2 &&
-        (noun2 == 0 || TokenWords[WordIndex].Index == noun2)) {
-        CurPartp = 2;
-        CurNoun2 = TokenWords[WordIndex++].Index;
+    if (TokenWords[WordIndex].Type == NOUN_TYPE && partp == 2 && (noun2 == 0 || TokenWords[WordIndex].Index == noun2)) {
+        CurrentPartp = 2;
+        CurrentNoun2 = TokenWords[WordIndex++].Index;
         return 1;
-    } else if (TokenWords[WordIndex].Type == PREPOSITION_TYPE && WordIndex < WordsInInput - 1 &&
-        (partp == 0 || TokenWords[WordIndex].Index == partp)) {
+    } else if (TokenWords[WordIndex].Type == PREPOSITION_TYPE && WordIndex < WordsInInput - 1 && (partp == 0 || TokenWords[WordIndex].Index == partp)) {
         if (TokenWords[WordIndex + 1].Type == NOUN_TYPE && (noun2 == 0 || TokenWords[WordIndex + 1].Index == noun2)) {
-            CurPartp = TokenWords[WordIndex++].Index;
-            CurNoun2 = TokenWords[WordIndex++].Index;
+            CurrentPartp = TokenWords[WordIndex++].Index;
+            CurrentNoun2 = TokenWords[WordIndex++].Index;
             return 1;
         }
     }
@@ -513,20 +517,20 @@ static int CommandFromTokens(int verb, int noun)
         }
     }
 
-    CurPartp = 0;
-    CurPrep = 0;
-    CurNoun2 = 0;
-    CurAdverb = 0;
+    CurrentPartp = 0;
+    CurrentPrep = 0;
+    CurrentNoun2 = 0;
+    CurrentAdverb = 0;
 
     if (TokenWords[WordIndex].Type == VERB_TYPE) {
         verb = TokenWords[WordIndex++].Index;
     }
 
     if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == ADVERB_TYPE) {
-        CurAdverb = TokenWords[WordIndex++].Index;
+        CurrentAdverb = TokenWords[WordIndex++].Index;
         debug_print("Found adverb \"");
-        PrintDictWord(CurAdverb, Adverbs);
-        debug_print("\" (%d)\n", CurAdverb);
+        PrintDictWord(CurrentAdverb, Adverbs);
+        debug_print("\" (%d)\n", CurrentAdverb);
         if (WordIndex < WordsInInput && verb == 0 && TokenWords[WordIndex].Type == VERB_TYPE) {
             verb = TokenWords[WordIndex++].Index;
         }
@@ -545,8 +549,8 @@ static int CommandFromTokens(int verb, int noun)
     debug_print("\" (%d)\n", verb);
 
     if (WordIndex >= WordsInInput) {
-        CurVerb = verb;
-        CurNoun = noun;
+        CurrentVerb = verb;
+        CurrentNoun = noun;
         return 0;
     }
 
@@ -556,17 +560,17 @@ static int CommandFromTokens(int verb, int noun)
 
     if (noun == 0) {
         if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == PREPOSITION_TYPE) {
-            CurPrep = TokenWords[WordIndex++].Index;
+            CurrentPrep = TokenWords[WordIndex++].Index;
             if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == NOUN_TYPE) {
                 noun = TokenWords[WordIndex++].Index;
             }
         }
     }
 
-    if (CurPrep != 0) {
+    if (CurrentPrep != 0) {
         debug_print("Found preposition \"");
-        PrintDictWord(CurPrep, Prepositions);
-        debug_print("\" (%d)\n", CurPrep);
+        PrintDictWord(CurrentPrep, Prepositions);
+        debug_print("\" (%d)\n", CurrentPrep);
     }
 
     if (noun != 0) {
@@ -576,18 +580,18 @@ static int CommandFromTokens(int verb, int noun)
     }
 
     if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == ADVERB_TYPE) {
-        CurAdverb = TokenWords[WordIndex++].Index;
-        if (CurAdverb > 0) {
+        CurrentAdverb = TokenWords[WordIndex++].Index;
+        if (CurrentAdverb > 0) {
             debug_print("Found adverb \"");
-            PrintDictWord(CurAdverb, Adverbs);
-            debug_print("\" (%d)\n", CurAdverb);
+            PrintDictWord(CurrentAdverb, Adverbs);
+            debug_print("\" (%d)\n", CurrentAdverb);
         }
     }
 
-    if (WordIndex < WordsInInput && CurPrep != 0) {
+    if (WordIndex < WordsInInput && CurrentPrep != 0) {
         if (TokenWords[WordIndex].Type == NOUN_TYPE) {
             if (noun != 0) {
-                CurNoun2 = TokenWords[WordIndex++].Index;
+                CurrentNoun2 = TokenWords[WordIndex++].Index;
             } else {
                 noun = TokenWords[WordIndex++].Index;
             }
@@ -595,28 +599,28 @@ static int CommandFromTokens(int verb, int noun)
     }
 
     if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == PREPOSITION_TYPE) {
-        CurPartp = TokenWords[WordIndex++].Index;
+        CurrentPartp = TokenWords[WordIndex++].Index;
         if (WordIndex < WordsInInput && TokenWords[WordIndex].Type == NOUN_TYPE) {
-            if (CurNoun2 != 0)
+            if (CurrentNoun2 != 0)
                 debug_print("Found a third noun. This should never happen.\n");
-            CurNoun2 = TokenWords[WordIndex++].Index;
+            CurrentNoun2 = TokenWords[WordIndex++].Index;
         } else {
-            CurPartp = CurPrep;
-            CurPrep = 0;
+            CurrentPartp = CurrentPrep;
+            CurrentPrep = 0;
         }
     }
 
-    if (CurPartp != 0) {
+    if (CurrentPartp != 0) {
         debug_print("Found participle \"");
-        PrintDictWord(CurPartp, Prepositions);
-        debug_print("\" (%d)\n", CurPartp);
+        PrintDictWord(CurrentPartp, Prepositions);
+        debug_print("\" (%d)\n", CurrentPartp);
     }
 
-    if (CurNoun2 != 0) {
+    if (CurrentNoun2 != 0) {
         debug_print("Found second noun \"");
-        PrintDictWord(CurNoun2, Nouns);
-        debug_print("\" (%d)\n", CurNoun2);
-    } else if (CurPartp != 0) {
+        PrintDictWord(CurrentNoun2, Nouns);
+        debug_print("\" (%d)\n", CurrentNoun2);
+    } else if (CurrentPartp != 0) {
         debug_print("Found participle but no second noun. This should never happen.\n");
     }
 
@@ -630,8 +634,8 @@ static int CommandFromTokens(int verb, int noun)
     else
         debug_print("No remaining input words.\n");
 
-    CurVerb = verb;
-    CurNoun = noun;
+    CurrentVerb = verb;
+    CurrentNoun = noun;
 
     return 0;
 }
@@ -658,11 +662,6 @@ static void LineInput(void)
         int length = ev.val1;
         buf[length] = 0;
 
-        if (Transcript) {
-            glk_put_string_stream(Transcript, buf);
-            glk_put_string_stream(Transcript, "\n");
-        }
-
         char *result = StripPunctuation(buf, &length);
         debug_print("After stripping punctuation: \"%s\"\n", result);
         result = ReplaceSynonyms(result, &length);
@@ -684,7 +683,8 @@ static void LineInput(void)
     } while (WordsInInput == 0 || InputWordStrings == NULL);
 }
 
-static int RemainderContainsVerb(void) {
+static int RemainderContainsVerb(void)
+{
     debug_print("RemainderContainsVerb: WordIndex: %d WordsInInput: %d\n", WordIndex, WordsInInput);
     if (WordIndex < WordsInInput - 1) {
         for (int i = WordIndex; i < WordsInInput; i++)
@@ -694,7 +694,8 @@ static int RemainderContainsVerb(void) {
     return 0;
 }
 
-void StopProcessingCommand(void) {
+void StopProcessingCommand(void)
+{
     if (RemainderContainsVerb()) {
         Output("\nThe rest of your input was ignored. ");
     }
@@ -726,7 +727,7 @@ int GetInput(void)
 
         result = CommandFromTokens(0, 0);
     } else {
-        result = CommandFromTokens(CurVerb, 0);
+        result = CommandFromTokens(CurrentVerb, 0);
     }
 
     return result;
