@@ -16,8 +16,11 @@
 
 #ifdef DEBUG
 
-#define debug_print(fmt, ...) \
-do { if (DEBUG_PRINT) fprintf(stderr, fmt, ##__VA_ARGS__); } while (0)
+#define debug_print(fmt, ...)                    \
+    do {                                         \
+        if (DEBUG_PRINT)                         \
+            fprintf(stderr, fmt, ##__VA_ARGS__); \
+    } while (0)
 
 #else
 
