@@ -9,9 +9,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "scottdefines.h"
 #include "parser.h"
 #include "scott.h"
+#include "scottdefines.h"
 
 #include "bsd.h"
 
@@ -19,7 +19,6 @@
 #define MAX_WORDS 128
 #define MAX_BUFFER 128
 
-extern strid_t Transcript;
 extern struct Command *CurrentCommand;
 
 glui32 **UnicodeWords = NULL;
@@ -30,7 +29,6 @@ static int lastnoun = 0;
 
 static glui32 *FirstErrorMessage = NULL;
 
-
 const char *EnglishDirections[NUMBER_OF_DIRECTIONS] = {
     NULL, "north", "south", "east", "west", "up", "down",
     "n", "s", "e", "w", "u", "d", " "
@@ -56,6 +54,7 @@ const char *ExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "load",
     "#restore",
     "transcript",
+    "#transcript",
     "script",
     "#script",
     "oops",
@@ -74,6 +73,7 @@ const char *ExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "#qsave",
     "except",
     "but",
+    "#flicker",
     "", "", "", "", ""
 };
 
@@ -87,6 +87,7 @@ const char *GermanExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "load",
     "#restore",
     "transcript",
+    "#transcript",
     "script",
     "#script",
     "oops",
@@ -105,6 +106,7 @@ const char *GermanExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "#qsave",
     "ausser",
     "bis",
+    "#flicker",
     "laden",
     "wiederherstellen",
     "transkript",
@@ -122,6 +124,7 @@ const char *SpanishExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "load",
     "#restore",
     "transcript",
+    "#transcript",
     "script",
     "#script",
     "oops",
@@ -140,6 +143,7 @@ const char *SpanishExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
     "#qsave",
     "excepto",
     "menos",
+    "#flicker",
     "reanuda",
     "cargar",
     "transcripcion",
@@ -149,16 +153,29 @@ const char *SpanishExtraCommands[NUMBER_OF_EXTRA_COMMANDS] = {
 
 extra_command ExtraCommandsKey[NUMBER_OF_EXTRA_COMMANDS] = {
     NO_COMMAND, RESTART, RESTART, SAVE, SAVE, RESTORE, RESTORE,
-    RESTORE, SCRIPT, SCRIPT, SCRIPT, UNDO, UNDO, UNDO, UNDO,
+    RESTORE, SCRIPT, SCRIPT, SCRIPT, SCRIPT, UNDO, UNDO, UNDO, UNDO,
     RAM, RAMLOAD, RAMLOAD, RAMLOAD, RAMLOAD, RAMLOAD, RAMSAVE,
-    RAMSAVE, RAMSAVE, RAMSAVE, EXCEPT, EXCEPT,
+    RAMSAVE, RAMSAVE, RAMSAVE, EXCEPT, EXCEPT, FLICKER,
     RESTORE, RESTORE, SCRIPT, UNDO, RESTART
 };
 
 const char *EnglishExtraNouns[NUMBER_OF_EXTRA_NOUNS] = {
-    NULL, "game", "story", "on", "off", "load",
-    "restore", "save", "move", "command", "turn",
-    "all", "everything", "it", " ", " ",
+    NULL,
+    "game",
+    "story",
+    "on",
+    "off",
+    "load",
+    "restore",
+    "save",
+    "move",
+    "command",
+    "turn",
+    "all",
+    "everything",
+    "it",
+    " ",
+    " ",
 };
 
 const char *GermanExtraNouns[NUMBER_OF_EXTRA_NOUNS] = {
@@ -341,8 +358,8 @@ glui32 *ToUnicode(const char *string)
             case '}':
                 unichar = 0xfc;
                 break;
-                case 12:
-                    unichar = 0xf6;
+            case 12:
+                unichar = 0xf6;
                 break;
             case '{':
                 unichar = 0xe4;
@@ -428,7 +445,8 @@ static char *FromUnicode(glui32 *unicode_string, int origlength)
     return result;
 }
 
-static int MatchYMCA(glui32 *string, int length, int index) {
+static int MatchYMCA(glui32 *string, int length, int index)
+{
     const char *ymca = "y.m.c.a.";
     int i;
     for (i = 0; i < 8; i++) {
@@ -466,53 +484,50 @@ void SplitIntoWords(glui32 *string, int length)
     for (int i = 0; string[i] != 0 && i < length && word_index < MAX_WORDS; i++) {
         foundspace = 0;
         switch (string[i]) {
-            case 'y':
-            {
-                int ymca = MatchYMCA(string, length, i);
-                if (ymca > 3)
-                {
-                    /* Start a new word */
-                    startpos[words_found] = i;
-                    wordlength[words_found] = ymca;
-                    words_found++;
-                    wordlength[words_found] = 0;
-                    i += ymca;
-                    if (i < length)
-                        foundspace = 1;
-                    lastwasspace = 0;
-                }
+        case 'y': {
+            int ymca = MatchYMCA(string, length, i);
+            if (ymca > 3) {
+                /* Start a new word */
+                startpos[words_found] = i;
+                wordlength[words_found] = ymca;
+                words_found++;
+                wordlength[words_found] = 0;
+                i += ymca;
+                if (i < length)
+                    foundspace = 1;
+                lastwasspace = 0;
             }
+        } break;
+            /* Unicode space and tab variants */
+        case ' ':
+        case '\t':
+        case '!':
+        case '?':
+        case '\"':
+        case 0x83: // ¿
+        case 0x80: // ¡
+        case 0xa0: // non-breaking space
+        case 0x2000: // en quad
+        case 0x2001: // em quad
+        case 0x2003: // em
+        case 0x2004: // three-per-em
+        case 0x2005: // four-per-em
+        case 0x2006: // six-per-em
+        case 0x2007: // figure space
+        case 0x2009: // thin space
+        case 0x200A: // hair space
+        case 0x202f: // narrow no-break space
+        case 0x205f: // medium mathematical space
+        case 0x3000: // ideographic space
+            foundspace = 1;
+            break;
+        case '.':
+        case ',':
+        case ';':
+            foundcomma = 1;
+            break;
+        default:
             break;
-                /* Unicode space and tab variants */
-            case ' ':
-            case '\t':
-            case '!':
-            case '?':
-            case '\"':
-            case 0x83: // ¿
-            case 0x80: // ¡
-            case 0xa0: // non-breaking space
-            case 0x2000: // en quad
-            case 0x2001: // em quad
-            case 0x2003: // em
-            case 0x2004: // three-per-em
-            case 0x2005: // four-per-em
-            case 0x2006: // six-per-em
-            case 0x2007: // figure space
-            case 0x2009: // thin space
-            case 0x200A: // hair space
-            case 0x202f: // narrow no-break space
-            case 0x205f: // medium mathematical space
-            case 0x3000: // ideographic space
-                foundspace = 1;
-                break;
-            case '.':
-            case ',':
-            case ';':
-                foundcomma = 1;
-                break;
-            default:
-                break;
         }
         if (!foundspace) {
             if (lastwasspace || foundcomma) {
@@ -572,11 +587,6 @@ void LineInput(void)
 
         unibuf[ev.val1] = 0;
 
-        if (Transcript) {
-            glk_put_string_stream_uni(Transcript, unibuf);
-            glk_put_char_stream_uni(Transcript, 10);
-        }
-
         lastwasnewline = 1;
 
         SplitIntoWords(unibuf, ev.val1);
@@ -591,7 +601,7 @@ void LineInput(void)
     return;
 }
 
- int WhichWord(const char *word, const char **list, int word_length,
+int WhichWord(const char *word, const char **list, int word_length,
     int list_length)
 {
     int n = 1;
@@ -768,10 +778,10 @@ static int FindExtaneousWords(int *index, int noun)
     if (list == NULL) {
         if (*index >= WordsInInput)
             *index = WordsInInput - 1;
-//        debug_print("FindExtaneousWords Error: I don't know what a \"%s\" is\n", CharWords[*index]);
+        //        debug_print("FindExtaneousWords Error: I don't know what a \"%s\" is\n", CharWords[*index]);
         CreateErrorMessage(sys[I_DONT_KNOW_WHAT_A], UnicodeWords[*index], sys[IS]);
     } else {
-//        debug_print("FindExtaneousWords Error: I don't understand\n");
+        //        debug_print("FindExtaneousWords Error: I don't understand\n");
         CreateErrorMessage(sys[I_DONT_UNDERSTAND], NULL, NULL);
     }
 
@@ -833,7 +843,7 @@ static struct Command *CommandFromStrings(int index, struct Command *previous)
         if (CurrentGame != GREMLINS_GERMAN && CurrentGame != GREMLINS_GERMAN_C64) {
             if (!previous) {
                 CreateErrorMessage(sys[I_DONT_KNOW_HOW_TO], UnicodeWords[i - 1],
-                                   sys[SOMETHING]);
+                    sys[SOMETHING]);
                 return NULL;
             } else {
                 verbindex = previous->verbwordindex;
@@ -856,10 +866,10 @@ static struct Command *CommandFromStrings(int index, struct Command *previous)
     if (i == WordsInInput) {
         if (lastverb) {
             return CreateCommandStruct(lastverb, verb, previous->verbwordindex, i,
-                                       previous);
+                previous);
         } else if (found_noun_at_verb_position) {
             CreateErrorMessage(sys[I_DONT_KNOW_HOW_TO], UnicodeWords[i - 1],
-                               sys[SOMETHING]);
+                sys[SOMETHING]);
             return NULL;
         } else {
             return CreateCommandStruct(verb, 0, i - 1, i, previous);
@@ -881,7 +891,7 @@ static struct Command *CommandFromStrings(int index, struct Command *previous)
         if (list == ExtraNouns && i < WordsInInput && noun - GameHeader.NumWords == ALL) {
             int stringlength = strlen(CharWords[i]);
             except = WhichWord(CharWords[i], ExtraCommands, stringlength,
-                                   NUMBER_OF_EXTRA_COMMANDS);
+                NUMBER_OF_EXTRA_COMMANDS);
         }
         if (ExtraCommandsKey[except] != EXCEPT && FindExtaneousWords(&i, noun) != 0)
             return NULL;
@@ -911,7 +921,7 @@ static struct Command *CommandFromStrings(int index, struct Command *previous)
         if (i < WordsInInput && verb - GameHeader.NumWords == ALL) {
             int stringlength = strlen(CharWords[i]);
             except = WhichWord(CharWords[i], ExtraCommands, stringlength,
-                               NUMBER_OF_EXTRA_COMMANDS);
+                NUMBER_OF_EXTRA_COMMANDS);
         }
         if (ExtraCommandsKey[except] != EXCEPT && FindExtaneousWords(&i, 0) != 0)
             return NULL;
@@ -969,7 +979,7 @@ static int CreateAllCommands(struct Command *command)
                 found = 1;
                 c->verb = command->verb;
                 c->noun = WhichWord(Items[i].AutoGet, Nouns, GameHeader.WordLength,
-                                    GameHeader.NumWords);
+                    GameHeader.NumWords);
                 c->item = i;
                 c->next = NULL;
                 c->nounwordindex = 0;
