@@ -5,19 +5,20 @@
 //  Created by Petter Sjölund on 2022-09-29.
 //
 
-#include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include "scott.h"
-#include "detectgame.h"
-#include "sagagraphics.h"
-#include "pcdraw.h"
-#include "atari8c64draw.h"
 #include "apple2draw.h"
+#include "atari8c64draw.h"
+#include "detectgame.h"
+#include "pcdraw.h"
+#include "sagagraphics.h"
+#include "scott.h"
 
 #include "saga.h"
 
+// clang-format off
 
 static const char *DOSFilenames[] =
   { "B01024R", "B01044I", "R0109",   "R0190",
@@ -35,7 +36,10 @@ static const char *DOSFilenames[] =
     "R0187",   "B01022R", "B01040R", "R0103",  "R0188",
     "B01023I", "B01042I", "R0104",   "R0189",  NULL };
 
-int LoadDOSImages(void) {
+// clang-format on
+
+int LoadDOSImages(void)
+{
     USImages = new_image();
 
     struct USImage *image = USImages;
@@ -105,12 +109,12 @@ uint8_t *ReadUSDictionary(uint8_t *ptr)
             Nouns[wordnum] = MemAlloc(charindex + 1);
             memcpy((char *)Nouns[wordnum], dictword, charindex + 1);
             debug_print("Nouns %d: \"%s\"\n", wordnum,
-                    Nouns[wordnum]);
+                Nouns[wordnum]);
         } else {
             Verbs[wordnum - nn] = MemAlloc(charindex + 1);
             memcpy((char *)Verbs[wordnum - nn], dictword, charindex + 1);
             debug_print("Verbs %d: \"%s\"\n", wordnum - nn,
-                    Verbs[wordnum - nn]);
+                Verbs[wordnum - nn]);
         }
         wordnum++;
 
@@ -123,7 +127,8 @@ uint8_t *ReadUSDictionary(uint8_t *ptr)
     return ptr;
 }
 
-int DrawUSImage(USImage *image) {
+int DrawUSImage(USImage *image)
+{
     last_image_index = image->index;
     if (image->systype == SYS_MSDOS)
         return DrawDOSImage(image);
@@ -134,7 +139,8 @@ int DrawUSImage(USImage *image) {
     return 0;
 }
 
-void DrawInventoryImages(void) {
+void DrawInventoryImages(void)
+{
     struct USImage *image = USImages;
     if (image != NULL) {
         do {
@@ -146,7 +152,8 @@ void DrawInventoryImages(void) {
     }
 }
 
-void DrawRoomObjectImages(void) {
+void DrawRoomObjectImages(void)
+{
     struct USImage *image = USImages;
     if (image != NULL) {
         do {
@@ -158,7 +165,8 @@ void DrawRoomObjectImages(void) {
     }
 }
 
-int DrawUSRoom(int room) {
+int DrawUSRoom(int room)
+{
     struct USImage *image = USImages;
     if (image != NULL) {
         do {
@@ -171,7 +179,8 @@ int DrawUSRoom(int room) {
     return 0;
 }
 
-void DrawUSRoomObject(int item) {
+void DrawUSRoomObject(int item)
+{
     struct USImage *image = USImages;
     if (image != NULL) {
         do {
@@ -196,28 +205,28 @@ void LookUS(void)
 
     if (CurrentGame == HULK_US && USImages && USImages->systype != SYS_APPLE2)
         switch (MyLoc) {
-            case 5: // Tunnel going outside
-            case 6:
-                room = 3;
-                break;
-            case 7: // Field
-            case 8:
-                room = 4;
-                break;
-            case 10: // Hole
-            case 11:
-                room = 9;
-                break;
-            case 13: // Dome
-            case 14:
-                room = 2;
-                break;
-            case 17: // warp
-            case 18:
-                room = 16;
-                break;
-            default:
-                break;
+        case 5: // Tunnel going outside
+        case 6:
+            room = 3;
+            break;
+        case 7: // Field
+        case 8:
+            room = 4;
+            break;
+        case 10: // Hole
+        case 11:
+            room = 9;
+            break;
+        case 13: // Dome
+        case 14:
+            room = 2;
+            break;
+        case 17: // warp
+        case 18:
+            room = 16;
+            break;
+        default:
+            break;
         }
 
     if (!DrawUSRoom(room)) {
@@ -290,11 +299,12 @@ static int SanityCheckScottFreeHeader(int ni, int na, int nw, int nr, int mc)
     return 1;
 }
 
-uint8_t *Skip(uint8_t *ptr, int count, uint8_t *eof) {
+uint8_t *Skip(uint8_t *ptr, int count, uint8_t *eof)
+{
 #if (DEBUG_PRINT)
     for (int i = 0; i < count && ptr + i + 1 < eof; i += 2) {
-        uint16_t val =  ptr[i] + ptr[i+1] * 0x100;
-        debug_print("Unknown value %d: %d (%x)\n", i/2, val, val);
+        uint16_t val = ptr[i] + ptr[i + 1] * 0x100;
+        debug_print("Unknown value %d: %d (%x)\n", i / 2, val, val);
     }
 #endif
     return ptr + count;
@@ -317,7 +327,7 @@ GameIDType LoadBinaryDatabase(uint8_t *data, size_t length, struct GameInfo info
     if (dict_start) {
         file_baseline_offset = dict_start - info.start_of_dictionary - 645;
         debug_print("LoadBinaryDatabase: file baseline offset:%d\n",
-                file_baseline_offset);
+            file_baseline_offset);
         offset = info.start_of_header + file_baseline_offset;
         ptr = SeekToPos(data, offset);
     } else {
@@ -357,7 +367,7 @@ GameIDType LoadBinaryDatabase(uint8_t *data, size_t length, struct GameInfo info
     ptr = ReadHeader(ptr);
 
     ParseHeader(header, US_HEADER, &ni, &na, &nw, &nr, &mc, &pr, &tr,
-                &wl, &lt, &mn, &trm);
+        &wl, &lt, &mn, &trm);
 
     PrintHeaderInfo(header, ni, na, nw, nr, mc, pr, tr, wl, lt, mn, trm);
 
@@ -496,7 +506,7 @@ GameIDType LoadBinaryDatabase(uint8_t *data, size_t length, struct GameInfo info
     }
 
     // Image strings lookup table
-    ptr = Skip(ptr, (ni + 1) * 4,  data + length);
+    ptr = Skip(ptr, (ni + 1) * 4, data + length);
 
 #pragma mark Actions
 
@@ -609,7 +619,6 @@ GameIDType LoadBinaryDatabase(uint8_t *data, size_t length, struct GameInfo info
     return info.gameID;
 }
 
-
 uint8_t *ReadFileIfExists(const char *name, size_t *size)
 {
     FILE *fptr = fopen(name, "r");
@@ -635,7 +644,8 @@ uint8_t *ReadFileIfExists(const char *name, size_t *size)
     return result;
 }
 
-int CompareFilenames(const char *str1, size_t length1, const char *str2, size_t length2) {
+int CompareFilenames(const char *str1, size_t length1, const char *str2, size_t length2)
+{
     while (length1 > 0 && str1[length1] != '.') {
         length1--;
     }
@@ -653,7 +663,8 @@ int CompareFilenames(const char *str1, size_t length1, const char *str2, size_t
     return 1;
 }
 
-const char *AddGameFileExtension(const char *filename, size_t gamefilelen, size_t *stringlength) {
+const char *AddGameFileExtension(const char *filename, size_t gamefilelen, size_t *stringlength)
+{
     char *new = NULL;
     size_t extpos = gamefilelen;
     while (extpos && game_file[extpos] != '.')
@@ -672,7 +683,8 @@ const char *AddGameFileExtension(const char *filename, size_t gamefilelen, size_
     return new;
 }
 
-const char *LookForCompanionFilenameInDatabase(const pairrec list[][2], size_t stringlen, size_t *stringlength2) {
+const char *LookForCompanionFilenameInDatabase(const pairrec list[][2], size_t stringlen, size_t *stringlength2)
+{
 
     for (int i = 0; list[i][0].filename != NULL; i++) {
         *stringlength2 = list[i][0].stringlength;
@@ -703,7 +715,8 @@ const char *LookForCompanionFilenameInDatabase(const pairrec list[][2], size_t s
     return NULL;
 }
 
-char *LookInDatabase(const pairrec list[][2], size_t stringlen) {
+char *LookInDatabase(const pairrec list[][2], size_t stringlen)
+{
     size_t resultlen;
     const char *foundname = LookForCompanionFilenameInDatabase(list, stringlen, &resultlen);
     if (foundname != NULL) {
