@@ -28,10 +28,6 @@
 #include "os.h"
 #include "glk.h"
 
-#ifdef GARGLK
-#include "garglk.h"                 /* for-size to-contents hack */
-#endif /* GARGLK */
-
 typedef struct os_banner_s *osbanid_t;
 typedef struct banner_contents_s *contentid_t;
 
@@ -294,11 +290,9 @@ void os_banner_styles_reset(osbanid_t banner)
         glk_stylehint_clear(wintype_AllTypes, style_BlockQuote, stylehint_BackColor);
     }
 
-#ifdef GARGLK
     /* reset our default colors with a superfluous hint */
     glk_stylehint_set(wintype_AllTypes, style_Normal, stylehint_TextColor, mainfg);
     glk_stylehint_set(wintype_AllTypes, style_Normal, stylehint_BackColor, mainbg);
-#endif /* GARGLK */
 }
 
 void os_banners_close(osbanid_t banner)
@@ -550,8 +544,7 @@ void os_banner_size_to_contents(void *banner_handle)
 #ifdef GARGLK
     if (banner->type == wintype_TextBuffer)
     {
-        int os_banner_size_garglk(winid_t);
-        int size = os_banner_size_garglk(banner->win);
+        int size = garglk_tads_os_banner_size(banner->win);
         os_banner_set_size(banner, size, OS_BANNER_SIZE_ABS, 0);
     }
 #endif /* GARGLK */
